#include "../publicKeyAlgs/Power/Windefs.h"
// #include "cCompressAlg.h"
#include "compressAlgs.h"
#include "cZLIBAlg.h"

typedef cCompressAlg *typeCreateAlg();
typedef unsigned int typeGetMaxCompressedSize( unsigned int size);

const char *GetCompressAlgName( BYTE algID )
{	switch ( algID )
	{
		case caID_ZIP	: return "ZIP";
		case caID_ZLIB	: return "ZLIB";
		case caID_BZip2 : return "BZip2";
		default : return NULL;
	}
}

const char *ca_getCompressAlgName( BYTE algID )
{
	return GetCompressAlgName( algID );
}

static int IDsTab[] = {
	caID_ZLIB,
};

int ca_getIDsOfAvailableAlgAlgorithms( int *pIDsCount, int **ppIDsTab )
{
	if ( pIDsCount != NULL )
		*pIDsCount = sizeof( IDsTab ) / sizeof( *IDsTab );

	if ( ppIDsTab != NULL )
		*ppIDsTab = IDsTab;

	return sae_ok;
}


struct sCompressAlgTabItem
{
	int algID;
	typeCreateAlg *createFunc;
	typeGetMaxCompressedSize *getMaxCompressedSizeFunc;
}
	CompressAlgTbl[] = 
		{
			caID_ZLIB, cCompressZLIB::createAlg, cCompressZLIB::getMaxCompressedSize,
		};


int ca_createAlgorithm( HCA *pHCA, int algID )
{
	int q = sizeof( CompressAlgTbl ) / sizeof( CompressAlgTbl[0] );

	while ( q-- )
	{
		if ( CompressAlgTbl[q].algID == algID )
		{
			*pHCA = CompressAlgTbl[q].createFunc();
			return cae_ok;
		}
	}

	return cae_invalidAlgorithmID;
}

unsigned int ca_getMaxCompressedSize( int algID, unsigned int originalSize )
{
	int q = sizeof( CompressAlgTbl ) / sizeof( CompressAlgTbl[0] );

	while ( q-- )
	{
		if ( CompressAlgTbl[q].algID == algID )
		{
			if ( CompressAlgTbl[q].getMaxCompressedSizeFunc == NULL )
				return 0;

			return CompressAlgTbl[q].getMaxCompressedSizeFunc( originalSize );
		}
	}

	return 0;
}

void ca_free( cCompressAlg *ha )
{
	delete ha;
}

int ca_compress( HCA hCA, BYTE *bfOut, int *pBfOutSize, const BYTE *bfIn, int bfInSize )
{
	return hCA->compress( bfOut, pBfOutSize, bfIn, bfInSize );
}

int ca_uncompress( HCA hCA, BYTE *bfOut, int *pBfOutSize, const BYTE *bfIn, int bfInSize )
{
	return hCA->uncompress( bfOut, pBfOutSize, bfIn, bfInSize );
}

int ca_compress( BYTE algID, BYTE *bfOut, int *pBfOutSize, const BYTE *bfIn, int bfInSize )
{
	HCA hca;
	int res = ca_createAlgorithm( &hca, algID );

	if ( res != cae_ok )
		return res;

	res = ca_compress( hca, bfOut, pBfOutSize, bfIn, bfInSize );

	ca_free( hca );

	return res;
}

int ca_uncompress( BYTE algID, BYTE *bfOut, int *pBfOutSize, const BYTE *bfIn, int bfInSize )
{
	HCA hca;
	int res = ca_createAlgorithm( &hca, algID );

	if ( res != cae_ok )
		return res;

	res = ca_uncompress( hca, bfOut, pBfOutSize, bfIn, bfInSize );

	ca_free( hca );

	return res;
}
