#include "cPEMCtx.h"
#include "..\PGPPkts\pktdefs.h"
#include "..\symmetricAlgs\symmetricAlgs.h"
#include "..\x509pkts\base64funcs.h"
#include "..\hashAlgs\hashAlgs.h"

/*

int PEM_do_header(EVP_CIPHER_INFO *cipher, unsigned char *data, long *plen,
	     pem_password_cb *callback,void *u)
	{
        ....

	EVP_BytesToKey(cipher->cipher,EVP_md5(),&(cipher->iv[0]),
		(unsigned char *)buf,klen,1,key,NULL);
        ....
}


int EVP_BytesToKey(const EVP_CIPHER *type, const EVP_MD *md,
	     const unsigned char *salt, const unsigned char *data, int datal,
	     int count, unsigned char *key, unsigned char *iv)
	{
	EVP_MD_CTX c;
	unsigned char md_buf[EVP_MAX_MD_SIZE];
	int niv,nkey,addmd=0;
	unsigned int mds=0,i;
	int rv = 0;
	nkey=type->key_len;
	niv=type->iv_len;
	OPENSSL_assert(nkey <= EVP_MAX_KEY_LENGTH);
	OPENSSL_assert(niv <= EVP_MAX_IV_LENGTH);

	if (data == NULL) return(nkey);

	EVP_MD_CTX_init(&c);
	for (;;)
		{
		if (!EVP_DigestInit_ex(&c,md, NULL))
			return 0;
		if (addmd++)
			if (!EVP_DigestUpdate(&c,&(md_buf[0]),mds))
				goto err;
		if (!EVP_DigestUpdate(&c,data,datal))
			goto err;
		if (salt != NULL)
			if (!EVP_DigestUpdate(&c,salt,PKCS5_SALT_LEN))
				goto err;
		if (!EVP_DigestFinal_ex(&c,&(md_buf[0]),&mds))
			goto err;

		for (i=1; i<(unsigned int)count; i++)
			{
			if (!EVP_DigestInit_ex(&c,md, NULL))
				goto err;
			if (!EVP_DigestUpdate(&c,&(md_buf[0]),mds))
				goto err;
			if (!EVP_DigestFinal_ex(&c,&(md_buf[0]),&mds))
				goto err;
			}
		i=0;
		if (nkey)
			{
			for (;;)
				{
				if (nkey == 0) break;
				if (i == mds) break;
				if (key != NULL)
					*(key++)=md_buf[i];
				nkey--;
				i++;
				}
			}
		if (niv && (i != mds))
			{
			for (;;)
				{
				if (niv == 0) break;
				if (i == mds) break;
				if (iv != NULL)
					*(iv++)=md_buf[i];
				niv--;
				i++;
				}
			}
		if ((nkey == 0) && (niv == 0)) break;
		}
	rv = type->key_len;
	err:
	EVP_MD_CTX_cleanup(&c);
	OPENSSL_cleanse(&(md_buf[0]),EVP_MAX_MD_SIZE);
	return rv;
}
*/

const int PKCS5_SALT_LEN = 8;
const int EVP_MAX_MD_SIZE = 512; // SHA-512
const int EVP_MAX_KEY_LENGTH = 32;
const int EVP_MAX_IV_LENGTH = 16;

int EVP_BytesToKey(
		HHA md, // = MD5
		const unsigned char *salt, // salt buffer 8 bytes 
		const unsigned char *data, int datal,
		int count,	// iteration count = 1
		unsigned char *key, int key_len,	// [out] key buffer
		unsigned char *iv, int iv_len		// [out] iv buffer
	)
{
	if ( ( data == NULL ) || ( datal == 0 ) )
		return 0;

	unsigned char md_buf[ EVP_MAX_MD_SIZE ];

	int addmd=0;
	unsigned int mds=0;
	unsigned int i;
	int rv = 0;

	if ( key_len > EVP_MAX_KEY_LENGTH ) return -1;
	if ( iv_len  > EVP_MAX_IV_LENGTH )  return -1;

	int nkey = key_len;
	int niv = iv_len;

	for (;;)
	{
		if ( ha_init( md ) != hae_ok )
			return 0;

		if ( addmd++ )
			if ( ha_process( md, &(md_buf[0]), mds ) != hae_ok )
				goto err;

		if ( ha_process( md, data, datal ) != hae_ok )
			goto err;

		if ( salt != NULL )
			if ( ha_process( md, salt, PKCS5_SALT_LEN ) != hae_ok )
				goto err;

		if ( ha_final( md, &(md_buf[0]) ) != hae_ok )
			goto err;

		mds = ha_getDigestSize( md );

		for ( i=1; i<(unsigned int)count; i++)
		{
			if ( ha_init( md ) != hae_ok )
				goto err;

			if ( ha_process( md, &(md_buf[0]), mds ) )
				goto err;

			if ( ha_final( md, &(md_buf[0]) ) != hae_ok )
				goto err;
		}

		i=0;
		if (nkey)
			{
			for (;;)
				{
				if (nkey == 0) break;
				if (i == mds) break;
				if (key != NULL)
					*(key++)=md_buf[i];
				nkey--;
				i++;
				}
			}
		if (niv && (i != mds))
			{
			for (;;)
				{
				if (niv == 0) break;
				if (i == mds) break;
				if (iv != NULL)
					*(iv++)=md_buf[i];
				niv--;
				i++;
				}
			}
		if ((nkey == 0) && (niv == 0)) break;
	}

	rv = key_len;
	err:
	memset( md_buf, 0, sizeof( md_buf ) );
	return rv;
}

//
//	findHeaderValue
//		returns pointer to header value if found.
//	Header sample 'Proc-Type: 4,ENCRYPTED\0x0aDEK-Info: DES-EDE3-CBC,3F17F5316E2BAC89\0x0a\0x0a'
//	if szItem == 'DEK-Info:', so return pointer 'DES-EDE3-CBC,3F17F5316E2BAC89\0x0a...'
//
static const char *findHeaderValue( const char *szItem, const char *szHeader )
{
	int len = strlen( szItem );

	if ( *szHeader == 0x0a )
		return NULL;

	if ( strncmp( szHeader, szItem, len ) == 0 )
	{
		while( szHeader[len] == ' ' )
			len++;

		return szHeader + len;
	}

	//
	//	Get next string
	//

	const char *nextHeader = NULL;

	for ( int i = 0; ; i++ )
	{
		if ( i > 100 )
			return NULL;

		if ( szHeader[i] == 0x0A )
		{
			nextHeader = szHeader + i + 1;
			break;
		}
	}

	return findHeaderValue( szItem, nextHeader );
}

struct stringBlock
{
	const char *B;
	int L;

	bool equ( const char *value )
	{
		int len = strlen( value );
		if ( len != L )
			return false;

		return strncmp( B, value, len ) == 0;
	}
};

static int findSymbol( const char *str, char ch )
{
	for( int i = 0; ; i++ )
	{
		if ( i > 100 )
			return i;

		if ( str[i] == 0 )
			return i;

		if ( str[i] == ch )
			return i;
	}
}

//
//	parseHeaderArgs
//	Sample:
//		headerValue -> 'DES-EDE3-CBC,3F17F5316E2BAC89\0x0a...'
//		separator -> ','
//		result
//			argT[0].B -> 'DES-EDE3-CBC,...'
//			argT[0].L -> 13
//			argT[1].B -> '3F17F5316E2BAC89\0x0a...'
//			argT[0].L -> 17
//		return 2.
//
static int parseHeaderArgs( stringBlock *argT, int argQ, const char *headerValue, int len = -1, const char separator = ',' )
{
	if ( argQ == 0 )
		return 0;

	if ( len == -1 )
		len = findSymbol( headerValue, 0x0a );

	while( *headerValue == ' ' )
	{
		++headerValue;
		--len;
	}

	argT[0].B = headerValue;

	const char *nextValue = NULL;

	for( int i = 0; ; i++ )
	{
		if ( i > 100 )
			return 0;

		if ( i == len )
		{
			argT[0].L = i;
			return 1;
		}

		if ( headerValue[i] == separator )
		{
			argT[0].L = i;

			nextValue = headerValue + i + 1;

			return 1 + parseHeaderArgs( argT + 1, argQ - 1, nextValue, len - ( i + 1 ), separator );
		}
	}
}

static bool hex2byte( BYTE *pByte, const char *hexStr )
{
	*pByte = 0;
	for( int i = 0; i<2; i++ )
	{
		BYTE bt;
		char h = hexStr[i];

		if ( ( h >= '0' ) && ( h <= '9' ) )
			bt = h - '0';
		else
		if ( ( h >= 'A' ) && ( h <= 'F' ) )
			bt = h - 'A' + 10;
		else
		if ( ( h >= 'a' ) && ( h <= 'f' ) )
			bt = h - 'a' + 10;
		else
			return false;

		*pByte = ( *pByte << 4 ) | bt ;
	}

	return true;
}

//
//	If success Returns size of parsed bytes
//	Otherwise returns -1
//
static int string2bytes( BYTE *bf, int bfSize, const char *str )
{
	int len = (int)strlen( str );
	int pos = 0;

	for( int i = 0; i < len; i++ )
	{
		char ch = str[i];
		if ( ch == ' ' )
			continue;

		if ( i + 1 > len )
			return pos;

		BYTE bt;
		if ( !hex2byte( &bt, str + i++ ) )
			return pos;

		if ( ( bf != NULL ) && ( bfSize > pos ) )
			bf[pos] = bt;

		pos++;
	}

	return pos;
}

//
//	getRealSize
//		checks padding and cut off padded size
//	RFC 1423,
//	In hexadecimal, the possible  paddings are:  01, 0202, 030303, 04040404, 0505050505, 060606060606, 07070707070707, and 0808080808080808
//
static int getRealSize( int *pRealSize, BYTE *bf, int paddedSize )
{
	int size = paddedSize;

	if ( size == 0 )
		return ppe_invalidBlock;

	BYTE pd = bf[ size - 1 ];

	if ( pd > 0x16 )
		return ppe_invalidBlock;

	if ( size < pd )
		return ppe_invalidBlock;

	size = paddedSize - pd;

	for ( int i = size; i < paddedSize; i++ )
	{
		if ( bf[ i ] != pd )
			return ppe_invalidBlock;
	}

	if ( pRealSize != NULL )
		*pRealSize = size;

	return ppe_ok;
}

int cPEMCtx::decrypt( cPTB &decBlock, const cPassword &pwd, const unsigned char *body, int bodySize )
{
	int res = bytesToKey( (const BYTE*)pwd.buffer, pwd.getLengthUTF8() );

	if ( res != ppe_ok )
		return res;

	return decrypt( decBlock, body, bodySize );
}

int cPEMCtx::bytesToKey( const BYTE *bf, int bfSize )
{
	int algKeySize = sa_getKeySize( m_algId );
	if ( algKeySize == 0 )
		return sae_invalidAlgorithmID;

	if ( m_ptbKey.Allocate( algKeySize ) == NULL )
		return RSA_MALLOC_ERROR;

	if ( m_saltSize != PKCS5_SALT_LEN )
		return x509e_invalidPkt;

	HHA ha;
	int res = ha_createAlgorithm( &ha, pgphaMD5 );
	
	if ( res != hae_ok )
		return res;

	res = EVP_BytesToKey(
			ha,
			m_salt,
			bf, bfSize,
			1,
			m_ptbKey.buffer, m_ptbKey.bufferSize,
			NULL, 0 );

	ha_free( ha );

	if ( res <= 0 )
		return x509e_internalError;

	return ppe_ok;
}

bool cPEMCtx::isPlainText()
{
	return m_algId == pgpsaPlaintext;
}

int cPEMCtx::parseHeaders( const char *armorHeaders )
{
	const char *procType = findHeaderValue( "Proc-Type:", armorHeaders );
	const char *dekInfo = findHeaderValue( "DEK-Info:", armorHeaders );

	stringBlock ptT[2];
	if ( parseHeaderArgs( ptT, maxLen( ptT ), procType, -1, ',' ) != 2 )
		return RSA_INVALID_BLOCK_TYPE;

	if ( !ptT[0].equ( "4" ) || !ptT[1].equ( "ENCRYPTED" ) )
	{
		m_algId = pgpsaPlaintext;
		return ppe_ok;
	}

	stringBlock diT[2];
	if ( parseHeaderArgs( diT, maxLen( diT ), dekInfo, -1, ',' ) != 2 )
		return RSA_INVALID_BLOCK_TYPE;

	stringBlock algT[3];
	int aiQ = parseHeaderArgs( algT, maxLen( algT ), diT[0].B, diT[0].L, '-' );
		
	if ( aiQ <= 2 )
		return RSA_INVALID_BLOCK_TYPE;

	if ( !algT[aiQ-1].equ( "CBC" ) )
		return sae_invalidAlgorithmID;

	if ( diT[0].equ( "AES-256-CBC" ) )
		m_algId = pgpsaAES_256;
	else
	if ( diT[0].equ( "DES-EDE3-CBC" ) )
		m_algId = pgpsaTripleDES;
	else
		return sae_invalidAlgorithmID;

	if ( m_ptbSalt.Allocate( diT[1].L / 2 ) == NULL )
		return sae_memoryAllocationError;

	int pos = string2bytes( m_ptbSalt.buffer, m_ptbSalt.bufferSize, diT[1].B );

	if ( pos != m_ptbSalt.bufferSize )
		return RSA_INVALID_KEY_BLOCK;

	m_salt = m_ptbSalt.buffer;
	m_saltSize = m_ptbSalt.bufferSize;

	return ppe_ok;
}

int cPEMCtx::decrypt( cPTB &decBlock, const unsigned char *body, int bodySize )
{
	HSA hsa;
	
	int res = sa_createAlgorithm( &hsa, m_algId, m_ptbKey.buffer, m_ptbKey.bufferSize );
	if ( res != ppe_ok )
		return res;

	if ( sa_getBlockSize( hsa ) != m_saltSize )
		return x509e_invalidPktSize;

	if ( decBlock.Allocate( bodySize ) == NULL )
		return RSA_MALLOC_ERROR;

	res = sa_decryptCBC( hsa, m_salt, decBlock.buffer, body, bodySize );

	sa_free( hsa );

	if ( res != sae_ok )
		return res;

	int realSize = 0;

	res = getRealSize( &realSize, decBlock.buffer, bodySize );

	if ( res != sae_ok )
		return ppe_invalidPassword;

	if ( realSize > bodySize )
		return sae_invalidBlockSize;

	decBlock.ReAllocate( realSize );

	return sae_ok;
}

//static 
int cPEMCtx::decodeBase64Block( cPTB &ptb, BYTE *ptr, int size, const char **pArmorHeaders )
{
	int encSize = 0;
	const BYTE *encBlock = NULL;

	int res = findEncodedBlock( 
			&encBlock, &encSize,
			ptr, size,
			"-----BEGIN RSA PRIVATE KEY-----",
			"-----END RSA PRIVATE KEY-----",
			pArmorHeaders );

	if ( res != ppe_ok )
		return res;

	int decSize = 0;
	
	res = base64Decode( NULL, &decSize, encBlock, encSize );
	if ( res != ppe_bufferTooSmall )
		return res;

	if ( ptb.Allocate( decSize ) == NULL )
		return RSA_MALLOC_ERROR;

	if ( decSize != ptb.bufferSize )
		return RSA_INIT_ERROR;

	res = base64Decode( ptb.buffer, &decSize, encBlock, encSize );

	return res;
}