#include "Packets.h"
#include "PGPPkts.h"
#include "PKTSignature.h"
#include "string2key.h"
#include "../publicKeyAlgs/publicKeyAlgs.h"
#include "../symmetricAlgs/symmetricAlgs.h"
#include <assert.h>
#include "../RNG/cRNG.h"
#include "../CompressAlgs/compressAlgs.h"
#include "../hashAlgs/hashAlgs.h"
#include "../publicKeyAlgs/cPKA.h"

#ifndef _WIN32
#include "../unix/misc.h"
#endif

class cPacket
{
public:
	BYTE tag;
	BYTE *data;
	int dataSize;
	int hdrSize;
	bool partial;

	int count;
	int countID;
	int subPktID;
	int subKeyPktID;

	bool m_stop;

	virtual ~cPacket()
	{

	}

	cPacket( int pktID = 0, int subKeyID = 0 )
	{
		tag = 0;
		data = NULL;
		dataSize = 0;
		hdrSize = 0;
		partial = false;

		count = 0;
		countID = pktID;
		subKeyPktID = subKeyID;

		subPktID = 0;

		m_stop = false;
	}

	virtual int Actor()
	{
		count++;
		return ppe_ok;
	}
};

static int walkPackets( BYTE *pktBuff, int pktBuffSize, cPacket *cPkt, int *pParsed = NULL )
{
	int res = ppe_ok;
//	int recCnt = 1;
	int parsed = 0;

	do
	{
		if ( cPkt->m_stop )
			break;

		do
		{
			res = pgp_PacketHeader_Parse( &cPkt->tag, &cPkt->hdrSize, &cPkt->dataSize, &cPkt->partial, pktBuff + parsed, pktBuffSize );

			if ( res != ppe_ok )
				return res;

			cPkt->data = pktBuff + parsed + cPkt->hdrSize;

			parsed += cPkt->dataSize + cPkt->hdrSize;

#ifdef _DEBUG
			const char *pktName;
			PKT_GetTagName( cPkt->tag, &pktName );
#endif // _DEBUG

			if ( !cPkt->countID  // If countID == 0 uses actor for all packets...
				|| ( cPkt->tag == cPkt->countID ) )
			{
				res = cPkt->Actor();

				if ( res != ppe_ok )
					return res;
			}
			else
			if ( cPkt->subPktID && ( cPkt->subPktID == cPkt->tag ) )
			{
				res = cPkt->Actor();

				if ( res != ppe_ok )
					return res;

			}
			else
			if ( cPkt->subKeyPktID && ( cPkt->subKeyPktID == cPkt->tag ) )
			{
				res = cPkt->Actor();

				if ( res != ppe_ok )
					return res;

			}
		}
		while ( cPkt->partial );
	}
	while( parsed < pktBuffSize );

	if ( pParsed )
		*pParsed = parsed;

	return res;
}

//#include <windows.h>
//#include <stdio.h>

//
//	Function:
//		pgp_EnumKeyIDs
//	Enumerates public key ID's of all keys in specified packet or key ring.
//
int pgp_EnumKeyIDs(
	BYTE *keyIDBuff,		// OUT - buffer for key IDs, 
							//  it is filled as sequence of available keyIDs with keyIDSize.
							// If returns pgpe_ok the *pKeyIDBuffSize / keyIDSize is number of 
							//	really filled key IDs
	int  *pKeyIDBuffSize,	// IN  - pointer to allocated size of key buffer
							// OUT - pointer to required size if ppe_bufferTooSmall returned

	BYTE *keyRingBuff,		// IN - Key ring packet to be parsed for available keys.
	int  keyRingBuffSize,
	bool secret
	)
{
	//
	//	Counts all public key packets..
	//
	cPacket cntPkt( secret ? pkt_SecretKey : pkt_PublicKey );

	int res = walkPackets( keyRingBuff, keyRingBuffSize, &cntPkt );

	if ( res != ppe_ok )
		return res;

	if ( cntPkt.count * pgp_keyIDSize > *pKeyIDBuffSize )
	{
		*pKeyIDBuffSize = cntPkt.count * pgp_keyIDSize;
		return ppe_bufferTooSmall;
	}

	*pKeyIDBuffSize = cntPkt.count * pgp_keyIDSize;

	if ( cntPkt.count == 0 )
		return ppe_ok;

	if ( keyIDBuff == NULL )
	{
		*pKeyIDBuffSize = 0;
		return ppe_internalError;
	}

	//
	//	Parses public key packets and fills keyID buffer...
	//
	class cEnumKeys : public cPacket
	{
		BYTE *m_keyIDBuff;
		int m_keyIDBuffSize;
		int m_keyNum;

	public:
		cEnumKeys( BYTE *keyIDBuff, int keyIDBuffSize, bool secret )
			: cPacket(
				secret ? pkt_SecretKey : pkt_PublicKey )
//				,secret ? pkt_SecretSubkey : pkt_PublicSubkey  )
		{
			m_keyIDBuff = keyIDBuff;
			m_keyIDBuffSize = keyIDBuffSize;
			m_keyNum = 0;
		}

		int Actor()
		{
//	MessageBox( 0, "Point 1", "pgp_EnumKeyIDs::Actor", MB_OK );
			if ( ( m_keyNum + 1 ) * pgp_keyIDSize > m_keyIDBuffSize )
				return ppe_internalError;

//	char message[ 1024 ];
//	sprintf( message, "%X, %d, %X, %d", data, dataSize, m_keyIDBuff, m_keyNum * pgp_keyIDSize );
//	MessageBox( 0, message, "pgp_EnumKeyIDs::Actor", MB_OK );
			int res = PKT_GetPublicKeyIDInternal(
						(sPublicKey *)data,
						dataSize,
						m_keyIDBuff + m_keyNum * pgp_keyIDSize
					);

			if ( res == ppe_ok )
				m_keyNum++;
			else
			if ( res == ppe_invalidPacketVersion )
				return ppe_ok;

			return res;
		}

	} enumPK( keyIDBuff, *pKeyIDBuffSize, secret );

//	char message[ 1024 ];
//	sprintf( message, "%X, %d, %X - %X, %d", keyRingBuff, keyRingBuffSize, &enumPK, keyIDBuff, *pKeyIDBuffSize );
//	MessageBox( 0, message, "pgp_EnumKeyIDs::Walk", MB_OK );

	return walkPackets( keyRingBuff, keyRingBuffSize, &enumPK );
}

//
//	pgp_GetPKData
//	Extracts first packet data from key ring packet
//	if pTag != NULL and *pTag != 0 returns ppe_invalidBlockType if packed tag not equal to assigned *pTag
//	f.e. if *pTag == pkt_PublicKey it automatically check if the packed is Public Key
//
int pgp_GetPktData( BYTE *pTag, BYTE **pData, int *pDataSize, BYTE *pktBuff, int pktBuffSize, int *pParsed )
{
	BYTE tag;
	BYTE *data;
	int dataSize;
	int hdrSize;
	bool partial = false;

	int parsed = 0;
	if ( pParsed != NULL )
		parsed = *pParsed;

	int res = pgp_PacketHeader_Parse( &tag, &hdrSize, &dataSize, &partial, pktBuff + parsed, pktBuffSize - parsed );

	if ( res != ppe_ok )
		return res;

	data = pktBuff + parsed + hdrSize;
	parsed += dataSize + hdrSize;

	if ( parsed > pktBuffSize )
		return ppe_invalidPacketSize;

	if ( pParsed	!= NULL ) *pParsed = parsed;
	if ( pTag		!= NULL ) *pTag = tag;
	if ( pData		!= NULL ) *pData = data;
	if ( pDataSize	!= NULL ) *pDataSize = dataSize;

	return ppe_ok;
}

static int compareKeyID( BYTE *keyID, BYTE *pkt, int pktSize )
{
	BYTE newKeyID[ pgp_keyIDSize ];

	int res = PKT_GetPublicKeyIDInternal(
				(sPublicKey *)pkt,
				pktSize,
				newKeyID
			);

	if ( res != ppe_ok )
		return res;

	// v.1.04.4 - 25.11.2014 sizeof( keyID ) );
	return memcmp( keyID, newKeyID, pgp_keyIDSize );
}

//
//	Function:
//		pgp_FindKeyByID
//	Imports public key with specified 8 BYTEs identificator keyID from a packet or key ring.
//
int pgp_FindKeyByID(
	BYTE **pKeyBuff,	// OUT - pointer to position of the key in common packet.
	int  *pKeyBuffSize,	// OUT - pointer to size of the key packet
	BYTE *keyID,		// IN  - 8 BYTEs identificator of key to be imported from following key ring.
						// Use pgp_EnumKeyIDs to get buffer with ID of available keys
	BYTE *keyRingBuff,  // IN  - Key ring packet.
	int keyRingBuffSize,// IN  - Key ring buffer size.

	bool isSecret,	// = false, if true finds secret key with specified ID otherwise finds public

	BYTE **pSignaturePkt,	// OUT - it is filled if the key ring contains a following signature
	int *pSignaturePktSize,	// OUT - the signature packet size.

	BYTE **pUserIDPkt,		// OUT - it is filled if the key ring contains a following user information
	int *pUserIDPktSize,	// OUT - the user information packet size.

	BYTE **pImgPkt,			//	OUT - Image packet.
	int *pImgPktSize		// 

//	BYTE *masterKeyID		// Buffer for master key ID
	)
{
	class cFindKeys : public cPacket
	{
	public:

		BYTE *m_keyID; // Key ID to be find.

		BYTE *m_keyPkt;
		int m_keyPktSize;

		BYTE *m_sigPkt;
		int m_sigPktSize;

		BYTE *m_userIDPkt;
		int m_userIDPktSize;

		BYTE *m_imgPkt;
		int m_imgPktSize;

		BYTE *m_subKeyPkt;
		int m_subKeyPktSize;

		cFindKeys( bool isSecret, BYTE *keyID ) :
			cPacket(
				isSecret ? pkt_SecretKey : pkt_PublicKey,
				isSecret ? pkt_SecretSubkey : pkt_PublicSubkey )
		{
			m_keyPkt = NULL;
			m_keyPktSize = 0;

			m_sigPkt = NULL;
			m_sigPktSize = 0;

			m_userIDPkt = NULL;
			m_userIDPktSize = 0;

			m_imgPkt = NULL;
			m_imgPktSize = 0;

			m_keyID = keyID;

			m_subKeyPkt = NULL;
		}

		int Actor()
		{
			if ( tag == pkt_Image ) 
			{
				m_imgPkt = data;
				m_imgPktSize = dataSize;

				subPktID = 0; // Reset the finding of followed packets.
				return ppe_ok;
			}
			else
			if ( tag == pkt_Signature ) 
			{
				if ( m_subKeyPkt ) // Finds signture of subpacket
				{
					m_sigPkt = data;
					m_sigPktSize = dataSize;
					subPktID = 0; // Stop finding subpackets
				}
				else
				{
					m_sigPkt = data;
					m_sigPktSize = dataSize;

					subPktID = pkt_Image; // Try to find a user image
				}

				return ppe_ok;
			}
			else
			if ( tag == pkt_UserID )
			{
				m_userIDPkt = data;
				m_userIDPktSize = dataSize;

				subPktID = pkt_Signature; // Find followed signature packet.
				return ppe_ok;
			}
			else
			if ( tag == subKeyPktID )
			{
				if ( compareKeyID( m_keyID, data, dataSize ) == 0 )
				{
					m_subKeyPkt = data;
					m_subKeyPktSize = dataSize;

					subPktID = pkt_Signature; // Find followed signature packet.
					subKeyPktID = 0; // Stop finding
				}
				else
					subPktID = 0;

				return ppe_ok;
			}
			else // pkt_SecretKey OR pkt_PublicKey
			{
				if ( compareKeyID( m_keyID, data, dataSize ) == 0 )
				{
					m_keyPkt = data;
					m_keyPktSize = dataSize;

					subPktID = pkt_UserID; // Find followed user ID packet
				}
				else
				{
					if ( subKeyPktID != 0 ) // Tries to find specified ID in subpackets.
					{
						m_keyPkt = data;
						m_keyPktSize = dataSize;
						subPktID = pkt_UserID; // Finds all data
					}
					else
						subPktID = 0; // Reset the finding of followed signature.
				}

				return ppe_ok;
			}
		}

	} findKey( isSecret, keyID );

	int res = walkPackets( keyRingBuff, keyRingBuffSize, &findKey );

	if ( res != ppe_ok )
		return res;

	//
	// Checks subkey first. It is Null if subkey with specified ID was not found
	//
	if ( findKey.m_subKeyPkt != NULL ) 
	{
		*pKeyBuff = findKey.m_subKeyPkt;
		*pKeyBuffSize = findKey.m_subKeyPktSize;
	}
	else
	{
		//
		//	Checks if keyID is same. It can be different if subkeys was parsed.
		//
		if ( compareKeyID( findKey.m_keyID, findKey.m_keyPkt, findKey.m_keyPktSize ) )
			return ppe_keyNotFound;

		*pKeyBuff = findKey.m_keyPkt;
		*pKeyBuffSize = findKey.m_keyPktSize;
	}

	if ( pSignaturePkt != NULL )
		*pSignaturePkt = findKey.m_sigPkt;

	if ( pSignaturePktSize != NULL )
		*pSignaturePktSize = findKey.m_sigPktSize;

	if ( pUserIDPkt != NULL )
		*pUserIDPkt = findKey.m_userIDPkt;

	if ( pUserIDPktSize != NULL )
		*pUserIDPktSize = findKey.m_userIDPktSize;

	if ( pImgPkt != NULL )
		*pImgPkt = findKey.m_imgPkt;

	if ( pImgPktSize != NULL )
		*pImgPktSize = findKey.m_imgPktSize;

	return res;
}

//
//	Function:
//		pgp_FindKeyByIDEx
//	It finds key with subkeys, even if a subkey ID was specified. 
//	User has to search subpackets if key was found but the master key ID 
//	is not equal to specified key.
//

void cpgp_keyInfo::init()
{
	memset( this, 0, sizeof( *this ) );
}

cpgp_pkt::cpgp_pkt()
{
	memset( this, 0, sizeof( *this ) );
}


class cFindKeysEx : public cPacket
{
public:
	bool m_found;
	BYTE *m_keyID; // Key ID to be find.
	char *m_userID;
	cpgp_keyInfo *m_cKey;
	bool m_secret;
	cpgp_keyInfo **m_pSubKey;

	cFindKeysEx( bool isSecret, BYTE *keyID, char *userID, cpgp_keyInfo *cKey, cpgp_keyInfo **pSubKey = NULL)
	{
		m_secret = isSecret;
		m_keyID = keyID;
		m_userID = userID;
		m_found = false;
		m_cKey = cKey;
		m_pSubKey = pSubKey;
	}

	int Actor()
	{
		if ( tag == pkt_Image ) 
		{
			cpgp_pkt *pkt = new cpgp_pkt( data, dataSize );

			if ( m_cKey->next )
			{
//				m_cKey->next->imgPkt = data;
//				m_cKey->next->imgPktSize = dataSize;
				pkt->next = m_cKey->next->img;
				m_cKey->next->img = pkt;
			}
			else
			{
//				m_cKey->imgPkt = data;
//				m_cKey->imgPktSize = dataSize;
				pkt->next = m_cKey->img;
				m_cKey->img = pkt;
			}
			return ppe_ok;
		}
		else
		if ( tag == pkt_Signature ) 
		{
			cpgp_pkt *pkt = new cpgp_pkt( data, dataSize );

			if ( m_cKey->next )
			{
//				m_cKey->next->sigPkt = data;
//				m_cKey->next->sigPktSize = dataSize;
				pkt->next = m_cKey->next->sig;
				m_cKey->next->sig = pkt;
			}
			else
			{
//				m_cKey->sigPkt = data;
//				m_cKey->sigPktSize = dataSize;
				pkt->next = m_cKey->sig;
				m_cKey->sig = pkt;
			}

			return ppe_ok;
		}
		else
		if ( tag == pkt_UserID )
		{
			if ( m_userID && ( memcmp( m_userID, data, dataSize ) == 0 ) )
			{
				m_found = true;
			}

			cpgp_pkt *pkt = new cpgp_pkt( data, dataSize );

			if ( m_cKey->next )
			{
//				m_cKey->next->userIDPkt = data;
//				m_cKey->next->userIDPktSize = dataSize;
				pkt->next = m_cKey->next->userID;
				m_cKey->next->userID = pkt;
			}
			else
			{
//				m_cKey->userIDPkt = data;
//				m_cKey->userIDPktSize = dataSize;
				pkt->next = m_cKey->userID;
				m_cKey->userID = pkt;
			}

			return ppe_ok;
		}
		else
		if ( tag == ( m_secret ? pkt_SecretSubkey: pkt_PublicSubkey ) )
		{
			cpgp_keyInfo *subKey = new cpgp_keyInfo;

			if ( subKey == NULL ) return ppe_memoryAllocation;

			subKey->next = m_cKey->next;
			m_cKey->next = subKey;

			int res = PKT_GetPublicKeyIDInternal(
						(sPublicKey *)data,
						dataSize,
						m_cKey->next->keyID
					);

			if ( res != ppe_ok ) return res;

			if ( m_keyID && ( memcmp( m_keyID, m_cKey->next->keyID, pgp_keyIDSize ) == 0 ) )
			{
				m_found = true;
				if ( m_pSubKey != NULL )
					*m_pSubKey = subKey;
			}

			m_cKey->next->keyPkt = data;
			m_cKey->next->keyPktSize = dataSize;

			return ppe_ok;
		}
		else
		if ( tag == ( m_secret ? pkt_SecretKey : pkt_PublicKey ) ) // Main key packet
		{
			if ( m_found )
			{
				m_stop = true;
				return ppe_ok;
			}

			m_cKey->freeSubitems(); // Specified key can be collected in subpuckets

			int res = PKT_GetPublicKeyIDInternal(
						(sPublicKey *)data,
						dataSize,
						m_cKey->keyID
					);

			if ( res != ppe_ok ) return res;

			if ( m_keyID && ( memcmp( m_keyID, m_cKey->keyID, pgp_keyIDSize ) == 0 ) )
			{
				m_found = true;
			}

			m_cKey->keyPkt = data;
			m_cKey->keyPktSize = dataSize;

			return ppe_ok;
		}

		return ppe_ok;
	}

};

int pgp_FindKeyByIDEx(
	cpgp_keyInfo *cKey,	// OUT - struct with class info to be filled. 
	BYTE *keyID,		// IN  - 8 BYTEs identificator of key to be imported from following key ring.
						// Use pgp_EnumKeyIDs to get buffer with ID of available keys
	BYTE *keyRingBuff,  // IN  - Key ring packet.
	int keyRingBuffSize,// IN  - Key ring buffer size.

	bool isSecret, 	// = false, if true finds secret key with specified ID otherwise finds public
	cpgp_keyInfo **pSubkeyInfo // = NULL - Pointer to found subkey with specified key ID, the item is linked to common list to kiyInfo
	)
{
	cFindKeysEx findKey( isSecret, keyID, NULL, cKey, pSubkeyInfo );

	if ( pSubkeyInfo != NULL )
		*pSubkeyInfo = NULL;

	int res = walkPackets( keyRingBuff, keyRingBuffSize, &findKey );

//	if ( findKey.m_found && ( res == ppe_invalidHeader ) ) // Some key packets contain additional information attached to tail
//		return ppe_ok;

	if ( ( res == ppe_ok ) && !findKey.m_found )
		return ppe_keyNotFound;

	return res;
}

bool pgp_GetKeyUserID( 
	cPlainTextBlock *ptbUserID,
	cpgp_keyInfo *masterKeyInfo,
	cpgp_keyInfo *subKeyInfo
	)
{
	bool useSubkey = ( subKeyInfo != masterKeyInfo );

	const char *text = "Undefined";

	if ( subKeyInfo->userID == NULL )
	{
		if ( useSubkey && ( masterKeyInfo->userID != NULL ) )
		{
			ptbUserID->WriteString( "Subkey of " ); 
			ptbUserID->WriteBytes( masterKeyInfo->userID->body, masterKeyInfo->userID->size );
		}
		else
		{
			assert( false );
			ptbUserID->WriteBytes( (BYTE *)text, strlen( text ), true );
			return false;
		}
	}
	else
	if ( !ptbUserID->WriteBytes( masterKeyInfo->userID->body, masterKeyInfo->userID->size ) )
	{
		assert( false );
		ptbUserID->WriteBytes( (BYTE *)text, strlen( text ), true );
		return false;
	}

	return true;
}

//
//	Function:
//		pgp_FindKeyByUserID
//	It finds key with subkeys. 
//	User has to search subpackets if key was found but the master key ID 
//	is not equal to specified key.
//
int pgp_FindKeyByUserID(
	cpgp_keyInfo *cKey,	// OUT - struct with class info to be filled. 
	char *userID,		// IN  - null terminated string with user ID
	BYTE *keyRingBuff,  // IN  - Key ring packet.
	int keyRingBuffSize,// IN  - Key ring buffer size.

	bool isSecret // = false // if true finds secret key with specified ID otherwise finds public
	)
{
	cFindKeysEx findKey( isSecret, NULL, userID, cKey );

	int res = walkPackets( keyRingBuff, keyRingBuffSize, &findKey );

	if ( ( res == ppe_ok ) && !findKey.m_found )
		return ppe_keyNotFound;

	return res;
}



//
//	Function : pgp_FormatSessionKeyPacket
//	Prepares key block with signature and encrypts it.
//
//	5.1. Public-Key Encrypted Session Key Packets (Tag 1)
//
int pgp_FormatPublicKeyEncryptedSessionKeyPacket(
	BYTE *pkt,		// OUT - Encrypted packet buffer.
	int  *pPktSize, // IN  - pointer to size of allocated buffer.
					// OUT - if no error it is pointer to size of really busied part
					//	 if error is ppe_bufferTooSmall it is pointer to required size .

	int  sessionAlgID,	// = pgpsaBlowfish,
						// Identificator of symmetric algorithm to be used for encryption of a data with
						// following session key
	BYTE *sessionKey,	// Session key for the symmetric algorithm.

	int sessionKeySize, // Size of the session key

	int  hashAlgID,		// = pgphaSHA1,
						// Identificator of hash algorithm.
						// It is used for internal signature

	BYTE *pkPkt,		// Public key packet
	int pkPktSize

	)
{
	int res;

	(void)hashAlgID;  // Unreferenced formal parameter.

	//
	//	1. Checks session algorithm...
	//
	int algKeySize = sa_getKeySize( sessionAlgID );

	if ( sessionKeySize == 0 )
		return ppe_invalidAlgorithmID;

	if ( algKeySize != sessionKeySize )
		return ppe_invalidKeySize;

	//
	//	2. Verifies public key packet.
	//
	BYTE pkAlgID;
	BYTE *pkMpi;
	int pkMpiSize;

	if ( pkPkt[0] < 4 )
	{
		sPublicKeyV3 *pk = (sPublicKeyV3 *)pkPkt;

		res = PKT_PublicKeyV3_Parse( pk, pkPktSize );

		if ( res != ppe_ok )
			return res;

		pkAlgID = pk->algID;
		pkMpi = pk->mpi;
		pkMpiSize = pkPktSize - ( (char *)(pk->mpi) - (char *)(pk) );
	}
	else
	{
		sPublicKey *pk = (sPublicKey *)pkPkt;

		res = PKT_PublicKey_Parse( pk, pkPktSize );

		if ( res != ppe_ok )
			return res;

		pkAlgID = pk->algID;
		pkMpi = pk->mpi;
		pkMpiSize = pkPktSize - ( (char *)(pk->mpi) - (char *)(pk) );
	}

	HPKA pka;

	res = pka_createAlgorithm( &pka, pkAlgID, pkMpi, pkMpiSize );

	if ( res != pkae_ok )
		return res;

	//
	//	3. Prepares formatted block to be encrypted.
	//
	cPlainTextBlock ptFormattedBlock;

	if ( ptFormattedBlock.Allocate( pka_getModuleSize( pka ) ) == NULL )
	{
		pka_free( pka );
		return ppe_memoryAllocation;
	}
/*
	res = PKT_PublicKeyEncryptionBlock_Format(
		ptFormattedBlock.buffer,
		ptFormattedBlock.bufferSize,
		sessionKey,
		sessionKeySize,
		02 );
*/
	res = RSA_PublicKeyEncryptionBlock_Format(
		true,
		ptFormattedBlock.buffer,
		ptFormattedBlock.bufferSize,
		sessionKey,
		sessionKeySize
		);
	if ( res != ppe_ok )
	{
		pka_free( pka );
		return res;
	}

	//
	//	4. Checks packet size and Encrypts formatted block.
	//

	int eskMpiSize = 0;
	sPublicKeyEncryptedSessionKey *esk = (sPublicKeyEncryptedSessionKey *)pkt;

	if ( *pPktSize > (int)sizeof( sPublicKeyEncryptedSessionKey ) )
	{
		eskMpiSize = *pPktSize - sizeof( sPublicKeyEncryptedSessionKey ) + sizeof( esk->mpi );
	}

	res = pka_encrypt( pka, esk->mpi, &eskMpiSize, ptFormattedBlock.buffer, ptFormattedBlock.bufferSize );

	pka_free( pka );

	*pPktSize = sizeof( sPublicKeyEncryptedSessionKey ) - sizeof( esk->mpi ) + eskMpiSize;

	if ( res != ppe_ok )
	{
		if ( res == pkae_bufferSizeTooSmall )
			res = ppe_bufferTooSmall;

		return res;
	}

	//
	//	5. Fills key packet header.
	//

	// see PKT_PublicKeyEncryptedSessionKey_Format

	esk->version = 3;
	esk->algID = ((sPublicKey *)pkPkt)->algID; //sessionAlgID;

	return PKT_GetPublicKeyIDInternal( (sPublicKey *)pkPkt, pkPktSize, esk->keyID );
}

//
//	Function
//		pgp_ParsePublicKeyEncryptedSessionKeyPacket
//	Check integrity of public key encrypted session key packet.
//
int pgp_ParsePublicKeyEncryptedSessionKeyPacket(
	BYTE **pKeyID,			// [OUT] pointer to public key ID
	BYTE *pAlgID,			// [OUT] pointer to public key algorithm ID
	BYTE **pEncMpi,			// [OUT] pointer to encrypted Mpi buffer position
	int  *pEncMpiSize,		// [OUT] pointer to size of encrypted Mpi buffer
	BYTE *pkt,				// [IN]The public key encrypted session key packet
	int	 pktSize
	)
{
	sPublicKeyEncryptedSessionKey *eskPkt; // Encrypted session key packet.

	eskPkt = (sPublicKeyEncryptedSessionKey *)pkt;

	int res = PKT_PublicKeyEncryptedSessionKey_Parse( eskPkt, pktSize, pEncMpiSize );

	if ( res != ppe_ok )
		return res;

	*pKeyID	 = eskPkt->keyID;
	*pAlgID	 = eskPkt->algID;
	*pEncMpi = eskPkt->mpi;

	return ppe_ok;
}

//
//	Function pgp_DecryptPublicKeyEncryptedSessionKeyFromPacket
//	Parses the packet, decrypts with specified public key ID the session key block with signature and
//	verifies the signature. If signature is invalid the function returns ppe_invalidSignature.
//
int pgp_DecryptPublicKeyEncryptedSessionKeyFromPacket(
	BYTE *pAlgID,			// OUT - pointer to identificator of symmetic key algorithm to be used 
							//	 for decryption of a data or packets with following session key
	BYTE *sessionKey,		// OUT - plain text buffer with session key
	int  *pSessionKeySize,	// IN  - pointer to allocated size of key buffer
							// OUT - pointer to required size if ppe_bufferTooSmall returned

	BYTE *pkt,			// The public key encrypted session key packet
	int	 pktSize,

	BYTE *skBuff,		// Secret key buffer
	int  skBuffSize,

	const char *pwd		// Null terminated password string.
	)
{
	sPublicKeyEncryptedSessionKey *eskPkt; // Encrypted session key packet.

	eskPkt = (sPublicKeyEncryptedSessionKey *)pkt;

	//
	//	Checks the packet.
	//

	int encSessionKeyMpiSize;

	int res = PKT_PublicKeyEncryptedSessionKey_Parse( eskPkt, pktSize, &encSessionKeyMpiSize );

	if ( res != ppe_ok )
		return res;

	BYTE *encSessionKeyMpi = eskPkt->mpi;

	//
	//	Tries to find secret key in specified key ring.
	//
	sSecretKey *secKey;
	int secKeySize;

	BYTE newKeyID[ pgp_keyIDSize ];

	res = PKT_GetPublicKeyIDInternal(
				(sSecretKey *)skBuff, skBuffSize,
				newKeyID
			);

	//
	//	Checks if the secret key packed is already unpacked.
	//
	if ( ( res == ppe_ok ) && ( memcmp( eskPkt->keyID, newKeyID, sizeof( newKeyID ) ) == 0 ) )
	{
		secKey = (sSecretKey *)skBuff;
		secKeySize = skBuffSize;
	}
	else
		res = pgp_FindKeyByID( (BYTE **)&secKey, &secKeySize, eskPkt->keyID, skBuff, skBuffSize, true );

	if ( res != ppe_ok )
		return res;

	//
	//	Creates secret key algorithm from secret key packet...
	//

	HPKA hSKA;

	res = PKT_CreateSecretKeyAlgFromPacket( &hSKA, secKey, secKeySize, pwd );

	if ( res != ppe_ok )
		return res;

	//
	//	Decrypts session key mpi buffer
	//

	int decBlockSize = ska_getModuleSize( hSKA );

	cPlainTextBlock decBlock; // decrypted block foramtted by PKCS-1 packet...

	if ( decBlock.Allocate( decBlockSize ) == NULL )
	{
		ska_free( hSKA );
		return ppe_memoryAllocation;
	}

	res = ska_decrypt( hSKA, decBlock.buffer, &decBlockSize, encSessionKeyMpi, encSessionKeyMpiSize );

	ska_free( hSKA );

	if ( res != pkae_ok )
		return pkae_internalError;

	//
	//	Parses formatted by PKCS-1 packet
	//
//	return PKT_PublicKeyEncryptionBlock_Parse( sessionKey, pSessionKeySize, decBlock.buffer, decBlockSize );

	BYTE *ptr = NULL;
	int sessKeySize = 0;
	res = RSA_SessionKeyBlock_Parse( &ptr, &sessKeySize, pAlgID, decBlock.buffer, decBlockSize );

	if ( res != pkae_ok )
		return res;

	if ( pSessionKeySize == NULL )
		return pkae_internalError;

	if ( *pSessionKeySize < sessKeySize )
	{
		*pSessionKeySize = sessKeySize;
		return pkae_bufferSizeTooSmall;
	}

	*pSessionKeySize = sessKeySize;

	if ( sessionKey != NULL )
		memcpy( sessionKey, ptr, *pSessionKeySize );

	return pkae_ok;
}

int pgp_VerifySignaturePacket(
	BYTE *sigPkt,
	int sigPktSize,
	cHashedList *hashList,
	BYTE *keyPkt,
	int keyPktSize,
	BYTE *issuerKeyRing,
	int issuerKeyRingSize,
	BYTE *issuerKeyIDBuff // = NULL
	)
{
	return
		PKT_VerifySignaturePacket(
				(sSignature *)sigPkt,
				sigPktSize,
				hashList,
				keyPkt,
				keyPktSize,
				issuerKeyRing,
				issuerKeyRingSize,
				issuerKeyIDBuff
			);
}

int pgp_GetSigType( BYTE *sigPtr, int signatureSize )
{
	BYTE version = sigPtr[0];

	(void)signatureSize;  // Unreferenced formal parameter.

	if ( version < 4 )
		return ((sSignatureV3 *)sigPtr)->type;
	else
		return ((sSignature *)sigPtr)->type;
}

//
//	Function: pgp_AddKeyPktToHashList
//		Adds the key packet body with old style header
//
int pgp_AddKeyPktToHashList(
		cHashedList *hashList,	// List of pointers to pakcets and it's sizes to be hashed
		BYTE *keyPkt,			// body of key packet
		int keyPktSize
	)
{
	//
	//	Prepares old style key packed header...
	//
	BYTE tmp[3];

	tmp[0] = 0x99;
	tmp[1] = BYTE((UINT)keyPktSize >> 8);
	tmp[2] = BYTE( keyPktSize );

	if (    !hashList->Add( tmp, 3, true ) // copy the buffer...
		 || !hashList->Add( keyPkt, keyPktSize ) 
	   )
		 return ppe_memoryAllocation;

	return ppe_ok;
}

//
//	Function: pgp_MakeKeySignature_0x10_HashList
//		Adds the required packets to hash list accoding with signature type 0x10 
//			   0x10: Generic certification of a User ID and Public Key packet.
//
//	See RFC2440 5.2.4. Computing Signatures
//
int pgp_MakeKeySignature_0x10_HashList(
		cHashedList *hashList,	// List of pointers to pakcets and it's sizes to be hashed
		BYTE *keyPkt,			// body of key packet
		int keyPktSize,
		BYTE *userID,			// body of user ID packet
		int userIDSize,
		BYTE sigVersion			// signature version
		)
{
	//
	//	Adds the key packet body with old style header...
	//
	int res = pgp_AddKeyPktToHashList( hashList, keyPkt, keyPktSize );

	if ( res != ppe_ok ) return res;

	BYTE tmp[5];

	if ( sigVersion > 3 )
	{
		//
		//	Adds header of user ID packet.
		//
/*
		A V3 certification hashes the contents of the name packet, without any
		header.A V4 certification hashes the constant 0xb4 (which is an
		old-style packet header with the length-of-length set to zero), a
		four-octet number giving the length of the username, and then the
		username data.
*/
		tmp[0] = 0xb4;
		tmp[1] = BYTE((UINT)userIDSize >> 24);
		tmp[2] = BYTE((UINT)userIDSize >> 16);
		tmp[3] = BYTE((UINT)userIDSize >>  8);
		tmp[4] = BYTE((UINT)userIDSize >>  0);

		if ( !hashList->Add( tmp, 5, true ) ) // copy it too
			return ppe_memoryAllocation;
	}

	if (  !hashList->Add( userID, userIDSize )  )
		 return ppe_memoryAllocation;

	return ppe_ok;
}

int pgp_VerifyKey(
		BYTE *pk,	  int pkSize,		// public key
		BYTE *userID, int userIDSize,	// user ID
		BYTE *sig,	  int sigSize,		// signature macket
		BYTE *kr,	  int krSize,		// if user key ID == issuer key ID, so key ring does not use and can be NULL
		BYTE *issuerKeyIDBuff // = NULL
	)
{
	BYTE ver = sig[0];

	cHashedList hashList;

	int res = pgp_MakeKeySignature_0x10_HashList( 
		&hashList,
		(BYTE *)pk,			// body of key packet
		pkSize,
		userID,			// body of user ID packet
		userIDSize,
		ver		 	// signature version
		);

	if ( res != ppe_ok ) return res;

	res = pgp_VerifySignaturePacket( sig, sigSize, &hashList, pk, pkSize, kr, krSize, issuerKeyIDBuff );

	return res;
}


//
//	Function.
//		pgp_ExtractSessionKeyFromSymmetricKeyEncrytpedPacket
//	Parses the packet.
//	Prepares symmetric key by s2k value and pwd.
//	If session key exists it will be decrypted to sessKeyBuff. 
//	Otherwise the s2k prepared key will be copied to the session key buffer.
//
int pgp_ExtractSessionKeyFromSymmetricKeyEncrytpedPacket(
		BYTE *pAlgID,			// pointer to symmetric algorithm identifier.
		BYTE *sessKeyBuff,
		int  *pSessKeyBuffSize, // Size of the buffer is small the function returns ppe_bufferTooSmall
								// and fills the value by required size.

		const BYTE *pkt,		// Symmetric-Key Encrypted Session-Key Packet
		int pktSize,

		const char *pwd		// Null terminated string.
		)
{
	sSymmetricKeyEncryptedSessionKeyPkt *keyPkt = (sSymmetricKeyEncryptedSessionKeyPkt *)pkt;

	if ( pktSize < (int)sizeof(*keyPkt) )
		return ppe_invalidPacketSize;

	S2K s2k;

	int s2kSize = s2k.Parse( keyPkt->s2kSpec );

	if ( s2kSize == 0 )
		return ppe_invalidS2KBlockType;

	s2k.encAlgID = keyPkt->algID;
	*pAlgID = keyPkt->algID;

	int sessKeyPos = ( sizeof(*keyPkt) - sizeof( keyPkt->s2kSpec ) ) + s2kSize;

	//
	//	Gets required size.
	//

	int pwdKeySize = 0;
	int pwdLen = 0;
	if ( pwd != NULL )
		pwdLen = strlen( pwd );

	int res = string2key( NULL, &pwdKeySize, pwd, pwdLen, &s2k );

	if ( res != ppe_bufferTooSmall )
		return res;

	//
	//	Check output buffer size.
	//

	bool useS2KKey = ( pktSize == sessKeyPos ); // The s2k generated key is to be used

	int sessKeySize = useS2KKey ?
							pwdKeySize :
							( pktSize - sessKeyPos ) - 1; // Alg ID lies first

	if ( *pSessKeyBuffSize < sessKeySize )
	{
		*pSessKeyBuffSize = sessKeySize;
		return ppe_bufferTooSmall;
	}

	*pSessKeyBuffSize = sessKeySize;


	//
	//	Generates s2k key to ptbKey.
	//
	cPlainTextBlock ptbKey;

	if ( ptbKey.Allocate( pwdKeySize ) == NULL )
		return ppe_memoryAllocation;

	res = string2key( ptbKey.buffer, &pwdKeySize, pwd, pwdLen, &s2k );

	if ( res != ppe_ok ) return res;

	if ( useS2KKey )
	{
		//
		//	Copies session key from s2k generated buffer.
		//

		assert( ptbKey.bufferSize <= *pSessKeyBuffSize );

		memcpy( sessKeyBuff, ptbKey.buffer, ptbKey.bufferSize );

		return ppe_ok;
	}
	else
	{
		//
		//	Decrypts session key from the packet to output buffer.
		//

		HSA sa;

		res = sa_createAlgorithm( &sa, s2k.encAlgID, ptbKey.buffer, ptbKey.bufferSize );
		if ( res != sae_ok ) return res;

		res = sa_initCFB( sa, NULL );
		if ( res != sae_ok ) { sa_free( sa ); return res; }

		// 1. One byte alg ID.
		res = sa_decryptCFB( sa, pAlgID, pkt + sessKeyPos++, 1 );
		if ( res != sae_ok ) { sa_free( sa ); return res; }

		// 2. The session key.
		res = sa_decryptCFB( sa, sessKeyBuff, pkt + sessKeyPos, sessKeySize );

		sa_free( sa );
		return res;
	}

}

//
//	Function.
//		pgp_ExtractSessionKeyFromSymmetricKeyEncrytpedPacket_Rudiment
//	Is same as previose but uses hasAlgId for encryption :(
//
int pgp_ExtractSessionKeyFromSymmetricKeyEncrytpedPacket_Rudiment(
		BYTE *pAlgID,			// pointer to symmetric algorithm identifier.
		BYTE *sessKeyBuff,
		int  *pSessKeyBuffSize, // Size of the buffer is small the function returns ppe_bufferTooSmall
								// and fills the value by required size.

		const BYTE *pkt,		// Symmetric-Key Encrypted Session-Key Packet
		int pktSize,

		const char *pwd		// Null terminated string.
		)
{
	sSymmetricKeyEncryptedSessionKeyPkt *keyPkt = (sSymmetricKeyEncryptedSessionKeyPkt *)pkt;

	if ( pktSize < (int)sizeof(*keyPkt) )
		return ppe_invalidPacketSize;

	S2K s2k;

	int s2kSize = s2k.Parse( keyPkt->s2kSpec );

	if ( s2kSize == 0 )
		return ppe_invalidS2KBlockType;

	s2k.encAlgID = keyPkt->algID;

	int sessKeyPos = ( sizeof(*keyPkt) - sizeof( keyPkt->s2kSpec ) ) + s2kSize;

	//
	//	Gets required size.
	//

	int pwdKeySize = 0;

	int res = string2key( NULL, &pwdKeySize , pwd, strlen( pwd ), &s2k );

	if ( res != ppe_bufferTooSmall )
		return res;

	//
	//	Check output buffer size.
	//

	bool useS2KKey = ( pktSize == sessKeyPos ); // The s2k generated key is to be used

	int sessKeySize = useS2KKey ?
							pwdKeySize :
							( pktSize - sessKeyPos ) - 1; // Alg ID lies first

	if ( *pSessKeyBuffSize < sessKeySize )
	{
		*pSessKeyBuffSize = sessKeySize;
		return ppe_bufferTooSmall;
	}

	*pSessKeyBuffSize = sessKeySize;


	//
	//	Generates s2k key to ptbKey.
	//
	cPlainTextBlock ptbKey;

	if ( ptbKey.Allocate( pwdKeySize ) == NULL )
		return ppe_memoryAllocation;

	res = string2key( ptbKey.buffer, &pwdKeySize, pwd, strlen( pwd ), &s2k );

	if ( res != ppe_ok ) return res;

	if ( useS2KKey )
	{
		//
		//	Copies session key from s2k generated buffer.
		//

		assert( ptbKey.bufferSize <= *pSessKeyBuffSize );

		memcpy( sessKeyBuff, ptbKey.buffer, ptbKey.bufferSize );

		return ppe_ok;
	}
	else
	{
		//
		//	Decrypts session key from the packet to output buffer.
		//

		HSA sa;

		res = sa_createAlgorithm( &sa, s2k.hashAlgID, ptbKey.buffer, ptbKey.bufferSize );
		if ( res != sae_ok ) return res;

		res = sa_initCFB( sa, NULL );
		if ( res != sae_ok ) { sa_free( sa ); return res; }

		// 1. One byte alg ID.
		res = sa_decryptCFB( sa, pAlgID, pkt + sessKeyPos++, 1 );
		if ( res != sae_ok ) { sa_free( sa ); return res; }

		// 2. The session key.
		res = sa_decryptCFB( sa, sessKeyBuff, pkt + sessKeyPos, sessKeySize );

		sa_free( sa );
		return res;
	}

}

int pgp_FormatSymmetricKeyEncrytpedSessionKeyPacket(
		BYTE *pkt,
		int  *pPktSize,		// Size of the buffer is small the function returns ppe_bufferTooSmall
							// and fills the value by required size.

		BYTE algID,				// Session algorithm ID

		const BYTE *sessKey,	// Session-Key buffer
		int sessKeySize,

		int keyAlgID,			// The algorithm for encryption of the key.

		const char *pwd,		// Null terminated string.

		BYTE hashAlgID,		// = 2 s2k hash algorithm ( pgphaSHA1 == 2 )

		const BYTE *salt,	// = NULL

		int iterationCount // = -1
		)
{
	sSymmetricKeyEncryptedSessionKeyPkt *keyPkt = (sSymmetricKeyEncryptedSessionKeyPkt *)pkt;
	S2K s2k;


	//
	//	Checks s2k size.
	//

	BYTE rnd[ sizeof( s2k.salt) ];

	if ( salt == NULL )
	{
		salt = rnd; 

		int res = cRNG::generate( rnd, sizeof( rnd ) );

		if ( res != ppe_ok )
			return ppe_randomBufferRequired;
	}

	s2k.SetDefault(
			salt,
			hashAlgID,	// 2 == SHA-1
			keyAlgID,
			iterationCount
		);

	int s2kSize = 0;

	int res = s2k.WriteToPacket( NULL, &s2kSize );

	if ( res != ppe_bufferTooSmall ) return res;

	//
	//	Checks output buffer size.
	//
	int keyPos = sizeof( *keyPkt ) - sizeof( keyPkt->s2kSpec ) + s2kSize;
	int keyPktSize = keyPos;

	if ( sessKeySize != 0 )
		keyPktSize += sessKeySize + 1; // Algorithm ID and session key

	if ( keyPktSize > *pPktSize )
	{
		*pPktSize = keyPktSize;
		return ppe_bufferTooSmall;
	}

	*pPktSize = keyPktSize;

	//
	//	Generates salted s2k 
	//

	s2k.SetDefault(
			salt, // rnd,
			hashAlgID, // 2 == SHA-1
			keyAlgID,
			iterationCount
		);

	//
	//	Formats packet header.
	//
	keyPkt->version = 4;
	keyPkt->algID = s2k.encAlgID;

	res = s2k.WriteToPacket( keyPkt->s2kSpec, &s2kSize );

	if ( sessKeySize == 0 )
	{
		return ppe_ok;
	}

	//
	//	Geterates key from pasword
	//
	int pwdKeySize = 0;

	res = string2key( NULL, &pwdKeySize , pwd, strlen( pwd ), &s2k );

	if ( res != ppe_bufferTooSmall )
		return res;

	cPlainTextBlock ptbKey;

	if ( ptbKey.Allocate( pwdKeySize ) == NULL )
		return ppe_memoryAllocation;

	res = string2key( ptbKey.buffer, &pwdKeySize, pwd, strlen( pwd ), &s2k );

	if ( res != ppe_ok ) return res;

	//
	//	Writes the both encrypted alg ID and session key.
	//
	HSA sa;

	res = sa_createAlgorithm( &sa, s2k.encAlgID, ptbKey.buffer, ptbKey.bufferSize );
	if ( res != sae_ok ) return res;

	res = sa_initCFB( sa, NULL );
	if ( res != sae_ok ) { sa_free( sa ); return res; }

	// 1. One byte alg ID.
	res = sa_encryptCFB( sa, pkt + keyPos++, &algID, 1 );
	if ( res != sae_ok ) { sa_free( sa ); return res; }

	// 2. The session key.
	res = sa_encryptCFB( sa, pkt + keyPos, sessKey, sessKeySize );

	sa_free( sa );
	return res;
}

//
//	Function:
//		pgp_PacketHeader_Parse ( see also PKT_PacketHeader_Format )
//	Reads packet header and extracts:
//		1. packet tag
//		2. size of the packet header ( pointer to data is pkt + hdrSize )
//		3. size of the data
//
//	Note, the function can read packets with partial data blocks
//
int pgp_PacketHeader_Parse(
			BYTE *pTag,		// [Out] Pointer to content tag
			int *pHdrSize,	// [Out] Pointer to the packet header size
			int *pDataSize,	// [Out] Pointer to the packet body size
			bool *pPartial,	// Pointer to partial value
							// [In ] False if read new packet,
							//		  ( *pTag is detected and changed )
							//		 True if read next part of partial packet, 
							//        ( *pTag is not detected and changed )
							// [Out] if True the data can be extracted again form pkt + *pHdrSize + *pDataSize;
							//		 if false the extracted data is last.
			BYTE *pkt,		// The packet with header.
			int pktSize		// Total size of available packet buffer.
			)
{
	if ( ( pktSize == 0 ) || ( pkt == NULL ) )
	{
		assert( false );
		return pkae_invalidInputData;
	}

	if ( !*pPartial )
	{
		//
		//	reads new block information.
		//
		if (( pkt[0] & 0x80) == 0 )
			return ppe_invalidHeader;

		BOOL newFormat = ( (pkt[0] & 0x40) != 0 );

		*pTag = pkt[0] & 0x3f;

		if ( !newFormat ) // Old version packet...
		{
			int lengthType = pkt[0] & 0x03; 
			*pTag /= 4;

			int lenghtByteQ = 0;

			*pHdrSize = 1;

			switch ( lengthType )
			{
				case 0: lenghtByteQ =  1; break;
				case 1: lenghtByteQ =  2; break;
				case 2: lenghtByteQ =  4; break;

				case 3: // The packet is of indeterminate length
					*pDataSize = -1;
					return ppe_ok;

				default:
					return ppe_invalidHeader;
			}

			for( *pDataSize = 0;lenghtByteQ--; )
			{
				*pDataSize = *pDataSize << 8;
				*pDataSize |= pkt[ (*pHdrSize)++ ];
			}

			if ( ( *pDataSize + *pHdrSize ) > pktSize )
				return ppe_invalidHeader;

			return ppe_ok;
		}
	}

	int i = *pPartial ? 0 : 1;

	*pPartial = false;

	if ( pkt[i] < 192 ) // One-Octet Lengths
	{
		*pDataSize  = pkt[i];
		*pHdrSize = i + 1;
	}
	else				// Two-Octet Lengths
	if ( pkt[i] < 223 )
	{
		*pDataSize = (((unsigned int)pkt[i] - 192) << 8) + pkt[i+1] + 192;
		*pHdrSize = i + 2;
	}
	else
	if ( pkt[i] == 255 ) // Five-Octet Lengths
	{
		*pDataSize =
			((unsigned int)pkt[i+1] << 24) |
			((unsigned int)pkt[i+2] << 16) |
            ((unsigned int)pkt[i+3] << 8)  |
			 (unsigned int)pkt[i+4];

		*pHdrSize = i + 4;

	}
	else // Partial Body Lengths
	{
		*pDataSize = 1 << ( pkt[i] & 0x1f );
		*pHdrSize = i + 1;
		*pPartial = true;
	}

	if ( ( *pHdrSize + *pDataSize  ) > ( pktSize ) )
		return ppe_invalidPacketSize;

	return ppe_ok;
}

//
//	Function:
//		pgp_PacketHeader_Format
//
//	Forms packet with header
//
int pgp_PacketHeader_Format(
			BYTE *pkt,		// [Out]The packet with header.
			int  *pktSize,	// [In] Size of allcated buffer for the packet.
							// [Out] Space written or required if the buffer is too small.
							//	If last it returns error ppe_bufferTooSmall.
			BYTE tag,		// Content tag, 0 if it is partial tail. Will be packed without "Packet Tag" byte.
			BYTE *data,		// Packet body ( data )
			int	 size,		// The packet body size
			bool partialSize// Pack as partial length ( following packet has to be packed as partial type ( tag 0 ) ) 
							// the length is a power of 2, from 1 to 1,073,741,824 (2 to the 30th power)
							// The first partial length MUST be at least 512 octets long ( rfc 2440 )
		)
{
	int i = 0;

	if ( tag != 0 )
	{
		if ( tag >= 0x40 )
			return ppe_invalidHeader;

		if ( pkt ) { pkt[i] = 0x80 | 0x40 | tag; } i++;
	}
	else
	if ( tag == 0 ) 
	{
		// Partial packet type.
		// Does nothing, it has no "Packet Tag" byte.
	}

	if ( partialSize )
	{
		if ( size % 512 )
			return ppe_invalidHeader;

		int exp = 0;

		for( int t = size ;; exp++)
		{
			if ( t & 1 )
				break;

			t /= 2; 
		}

		if ( size != ( 1 << exp ) )
			return ppe_invalidHeader;

		if ( exp >= 0x1F )
			return ppe_invalidHeader;

		if ( pkt ) { pkt[i] = 0xE0 | (BYTE)exp; } i++;
	}
	else
	if ( size < 192 )
	{
		if ( pkt ) { pkt[i] = (BYTE)size; } i++;
	}
	else
	if ( size < 8383 )
	{
		//	pkcs 2440 
		//	4.2.2.2. Two-Octet Lengths
		//		bodyLen = ((1st_octet - 192) << 8) + (2nd_octet) + 192

//		BYTE firstOctet = BYTE(( size ) >> 8 ) + 192 );
		BYTE firstOctet = BYTE(( ( size - 192 ) >> 8 ) + 192 );
		if ( pkt ) { pkt[i] = firstOctet; } i++;

//		int snd = size  - 192  - (( pkt[ i - 1 ]  - 192 ) << 8 );
		WORD secondOctet = WORD(size) - 192 - ( WORD( firstOctet - 192 ) << 8 );

		assert( secondOctet <= 0xFF );

		if ( pkt ) { pkt[i] = BYTE(secondOctet); } i++;

		assert( ((WORD(firstOctet) - 192) << 8) + (secondOctet) + 192 == size );
	}
	else
	{
		if ( pkt ) { pkt[i] = 0xFF;				} i++;
		if ( pkt ) { pkt[i] = BYTE(size >> 24); } i++;
		if ( pkt ) { pkt[i] = BYTE(size >> 16); } i++;
		if ( pkt ) { pkt[i] = BYTE(size >>  8); } i++;
		if ( pkt ) { pkt[i] = BYTE(size);		} i++;
	}

	int allocatedSize = *pktSize;

	*pktSize = size + i;

	if ( *pktSize > allocatedSize )
		return ppe_bufferTooSmall;

	if ( pkt && ( pkt + i != data ) ) // Do not copy self packet
		memcpy( pkt + i, data, size );

	return ppe_ok;
}

int pgp_GenerateKeyFromSeedBuffer( BYTE *key, int keySize, const BYTE *seed, int seedSize, BYTE hashAlgID )
{
	S2K s2k;

	s2k.mode = s2k.simple;
	s2k.hashAlgID = hashAlgID;
	s2k.encAlgID = 0;

	return string2key( key, &keySize, (const char *)seed, seedSize, &s2k );
}

int pgp_SetRandomBuffer( BYTE *buff, int buffSize )
{
	cRNG::setPool( buff, buffSize );
	return ppe_ok;
}

//
//	1. Finds key by ID
//	2. Gets it start position
//	3. If key is found gets it finish position.
//	   It may be:
//		3.1. The packet end if no more keys ( m_finishPos == 0 )
//		3.2. Or start position of next key
//
class cFindKeyRange : public cPacket
{
public:
	bool 
		m_found,
		m_subKeyFound,
		m_chooseSubkeyOnly;

	BYTE *m_keyID; // Key ID to be find.

	BYTE *m_startPos;
	BYTE *m_finishPos;

	cFindKeyRange( bool isSecret, bool findInSubpackets, BYTE *keyID, bool chooseSubkeyOnly = true ) :
		cPacket(
			( isSecret ? pkt_SecretKey : pkt_PublicKey ),
			( findInSubpackets ? ( isSecret ? pkt_SecretSubkey : pkt_PublicSubkey ) : 0 )
		)
	{
		m_startPos = 0;
		m_finishPos = 0;

		m_keyID = keyID;

		m_found = false;
		m_subKeyFound = false;
		m_chooseSubkeyOnly = chooseSubkeyOnly;
	}

	int Actor()
	{
		if ( tag == subKeyPktID )
		{
			if ( compareKeyID( m_keyID, data, dataSize ) == 0 )
			{
				if ( m_chooseSubkeyOnly )
				{
					m_startPos = data - hdrSize;
					m_subKeyFound = true;
				}

				m_found = true;
			}
			else
			if ( m_chooseSubkeyOnly && m_subKeyFound )
			{
				m_finishPos = data - hdrSize;
				m_stop = true;
				return ppe_ok;
			}

			return ppe_ok;
		}
		else // pkt_SecretKey OR pkt_PublicKey
		{
			assert( tag == countID );

			if ( m_found )
			{
				m_finishPos = data - hdrSize;
				m_stop = true;
				return ppe_ok;
			}

			//
			//	The assigment here allows to find start position of master key
			//
			m_startPos = data - hdrSize;

			if ( compareKeyID( m_keyID, data, dataSize ) == 0 )
			{
				m_found = true;
			}

			return ppe_ok;
		}
	}

};

//
//	Function:
//		pgp_GetEntireKeyByID
//	Gets entire key packet ( with all subpackets ) by specified ID from key ring 
//		1. Finds main key with the ID or subpacket with the ID
//		2. Gets entire packet position and size
//
int pgp_GetEntireKeyByID(
	BYTE **pEntireKeyPos,
	int *pEntireKeySize,
	BYTE *keyRingBuff,		// Key ring packet.
	int  keyRingBuffSize,	// Key ring buffer size.
	BYTE *keyID,
	bool isSecret
	)
{
	cFindKeyRange findKeyRange( isSecret, true, keyID, false ); // findInsubkeys = true, chooseSubkeyOnly = false

	int res = walkPackets( keyRingBuff, keyRingBuffSize, &findKeyRange );

	if ( res != ppe_ok )
		return res;

	if ( !findKeyRange.m_found )
		return ppe_keyNotFound;

	*pEntireKeyPos = findKeyRange.m_startPos;

	if ( findKeyRange.m_finishPos == 0 ) // no more keys were found
		findKeyRange.m_finishPos = ( keyRingBuff + keyRingBuffSize );

	*pEntireKeySize = (int)( findKeyRange.m_finishPos - findKeyRange.m_startPos );

	return ppe_ok;
}

//
//	Function:
//		pgp_RemoveKeyByID
//	Removes key by specified ID ( with all subpackets ) from key ring 
//		1. Finds main key with the ID. ( see pgp_FindKeyByIDEx descryption )
//		2. Walks all subkeys 
//		3. Copys next keys to the position.
//		4. Returns new decreased size.
//
int pgp_RemoveKeyByID(
	BYTE *keyRingBuff,		// Key ring packet.
	int  *pKeyRingBuffSize,	// [IN] Key ring buffer size.
							// [OUT] Key decreased size
	BYTE *keyID,
	bool isSecret
	)
{
	cFindKeyRange findKeyRange( isSecret, true, keyID );

	int res = walkPackets( keyRingBuff, *pKeyRingBuffSize, &findKeyRange );

	if ( res != ppe_ok )
		return res;

	if ( !findKeyRange.m_found )
		return ppe_keyNotFound;

	if ( findKeyRange.m_finishPos == 0 ) // no more keys
		*pKeyRingBuffSize = (int)( findKeyRange.m_startPos - keyRingBuff );
	else
	{
		int packetTailSize = (int)( keyRingBuff + *pKeyRingBuffSize - findKeyRange.m_finishPos );
		int commonKeySize = (int)( findKeyRange.m_finishPos - findKeyRange.m_startPos );

		memmove( findKeyRange.m_startPos, findKeyRange.m_finishPos, packetTailSize );
		*pKeyRingBuffSize -= commonKeySize;
	}

	return ppe_ok;
}

//
//	Function:
//		pgp_AddPacketToKeyRing
//	Adds a packet to the end of key ring
//
int pgp_AddPacketToKeyRing(
	BYTE *keyRingBuff,		// Key ring packet.
	int  *pKeyRingBuffSize,	// [IN] Key ring buffer size.
							// [OUT] Key increased size
	BYTE *pkt,
	int pktSize
	)
{
	// Unreferenced formal parameters.
	(void)keyRingBuff;
	(void)pKeyRingBuffSize;
	(void)pkt;
	(void)pktSize;

	return ppe_internalError;
}

//
//	Function:
//		pgp_WritePublicKeyToPacketV4
//	Creates public key v4 packet from algorithm handle
//
int pgp_WritePublicKeyToPacketV4( BYTE *buff, int *pBuffSize, HPKA pka )
{
	int hdrSize = sizeof( sPublicKey ) - 2; // sizeof ( sPublicKey::mpi );
	int required = 0;
	int res = pka_writeKeyToMpiBuffer( pka, NULL, &required );
	if ( res != ppe_bufferTooSmall )
		return res;

	if ( ( required + hdrSize ) > *pBuffSize )
	{
		*pBuffSize = required + hdrSize;
		return ppe_bufferTooSmall;
	}

	sPublicKey *pkt = (sPublicKey *)buff;
	pkt->version = 4;
	pkt->time.setTime( pka->time );
	pkt->algID = (BYTE)pka_getAlgID( pka );

	*pBuffSize = required + hdrSize;

	return pka_writeKeyToMpiBuffer( pka, buff + hdrSize, &required );
}

int pgp_WritePublicKeyToPacket( BYTE *buff, int *pBuffSize, HPKA pka, int version )
{
	if ( version > 3 )
		return pgp_WritePublicKeyToPacketV4( buff, pBuffSize, pka );

	int hdrSize = sizeof( sPublicKeyV3 ) - 2; // sizeof ( sPublicKey::mpi );
	int required = 0;
	int res = pka_writeKeyToMpiBuffer( pka, NULL, &required );
	if ( res != ppe_bufferTooSmall )
		return res;

	if ( ( required + hdrSize ) > *pBuffSize )
	{
		*pBuffSize = required + hdrSize;
		return ppe_bufferTooSmall;
	}

	sPublicKeyV3 *pkt = (sPublicKeyV3 *)buff;
	pkt->version = (BYTE)version;
	pkt->time.setTime( pka->time );
	pkt->days.setDays( pka->days );
	pkt->algID = (BYTE)pka_getAlgID( pka );

	*pBuffSize = required + hdrSize;

	return pka_writeKeyToMpiBuffer( pka, buff + hdrSize, &required );
}

//
//	Function:
//		pgp_CompressedPacket_Parse
//	Retrieves compressed algorithm ID and pointers to compressed data.
//
//	RFC2440 pkt_CompressedData (Tag 8)
//	The body of this packet consists of:
//     - One octet that gives the algorithm used to compress the packet.
//     - The remainder of the packet is compressed data.
//
int pgp_CompressedPacket_Parse(
		BYTE *pAlgID,
		BYTE **ppData,
		int	 *pDataSize,
		const BYTE *pkt,
		int pktSize )
{
	if ( pktSize == 0 )
		return ppe_invalidBlockType;

	if ( pkt[0] > caID_last )
		return ppe_invalidBlockType;

	if ( pAlgID != NULL )
		*pAlgID = pkt[0];

	if ( ppData != NULL )
		*ppData = (BYTE*)pkt + 1;

	if ( pDataSize != NULL )
		*pDataSize = pktSize - 1;

	return ppe_ok;
}

//
//	Function:
//		pgp_SymEncryptedAndIntegrityProtectedDataPacket_Parse
//
//	See: openpgp-rfc2440bis-22
//		Sym. Encrypted and Integrity Protected Data Packet (Tag 18)
//		pkt_SymEncryptedAndIntegrityProtectedData
//
int pgp_SymEncryptedAndIntegrityProtectedDataPacket_Parse(
		const BYTE **pPkt,
		int *pPktSize,
		int saBlockSize,
		const BYTE *bf,
		int bfSize
	)
{
	// See: openpgp-rfc2440bis-22

	if ( bfSize < saBlockSize + 2 )
		return ppe_invalidBlockType;

	if ( memcmp( bf + saBlockSize - 2, bf + saBlockSize, 2 ) != 0 )
		return ppe_invalidBlockType;

	HHA hHa;
	int res = ha_createAlgorithm( &hHa, pgphaSHA1 );
	if ( res != hae_ok ) return res;

	if ( hHa == NULL )
		return ppe_invalidAlgorithmID;

	int digestSize = ha_getDigestSize( hHa );

	cPlainTextBlock ptbDigest;
	if ( ptbDigest.Allocate( digestSize ) == NULL )
		return hae_memoryAllocation;

	if ( ha_makeDigest( hHa, ptbDigest.buffer, (BYTE*)bf, bfSize - digestSize ) != hae_ok )
		return ppe_invalidAlgorithmID;

	ha_free( hHa );

	if ( memcmp( ptbDigest.buffer, bf + bfSize - digestSize, digestSize ) != 0 )
		return ppe_invalidBlockType;

	*pPkt = bf + saBlockSize + 2;
	*pPktSize = bfSize - saBlockSize + 2 - digestSize;

	return res;
}

//
//	Function:
//		pgp_SymEncryptedAndIntegrityProtectedDataPacket_Format
//
//	See: openpgp-rfc2440bis-22
//		Sym. Encrypted and Integrity Protected Data Packet (Tag 18)
//		pkt_SymEncryptedAndIntegrityProtectedData
//	1. Generates random saBlockSize bytes at packet beginnig
//		( use the cRNG::setPool if the function returns ppe_randomBufferRequired )
//	2. Duplicates two last random bytes
//	3. Copies input data to output packet
//	4. Adds SHA1 hash of the output packet to the packet end.
//
int pgp_SymEncryptedAndIntegrityProtectedDataPacket_Format(
		BYTE *pPkt,
		int *pPktSize,
		int saBlockSize,
		const BYTE *bf,
		int bfSize
	)
{
	// See: openpgp-rfc2440bis-22

	BYTE hashAlgId = pgphaSHA1;
	int digestSize = 0;

	int res = ha_getDigestSize( &digestSize, hashAlgId );
	if ( res != hae_ok )
		return res;

	int required = saBlockSize + 2 + bfSize + digestSize;

	if ( *pPktSize < required )
	{
		*pPktSize = required;
		return ppe_bufferTooSmall;
	}

	res = cRNG::generate( pPkt, saBlockSize );

	if ( res != ppe_ok )
		return ppe_randomBufferRequired;

	memcpy_s( pPkt + saBlockSize, 2, pPkt + saBlockSize - 2, 2 );

	memcpy_s( pPkt + saBlockSize + 2, bfSize, bf, bfSize );

	HHA hHa;
	res = ha_createAlgorithm( &hHa, hashAlgId );
	if ( res != hae_ok ) return res;

	if ( hHa == NULL )
		return ppe_invalidAlgorithmID;

	res = ha_makeDigest( hHa, pPkt + saBlockSize + 2 + bfSize, pPkt, saBlockSize + 2 + bfSize );
	ha_free( hHa );

	*pPktSize = required;

	return res;
}

//
//	Function:
//		pgp_LiteralDataPacket_Format
//
//	See: openpgp-rfc2440bis-22
//
int pgp_LiteralDataPacket_Format(
		BYTE *pkt,
		int *pPktSize,
		int codePage,
		const char *specialName,
		UINT date,
		const BYTE *bf,
		int bfSize
	)
{
	if ( specialName == NULL )
		specialName = "";

	int len = strlen( specialName ) + 1;

	int required = 1 + len + sizeof(UINT) + bfSize;

	if ( *pPktSize < required )
	{
		*pPktSize = required;
		return ppe_bufferTooSmall;
	}

	*pPktSize = required;

#ifndef CP_UTF8
#define CP_UTF8 65001
#define CP_ACP  0
#endif

	switch ( codePage )
	{
		case CP_UTF8: 	pkt[0] = 0x75; break;
		case CP_ACP: 	pkt[0] = 0x74; break;
		default:		pkt[0] = 0x62; break; // binary data
	}

	int shift = 1;

	memcpy_s( pkt + shift, len, specialName, len );
	shift += len;

	memcpy_s( pkt + shift, *pPktSize - shift, &date, sizeof( date ) );
	shift += sizeof( date );

	memcpy_s( pkt + shift, *pPktSize - shift, bf, bfSize );

	return ppe_ok;
}


//
//	Function:
//		pgp_LiteralDataPacket_Parse
//
//	See: openpgp-rfc2440bis-22
//
int pgp_LiteralDataPacket_Parse(
		const BYTE **pPkt,
		int *pPktSize,
		int *pCodePage,
		char **pSpecialName,
		UINT *pDate,
		const BYTE *bf,
		int bfSize
	)
{
	BYTE cp = *bf;
	if ( cp == 0x62 ) // binary
	{
		//if ( pCodePage ) *pCodePage = CP_UTF8;
	}
	else
	if ( cp == 0x74 ) // text
	{
		if ( pCodePage ) *pCodePage = CP_ACP;
	}
	else
	if ( cp == 0x75 ) // UTF_8
	{
		if ( pCodePage ) *pCodePage = CP_UTF8;
	}

	int shift = 1;

	if ( pSpecialName ) *pSpecialName = (char*)bf + shift;

	int len = strlen( (char*)bf + shift ) + 1;

	shift += len;

	if ( shift + (int)sizeof( UINT ) > bfSize ) return ppe_invalidBlock;

	if ( pDate != NULL ) *pDate = *(UINT *)(bf + shift);

	shift += sizeof( UINT );

	if ( pPkt != NULL ) *pPkt = bf + shift;
	if ( pPktSize != NULL ) *pPktSize = bfSize - shift;

	return ppe_ok;
}
