#include "Packets.h"
#include "../publicKeyAlgs/Power/Windefs.h"
#include "PGPPkts.h"
#include "PKTSignature.h"
#include "../publicKeyAlgs/publicKeyAlgs.h"
#include "../hashAlgs/hashAlgs.h"
#include "../SMemory/SBlocks.h"


cSigSubpacketParser::cSigSubpacketParser( BYTE *pkt, int pktSize )
{
	m_pkt = pkt; 
	m_cur = 0;
	m_size = pktSize;
	m_error = ppe_ok;
}

BOOL cSigSubpacketParser::GetSubPacket( )
{
	if ( m_error != ppe_ok )
		return FALSE;

	if ( m_cur == m_size )
	{	m_error = ppe_ok;
		return FALSE;
	}

	int lengthOfLength = 0;

	if ( m_pkt[ m_cur ] < 192 )
	{
		lengthOfLength = 1;
		m_spSize = m_pkt[ m_cur ];
	}
	else
	if ( m_pkt[ m_cur ] < 255 )
	{
		lengthOfLength = 2;
		m_spSize = (( int( m_pkt[ m_cur ]) - 192) << 8) + m_pkt[ m_cur + 1] + 192;
	}
	else // == 255
	{
		lengthOfLength = 5;
		m_spSize =
			( UINT( m_pkt[ m_cur + 1 ] ) << 24) |
			( UINT( m_pkt[ m_cur + 2 ] ) << 16) |
			( UINT( m_pkt[ m_cur + 3 ] ) << 8 ) |
			UINT( m_pkt[ m_cur + 4 ] );
	}

	if ( ( m_cur + lengthOfLength + m_spSize ) > m_size )
	{	m_error = ppe_invalidPacketSize;
		return FALSE;
	}

	m_spBody = m_pkt + m_cur + lengthOfLength;

	m_cur += lengthOfLength + m_spSize;

	m_spType = m_spBody[ 0 ];

	m_spTypeCritical = m_spType & 0x80;

	if ( m_spTypeCritical )
		m_spType &= ~0x80;

	if ( GetTypeName( m_spType ) == NULL )
	{
		m_error = ppe_invalidSignatureSubpacketType;
		return FALSE;
	}


	if ( !IsValidSubpacketType( m_spBody, m_spSize, &m_spTypeCritical ) && m_spTypeCritical )
	{
		m_error = ppe_invalidSignatureSubpacketType;
		return FALSE;
	}

	m_error = ppe_ok;
	return TRUE;
}

BOOL cSigSubpacketParser::IsValidSubpacketType( BYTE *pkt, int size, BOOL *pCritical )
{
	if ( size < 1 )
		return FALSE;


	if ( pCritical )
		*pCritical = pkt[0] & 0x80;

	switch( pkt[0] & ~0x80 )
	{
		case ssp_signatureCreationTime				: break;
		case ssp_signatureExpirationTime			: break;
		case ssp_exportableCertification			: break;
		case ssp_trustSignature						: break;
		case ssp_regularExpression					: break;
		case ssp_revocable							: break;
		case ssp_keyExpirationTime					: break;
		case ssp_placeholderForBackwardCompatibility: break;
		case ssp_preferredSymmetricAlgorithms		: break;
		case ssp_revocationKey						: break;
		case ssp_issuerKeyID						: break;
		case ssp_notationData						: break;
		case ssp_preferredHashAlgorithms			: break;
		case ssp_preferredCompressionAlgorithms		: break;
		case ssp_keyServerPreferences				: break;
		case ssp_preferredKeyServer					: break;
		case ssp_primaryUserId						: break;
		case ssp_policyURL							: break;
		case ssp_keyFlags							: break;
		case ssp_signerUserId						: break;
		case ssp_reasonForRevocation				: break;
		default:
			return FALSE;
	}

	return TRUE;
}

const char *cSigSubpacketParser::GetTypeName( BYTE spType )
{
	switch ( (spType & ~0x80 ))
	{
		case ssp_signatureCreationTime					: return "signature creation time";
		case ssp_signatureExpirationTime				: return "signature expiration time";
		case ssp_exportableCertification				: return "exportable certification";
		case ssp_trustSignature							: return "trust signature";
		case ssp_regularExpression						: return "regular expression";
		case ssp_revocable								: return "revocable";
		case ssp_keyExpirationTime						: return "key expiration time";
		case ssp_placeholderForBackwardCompatibility	: return "placeholder for backward compatibility";
		case ssp_preferredSymmetricAlgorithms			: return "preferred symmetric algorithms";
		case ssp_revocationKey							: return "revocation key";
		case ssp_issuerKeyID							: return "issuer key ID";
		case ssp_notationData							: return "notation data";
		case ssp_preferredHashAlgorithms				: return "preferred hash algorithms";
		case ssp_preferredCompressionAlgorithms 		: return "preferred compression algorithms";
		case ssp_keyServerPreferences					: return "key server preferences";
		case ssp_preferredKeyServer						: return "preferred key server";
		case ssp_primaryUserId							: return "primary user id";
		case ssp_policyURL								: return "policy URL";
		case ssp_keyFlags								: return "key flags";
		case ssp_signerUserId							: return "signer's user id";
		case ssp_reasonForRevocation					: return "reason for revocation";
		default : return NULL;
	}
}

const char *PKT_GetHashAlgName( BYTE algID )
{
	return GetHashAlgName( algID );
}

//
//	Function:
//		PKT_Signature_Parse
//
//	5.2. Signature Packet (Tag 2)
//
int PKT_SignatureV3_Parse(
		sSignatureV3 *sig,
		int pktSize
	)
{
	if ( sig->version > 3 )
		return ppe_invalidPacketVersion;

	int mpiCnt = PKT_GetSignatureMPICount( sig->publicKeyAlgID );

	if ( mpiCnt == 0 )
		return ppe_invalidAlgorithmID;

	int mpiLength = PKT_CheckMPIBuffer( sig->mpi, mpiCnt, pktSize - sizeof( *sig ) + sizeof( sig->mpi ) );

	if ( mpiLength == 0 )
		return ppe_invalidMPIBuffer;

	return ppe_ok;
}

int PKT_Signature_Parse(
		sSignature *sig,
		int pktSize,
		sSignatureData *sigData
	)
{
	return PKT_Signature_Parse(
		sig,
		pktSize,
		&(sigData->hashedSubPkt),
		&(sigData->hashedSize),
		&(sigData->unhashedSubPkt),
		&(sigData->unhashedSize),
		&(sigData->hashLeft16Bits),
		&(sigData->mpi) );
}

int PKT_Signature_Parse(
		sSignature *sig,
		int pktSize,
		BYTE **pHashedSubPkt,
		WORD *pHashedSize,
		BYTE **pUnhashedSubPkt,
		WORD *pUnhashedSize,
		BYTE **pHashLeft16Bits,
		BYTE **pMpi
		)
{
	if ( sig->version != 4 )
	{
		return ppe_invalidPacketVersion;
	}

	if ( PKT_Signature_GetTypeName( sig->type ) == NULL )
		return ppe_invalidSignatureType;

	BYTE *ptr = (BYTE *)sig + sizeof( sSignature );

	WORD size = ( WORD(ptr[0]) << 8 ) | ptr[1];
	ptr += sizeof( WORD );

	if ( pHashedSubPkt ) *pHashedSubPkt = ptr;
	ptr += size;

	if ( pHashedSize ) *pHashedSize = size;

	size = ( WORD(ptr[0]) << 8 ) | ptr[1];
	ptr += sizeof( WORD );

	if ( pUnhashedSubPkt ) *pUnhashedSubPkt = ptr;
	ptr += size;

	if ( pUnhashedSize ) *pUnhashedSize = size;

	if ( pHashLeft16Bits ) *pHashLeft16Bits = ptr;
	ptr += sizeof( WORD );

	if ( pMpi ) *pMpi = ptr;

	if ( ptr > (BYTE *)sig + pktSize )
		return ppe_invalidPacketSize;

	if ( PKT_GetHashAlgName( sig->hashAlgID ) == NULL )
		return ppe_invalidAlgorithmID;

	if ( PKT_GetPublicKeyAlgName( sig->publicKeyAlgID ) == NULL )
		return ppe_invalidAlgorithmID;

	int mpiCnt = PKT_GetSignatureMPICount( sig->publicKeyAlgID );

	if ( mpiCnt == 0 )
		return ppe_invalidAlgorithmID;

	if ( PKT_CheckMPIBuffer( *pMpi, mpiCnt, pktSize - ( (char *)*pMpi - (char *)sig ) ) == 0 )
		return ppe_invalidMPIBuffer;

	return ppe_ok;
}

const char *PKT_Signature_GetTypeName( BYTE type )
{
	switch ( type )
	{
		case sig_SignatureOfABinaryDocument							: return "Signature of a binary document";
		case sig_SignatureOfACanonicalTextDocument					: return "Signature of a canonical text document";
		case sig_StandaloneSignature								: return "Standalone signature";
		case sig_GenericCertificationOfAUserIDAndPublicKeyPacket	: return "Generic certification of a user ID and public key packet";
		case sig_PersonaCertificationOfAUserIDAndPublicKeyPacket	: return "Persona certification of a user ID and public key packet";
		case sig_CasualCertificationOfAUserIDAndPublicKeyPacket		: return "Casual certification of a user ID and public key packet";
		case sig_PositiveCertificationOfAUserIDAndPublicKeyPacket	: return "Positive certification of a user ID and public key packet";
		case sig_SubkeyBindingSignature								: return "Subkey binding signature";
		case sig_SignatureDirectlyOnAKey							: return "Signature directly on a key";
		case sig_KeyRevocationSignature								: return "Key revocation signature";
		case sig_SubkeyRevocationSignature							: return "Subkey revocation signature";
		case sig_CertificationRevocationSignature					: return "Certification revocation signature";
		case sig_TimestampSignature									: return "Timestamp signature";
		default : return NULL;
	}
}

int PKT_MakeSignatureBlockV3(
		BYTE *sigBlock,
		int sigBlockSize,
		BYTE pkAlgID,
		BYTE hashAlgID,
		BYTE *digest,
		int digestSize
	)
{
	if ( pkAlgID == pgppkaDSA )
	{
		if (( sigBlockSize != 20 ) || ( digestSize != 20 ) )
			return ppe_internalError;

		memcpy( sigBlock, digest, digestSize );

		return ppe_ok;
	}
	else
	if ( pkAlgID != pgppkaRSA )
		return ppe_invalidAlgorithmID;

	const static BYTE 
		rsaHashPrefix_MD2[] =
			{
				0x30, 0x20, 0x30, 0x0C, 0x06, 0x08, 0x2A, 0x86,
				0x48, 0x86, 0xF7, 0x0D, 0x02, 0x02, 0x05, 0x00,
				0x04, 0x10
			};

	const static BYTE 
		rsaHashPrefix_MD5[] =
			{
				0x30, 0x20, 0x30, 0x0C, 0x06, 0x08, 0x2A, 0x86,
				0x48, 0x86, 0xF7, 0x0D, 0x02, 0x05, 0x05, 0x00,
				0x04, 0x10
			};

	const static BYTE 
		rsaHashPrefix_RIPEMD160[] =
			{
				0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2B, 0x24,
				0x03, 0x02, 0x01, 0x05, 0x00, 0x04, 0x14
			};

	const static BYTE 
		rsaHashPrefix_SHA1[] = 
			{
				0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0E,
				0x03, 0x02, 0x1A, 0x05, 0x00, 0x04, 0x14
			};

	const BYTE *rsaHashPrefix;
	int rsaHashPrefixSize;

	switch ( hashAlgID )
	{
		case pgphaMD5 :
			rsaHashPrefix = rsaHashPrefix_MD5;
			rsaHashPrefixSize = sizeof( rsaHashPrefix_MD5 );
			break;
		case pgphaSHA1 :
			rsaHashPrefix = rsaHashPrefix_SHA1;
			rsaHashPrefixSize = sizeof( rsaHashPrefix_SHA1 );
			break;
		case pgphaRIPEMD160	:
			rsaHashPrefix = rsaHashPrefix_RIPEMD160;
			rsaHashPrefixSize = sizeof( rsaHashPrefix_RIPEMD160 );
			break;
		case pgphaMD2 :
			rsaHashPrefix = rsaHashPrefix_MD2;
			rsaHashPrefixSize = sizeof( rsaHashPrefix_MD2 );
			break;
		default : return ppe_invalidAlgorithmID;
	}

	int pos = sigBlockSize - digestSize - rsaHashPrefixSize;

	memcpy( sigBlock + pos, rsaHashPrefix, rsaHashPrefixSize );
	memcpy( sigBlock + pos + rsaHashPrefixSize, digest, digestSize );

	return RSA_PublicKeyEncryptionBlock_Format( false, sigBlock, sigBlockSize, sigBlock + pos, digestSize + rsaHashPrefixSize );
//	return PKT_PublicKeyEncryptionBlock_Format( sigBlock, sigBlockSize, sigBlock + pos, digestSize + rsaHashPrefixSize, 01 );
}

int PKT_VerifySignature(
		BYTE hashAlgID,
		BYTE pkAlgID,
		cHashedList *hashList,
		BYTE *hashAlders,
		BYTE *mpi,
		int mpiSize,
		BYTE *issuerKeyPkt,
		int issuerKeySize )
{
	//
	//	2. Makes digest from hash list
	//

	HHA ha;

	int res = ha_createAlgorithm( &ha, hashAlgID );

	if ( res != hae_ok )
		return res;

	if ( ha == NULL )
		return ppe_invalidAlgorithmID;

	for( sHashedItem *cur = hashList->GetFirst(); cur; cur = hashList->GetNext() )
	{
		res = ha_process( ha, cur->m_pkt, cur->m_pktSize );
		if ( res != hae_ok )
		{
			ha_free( ha );
			return res;
		}
	}

	cPlainTextBlock digest;
	if ( digest.Allocate( ha_getDigestSize( ha ) ) == NULL )
	{
		ha_free( ha );
		return ppe_memoryAllocation;
	}

	res = ha_final( ha, digest.buffer );

	ha_free( ha );

	if ( res != hae_ok )
		return res;

	//
	//	3. Checks
	//

	bool check = true;

	if ( check && ( memcmp( digest.buffer, hashAlders, 2 ) != 0 )  )
		return ppe_invalidSignature;

	//
	//	4.Creates algorithm and verifies digest.
	//
	HPKA pka = NULL;

	res = PKT_CreatePublicKeyAlgFromPacket(  &pka, (sPublicKey *)issuerKeyPkt, issuerKeySize );

	if ( res != ppe_ok )
		return res;

	if ( pka == NULL )
		return ppe_memoryAllocation;


	cPlainTextBlock sigBlock;

	if ( sigBlock.Allocate( pka_getBlockSize( pka ) ) == NULL )
	{
		pka_free( pka );
		return ppe_memoryAllocation;
	}

	res = PKT_MakeSignatureBlockV3( sigBlock.buffer, sigBlock.bufferSize, pkAlgID, hashAlgID, digest.buffer, digest.bufferSize );

	if ( res != ppe_ok )
	{
		pka_free( pka );
		return res;
	}

	res = pka_verify( pka, mpi, mpiSize, sigBlock.buffer, sigBlock.bufferSize );

	pka_free( pka );

	return res;
}

int PKT_VerifySignatureEx(
		BYTE *sigPkt,
		int sigPktSize,
		cHashedList *hashList,
		BYTE *issuerKeyPkt,
		int issuerKeySize )
{
	BYTE hashAlgID;
	BYTE pkAlgID;
	BYTE *hashAlders;
	BYTE *mpi;
	int mpiSize;

	BYTE version = sigPkt[0];

	if ( version < 4 )
	{
		sSignatureV3 *sig = (sSignatureV3 *)sigPkt;
		hashAlgID = sig->hashAlgID;
		pkAlgID = sig->publicKeyAlgID;
		hashAlders = sig->hashAlders;
		mpi = sig->mpi;
	}
	else
	{
		sSignature *sig = ( sSignature *)sigPkt;
		hashAlgID = sig->hashAlgID;
		pkAlgID = sig->publicKeyAlgID;

		int res = PKT_Signature_Parse( sig, sigPktSize, NULL, NULL, NULL, NULL, &hashAlders, &mpi );

		if ( res != ppe_ok ) return res;

	}

//	mpiSize = sigPktSize - ((int)mpi - (int)sigPkt);
	mpiSize = sigPktSize - (int)(mpi - sigPkt);

	return PKT_VerifySignature( hashAlgID, pkAlgID, hashList, hashAlders, mpi, mpiSize, issuerKeyPkt, issuerKeySize );
}


//
//	Function:
//		verifySignature
//
//	1. Gets issuer key ( if the key is self signed so uses the key instead ).
//	2. Makes digest from hash list
//	3. Checks digest with two alder bytes.
//	4. Creates issuer public key algorithm and verifies digest.
//
static int verifySignature(
		BYTE hashAlgID,
		BYTE pkAlgID,
		cHashedList *hashList,
		BYTE *hashAlders,
		BYTE *mpi,
		int mpiSize,
		BYTE *issuerKeyID,
		BYTE *keyPkt,
		int keySize,
		BYTE *issuerKeyRing,
		int issuerKeyRingSize
		)
{
	//
	//	1. Gets issuer key.
	//

	BYTE *issuerKeyPkt;
	int issuerKeySize;

	BOOL selfSigned = false;

	int res;

	if ( ( keyPkt != NULL ) || ( keySize != 0 ) )
	{
		BYTE keyID[ pgp_keyIDSize ];
		res = PKT_GetPublicKeyIDInternal( (sPublicKey *)keyPkt, keySize, keyID );
		if ( res != ppe_ok )
			return res;

		selfSigned = memcmp( keyID, issuerKeyID, pgp_keyIDSize ) == 0;
	}

	if ( selfSigned )
	{
		issuerKeyPkt = keyPkt;
		issuerKeySize = keySize;
	}
	else
	{
		if ( ( issuerKeyRing == NULL ) || ( issuerKeyRingSize == 0 ) )
			return ppe_signatureIssuerKeyRingDoesNotSpecified;

		res = pgp_FindKeyByID(
			&issuerKeyPkt,
			&issuerKeySize,
			issuerKeyID,
			issuerKeyRing,
			issuerKeyRingSize );

		if ( res != ppe_ok )
		{
			if ( res == ppe_keyNotFound )
				return ppe_signatureIssuerNotFound;
			else
				return res;
		}
	}

	return PKT_VerifySignature( hashAlgID, pkAlgID, hashList, hashAlders, mpi, mpiSize, issuerKeyPkt, issuerKeySize );
}

int PKT_VerifySignaturePacketV3(
	sSignatureV3 *sig,
	int sigPktSize,
	cHashedList *hashList,
	BYTE *keyPkt,
	int keySize,
	BYTE *issuerKeyRing,
	int issuerKeyRingSize 
	)
{
	int res = PKT_SignatureV3_Parse( sig, sigPktSize );
	if ( res != ppe_ok )
		return res;

	if ( !hashList->Add( &(sig->type), sig->hashedLength ) )
		return ppe_memoryAllocation;

	return verifySignature( 
				sig->hashAlgID,
				sig->publicKeyAlgID,
				hashList,
				sig->hashAlders,
				sig->mpi,
				sigPktSize - sizeof( *sig ) + sizeof( sig->mpi ),
				sig->keyID,
				keyPkt,
				keySize,
				issuerKeyRing,
				issuerKeyRingSize
				);
}

sTime *PKT_findSignatureTime( BYTE *sp, int spSize )
{
	BYTE version = sp[0];

	if ( version == 3 )
		return &((sSignatureV3 *)sp)->time;

	cSigSubpacketParser csp( sp, spSize );

	while ( csp.GetSubPacket() )
	{
		if ( ( csp.m_spType == ssp_signatureCreationTime ) && ( csp.m_spSize == 5 ) )
			return (sTime *)( csp.m_spBody + 1 );
	}

	return NULL;
}

BYTE *PKT_findIssuerKeyID( BYTE *sp, int spSize )
{
	BYTE version = sp[0];

	if ( version == 3 )
		return ((sSignatureV3 *)sp)->keyID;

	cSigSubpacketParser csp( sp, spSize );

	while ( csp.GetSubPacket() )
	{
		if ( ( csp.m_spType == ssp_issuerKeyID ) && ( csp.m_spSize == 9 ) )
			return csp.m_spBody + 1;
	}

	return NULL;
}

int PKT_VerifySignaturePacket(
	sSignature *sigPkt,
	int sigPktSize,
	cHashedList *hashList,
	BYTE *keyPkt,
	int keyPktSize,
	BYTE *issuerKeyRing,
	int issuerKeyRingSize,
	BYTE *issuerKeyIDBuff // = NULL
	)
{
	BYTE version = sigPkt->version;

	if ( version < 4 )
	{
		sSignatureV3 *sig = (sSignatureV3 *)sigPkt;

		if ( issuerKeyIDBuff )
			memcpy( issuerKeyIDBuff, sig->keyID, pgp_keyIDSize );

		return
			PKT_VerifySignaturePacketV3(
				sig,
				sigPktSize,
				hashList,
				keyPkt,
				keyPktSize,
				issuerKeyRing,
				issuerKeyRingSize
			);
	}
	else
	{
		BYTE *spHashed;
		WORD spHashedSize;
		BYTE *spUnhashed;
		WORD spUnhashedSize;
		BYTE *hashAlders;
		BYTE *MPIs;

		int res = PKT_Signature_Parse( sigPkt, sigPktSize, &spHashed, &spHashedSize, &spUnhashed, &spUnhashedSize, &hashAlders, &MPIs );
		if ( res != ppe_ok )
			return res;

		int mpiSize = sigPktSize - int( MPIs - (BYTE*)sigPkt );
		BYTE *issuerKeyID = PKT_findIssuerKeyID ( spUnhashed, spUnhashedSize );
		if ( issuerKeyID == NULL )
			return ppe_signatureIssuerSubpacketNotFound;

		if ( issuerKeyIDBuff )
			memcpy( issuerKeyIDBuff, issuerKeyID, pgp_keyIDSize );

/*
   A V4 signature hashes the packet body
   starting from its first field, the version number, through the end of
   the hashed subpacket data. Thus, the fields hashed are the signature
   version, the signature type, the public key algorithm, the hash
   algorithm, the hashed subpacket length, and the hashed subpacket
   body.
*/
		long reallyHashedSize = long ( (char *)(spHashed + spHashedSize) - (char *)sigPkt );

		if ( !hashList->Add( (BYTE *)sigPkt, (int)reallyHashedSize  ) )
			return ppe_memoryAllocation;
/*
   V4 signatures also hash in a final trailer of six octets: the version
   of the signature packet, i.e. 0x04; 0xFF; a four-octet, big-endian
   number that is the length of the hashed data from the signature
   packet (note that this number does not include these final six
   octets.
*/
		BYTE tmp[6];

		tmp[0] = 0x04;
		tmp[1] = 0xFF;

		//
		// Note, the following is not 'big-endian' ( see the note about last trailer above ),
		// but only the little-endian way leads to valid signature :)
		//
		tmp[2] = (BYTE)((UINT)reallyHashedSize >> 24);
		tmp[3] = (BYTE)((UINT)reallyHashedSize >> 16);
		tmp[4] = (BYTE)((UINT)reallyHashedSize >>  8);
		tmp[5] = (BYTE)((UINT)reallyHashedSize      );

		if ( !hashList->Add( tmp, 6 ) )
			return ppe_memoryAllocation;

		return verifySignature(
					sigPkt->hashAlgID,
					sigPkt->publicKeyAlgID,
					hashList,
					hashAlders,
					MPIs,
					mpiSize,
					issuerKeyID,
					keyPkt,
					keyPktSize,
					issuerKeyRing,
					issuerKeyRingSize
				);
	}
}
