#include "Packets.h"
#include "PGPPkts.h"
#include <assert.h>

#include "../hashAlgs/hashAlgs.h" // For Public Key Fingerprint generation...
				  // See PKT_GetPublicKeyFingerprint and PKT_GetPublicKeyID functions.
#include "../symmetricAlgs/symmetricAlgs.h"
#include "../SMemory/SBlocks.h"
#include "../RNG/cRNG.h"
#include "../publicKeyAlgs/cPKA.h"

//
//	Non secure random generator...
//
#include <stdlib.h>
#include <time.h>

static void generateRandomBytes( BYTE *bf, int bfSize )
{
	srand( (int)time(0) ^ (int)clock() );
	while( bfSize-- )
		bf[ bfSize ] = (BYTE)rand();
}


//
//	Public key encryption block functions
//
static void RSA_makePKEBPaddingString( BYTE *bf, int bfSize, bool publicEncryption )
{

	if ( publicEncryption )
	{
		generateRandomBytes( bf, bfSize );
		bf[ bfSize ] = 0x00;
		//
		//	increment zero bytes.
		//
		while ( bfSize-- )
			if ( bf[ bfSize ] == 0 )
				bf[ bfSize ] = 1;
	}
	else
	{
		memset( bf, 0xFF, bfSize );
		bf[ bfSize ] = 0x00;
	}
}
/*
static void makePKEBPaddingString( BYTE *bf, int bfSize, BYTE blockType )
{
	switch (blockType)
	{
		case 0x01:
			memset( bf, 0, bfSize );
			bf[ bfSize ] = 0xFF;
			break;
		case 0x00:
			memset( bf, 0xFF, bfSize );
			bf[ bfSize ] = 0;
			break;
		default:
			generateRandomBytes( bf, bfSize );

			bf[ bfSize ] = 0;
			//
			//	increment zero bytes.
			//
			while ( bfSize-- )
				if ( !bf[ bfSize ] )
					bf[ bfSize ] = 1;
	}
}

static void makePKEBPaddingString( BYTE *bf, int bfSize, BYTE blockType )
{
	switch (blockType)
	{
		case 0x00:
			memset( bf, 0, bfSize );
			break;
		case 0x01:
			memset( bf, 0xFF, bfSize );
			break;
		default:
			generateRandomBytes( bf, bfSize );

			//
			//	increment zero bytes.
			//
			while ( bfSize-- )
				if ( !bf[ bfSize ] )
					bf[ bfSize ] = 1;
	}
}
*/
//
//	5.1. Public-Key Encrypted Session Key Packets (Tag 1)
//
int PKT_GetEncryptedMessageMPICount( BYTE algID )
{
	int mpiCnt;

	switch ( algID )
	{
		case 1:
		case 2:
		case 3:		// RSA ID.
			mpiCnt = 1;
			break;
		case 16:
		case 20:	// ElGamal
			mpiCnt = 2;
			break;
//		case 17:	// DSA
//			mpiCnt = 2;
//			break;
		default:
			mpiCnt = 0;
			break;
	}

	return mpiCnt;
}

int PKT_GetSignatureMPICount( BYTE algID )
{
	int mpiCnt;

	switch ( algID )
	{
		case 1:
		case 2:
		case 3:		// RSA ID.
			mpiCnt = 1;
			break;
		case 16:
		case 20:	// ElGamal
			mpiCnt = 2;
			break;
		case 17:	// DSA
			mpiCnt = 2;
			break;
		default:
			mpiCnt = 0;
			break;
	}

	return mpiCnt;
}

//
//	5.5.3. Secret Key Packet Formats
//
//	Algorithm Specific Fields for RSA secret keys:
//	- multiprecision integer (MPI) of RSA secret exponent d.
//	- MPI of RSA secret prime value p.
//	- MPI of RSA secret prime value q (p < q).
//	- MPI of u, the multiplicative inverse of p, mod q.
//
//	Algorithm Specific Fields for DSA secret keys:
//	- MPI of DSA secret exponent x.
//
//	Algorithm Specific Fields for Elgamal secret keys:
//	- MPI of Elgamal secret exponent x.
//
int PKT_GetSecretKeyEncryptedPartMPICount( BYTE algID )
{
	int mpiCnt;

	switch ( algID )
	{
		case 1:
		case 2:
		case 3:		// RSA ID.
			mpiCnt = 4;
			break;
		case 16:
		case 20:	// ElGamal
			mpiCnt = 1;
			break;
		case 17:	// DSA
			mpiCnt = 1;
			break;
		default:
			mpiCnt = 0;
			break;
	}

	return mpiCnt;
}
/*
static int SHA1MakeDigest( BYTE *digest, BYTE *message, int messageSize )
{
	HHA  hHa = ha_createAlgorithm( 2 );

	if ( hHa == NULL )
		return ppe_invalidAlgorithmID;

	if ( ha_getDigestSize( hHa ) != 20 )
		return ppe_invalidAlgorithmID;

	if ( ha_makeDigest( hHa, digest, message, messageSize ) != hae_ok )
		return ppe_invalidAlgorithmID;

	ha_free( hHa );

	return ppe_ok;
}
*/
//
//	getFingerprintV3
//		hashs bodyes of two mpis
//
static int getFingerprintV3( sPublicKeyV3 *pk, int pkSize,  BYTE *bf20Bytes )
{
	int mpiBuffSize = pkSize - sizeof( sPublicKeyV3 ) + sizeof( pk->mpi );

	if ( PKT_CheckMPIBuffer( pk->mpi, 2, mpiBuffSize ) == 0 )
		return ppe_invalidMPIBuffer;

	HHA  hHa;

	int res = ha_createAlgorithm( &hHa, 1 );

	if ( res == hae_invalidAlgorithmID )
		return ppe_invalidAlgorithmID;

	if ( hHa == NULL )
		return ppe_memoryAllocation;

	//
	//	process modulus
	//
	BYTE *mpi = pk->mpi;
	int  mpiLen = (((int(mpi[0]) << 8) | mpi[1]) + 7 ) / 8;

	res = ha_process( hHa, mpi + 2, mpiLen );
	if ( res != hae_ok )
	{
		ha_free( hHa );
		return res;
	}

	//
	//	process public exponent
	//

	mpi += mpiLen + 2;
	mpiLen = (((int(mpi[0]) << 8) | mpi[1]) + 7 ) / 8;

	res = ha_process( hHa, mpi + 2, mpiLen );

	if ( res == hae_ok )
		ha_final( hHa, bf20Bytes );

	ha_free( hHa );

	return res;
}

//
//	getFingerprintV4
//		Creates finger print for subkeys and secret keys...
//
static int getFingerprintV4( sTime time, BYTE algID, BYTE *mpi, int mpiSize,  BYTE *bf20Bytes )
{
	int mpiCnt = PKT_GetPublicKeyMPICount( algID );

	if ( mpiCnt == 0 )
		return ppe_invalidAlgorithmID;

	int mpiLen = PKT_CheckMPIBuffer( mpi, mpiCnt, mpiSize );

	if ( mpiLen == 0 )
		return ppe_invalidPacketSize;

	HHA  hHa;

	int res = ha_createAlgorithm( &hHa, 2 );

	if ( res == hae_invalidAlgorithmID )
		return ppe_invalidAlgorithmID;

	if ( hHa == NULL )
		return ppe_memoryAllocation;

#pragma pack(1)
	struct spkt
	{
		BYTE hdr;
		BYTE len1;
		BYTE len0;
		sPublicKey pk;
	} pkt;
#pragma pack()

	int len = mpiLen + sizeof( sPublicKey ) - 2;

	pkt.hdr = 0x99;
	pkt.len1 = BYTE( len >> 8 );
	pkt.len0 = BYTE( len );

	pkt.pk.version = 4;
	pkt.pk.time = time;
	pkt.pk.algID = algID;

	ha_process( hHa, (BYTE *)&pkt, sizeof( spkt ) - 2 );
	ha_process( hHa, mpi, mpiLen );
	ha_final( hHa, bf20Bytes );

	ha_free( hHa );

	return ppe_ok;
}

//
//	11.2. Key IDs and Fingerprints
//
int PKT_GetPublicKeyFingerprint( BYTE *pk, int totalPktSize, BYTE *bf20Bytes )
{
	BYTE tag;
	int dataSize;
	int hdrSize;
	bool partial = 0;

	int res = pgp_PacketHeader_Parse( &tag, &hdrSize, &dataSize, &partial, pk, totalPktSize );

	if ( res != ppe_ok )
		return res;

	memset( bf20Bytes, 0, 20 );

	BYTE ver = pk[ hdrSize ];

	if ( ver != 4 )
		return getFingerprintV3( (sPublicKeyV3 *)(pk + hdrSize), totalPktSize, bf20Bytes ) ;

	sPublicKey *spk = (sPublicKey *)(pk + hdrSize);

	return getFingerprintV4( spk->time, spk->algID, spk->mpi, dataSize - sizeof( sPublicKey ) + 2, bf20Bytes );

//	return SHA1MakeDigest( bf20Bytes, pk, hdrSize + dataSize );
}

int PKT_GetPublicKeyFingerprintInternal( sPublicKey *pk, int totalPktSize, BYTE *bf20Bytes )
{
	memset( bf20Bytes, 0, 20 );

	if ( pk->version < 4 )
		return getFingerprintV3( (sPublicKeyV3 *)pk, totalPktSize, bf20Bytes );
	else
	if ( pk->version == 4 )
		return getFingerprintV4( pk->time, pk->algID, pk->mpi, totalPktSize - sizeof( sPublicKey ) + sizeof( pk->mpi ), bf20Bytes );

	return ppe_invalidPacketVersion;
}

//
//	Function:
//		getPublicKeyIDV3
//	returns low 8 bytes of modulus.
//
static int getPublicKeyIDV3( sPublicKeyV3 *pk, int totalPktSize, BYTE *bf8Bytes )
{
	int  mpiLen = PKT_CheckMPIBuffer( pk->mpi, 1, totalPktSize - sizeof(sPublicKeyV3) + sizeof( pk->mpi) );

	for ( int i = 8; i--; )
		bf8Bytes[ i ] = *(pk->mpi + mpiLen - 8 + i);

	return ppe_ok;
}

int PKT_GetPublicKeyID( BYTE *pk, int totalPktSize, BYTE *bf8Bytes )
{
	BYTE tag;
	int dataSize;
	int hdrSize;
	bool partial = 0;

	int res = pgp_PacketHeader_Parse( &tag, &hdrSize, &dataSize, &partial, pk, totalPktSize );

	if ( res != ppe_ok )
		return res;

	memset( bf8Bytes, 0, 8 );

	BYTE ver = pk[ hdrSize ];

	if ( ver == 4 )
	{
		BYTE bf[20];
		memset( bf, 0, sizeof( bf ) );

		res = PKT_GetPublicKeyFingerprint( pk, totalPktSize, bf );

		if ( res == ppe_ok )
			memcpy( bf8Bytes, bf + 12, 8 );
	}
	else
	if ( ver < 4 )
	{
		return getPublicKeyIDV3( (sPublicKeyV3 *)(pk + hdrSize), totalPktSize, bf8Bytes );
	}
	else
		return ppe_invalidPacketVersion;

	return res;
}

int PKT_GetPublicKeyIDInternal( sPublicKey *pk, int totalPktSize, BYTE *bf8Bytes )
{
	memset( bf8Bytes, 0, 8 );

	if ( pk->version == 4 )
	{
		BYTE bf[20];
		memset( bf, 0, sizeof( bf ) );

		int res = PKT_GetPublicKeyFingerprintInternal( pk, totalPktSize, bf );

		if ( res == ppe_ok )
			memcpy( bf8Bytes, bf + 12, 8 );

		return res;
	}
	else
	if ( pk->version < 4 )
	{
		return getPublicKeyIDV3( (sPublicKeyV3 *)pk, totalPktSize, bf8Bytes );
	}
	else
		return ppe_invalidPacketVersion;

}

//
//	5.5.2. Public Key Packet Formats
//
int PKT_GetPublicKeyMPICount( BYTE algID )
{
	int mpiCnt;

	switch ( algID )
	{
		case 1:
		case 2:
		case 3:		// RSA ID.
			mpiCnt = 2;
			break;
		case 16:
		case 20:	// ElGamal
			mpiCnt = 3;
			break;
		case 17:	// DSA
			mpiCnt = 4;
			break;
		default:
			mpiCnt = 0;
			break;
	}

	return mpiCnt;
}

const char *PKT_GetPublicKeyAlgName( BYTE algID )
{
	switch ( algID )
	{
		case 1:	 return "RSA";
		case 2:	 return "RSA Encrypt-Only";
		case 3:	 return "RSA Sign-Only";
		case 16: return "Elgamal (Encrypt-Only)";
		case 20: return "ElGamal";
		case 17: return "DSA";
		default: return NULL;
	}
}

const char *PKT_GetSymmetricAlgName( BYTE algID )
{
	switch ( (DWORD)algID )
	{
		case pgpsaPlaintext	: return "Plaintext or unencrypted data";
		case pgpsaIDEA		: return "IDEA";
		case pgpsaTripleDES	: return "Triple-DES";
		case pgpsaCAST5		: return "CAST5";
		case pgpsaBlowfish	: return "Blowfish";
		case pgpsaSAFER_SK128: return "SAFER-SK128";
		case pgpsa2KeyTripleDES: return "2-Key Triple-DES";
//		case ___saDES		: return "DES";
		case pgpsa128BitRC4	: return "128BitRC4";
		case pgpsa40BitRC4	: return "40BitRC4";
		case pgpsaAES_128	: return "AES-128";
		case pgpsaAES_192	: return "AES-192";
		case pgpsaAES_256	: return "AES-256";
		case pgpsaTFISH		: return "TWOFISH";
		case pgpsa128BitRC2	: return "128BitRC2";
		case pgpsa40BitRC2	: return "40BitRC2";
		default : return NULL;
	}
}

//
//	Returns real parsed size
//
int PKT_CheckMPIBuffer( BYTE *mpis, int cnt, int size )
{
	int pos = 0;
	int bits, bytes;

	while( cnt-- )
	{
		if ( pos + 2 >= size )
			return 0;

		bits = ((int)mpis[ pos ] << 8 ) | mpis[ pos + 1 ];

		bytes = ( bits + 7 ) / 8;

		pos += 2 + bytes;

		if ( pos > size )
			return 0;
	}

	return pos;
}

//
//	Function: 
//		PKT_PublicKeyEncryptionBlock_Format
//
//	Formats public key encryption block.
//
//	PKCS-1.
//	8.1 Encryption-block formatting.
//		Block type values are 00, 01, or 02.
//		For a private-key operation, the block type shall be 00 or 01.
//		For a public-key operation, it shall be 02.
//
static bool checkPKEBBlockType( BYTE type )
{
	switch ( type )
	{
		case 0:
		case 1:
		case 2:		return true;
		default:	return false;
	}
}
/*
int PKT_PublicKeyEncryptionBlock_Format(
		BYTE *outBuf,
		int outBufSize,
		const BYTE *data,
		int dataSize,
		BYTE blockType )
{
	//
	//    EB = 00 || BT || PS || 00 || D .       
	//

	if ( !checkPKEBBlockType( blockType ) )
		return ppe_invalidBlockType;

	int psLen = outBufSize - dataSize - 3;

	if ( ( psLen <= 0 ) ||
		(( blockType == 0x02 ) && ( psLen < 8 ) ) // See Note 6.
		)
		return ppe_bufferTooSmall;

	memset( outBuf, 0, outBufSize - dataSize );

	outBuf[0] = 0;
	outBuf[1] = blockType;
	makePKEBPaddingString( outBuf + 2, psLen, blockType );
	memcpy( outBuf + psLen + 3, data, dataSize );

	return ppe_ok;
}
*/
//
// PKCS1 v2, Encoding methods
//
int RSA_PublicKeyEncryptionBlock_Format(
		bool publicKeyEncryption,// if false the method is used for signatures by private -> block type 01
								//  if true the method is used for encryption by public   -> block type 02
		BYTE *outBuf,
		int outBufSize,
		const BYTE *data,
		int dataSize
		)
{
	//
	//    EB = 00 || BT || PS || 00 || D .       
	//

	if ( dataSize > outBufSize - 10 )
		return ppe_messageTooLong; // PKCS1 v2 p 9.1.2.1, Encoding operation, Step 1 ( Encoding method for encryption )
								  //  PKCS1 v2 p 9.2.1,   EMSA-PKCS1-v1_5,    Step 3 ( Encoding method for signatures )

	int psLen = outBufSize - dataSize - 3;

	BYTE blockType = publicKeyEncryption ? 0x02 : 0x01;

	memset( outBuf, 0, outBufSize - dataSize );

	outBuf[0] = 0;
	outBuf[1] = blockType;
	RSA_makePKEBPaddingString( outBuf + 2, psLen, publicKeyEncryption );
	memcpy( outBuf + psLen + 3, data, dataSize );

	return ppe_ok;
}

//
//	Function:
//		RSA_PublicKeyEncryptionBlock_Parse
//
//	Parses and extracts data from public key encrtion block.
//
int RSA_PublicKeyEncryptionBlock_Parse(
		bool publicKeyEncryption, // if false the method is used for signatures by private   -> block type 01 || FF ... || 00
								//  if true the method is used for encryption by public key -> block type 02 || RANDOM || 00
		const BYTE *inBuf,
		int inBufSize,
		const BYTE **pData, // position of data in input buffer
		int *pDataSize
		)
{
	if ( (inBufSize < 3) || inBuf[0] )
		return ppe_invalidBlockType;

	BYTE blockType = inBuf[1];

	int dataPos = 2;

	switch ( blockType )
	{
		case 0x01:
			if ( publicKeyEncryption )
				return ppe_invalidBlockType;
			// finds first zero octet
			while ( inBuf[ dataPos++ ] == 0xFF  )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;

			if ( inBuf[ dataPos - 1 ] != 0 )
				return ppe_invalidBlockType;

			break;
	/*	case 0x00:
			if ( publicKeyEncryption )
				return ppe_invalidBlockType;
			// finds first non zero octet
			while ( inBuf[ dataPos++ ] == 0 )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;

			if ( inBuf[ dataPos - 1 ] != 0xFF )
				return ppe_invalidBlockType;

			break;
	*/
		case 2:
			if ( !publicKeyEncryption )
				return ppe_invalidBlockType;
			// finds first zero octet
			while ( inBuf[ dataPos++ ] != 0  )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;
			break;
		default:
			return ppe_invalidBlockType;
	}

	if ( pDataSize != NULL )
		*pDataSize = inBufSize - dataPos;

	if ( pData != NULL )
		*pData = inBuf + dataPos;

	return ppe_ok;
}

//
//	Function:
//		PKT_PublicKeyEncryptionBlock_Parse
//
//	Parses and extracts data from public key encrtion block.
//
int PKT_PublicKeyEncryptionBlock_Parse(
		BYTE *data,
		int *pDataSize,
		BYTE *inBuf,
		int inBufSize
		)
{
	if ( (inBufSize < 3) || inBuf[0] )
		return ppe_invalidBlockType;

	BYTE blockType = inBuf[1];

	if ( !checkPKEBBlockType( blockType ) )
		return ppe_invalidBlockType;

	int dataPos = 2;

	switch ( blockType )
	{
		case 0x01:
			if ( inBuf[ dataPos ] == 0xFF )
			{
				// Goes to next case.
			}
			else
			{
				// finds first non zero octet
				while ( inBuf[ dataPos++ ] == 0 )
					if ( dataPos >= inBufSize - 1 )
						return ppe_invalidBlockType;

				if ( inBuf[ dataPos - 1 ] != 0xFF )
					return ppe_invalidBlockType;

				break;
			}
		case 0x00:
			// finds first zero octet
			while ( inBuf[ dataPos++ ] == 0xFF  )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;

			if ( inBuf[ dataPos - 1 ] != 0 )
				return ppe_invalidBlockType;

			break;
		default:
			// finds first zero octet
			while ( inBuf[ dataPos++ ] != 0  )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;
			break;
	}

	if ( pDataSize == NULL )
		return ppe_ok;

	if ( inBufSize - dataPos > (*pDataSize) )
	{
		*pDataSize = inBufSize - dataPos;
		return ppe_bufferTooSmall;
	}

	*pDataSize = inBufSize - dataPos;

	memcpy( data, inBuf + dataPos, *pDataSize );

	return ppe_ok;
}

int PKT_PublicKeyEncryptionBlock_Parse_Rudiment(
		BYTE *data,
		int *pDataSize,
		BYTE *inBuf,
		int inBufSize )
{
	if ( (inBufSize < 3) || inBuf[0] )
		return ppe_invalidBlockType;

	BYTE blockType = inBuf[1];

	if ( !checkPKEBBlockType( blockType ) )
		return ppe_invalidBlockType;

	int dataPos = 2;

	switch ( blockType )
	{
/* Error 00 type filled by 00 - fixed in v 1.01
		case 0x00:
			// finds first non zero octet
			while ( inBuf[ dataPos ] == 0 )
				if ( ++dataPos >= inBufSize )
					return ppe_invalidBlockType;
			break;
		default:
			// finds first zero octet
			while ( inBuf[ dataPos++ ] != 0  )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;
			break;
*/
// Valid, but we have to say that the old 1 and FF,FF,.. 00 is valid
		case 0x01:
			if ( inBuf[ dataPos ] == 0 ) // If FF uses default check 
			{
				// finds first non zero octet
				while ( inBuf[ dataPos++ ] == 0 )
					if ( dataPos >= inBufSize - 1 )
						return ppe_invalidBlockType;

				if ( inBuf[ dataPos - 1 ] != 0xFF )
					return ppe_invalidBlockType;

				break;
			}
		case 0x00:
			// finds first zero octet
			while ( inBuf[ dataPos++ ] == 0xFF  )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;

			if ( inBuf[ dataPos - 1 ] != 0 )
				return ppe_invalidBlockType;

			break;
		default:
			// finds first zero octet
			while ( inBuf[ dataPos++ ] != 0  )
				if ( dataPos >= inBufSize - 1 )
					return ppe_invalidBlockType;
			break;
	}

	if ( inBufSize - dataPos > (*pDataSize) )
	{
		*pDataSize = inBufSize - dataPos;
		return ppe_bufferTooSmall;
	}

	*pDataSize = inBufSize - dataPos;

	memcpy( data, inBuf + dataPos, *pDataSize );

	return ppe_ok;
}
//
//	RFC 2440.										[Pages 12,13,14]
//	4.2. Packet Headers
//
//	Packet tags.
//
//       0        -- Reserved - a packet tag must not have this value
//       1        -- Public-Key Encrypted Session Key Packet
//       2        -- Signature Packet
//       3        -- Symmetric-Key Encrypted Session Key Packet
//       4        -- One-Pass Signature Packet
//       5        -- Secret Key Packet
//       6        -- Public Key Packet
//       7        -- Secret Subkey Packet
//       8        -- Compressed Data Packet
//       9        -- Symmetrically Encrypted Data Packet
//      10        -- Marker Packet
//      11        -- Literal Data Packet
//      12        -- Trust Packet
//      13        -- User ID Packet
//      14        -- Public Subkey Packet
//      60 to 63  -- Private or Experimental Values
//


int PKT_GetTagName( BYTE tag, const char **pStr )
{
	switch ( tag )
	{
		case  1: *pStr = "Public-Key Encrypted Session Key Packet";		break;
		case  2: *pStr = "Signature Packet";					break;
		case  3: *pStr = "Symmetric-Key Encrypted Session Key Packet";		break;
		case  4: *pStr = "One-Pass Signature Packet";				break;
		case  5: *pStr = "Secret Key Packet";					break;
		case  6: *pStr = "Public Key Packet";					break;
		case  7: *pStr = "Secret Subkey Packet";				break;
		case  8: *pStr = "Compressed Data Packet";				break;
		case  9: *pStr = "Symmetrically Encrypted Data Packet";			break;
		case 10: *pStr = "Marker Packet";					break;
		case 11: *pStr = "Literal Data Packet";					break;
		case 12: *pStr = "Trust Packet";					break;
		case 13: *pStr = "User ID Packet";					break;
		case 14: *pStr = "Public Subkey Packet";				break;
		case 17: *pStr = "User Attribute Packet";				break;
		case 18: *pStr = "Sym. Encrypted and Integrity Protected Data Packet";	break;
		case 19: *pStr = "Modification Detection Code Packet";			break;
		case pkt_0x3d : *pStr = "pkt_0x3d";					break;
		default:
			*pStr = "Undefined Packet";
			return ppe_invalidBlockType;
	}

	return ppe_ok;
}

//
//	Function:
//		PKT_PublicKeyEncryptedSessionKey_Parse
//	See RFC 2440
//			5.1. Public-Key Encrypted Session Key Packets (Tag 1)
//
//	Assigns positions of keyID, algId, and encrypted message MPI.
//
int PKT_PublicKeyEncryptedSessionKey_Parse(
		sPublicKeyEncryptedSessionKey *pkt,
		int pktSize,
		int *pKeyMpiSize
		)
{
	if ( pktSize < (int)sizeof(sPublicKeyEncryptedSessionKey) )
		return ppe_invalidPacketSize;

	//
	//	Check version number. V2 and V3 is valid.
	//
	if ( ( pkt->version != 2 ) && ( pkt->version !=3) )
		return ppe_invalidPacketVersion;

	int mpiCnt = PKT_GetEncryptedMessageMPICount( pkt->algID );

	if ( mpiCnt == 0 )
		return ppe_invalidAlgorithmID;

	int mpiSize = pktSize - ( sizeof( *pkt ) - sizeof( pkt->mpi ) );

	int parsedMpiSize = PKT_CheckMPIBuffer( pkt->mpi, mpiCnt, mpiSize );

	if ( parsedMpiSize == 0 )
		return ppe_invalidMPIBuffer;

	if ( pKeyMpiSize )
		*pKeyMpiSize = parsedMpiSize;

	return ppe_ok;
}

int PKT_PublicKeyEncryptedSessionKey_Format(
		sPublicKeyEncryptedSessionKey *pkt,
						// [OUT] Packet buffer
		int  *pPktSize, // [IN]  Pointer to size of allocated buffer
						// [OUT] space really busied or required for the packet
		BYTE *keyID,	// [IN]  Key indentificator
		BYTE algID,		// [IN]  Public key algorithm
		BYTE *keyMpi,	// [IN]  Encrypted key
						// if RSA		one MPI = key^e mod p
						// if Elgamal	two MPIs: 1) g^k mod p, 2) m * (y^k) mod p
		int  keyMpiSize	// [IN]  Size of encrypted key buffer.
		)
{
	//
	//	Check size of key packet header with MPI buffer.
	//

	int hdrSize = sizeof( *pkt ) - sizeof( pkt->mpi );

	if ( *pPktSize < (int)( keyMpiSize + hdrSize ) )
	{
		*pPktSize = ( keyMpiSize + hdrSize );
		return ppe_bufferTooSmall;
	}

	*pPktSize = ( keyMpiSize + hdrSize );

	pkt->version = 3;
	memcpy( pkt->keyID, keyID, 8 );
	pkt->algID = algID;

	memcpy( pkt->mpi, keyMpi, keyMpiSize );

	return ppe_ok;
}

static WORD getCheckSumm( const BYTE *byteBuff, int byteBuffSize )
{
	WORD summ = 0;

	for ( int i = 0; i < byteBuffSize; i += 2 )
	{
		WORD w = ( WORD(byteBuff[ i ]) << 8 ) | byteBuff[ i + 1 ];
		summ = summ + w;
	}

	return summ;
}

static WORD getCheckSummByte( const BYTE *byteBuff, int byteBuffSize )
{
	WORD summ = 0;

	for ( int i = 0; i < byteBuffSize; i ++ )
		summ = summ + byteBuff[ i ];

	return summ;
}


//
//	5.1. Public-Key Encrypted Session Key Packets (Tag 1)
//
//	Ecrypted message format "m"
//
//   The value "m" in the above formulas is derived from the session key
//   as follows.  First the session key is prefixed with a one-octet
//   algorithm identifier that specifies the symmetric encryption
//   algorithm used to encrypt the following Symmetrically Encrypted Data
//   Packet.  Then a two-octet checksum is appended which is equal to the
//   sum of the preceding session key octets, not including the algorithm
//   identifier, modulo 65536.  This value is then padded as described in
//   PKCS-1 block type 02 [RFC2313] to form the "m" value used in the
//   formulas above.
//

//
// Attention!
// There are two differences in generation session key packet
// by pgp_SessionKeyBlock_Format and old RSA_SessionKeyBlock_Format
//
//	1. Value D
//		(pgp) = Session key alg ID || Session key || Check summ
//		(old) = Session key alg ID || Check summ || Session key
//	2. The checksumm generation 
//		(pgp) uses getCheckSummByte
//		(old) uses getCheckSumm
//
// Note, RSA_SessionKeyBlock_Parse ( instead of RSA_SessionKeyBlock_Parse_OLD )
//	checks both variants in parcing( first the old, because it is used by BCArchive, second as pgp checks ).
//

int pgp_SessionKeyBlock_Format( 
		BYTE *outBuf,
		int outBufSize,
		BYTE algID,
		const BYTE *sessKey,
		int sessKeySize
	)
{
	WORD checkSumm = getCheckSummByte( sessKey, sessKeySize );
	int dataSize = sizeof( algID ) + sessKeySize + sizeof( checkSumm );

	//
	//    EB = 00 || BT || PS || 00 || D .  Where D = Session key alg ID || Session key || Check summ     
	//

	if ( dataSize > outBufSize - 10 )
		return ppe_messageTooLong; // PKCS1 v2 p 9.1.2.1, Encoding operation, Step 1 ( Encoding method for encryption )
								  //  PKCS1 v2 p 9.2.1,   EMSA-PKCS1-v1_5,    Step 3 ( Encoding method for signatures )

	int psLen = outBufSize - dataSize - 3;

	BYTE blockType = 0x02;

	memset( outBuf, 0, outBufSize - dataSize );

	outBuf[0] = 0;
	outBuf[1] = blockType;
	RSA_makePKEBPaddingString( outBuf + 2, psLen, true );
	outBuf[ psLen + 3 ] = algID;
	memcpy( outBuf + psLen + 4, sessKey, sessKeySize );
	outBuf[ psLen + 4 + sessKeySize ] = BYTE( checkSumm >> 8 );
	outBuf[ psLen + 4 + sessKeySize + 1 ] = BYTE( checkSumm );

	return ppe_ok;
}

int RSA_SessionKeyBlock_Format( 
		BYTE *outBuf,
		int outBufSize,
		BYTE algID,
		const BYTE *sessKey,
		int sessKeySize
	)
{
	WORD checkSumm = getCheckSumm( sessKey, sessKeySize );
	int dataSize = sizeof( algID ) + sizeof( checkSumm) + sessKeySize;

	//
	//    EB = 00 || BT || PS || 00 || D .  Where D = Session key alg ID || Check summ || Session key     
	//

	if ( dataSize > outBufSize - 10 )
		return ppe_messageTooLong; // PKCS1 v2 p 9.1.2.1, Encoding operation, Step 1 ( Encoding method for encryption )
								  //  PKCS1 v2 p 9.2.1,   EMSA-PKCS1-v1_5,    Step 3 ( Encoding method for signatures )

	int psLen = outBufSize - dataSize - 3;

	BYTE blockType = 0x02;

	memset( outBuf, 0, outBufSize - dataSize );

	outBuf[0] = 0;
	outBuf[1] = blockType;
	RSA_makePKEBPaddingString( outBuf + 2, psLen, true );
	outBuf[ psLen + 3 ] = algID;
	outBuf[ psLen + 4 ] = BYTE( checkSumm >> 8 );
	outBuf[ psLen + 5 ] = BYTE( checkSumm );
	memcpy( outBuf + psLen + 6, sessKey, sessKeySize );

	return ppe_ok;
}

int RSA_SessionKeyBlock_Parse( 
		BYTE **pSessKey, // Position of session key in inBuf
		int *pSesKeySize,
		BYTE *pSessKeyAlg,
		BYTE *inBuf,
		int inBufSize
	)
{
	//
	//    EB = 00 || BT || PS || 00 || D .  
	//

	int dataSize = 0;
	int res = PKT_PublicKeyEncryptionBlock_Parse( NULL, &dataSize, inBuf, inBufSize );

	if ( res != ppe_bufferTooSmall )
		return res;

	if ( dataSize < 3 )
		return ppe_invalidBlockType;

	if ( pSessKeyAlg != NULL )
		*pSessKeyAlg = inBuf[ inBufSize - dataSize ];

	if ( pSesKeySize != NULL )
	{
		*pSesKeySize = dataSize - 3;
	}

	//
	//	Old version generates and uses the check summ
	//	Where D = Session key alg ID || Check summ || Session key
	//	The invalid variant is used by BCArchive for key block parsing.
	//

	WORD checkSumm = getCheckSumm( inBuf + inBufSize - dataSize + 3, dataSize - 3 );

	if ( ( BYTE( checkSumm >> 8 ) == inBuf[ inBufSize - dataSize + 1 ] ) &&
		 ( BYTE( checkSumm ) == inBuf[ inBufSize - dataSize + 2 ] ) )
	{
		if ( pSessKey != NULL )
			*pSessKey = inBuf + inBufSize - dataSize + 3;

		return ppe_ok;
	}

	// New version 9.11.2007 implements 
	// There are two differences
	//	1. Check summ is placed after message.
	//	2. Check summ is simple BYTE summ accumulated at WORD, see getCheckSummByte

	//
	//	Version: GnuPG v1.2.6 (GNU/Linux)
	//
	//	RFC 2440  OpenPGP Message Format            November 1998 
	//	[Page 16]
	//	The value "m" in the above formulas is derived from the session key 
	//	as follows.  First the session key is prefixed with a one-octet 
	//	algorithm identifier that specifies the symmetric encryption 
	//	algorithm used to encrypt the following Symmetrically Encrypted Data 
	//	Packet.  Then a two-octet checksum is appended which is equal to the <<<<< !APPENDED!
	//	sum of the preceding session key octets, not including the algorithm <<<<< !Simple byte check summ!
	//	identifier, modulo 65536.  This value is then padded as described in 
	//	PKCS-1 block type 02 [RFC2313] to form the "m" value used in the 
	//	formulas above.
	//

	//	Where D = Session key alg ID || Session key  || Check summ

	checkSumm = getCheckSummByte( inBuf + inBufSize - dataSize + 1, dataSize - 3 ); // getCheckSumm instead

	if ( ( BYTE( checkSumm >> 8 ) == inBuf[ inBufSize - 2 ] ) &&
		 ( BYTE( checkSumm ) == inBuf[ inBufSize - 1 ] ) )
	{
		if ( pSessKey != NULL )
			*pSessKey = inBuf + inBufSize - dataSize + 1; // +3 instead

		return ppe_ok;
	}

	return ppe_invalidBlockType;
}

int RSA_SessionKeyBlock_Parse_OLD( 
		BYTE **pSessKey, // Position of session key in inBuf
		int *pSesKeySize,
		BYTE *pSessKeyAlg,
		BYTE *inBuf,
		int inBufSize
	)
{
	//
	//    EB = 00 || BT || PS || 00 || D .  Where D = Session key alg ID || Check summ || Session key     
	//

	int dataSize = 0;
	int res = PKT_PublicKeyEncryptionBlock_Parse( NULL, &dataSize, inBuf, inBufSize );

	if ( res != ppe_bufferTooSmall )
		return res;

	if ( dataSize < 3 )
		return ppe_invalidBlockType;

	if ( pSessKeyAlg != NULL )
		*pSessKeyAlg = inBuf[ inBufSize - dataSize ];

	WORD checkSumm = getCheckSumm( inBuf + inBufSize - dataSize + 3, dataSize - 3 );
	if ( BYTE( checkSumm >> 8 ) != inBuf[ inBufSize - dataSize + 1 ] )
		return ppe_invalidBlockType;
	if ( BYTE( checkSumm ) != inBuf[ inBufSize - dataSize + 2 ] )
		return ppe_invalidBlockType;

	if ( pSesKeySize != NULL )
	{
//
// It does not copy to buffer it assigns positions only
//		so it is commented.
//
//		if ( *pSesKeySize < dataSize - 3 )
//		{
//			*pSesKeySize = dataSize - 3;
//			return ppe_bufferTooSmall;
//		}

		*pSesKeySize = dataSize - 3;
	}

	if ( pSessKey != NULL )
		*pSessKey = inBuf + inBufSize - dataSize + 3;

	return ppe_ok;
}

//
//	Public Key Packet
//
//	5.5.1.1. Public Key Packet (Tag 6)
//
//	Checks:
//	 version,
//	 alg validity,
//	 mpi buffer size.
//
int PKT_PublicKey_Parse( sPublicKey *pkt, int pktSize, int *pParsed, BYTE *pAlgID )
{
	if ( pkt->version < 4 )
	{
		sPublicKeyV3 *pktV3 = ( sPublicKeyV3 * )pkt;

		if ( pAlgID )
			*pAlgID = pktV3->algID;

		return PKT_PublicKeyV3_Parse( pktV3, pktSize, pParsed );
	}

	if ( pAlgID )
		*pAlgID = pkt->algID;

	if ( pktSize < (int)sizeof( sPublicKey ) )
		return ppe_invalidPacketSize;

	int mpiCnt = PKT_GetPublicKeyMPICount( pkt->algID );

	if ( mpiCnt == 0 )
		return ppe_invalidAlgorithmID;

	int mpiSize = PKT_CheckMPIBuffer( pkt->mpi, mpiCnt, pktSize -  ( sizeof( *pkt) - sizeof( pkt->mpi) ) );

	if ( mpiSize == 0 )
		return ppe_invalidMPIBuffer;

	if ( pParsed )
		*pParsed = sizeof( *pkt ) - sizeof( pkt->mpi ) + mpiSize;

	return ppe_ok;
}

int PKT_PublicKeyV3_Parse( sPublicKeyV3 *pkt, int pktSize, int *pParsed )
{
	if ( ( pkt->version < 2 ) && ( pkt->version > 3 ) )
		return ppe_invalidPacketVersion;

	if ( pktSize < (int)sizeof( sPublicKeyV3 ) )
		return ppe_invalidPacketSize;

	switch ( pkt->algID )
	{
		case 1:
		case 2:
		case 3: break;
		default : return ppe_invalidAlgorithmID;
	}

	int mpiCnt = PKT_GetPublicKeyMPICount( pkt->algID );

	if ( mpiCnt == 0 )
		return ppe_invalidAlgorithmID;

	int mpiSize = PKT_CheckMPIBuffer( pkt->mpi, mpiCnt, pktSize -  ( sizeof( *pkt) - sizeof( pkt->mpi) ) );

	if ( mpiSize == 0 )
		return ppe_invalidMPIBuffer;

	if ( pParsed )
		*pParsed = sizeof( *pkt ) - sizeof( pkt->mpi ) + mpiSize;

	return ppe_ok;
}


//
//	Function:
//		PKT_SecretKey_Parse
//	See RFC 2440
//		5.5.3. Secret Key Packet Formats
//
//	1. Checks public part:
//		version,
//		alg validity,
//		mpi buffer size.
//
//	2. Fills secret key specific fields.
//		s2kMode,
//		symmetric algorithm ID
//		Initial Vector.
//		secret mpi buffer,
//		check summ
//
int PKT_SecretKey_Parse(
	sSecretKey *pkt,	// pointer to secret key packet
	int pktSize,		// total pkt size

	int *pPublicMpiSize,

	BYTE *pSymAlgID,	// pointer to identifier of symmetric the algorithm used for decryption of private part
	S2K  *pS2K,			// pointer to mode for string to key conversion for symmetrin algorithm
	BYTE **pIV,			// pointer to initial vector

	BYTE **pEncMpi,		// pointer to encrypted buffer with secret mpis
	int *pEncMpiSize	// pointer to size of buffer to be decrypted
//	WORD *pCheckSumm	// pointer to two byte of check summ
	)
{
	//
	//	1. Check public part...
	//
	int publicValueSize = 0;

	int res = PKT_PublicKey_Parse( pkt, pktSize, &publicValueSize );

	if ( res != ppe_ok )
		return res;

	if ( publicValueSize < (int)sizeof( sPublicKey ) )
		return ppe_invalidMPIBuffer;

	if ( pPublicMpiSize )
	{
		if ( pkt->version < 4 )
			*pPublicMpiSize = publicValueSize - ( sizeof( sPublicKeyV3 ) - 2 );
		else
			*pPublicMpiSize = publicValueSize - ( sizeof( sPublicKey ) - 2 );
	}

	//
	//	2. Parse secret part
	//

	BYTE *sec = (BYTE *)pkt + publicValueSize;

	*pSymAlgID = *(sec++);

	if ( *pSymAlgID == 0 )
	{
		pS2K->mode = S2K::initialisationRequired;
		*pIV = NULL;
	}
	else
	if ( ( *pSymAlgID == 255 ) || ( *pSymAlgID == 254 ) ) // ??? several keys has the 254 s2k identifier instead 255 as documented.
	{
		*pSymAlgID = *(sec++);

		int res = pS2K->Parse( sec );
		if ( res == 0 )
			return ppe_invalidS2KBlockType;
		sec += res;

		int blockSize = sa_getBlockSize( *pSymAlgID );
		*pIV = sec;
		sec += blockSize; // 8;
	}
	else // *pSymAlgID != 0
	{
		pS2K->mode = S2K::initialisationRequired;

		int blockSize = sa_getBlockSize( *pSymAlgID );
		*pIV = sec;
		sec += blockSize; // 8;
	}

	pS2K->encAlgID = *pSymAlgID;

	*pEncMpi = sec;

	int herePos = (int)( sec - (BYTE *)pkt );

	*pEncMpiSize = pktSize - 2 - herePos; // - 2 byte check sum

	if ( *pEncMpiSize <= 2 )
		return ppe_invalidMPIBuffer;

//	*pCheckSumm = ( WORD(((BYTE *)pkt)[ pktSize - 2 ]) << 8 ) | ((BYTE *)pkt)[ pktSize - 1 ];

	return ppe_ok;
}
/*
int
pgpBnGet(BigNum *bn, PGPByte const *buf, unsigned size,
	PGPCFBContext *cfb, unsigned *checksump, int old)
{
	PGPByte tmp[64];	// This can be any (non-zero) size >= 2
	unsigned t, l;

	if (!cfb) {
		int i;

		i = pgpBnGetPlain(bn, buf, size);
		if (i >= 0 && checksump) {
			for (t = 0; t < (unsigned)i; t++)
				*checksump += buf[t];
		}
		return i;
	}

	if (size < 2)
		return 0;
	if (old) {
		// Length it bits is not encrypted
		PGPCFBSync(cfb);
		if (checksump)
			*checksump += (unsigned)buf[0] + buf[1];
		t = ((unsigned)buf[0] << 8) + buf[1];
	} else {
		// Length in bits is encrypted 
		pgpCFBDecryptInternal(cfb, buf, 2, tmp );
		if (checksump)
			*checksump += (unsigned)tmp[0] + tmp[1];
		t = ((unsigned)tmp[0] << 8) + tmp[1];
	}
	buf += 2;
	t = (t+7)/8;
	if (size < t+2)
		return 0;

	//
	// Descrypt and convert in pieces.  It's done from the
	// most-significnt end to force allocation of the result
	// number all at once rather than reallocating it bit by bit.
	// 
	size = t;
	while (size) {
		l = size < sizeof(tmp) ? size : sizeof(tmp);

		pgpCFBDecryptInternal(cfb, buf, l, tmp);
		size -= l;
		buf += l;
		if (bnInsertBigBytes(bn, tmp, size, l) < 0) {
			pgpClearMemory( tmp,  sizeof(tmp));
			return kPGPError_OutOfMemory;
		}

		// Checksum
		if (checksump) {
			do {
				*checksump += tmp[--l];
			} while (l);
		}
	}
	pgpClearMemory( tmp,  sizeof(tmp));

	return (int)t+2;
}
*/

//
//	Function:
//		mpiSizeInByte
//	Gets real size of one mpi and checks with total buffer size.
//	Return 0 if mpi size is to large for specified buffer, otherwise return size in bytes.
//	Note, it does not check the mpi bit size ( so it's body can be encrypted ).
//
static UINT mpiSizeInByte( const BYTE *mpi, int bfSize )
{
	if ( bfSize < 2 )
		return 0;

	UINT size = ((UINT)(mpi[0]) << 8) | mpi[1];
	size = ( size + 7 ) / 8 + 2;

	if ( (int)size > bfSize )
		return 0;

	return size;
}

//
//	Function:
//		decryptOneMpi
//	Decrypts and parses one ecrypted mpi, fills check summ.
//	Return real size of the parsed one mpi buffer.
//
static int decryptOneMpi(
	HSA   alg,			// symmetric algorithm for decryption
	BYTE  *mpi,			// decrypted output mpi buffer
	const BYTE *encMpi,	// encrypted input buffer
	int	  encMpiSize,	// size of encrypted buffer
	WORD  *pCheckSum,	// parsed check summ.
	bool  old = false
	)
{
	int size = 0;

	if ( alg == NULL ) // Was not encrypted
	{
		size = mpiSizeInByte( encMpi, encMpiSize );
		if ( size == 0 )
			return 0;

		memcpy( mpi, encMpi, size );
	}
	else
	{
		if ( old )
		{
			size = mpiSizeInByte( encMpi, encMpiSize );
			if ( size == 0 )
				return 0;

			sa_syncCFB( alg );

			memcpy( mpi, encMpi, 2 );
		}
		else
		{
			sa_decryptCFB( alg, mpi, encMpi, 2 );

			size = mpiSizeInByte( mpi, encMpiSize );
			if ( size == 0 )
				return 0;
		}

		int res = sa_decryptCFB( alg, mpi + 2, encMpi + 2, size - 2 );
		if ( res != sae_ok )
			return 0;

		if ( PKT_CheckMPIBuffer( mpi, 1, size ) == 0 )
			return 0;
	}

	if ( pCheckSum )
	{
		for( int i = 0; i < size; i++ )
		{
			*pCheckSum = *pCheckSum + mpi[ i ];
		}
	}

	return size;
}

static int makeSHA1hash( BYTE *keyBuff, const BYTE *str, int strSize )
{
	HHA hHa;
	int res = ha_createAlgorithm( &hHa, pgphaSHA1 );

	if ( res != hae_ok )
		return res;

	if ( hHa == NULL )
		return ppe_memoryAllocation;

//	int digestSize = ha_getDigestSize( hHa );

	res = ha_makeDigest( hHa, keyBuff, (BYTE *)str, strSize );

	ha_free( hHa );

	return res;
}


//
//	Function:
//		decryptSecretKeyBuff
//			Parses and decrypts required quantity of encrypted mpis
//			Checks the ckeck summ.
//
static int decryptSecretKeyBuff(
	HSA hSA,
	BYTE *out,
	BYTE *in,
	int secPktSize, // size of mpis and check summ...
	int mpiCnt,
	bool old
	)
{
	int parsed = 0;

	WORD checkSum = 0;

	while( mpiCnt-- ) 
	{
		int size = decryptOneMpi( hSA, out + parsed, in + parsed, secPktSize - parsed, &checkSum, old );
		if ( size == 0 )
			return ppe_invalidPassword;
//			return ppe_invalidMPIBuffer;

		parsed += size;
	}

	//
	//	Check the check sum
	//
	if ( parsed + 2 > secPktSize )
		return ppe_invalidMPIBuffer;

	BYTE chkSumBytes[20];

	chkSumBytes[0] = BYTE(checkSum >> 8);
	chkSumBytes[1] = BYTE(checkSum);

	BYTE chkSumBytesInPkt[20];

	if ( old || ( hSA == NULL ) )
	{
		memcpy( chkSumBytesInPkt, in + parsed, 2 );
	}
	else
	{
		sa_decryptCFB( hSA, chkSumBytesInPkt, in + parsed, 2 );
	}

	if ( memcmp( chkSumBytes, chkSumBytesInPkt, 2 ) != 0 )
	{
		// tries to check SHA1 digest
		if ( parsed + 20 <= secPktSize )
		{
			sa_decryptCFB( hSA, chkSumBytesInPkt + 2, in + parsed + 2, 20 - 2 );
			makeSHA1hash( chkSumBytes, out, parsed );

			if ( memcmp( chkSumBytes, chkSumBytesInPkt, 20 ) != 0 )
				return ppe_invalidPassword;
		}
		else
			return ppe_invalidPassword;
	}

	return ppe_ok;
}

//
//	Function:
//		PKT_CreateSecretKeyAlgFromPacket
//	1. Parses and ckecks secret key packet.
//	2. Extracts public data
//	3. Decrypt secret data with specified password.
//	4. Allocates secret key algorithm in pHSA value, 
//		the hadler can be used for decryption ( ska_decrypt ) and signing ( ska_sing ),
//		it has to be freed by ( ska_free )
//
int PKT_CreateSecretKeyAlgFromPacket( HPKA *pHSKA, sSecretKey *skPkt, int skPktSize, const char *pwd )
{
	//
	//	Parse secret key packet.
	//

	int secMpiSize = 0;

	S2K  s2k;
	BYTE symAlgID = 0;
	BYTE *iv = NULL;
	BYTE *encMpi = NULL;
	int publicMpiSize;

	int res = PKT_SecretKey_Parse( skPkt, skPktSize, &publicMpiSize, &symAlgID, &s2k, &iv, &encMpi, &secMpiSize );

	if ( res != ppe_ok )
		return res;

	bool old = skPkt->version < 4;
	int skAlgID = old ? ((sPublicKeyV3 *)skPkt)->algID : skPkt->algID;
	BYTE *publicMpi = old ? ((sPublicKeyV3 *)skPkt)->mpi : skPkt->mpi;

	int secMpiCnt = PKT_GetSecretKeyEncryptedPartMPICount( (BYTE)skAlgID );
	if ( secMpiCnt == 0 )
		return ppe_invalidAlgorithmID;

	//
	//	Prepare key for symmetric algorithm to decrypt secret key.
	//	If symAlgID == 0 the data is not encrypted.
	//
	HSA hSa = NULL;

	if ( symAlgID != 0 )
	{
		int keySize = sa_getKeySize( symAlgID );

		if ( keySize == 0 )
			return ppe_invalidAlgorithmID;

		cPlainTextBlock ptbKey;
		if ( ptbKey.Allocate( keySize ) == NULL )
			return ppe_memoryAllocation;

		if ( pwd == NULL )
			return ppe_invalidPassword;

		res = string2key( ptbKey.buffer, &ptbKey.bufferSize, pwd, strlen( pwd ), s2k.mode == S2K::initialisationRequired ? NULL : &s2k );

		if ( res != ppe_ok )
			return res;

		res = sa_createAlgorithm( &hSa, symAlgID, ptbKey.buffer, ptbKey.bufferSize );

		if ( res != ppe_ok )
			return res;

		res = sa_initCFB( hSa, iv );
		if ( res != ppe_ok )
		{
			sa_free( hSa );
			return res;
		}
	}

	//
	//	Decrypt secret key part...
	//

	cPlainTextBlock secMpi;
	if ( secMpi.Allocate( secMpiSize ) == NULL )
	{
		if ( hSa )
			sa_free( hSa );

		return ppe_memoryAllocation;
	}

	res = decryptSecretKeyBuff( hSa, secMpi.buffer, encMpi, secMpiSize + sizeof( WORD ), secMpiCnt, old );

	if ( hSa )
		sa_free( hSa );

	if ( res != ppe_ok )
		return res;


	//
	//	Create secret key algorithm
	//

	res = ska_createAlgorithm( pHSKA, skAlgID, publicMpi, publicMpiSize, secMpi.buffer, secMpi.bufferSize );

	if ( res == pkae_invalidAlgorithmID )
		return ppe_invalidAlgorithmID;

	if ( *pHSKA == NULL )
		return ppe_memoryAllocation;

	(*pHSKA)->time = old ? ((sPublicKeyV3 *)skPkt)->time.getTime() : skPkt->time.getTime();
	(*pHSKA)->days = old ? ((sPublicKeyV3 *)skPkt)->days.gatDays() : 0;

	return ppe_ok;
}

//
//	Function:
//		PKT_CreatePublicKeyAlgFromPacket
//	1. Parses and ckecks public key packet.
//	2. Allocates public key algorithm in pHSA value, 
//		the hadler can be used for encryption ( pka_encrypt ) and verifying ( pka_verify ),
//		it has to be freed by ( pka_free )
//
int PKT_CreatePublicKeyAlgFromPacket( HPKA *pHPKA, sPublicKey *pkPkt, int pkPktSize )
{
	//
	//	Parse public key packet.
	//

	int publicMpiSize;

	int res = PKT_PublicKey_Parse( pkPkt, pkPktSize, &publicMpiSize );

	if ( res != ppe_ok )
		return res;

	bool old = pkPkt->version < 4;
	int pkAlgID = old ? ((sPublicKeyV3 *)pkPkt)->algID : pkPkt->algID;
	BYTE *publicMpi = old ? ((sPublicKeyV3 *)pkPkt)->mpi : pkPkt->mpi;


	//
	//	Create public key algorithm
	//

	*pHPKA = NULL;

	res = pka_createAlgorithm( pHPKA, pkAlgID, publicMpi, publicMpiSize );

	if ( res == pkae_invalidAlgorithmID )
		return ppe_invalidAlgorithmID;

	if ( *pHPKA == NULL )
		return ppe_memoryAllocation;

	(*pHPKA)->time = old ? ((sPublicKeyV3 *)pkPkt)->time.getTime() : pkPkt->time.getTime();
	(*pHPKA)->days = old ? ((sPublicKeyV3 *)pkPkt)->days.gatDays() : 0;

	return ppe_ok;
}
