#include "../publicKeyAlgs/Power/Windefs.h"
#include "string2key.h"
#include "../PGPPkts/pktdefs.h" // For error definition
#include "../hashAlgs/hashAlgs.h"
#include "../symmetricAlgs/symmetricAlgs.h"
#include "../SMemory/SBlocks.h"
#include <assert.h>
/*
static int makeMD5hash( BYTE *keyBuff, int *pKeyBuffSize, const BYTE *str, int strSize )
{
	HHA hHa;
	int res = ha_createAlgorithm( &hHa, 1 );

	if ( res != hae_ok )
		return res;

	if ( hHa == NULL )
		return ppe_memoryAllocation;

	int digestSize = ha_getDigestSize( hHa );

	if ( *pKeyBuffSize < digestSize )
	{
		ha_free( hHa );
		*pKeyBuffSize = digestSize;
		return ppe_bufferTooSmall;
	}

	res = ha_makeDigest( hHa, keyBuff, (BYTE *)str, strSize );

	ha_free( hHa );

	return res;

}
*/
int string2key( BYTE *keyBuff, int *pKeyBuffSize, const char *str, int strSize, S2K *s2k )
{
	S2K emptyS2k;

	if ( s2k == NULL )
	{
//		return makeMD5hash( keyBuff, pKeyBuffSize, (const BYTE *)str, strSize );

		s2k = &emptyS2k;
		memset( s2k, 0, sizeof( *s2k ));
		s2k->hashAlgID = pgphaMD5;
	}

	int res = ppe_ok;

	if ( s2k->mode == s2k->initialisationRequired )
	{
		s2k->SetDefault();
	}

	//
	//	Check key buffer size...
	//
	int algKeySize;

	if ( s2k->encAlgID == 0 ) // If the algorithm is not specified, generates key for all available buffer.
	{
		algKeySize = *pKeyBuffSize;

		if ( algKeySize == 0 )
			return ppe_internalError;
	}
	else
	{
		algKeySize = sa_getKeySize( s2k->encAlgID );

		if ( algKeySize == 0 )
			return ppe_invalidAlgorithmID;

		if ( algKeySize > *pKeyBuffSize )
		{
			*pKeyBuffSize = algKeySize;
			return ppe_bufferTooSmall;
		}

		*pKeyBuffSize = algKeySize;
	}


	//
	//	Initiate hash algorithm;
	//

	HHA hHa;
	res = ha_createAlgorithm( &hHa, s2k->hashAlgID );

	if ( res == hae_invalidAlgorithmID )
		return ppe_invalidAlgorithmID;

	if ( hHa == NULL )
		return ppe_memoryAllocation;

	int digestSize;

	digestSize = ha_getDigestSize( hHa );

	cPlainTextBlock ptDigest;

	if ( ptDigest.Allocate( digestSize ) == NULL )
	{
		ha_free( hHa );
		return ppe_memoryAllocation;
	}

	int length = strSize;

	if ( s2k->mode & S2K::salted )
		length += sizeof( s2k->salt );

	int count = 0;

	if ( s2k->mode & S2K::iterated )
		count = ( 0x10 + ( s2k->count & 0xF ) ) << ( ( s2k->count >> 4 ) + 6 );

	if ( count < length )
		count = length;

	int pos = 0;

	for ( int pass = 0; pos < algKeySize; pass++ )
	{
		ha_init( hHa );

		//
		//	Adds zero bytes
		//
		BYTE zero = 0;

		for ( int i = pass; i--; )
			ha_process( hHa, &zero, 1 );

		int rest = count;

		for ( ; rest > length; rest -= length )
		{
			if ( s2k->mode & S2K::salted )
				ha_process( hHa, s2k->salt, sizeof( s2k->salt) );

			ha_process( hHa, (BYTE*)str, strSize );
		}

		if ( s2k->mode & S2K::salted )
		{
			if ( rest > 8 )
			{
				ha_process( hHa, s2k->salt, sizeof( s2k->salt ) );
				rest -= sizeof( s2k->salt );
			}
			else
			{
				ha_process( hHa, s2k->salt, rest);
				rest = 0;
			}
		}

		assert( rest <= strSize );

		ha_process( hHa, (BYTE *)str, rest );

		res = ha_final( hHa, ptDigest.buffer );

		if ( res != hae_ok )
			break;

		rest = algKeySize - pass * digestSize;

		if ( rest <= digestSize )
		{
			memcpy( keyBuff + pos, ptDigest.buffer, rest); // key->WriteBytes( ptDigest.buffer, rest );
			pos += rest;
			break;
		}
		else
		{	memcpy( keyBuff + pos, ptDigest.buffer, digestSize ); //key->WriteBytes( ptDigest.buffer, digestSize );
			pos += digestSize;
		}
	}

	ha_free( hHa );

	return res;
}

//
//	Parse S2K buffer, returns number of parsed bytes.
//
int S2K::Parse( unsigned char *ptr )
{
	int size = 0;

	mode = ptr[ size++ ];
	hashAlgID = ptr[ size++ ];

	if ( mode == simple )
		return size;
	else
	if ( ( mode == salted ) || ( mode == mixed ) )
	{
		memcpy( salt, ptr + size, sizeof( salt ) );
		size += sizeof( salt );

		if ( mode == mixed )
			count = ptr[ size ++ ];

		return size;
	}

	//
	//	Invalid mode..
	//
	return 0;
}

//
//	Writes s2k to packet 
//
int S2K::WriteToPacket(
		BYTE *pkt,		// Buffer for writing
		int *pPktSize	// [IN]  Size of the buffer
						// [OUT] Number of bytes required/written,
						//	function returns ppe_bufferTooSmall
		)
{
	int sizeRequired =
		   1 // Mode
		 + 1 // Hash Alg ID
		 + (( mode & salted   ) ? sizeof( salt ) : 0 )    // salt
		 + (( mode & iterated ) ? 1 : 0 )				// count
		 ;

	if ( *pPktSize < sizeRequired )
	{
		*pPktSize = sizeRequired;
		return ppe_bufferTooSmall;
	}

	*pPktSize = 0;

	pkt[ (*pPktSize) ++ ] = mode;
	pkt[ (*pPktSize) ++ ] = hashAlgID;

	if ( mode & salted )
	{
		memcpy( pkt + *pPktSize, salt, sizeof( salt ) );
		*pPktSize += sizeof( salt );
	}

	if ( mode & iterated )
		pkt[ (*pPktSize) ++ ] = count;

	assert( *pPktSize == sizeRequired );

	return ppe_ok;
}
