#pragma once

#include "../SMemory/SBlocks.h"

//
//	class cPGPPacket
//		Accumulates data at temporary body.
//		Writes data to output packet partially at Final.
//
class cPGPPacket
{
	cPlainTextBlock m_body;
	BYTE m_tag;

public:
	cPGPPacket( BYTE tag = 0, BYTE *body = NULL, int len = 0 )
	{
		Init( tag );
		Add( body, len );
	}

	bool Init( BYTE tag )
	{
		m_tag = tag;
		m_body.Free();
		return true;
	}

	template < class T > bool Add( T t )
	{
		return Add( (BYTE*)&t, sizeof(T) );
	}

	bool Add( BYTE *body, int len )
	{
		return m_body.WriteBytes( body, len ) != 0;
	}

	BYTE *GetSpace( int len )
	{
		int pos = m_body.bufferSize;

		if ( m_body.WriteBytes( NULL, len ) == 0 )
			return NULL;

		assert( m_body.bufferSize == pos + len );

		return m_body.buffer + pos;
	}

	enum { maxStep = 0x200 }; // RFC 2440. The first partial length MUST be at least 512 octets long

	bool Final( cPlainTextBlock *ptbOut )
	{
		int res;

		BYTE tag = m_tag;

		int step = maxStep;
		int pos = 0;

		do
		{
			bool partialSize = true;

			if ( pos + step > m_body.bufferSize )
			{
				step = m_body.bufferSize - pos;
				partialSize = false;
			}

			int pktSize = 0;

			res = pgp_PacketHeader_Format( NULL, &pktSize, tag, m_body.buffer + pos, step, partialSize );
			if ( res != ppe_bufferTooSmall )
				return false;
			
			int ptbOutPos = ptbOut->bufferSize;

			if ( ptbOut->WriteBytes( NULL, pktSize ) == 0 )
				return false;

			res = pgp_PacketHeader_Format( ptbOut->buffer + ptbOutPos, &pktSize, tag, m_body.buffer + pos, step, partialSize );
			if ( res != ppe_ok )
				return false;

			tag = 0; // Next packet is partial type ( without 'Packet Tag' byte )
			pos += step;
		}
		while( pos < m_body.bufferSize );

		return true;
	}
};
