//
//	Copyright (c) 2000-2001 Jetico Inc., Finland
//	All rights reserved.
//
//	File:			PGPdefs.h
//
#ifndef __pktdefs__
#define __pktdefs__

#include "../Includes/windefs.h"
#include "../SMemory/smemory.h"

#include "../crtLibErrs.h"

const int pgp_keyIDSize = 8;

//
//	Packet tag enums...
//
enum { 
	pkt_PublicKeyEncryptedSessionKey = 1,
	pkt_Signature = 2,
	pkt_SymmetricKeyEncryptedSessionKey = 3,
	pkt_OnePassSignature = 4,
	pkt_SecretKey = 5,
	pkt_PublicKey = 6,
	pkt_SecretSubkey = 7,
	pkt_CompressedData = 8,
	pkt_SymmetricallyEncryptedData = 9,
	pkt_Marker = 10,
	pkt_LiteralData = 11,
	pkt_Trust = 12,
	pkt_UserID = 13,
	pkt_PublicSubkey = 14,
	pkt_Image = 17,	//  -- User Attribute Packet
	pkt_SymEncryptedAndIntegrityProtectedData =    18, //  -- Sym. Encrypted and Integrity Protected Data Packet
	pkt_ModificationDetectionCodePacket =	19, //  -- Modification Detection Code Packet
	pkt_0x3d = 0x3d
};

//
// PGP Public key algorithm ID enums
//
enum { 
	pgppkaRSA		= 1,	// RSA (Encrypt or Sign)
	pgppkaRSA_e		= 2,	// RSA Encrypt-Only
	pgppkaRSA_s		= 3,	// RSA Sign-Only
	pgppkaElgamal_e	= 16,	// Elgamal (Encrypt-Only)
	pgppkaDSA		= 17,	// DSA (Digital Signature Standard)
	pgppkaElgamal	= 20	// Elgamal (Encrypt or Sign)
};

#define isSignatreAlgorithmOnly( algID ) (( algID == pgppkaDSA ) || ( algID == pgppkaRSA_s ))
#define isPKEncryptionAlgorithm( algID ) (( algID == pgppkaRSA ) || ( algID == pgppkaRSA_e ) || ( algID == pgppkaElgamal ) || ( algID == pgppkaElgamal_e ) )

//
//	PGP Symmetric key algorithm ID enums
//

//
// RFC2440 says 100 to 110 - Private/Experimental algorithm.
//	but PGP source code defined values	
#define	kPGPCipherAlgorithm_RC2_40              100
#define	kPGPCipherAlgorithm_RC2_128             101
//
enum {
	pgpsaPlaintext		= 0, // Plaintext or unencrypted data
	pgpsaIDEA			= 1, // IDEA
	pgpsaTripleDES		= 2, // Triple-DES (DES-EDE, as per spec -
							 // 168 bit key derived from 192)
	pgpsaCAST5			= 3, // CAST5 (128 bit key, as per RFC 2144)
	pgpsaBlowfish		= 4, // Blowfish (128 bit key, 16 rounds)
	pgpsaSAFER_SK128	= 5, // SAFER-SK128 (13 rounds)

	pgpsa2KeyTripleDES	= 6, // 2-Key Triple-DES ( 112 bit key derived from 128)

	// GNUPG AES ( RIJNDAEL ) definishions
	pgpsaAES_128		= 7,
	pgpsaAES_192		= 8,
	pgpsaAES_256		= 9,
	pgpsaTFISH			= 10,

	// They are not implemented but may by parsed at oidString.cpp see pkcs12_pbe_T table
	pgpsa128BitRC4		= -1,
	pgpsa40BitRC4		= -2,
	pgpsa128BitRC2		= kPGPCipherAlgorithm_RC2_128,
	pgpsa40BitRC2		= kPGPCipherAlgorithm_RC2_40,
};


//
// BC additional symmetric encryption algorithms
//
enum
{
	bcsaMinAlgID = 0x80,
	bcsaBFish	 = bcsaMinAlgID + 0x0, // 128,
	bcsaBFish448 = bcsaMinAlgID + 0x1, // 176,
	bcsaCAST	 = bcsaMinAlgID + 0x2, // 224,
	bcsaDES		 = bcsaMinAlgID + 0x3, // 64,
	bcsaDummy	 = bcsaMinAlgID + 0x4, // 254,
	bcsaGOST	 = bcsaMinAlgID + 0x5, // 170,
	bcsaIDEA	 = bcsaMinAlgID + 0x6, // 32,
	bcsaRC6		 = bcsaMinAlgID + 0x7, // 230,
	bcsaRIJN	 = bcsaMinAlgID + 0x8, // 240,
	bcsaSERP	 = bcsaMinAlgID + 0x9, // 232,
	bcsaTFISH	 = bcsaMinAlgID + 0xA, // 160
};

//
//	Symmetric encryption modes
//
enum { 
	pgpsamodeCBC = 0,
	pgpsamodeCFB = 1,
	pgpsamodeECB = 2,
	pgpsamodeOFB = 3, // ??? was not find in ducumments.
};

//
//	PGP Hash algorithm ID enums
//
enum {
	pgphaMD5		= 1, // MD5
	pgphaSHA1		= 2, // SHA-1
	pgphaRIPEMD160	= 3, // RIPE-MD/160 // 1.0.10118.3.0.49
	pgphaMD2		= 5, // MD2

	pgphaMD4		= 7, // ?
	pgphaSHA256		= 8,	// 2.16.840.1.101.3.4.2.1
	pgphaSHA384		= 9,	// 2.16.840.1.101.3.4.2.2
	pgphaSHA512		= 10,	// 2.16.840.1.101.3.4.2.3
	pgphaSHA224		= 11,	// 2.16.840.1.101.3.4.2.4

};

//
// BC additional hash algorithms
//
enum 
{
	bchaMinAlgID		= 0x80,
	bchaSHA256			= bchaMinAlgID + 0x0,
	bchaWhirlpool512	= bchaMinAlgID + 0x1,	// 1.0.10118.3.0.55
	bchaSHA3_512		= bchaMinAlgID + 0x2,	// Keccak1600 ( internal state is array 5x5 of u64 = 1600 )
	bchaSkein512		= bchaMinAlgID + 0x3,
};

struct sHashedItem
{
	BYTE *m_pkt;
	int m_pktSize;
	sHashedItem *m_next;
	bool m_allocated;

	sHashedItem( BYTE *pkt, int pktSize, bool allocateAndCopy )
	{
		m_allocated = allocateAndCopy;

		if ( m_allocated )
		{
			m_pkt = new BYTE[ pktSize ];

			if ( m_pkt == NULL )
			{
				m_pktSize = 0;
				return;
			}

			SMemCpy( m_pkt, pkt, pktSize );
		}
		else
			m_pkt		= pkt;

		m_pktSize	= pktSize;
		m_next		= NULL;
	}

	~sHashedItem()
	{
		if ( m_allocated )
		{
			SMemSet( m_pkt, 0, m_pktSize );
			delete m_pkt;
		}

		SMemSet( this, 0, sizeof( *this ) );
	}
};

class cHashedList
{
	sHashedItem  *head, *last, *cur;
public:
	cHashedList()
	{
		SMemSet( this, 0, sizeof( *this ) );
	}

	~cHashedList()
	{
		while( head )
		{
			last = head;
			head = head->m_next;
			delete last;
		}
	}

	bool Add( BYTE *pkt, int pktSize, bool allocateAndCopy = false)
	{
		sHashedItem *tmp;
		tmp = new sHashedItem( pkt, pktSize, allocateAndCopy );
		if ( tmp == NULL )
			return false;

		if ( head == NULL )
			head = last = tmp;
		else
		{
			last->m_next = tmp;
			last = tmp;
		}

		return true;
	}

	sHashedItem *GetFirst()
	{
		cur = head;
		return cur;
	}

	sHashedItem *GetNext()
	{
		if ( cur == NULL )
			return NULL;

		cur = cur->m_next;
		return cur;
	}
};

#endif // __pktdefs__
