#ifndef __string2key__
#define __string2key__

#include <string.h>

class S2K
{
  public:

	enum {
		simple = 0,
		salted = 1,
		iterated = 2,
		mixed = salted | iterated,
		initialisationRequired = 4
	};

	BYTE mode;
	BYTE hashAlgID;
	BYTE encAlgID;
	BYTE salt[8];
	BYTE count;

	static int GetIterationCountInt( BYTE count )
	{
		return ( 0x10 + ( count & 0xF ) ) << ( ( count >> 4 ) + 6 );
	}

	static BYTE GetIterationCountByte( int intCount )
	{
		BYTE count = 0;

		if ( ( intCount == -1 ) || ( intCount >= 0x10000 ) )
			count = (BYTE)0x60;
		else
		{
			//
			//	if ( s2k->mode & S2K::iterated )
			//	 	intCount = ( 0x10 + ( s2k->count & 0xF ) ) << ( ( s2k->count >> 4 ) + 6 );
			//
			//		intCount = ( 0x10 + cl ) << ( ch + 6 );
			//		count == (ch << 4) | cl;
			//
			BYTE ch = 0,cl = 0;
			for( ch = 0x6;  ; ch-- ) 
			{
				BYTE l = (BYTE)( intCount >> (  ch + 6 ) );

				if ( l >= 0x10 ) 
				{
					cl = l & 0xF;
					break;
				}

				if ( ch == 0 )
					break;
			}

			count = ( ch << 4 ) | cl;
		}

		return count;
	}

	void SetDefault( const BYTE *rndSalt = NULL, int hashAlg = 1, int encAlg = 1, int intCount = -1 )
	{
		mode = iterated;

		if ( rndSalt )
		{
			memcpy( salt, rndSalt, sizeof( salt ) );
			mode |= salted; 
		}

		count = GetIterationCountByte( intCount );

		hashAlgID = (BYTE)hashAlg;
		encAlgID = (BYTE)encAlg;
	}

	S2K()
	{
		memset( this, 0, sizeof( *this ) );

		mode = initialisationRequired;
	}

	//
	//	Parse S2K buffer, returns number of parsed bytes.
	//
	int Parse( BYTE *ptr );

	//
	//	Writes s2k to packet 
	//
	int WriteToPacket(
		BYTE *pkt,		// Buffer for writing
		int *pPktSize	// [IN]  Size of the buffer
						// [OUT] Number of bytes required/written,
						//	function returns ppe_bufferTooSmall
		);
};

//
//	string2key:
//		Converts BYTE string to key. Note, the bytes can be zero.
//		if s2k->encAlgID == 0, generate key for all specified *pKeyBuffSize
//
int string2key( BYTE *keyBuff, int *pKeyBuffSize, const char *str, int strSize, S2K *s2k );

#endif // __string2key__
