#ifndef __cRNG__
#define __cRNG__

//
//	generateRandomKey
//	uses PKSC-12 rules for key generation
//
int generateRandomKey( BYTE *outBuff, int outBuffSize, BYTE *seed, int seedSize );

class cSimpleRng
{
	BYTE *m_pool;
	int m_poolSize;
	int m_pos;

public:

	bool generatePool( int poolSize, BYTE *seed, int seedSize );
	void freePool();

	cSimpleRng( int buffSize = 0, BYTE *seed = NULL, int seedSize = 0 )
	{
		memset( this, 0, sizeof( *this ) );

		if ( buffSize != 0 )
			generatePool( buffSize, seed, seedSize );
	}

	~cSimpleRng()
	{
		freePool();
	}

	int get( BYTE *buff, int buffSize );
	BYTE *get( int buffSize );
};


class cRNG
{
	static cRNG extRNG;
	int m_poolSize;
	BYTE *m_pool;
	BYTE *m_pos;

public:

	static void setPool( BYTE *pool, int poolSize )
	{
		extRNG.m_pos = extRNG.m_pool = pool;
		extRNG.m_poolSize = poolSize;
	}

	cRNG()
	{
		memset( this, 0, sizeof( *this ));
	}

	static void clearPool()
	{
		if ( extRNG.m_pool )
			memset( extRNG.m_pool, 0, extRNG.m_poolSize );

		memset( &extRNG, 0, sizeof( extRNG ));
	}

	//
	//	The functions take data from pool, they do not randomise the pool data.
	//	Returns pkae_randomBufferRequired if buffer is on initialised or it's size is too small.
	//
	static int generate( BYTE *buff, int buffSize );
	static int m_lastRequiredSize;
	static BYTE *get( int buffSize );
};

#endif // __cRNG__
