//#include <windows.h>
#include "../publicKeyAlgs/Power/Windefs.h"
#include "../x509pkts/pkcs12_funcs.h"
#include "cRNG.h"
#include "../crtLibErrs.h"

bool cSimpleRng::generatePool( int poolSize, BYTE *seed, int seedSize )
{
	if ( m_pool )
		freePool();

	m_pool = new BYTE[ poolSize ];
	if ( m_pool == NULL )
		return false;

	m_poolSize = poolSize;
	m_pos = 0;

	if ( P12_producePseudoRandomBytes( m_pool, m_poolSize, NULL, 0, 1, seed, seedSize ) != x509e_ok )
		return false;

	return true;
}

void cSimpleRng::freePool()
{
	if ( m_pool == NULL )
		return;

	memset( m_pool, 0, m_poolSize );
	delete m_pool;

	memset( this, 0, sizeof( *this ) );
}

int cSimpleRng::get( BYTE *buff, int buffSize )
{
	BYTE *rndBuff = get( buffSize );

	if ( rndBuff == NULL )
		return pkae_randomBufferRequired;

	memcpy( buff, rndBuff, buffSize );

	return pkae_ok;
}

BYTE *cSimpleRng::get( int buffSize )
{
	if ( m_pos + buffSize > m_poolSize )
		return NULL;

	m_pos += buffSize;

	return m_pool + m_pos - buffSize;
}

//
//	External random generator. It is used for generation of random data.
//	Pool buffer has not to be modified 
//
cRNG cRNG::extRNG;

// static
int cRNG::m_lastRequiredSize = 0;

int cRNG::generate( BYTE *buff, int buffSize )
{
	BYTE *rndBuff = get( buffSize );

	if ( rndBuff == NULL )
	{
		m_lastRequiredSize = buffSize;
		return pkae_randomBufferRequired;
	}

	memcpy( buff, rndBuff, buffSize );

	return pkae_ok;
}

BYTE *cRNG::get( int buffSize )
{
	if ( extRNG.m_pos + buffSize > extRNG.m_pool + extRNG.m_poolSize )
		return NULL;

	extRNG.m_pos += buffSize;

	return extRNG.m_pos - buffSize;
}

int generateRandomKey( BYTE *buff, int buffSize, BYTE *seed, int seedSize )
{
	BYTE salt[8] = {0,1,2,3,4,5,6,7};
	int saltSize = sizeof( salt );

	return P12_producePseudoRandomBytes( buff, buffSize, salt, saltSize, 1, seed, seedSize, P12_prgm4key );
}
