/*******************************************************************
 *
 *	Copyright (c) 1994-2002 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: sha1random.h
 *	Revision: 
 *	Created:
 *	Description: declaration of random generator based on SHA-1 
 *		Secure Hash Algorithm
 *	The random generator perfectly renerates long size random buffer.
 *	Note! It cannot be used for generation of key materials.
 *
 *******************************************************************/

#ifndef __SHA1RANDOM_H__
#define __SHA1RANDOM_H__

#include <stddef.h>
#include "../hashAlgs/SHA1/Sha1.h"

//-------------------------------------------------------------
// SHA1RandomGenerate high level generation function...
//	return NO_ERROR (0) if successful
//-------------------------------------------------------------

extern int SHA1RandomGenerate( unsigned char *byteBuffer, size_t byteBufferSize, unsigned char *seedBuffer, size_t seedBufferSize );

/**************************************************************************************\
 * Function:  SHA1RandomAllocate
 *  void **sRandom    pointer to internal structure of random generator;
 *
 * Return value:
 *  If successfull return NO_ERROR.
\**************************************************************************************/
int SHA1RandomAllocate(void **sRandom);

/**************************************************************************************\
 * Function:  SHA1RandomFree
 *  void **sRandom    pointer to internal structure of random generator;
 *
 * Return value:
 *  If successfull return NO_ERROR.
\**************************************************************************************/
int SHA1RandomFree(void **sRandom);


/*************************************************************************************\
*
* Function:  SHA1RandomInitialize
*	void *sRandom	- pointer to internal structure of random generator;
*
\*************************************************************************************/
int	SHA1RandomInitialize( void *sRandom );


/*************************************************************************************\
*
* Function:  SHA1RandomUpdate
*	void *sRandom			- pointer to internal structure of random generator;
*	byte   *seedBuffer      - pointer to initialisation buffer;
*	size_t  seedBufferSize	- size of the initialization buffer;
*
* Description:
*  Use this function to initialize current random state by seedBuffer.
*  The seedBufferSize does not limited. State will always padded by SHA1 digest
*  algorithm.
*
\*************************************************************************************/
int SHA1RandomRandomize(void *sRandom,
                         unsigned char *seedBuffer, size_t seedBufferSize);


/*************************************************************************************\
*
* Function: SHA1RandomGenerateBytes (
*	void *sRandom		- pointer to internal structure of random generator;
*	byte *buffer		- pointer to buffer for new generated random data
*	size_t bufferSize	- size in bytes of required random data
*
* Description:
*  sRandom structure must be initialized by SHA1RandomInit.
*  SHA1RandomGenerate function can by called numerous times.
\*************************************************************************************/
int SHA1RandomGetRandomBytes(
	void *sRandom,
	unsigned char *buffer ,
	size_t bufferSize);


/*************************************************************************************\
*
* Function: SHA1RandomFinal (
*	void *sRandom		- pointer to void structure;
*
* Description:
*  secure zeroizes sRandom structure ..and deallocate it.
\*************************************************************************************/
int SHA1RandomFinalize(void *sRandom);

#endif /* __SHA1RANDOM_H__ */
