#include "SBlocks.h"
#include "smemory.h"
#include <assert.h>
#include <stdio.h>
#include <errno.h>

#ifndef _WIN32
//#include "../../misc/misc.h"
#define _myFree SFree
#define _myAlloc SMalloc
#define CheckMem( bf ) SMemCheck( bf )
#else

#ifdef _DEBUG_MEMORY

void CheckMem( BYTE *bf )
{
	bf -= 8;

	SIZE_T sizeB;
	SIZE_T sizeT;

	assert( memcmp( bf, "DBGA", 4 ) == 0 );
	memcpy( &sizeB, bf + 4, 4 );
	memcpy( &sizeT, bf + 8 + sizeB, 4 );
	assert( sizeT == sizeB );

	assert( memcmp( bf + 8 + sizeB + 4, "TAIL", 4 ) == 0 );
}

HGLOBAL DbgGlobalFree( HGLOBAL hMem )
{
	BYTE *bf = (BYTE *)hMem;

	CheckMem( bf );

	bf -= 8;

	return GlobalFree( (HGLOBAL)bf );
 }

HGLOBAL DbgGlobalAlloc( UINT uFlags, SIZE_T dwBytes )
{
	HGLOBAL hGlbl = GlobalAlloc( uFlags, dwBytes + 8 + 8 );
	BYTE *bf = (BYTE *)hGlbl;

	if ( bf == NULL )
		return NULL;

	memcpy( bf, "DBGA", 4 );
	memcpy( bf + 4, &dwBytes, 4 );

	memcpy( bf + 8 + dwBytes, &dwBytes, 4 );
	memcpy( bf + 8 + dwBytes + 4, "TAIL", 4 );

	return (HGLOBAL)(bf + 8);
}

#define _myFree( X ) DbgGlobalFree( HGLOBAL( X ) )
#define _myAlloc (BYTE*)DbgGlobalAlloc

#else
#define _myFree GlobalFree
#define _myAlloc( __size ) GlobalAlloc( GMEM_FIXED, __size )
#define CheckMem( bf )
#endif

int cUniversalString::SetWCS( const wchar_t *wStr, int wcsLen /* = -1 */ )
{
	if ( wcsLen == -1 )
	{
		if ( wStr == NULL )
			return 0;

		wcsLen = (int)wcslen( wStr );
	}

	int newSize = ( wcsLen + 1 ) * sizeof( wchar_t );

	if ( ( wcBuffer != NULL ) && ( wcBufferSize < newSize ) )
	{
		FreeBuffer( (BYTE *)wcBuffer );
		wcBuffer = NULL;
		wcBufferSize = 0;
	}

	if ( wcBuffer == NULL )
	{
		wcBuffer = (wchar_t *)AllocBuffer( newSize );

		if ( wcBuffer == NULL )
		{
			wcBufferSize = 0;
			return 0;
		}

		wcBufferSize = newSize;
	}

	memset( wcBuffer, 0, wcBufferSize );

	if ( wStr != NULL )
		memcpy( wcBuffer, wStr, newSize );

	return wcBufferSize;
}

int cUniversalString::Set( const wchar_t *wcs, int wcsLen /* = -1 */ )
{
	ClearBuffers();

	size_t num = 0;

	errno_t err = wcstombs_s( &num, NULL, 0, wcs, wcsLen );

	if ( err != 0 )
		return 0;

	bufferSize = (int)num;

	buffer = ( char *)AllocBuffer( bufferSize );
	if ( buffer == 0 )
		return 0;

	err = wcstombs_s( &num, buffer, bufferSize, wcs, wcsLen );

	if ( err != 0 ) // == EILSEQ )
		return 0;

	if ( wcsLen == -1 )
		wcsLen = (int)wcslen( wcs );

	if ( AllocateUnicodeBuffer( wcsLen + 1 ) == 0 )
		return 0;

	BYTE *leBf = (BYTE*)wcs;

	unicodeBuffer[0] = 0;
	memcpy_s( unicodeBuffer + 1, unicodeBufferSize  - 3, leBf, wcsLen * sizeof( wchar_t ) - 1 );
	unicodeBuffer[ wcsLen * sizeof( wchar_t )  ] = 0;
	unicodeBuffer[ wcsLen * sizeof( wchar_t ) + 1 ] = 0;

	wcBufferSize = unicodeBufferSize;
	wcBuffer = (wchar_t *)AllocBuffer( wcBufferSize );
	if ( wcBuffer == NULL )
		return 0;

	wcsncpy_s( wcBuffer, wcBufferSize / sizeof( wchar_t ), wcs, _TRUNCATE );

	return wcBufferSize;
}

int cUniversalString::MakeFromUnicodeString()
{
	// Converts ASN.1 unicode string to wchar_t 
	if ( wcBuffer )
	{
		memset( wcBuffer, 0, wcBufferSize );
		FreeBuffer( (BYTE *)wcBuffer );
	}

	//
	//	v. 1.03.5 - 12.10.2011
    //		Unix wchar_t has 4 byte size. but unicode buffer has to be complient with PKCS 12
	//		See comments at cUniversalString::wstr2unicodeBuff
	//
	int sizeInSymbols = unicodeBufferSize / 2;
	wcBufferSize = ( sizeInSymbols + 1 )* sizeof( wchar_t );
	wcBuffer = (wchar_t *)AllocBuffer( wcBufferSize );

	if ( wcBuffer == NULL )
		wcBufferSize = 0;
	else
	{
		for( int i = 0; i < sizeInSymbols; i++ )
			wcBuffer[i] =  ( ( (wchar_t)unicodeBuffer[ i * 2 ]) << 8 ) | unicodeBuffer[ i * 2 + 1 ];
	}

	wcBuffer[ sizeInSymbols ] = L'\0';

	// Convert wchar_t to char

	if ( buffer )
	{
		memset( buffer, 0, bufferSize );
		FreeBuffer( (BYTE*)buffer );
	}

#ifdef _WIN32
	// 10.12.2013 Makes UTF8 byte buffer.
	int bufferSize = WideCharToMultiByte( CP_UTF8, 0, wcBuffer, -1, NULL, 0, NULL, NULL );
	buffer = (char *)AllocBuffer( bufferSize );

	if ( buffer == NULL )
		bufferSize = 0;
	else
		WideCharToMultiByte( CP_UTF8, 0, wcBuffer, -1, buffer, bufferSize, NULL, NULL );
#else
	bufferSize = unicodeBufferSize / 2;
	buffer = (char *)AllocBuffer( bufferSize );
		size_t num;

	if ( buffer == NULL )
		bufferSize = 0;
	else
		wcstombs_s( &num, buffer, bufferSize, wcBuffer, _TRUNCATE );
#endif
	return bufferSize;
}

#endif // #ifndef _WIN32

//
//	The call can be send as void handler ( see static WriteBytes ).
//	But it has a signature value. So validity of the block can be checked.
//
int cPlainTextBlock::WriteBytesStatic( const void *ptr, BYTE *data, int dataSize )
{
	cPlainTextBlock *ptb = (cPlainTextBlock *)ptr;

	if ( !ptb->checkSig() )
		return 0;

	return ptb->WriteBytes( data, dataSize );
}

void cPlainTextBlock::ClearBuffer()
{
	if ( buffer )
		memset( buffer, 0, bufferSize );
}

void cPlainTextBlock::SeekToBegin()
{
	//
	//	Clear previously filled data...
	//
	ClearBuffer();

	bufferSize = 0;
}

BYTE *cPlainTextBlock::AddBytes( const BYTE *data, int dataSize )
{
	if ( bufferSize + dataSize > allocatedSize )
		if ( ReAllocate( bufferSize + dataSize ) == NULL )
			return 0;

	if ( data != NULL )
		memcpy( buffer + bufferSize, data, dataSize );

	BYTE *newPos = buffer + bufferSize;

	bufferSize += dataSize;

	return newPos;
}

int cPlainTextBlock::WriteBytes( const BYTE *data, int dataSize, bool init )
{
	if ( init )
		SeekToBegin();

	if ( bufferSize + dataSize > allocatedSize )
		if ( ReAllocate( bufferSize + dataSize ) == NULL )
			return 0;

	if ( data != NULL )
		memcpy( buffer + bufferSize, data, dataSize );

	bufferSize += dataSize;

	return dataSize;
}

int cPlainTextBlock::PutByte( BYTE bt )
{
	return WriteBytes( &bt, 1 );
}

void cPlainTextBlock::Free()
{
	if ( buffer != NULL )
	{
	    CheckMem( buffer );
	    _myFree( buffer );
	    buffer = NULL;
	}

	allocatedSize = bufferSize = 0;
}

//
//	Note, ReAllocate does not change bufferSize - current filled data does not changed...
//
BYTE *cPlainTextBlock::ReAllocate( int size )
{
	if ( allocatedSize == 0 )
	{
		BYTE *bf = Allocate( size );
		bufferSize = 0; // See note above
		return bf;
	}

	CheckMem( buffer );

	if ( bufferSize >= size )
	{
		memset( buffer + size, 0, bufferSize - size );
		bufferSize = size;
		return buffer;
	}

	if ( allocatedSize >= size )
		return buffer;

	BYTE *newBuffer = (BYTE *)_myAlloc( size );

	if ( newBuffer == NULL )
		return NULL;

	allocatedSize = size;

	memcpy( newBuffer, buffer, bufferSize );

	memset( newBuffer + bufferSize, 0, allocatedSize - bufferSize );

	memset( buffer, 0, bufferSize );

	_myFree( buffer );

	buffer = newBuffer;

	return buffer;
}

BYTE *cPlainTextBlock::Allocate( int size )
{
	if ( buffer )
		Free();

	buffer = (BYTE *)_myAlloc( size  );

	if ( buffer )
	{
		allocatedSize = bufferSize = size;
		ClearBuffer();
	}

	return buffer;
}

BYTE *cPlainTextBlock::Assign( const BYTE *inBuffer, int size )
{
	if ( Allocate( size ) == NULL )
		return NULL;

	memcpy( buffer, inBuffer, size );

	return buffer;
}


cPlainTextBlock::~cPlainTextBlock()
{
	Free();
}

/////////////////////
//
//	class cUniversalString
//
void cUniversalString::ClearBuffers()
{
	if ( buffer )
		memset( buffer, 0, bufferSize );

	if ( unicodeBuffer )
		memset( unicodeBuffer, 0, unicodeBufferSize );

	if ( wcBuffer )
		memset( wcBuffer, 0, wcBufferSize );
}

int cUniversalString::Set( const char *str, int size /* = -1 */ )
{
	ClearBuffers();
	if ( size == -1 )
		size = (int)strlen( str );

	buffer = ( char *)AllocBuffer( size + 1 );

	if ( buffer == NULL )
		return 0;

	memcpy( buffer, str, size );
	buffer[ size ] = 0;

	bufferSize = size + 1;

	return bufferSize;
}

int cUniversalString::AllocateUnicodeBuffer( int sizeInSymbols )
{
	int size = sizeInSymbols * 2; // !!!! UNIX version -> sizeof( wchar_t ) == 4 !!!!! ;

	if ( unicodeBuffer != NULL )
	{
		memset( unicodeBuffer, 0, unicodeBufferSize );

		if ( unicodeBufferSize < size  )
		{
			FreeBuffer( unicodeBuffer );

			unicodeBufferSize = 0;
			unicodeBuffer = NULL;
		}
		else
			return unicodeBufferSize;
	}

	if ( unicodeBuffer == NULL )
	{
		unicodeBuffer = AllocBuffer( size );
		if ( unicodeBuffer == NULL )
		{
			unicodeBufferSize = 0;
			return 0;
		}
	}

	unicodeBufferSize = size;

	memset( unicodeBuffer, 0, size );

	return unicodeBufferSize;
}

//
//	SetUnicode_CheckBigEndianInput
//		Checks input buffer before copying to allocated buffer
//		Assumes input as double byte buffer in little or big endian format.
//		The allocated unicode buffer used for password generation, so it has to be filled according 
//		PKCS 12 instructions( see comments at cUniversalString::wstr2unicodeBuff. )
//		in big-endian format (most-significant byte first)
//
int cUniversalString::SetUnicode_CheckBigEndianInput( BYTE *unicodeInput, int sizeInBytes )
{
	if ( sizeInBytes % 2 )
	{
		//ODS_fls( L"Assertion!" );
		sizeInBytes += 1;
	}

	if ( sizeInBytes == -1 )
	{
		for( sizeInBytes = 0; ; sizeInBytes += 2 )
		{
			if ( unicodeInput[ sizeInBytes ] != 0 ) continue;
			if ( unicodeInput[ sizeInBytes + 1 ] != 0 ) continue;

			break;
		}
	}

	int size = sizeInBytes / 2;

	if ( AllocateUnicodeBuffer( size + 1 ) == 0 )
		return 0;

	bool bigEndian = ( unicodeInput[0] == 0 );

	// Really we have to place zero byte first to allocated unicode buffer

	if ( bigEndian )
		 memcpy( unicodeBuffer, unicodeInput, size * 2 );
	else
	if ( size != 0 )
	{
		memcpy( unicodeBuffer + 1, unicodeInput, size * 2 - 1 );
		unicodeBuffer[ 0 ] = 0;
	}

	unicodeBuffer[ size * 2 ] = 0;
	unicodeBuffer[ size * 2 + 1 ] = 0;

	return unicodeBufferSize;
}

//
//	SetUnicode
//		The unicode string is required for generation key from password by P12_producePseudoRandomBytes
//
int cUniversalString::SetUnicode( const wchar_t *wStr, int sizeInWideChars /* = -1*/ ) //, bool checkBigEndian /* = false */ )
{
	int size = sizeInWideChars;
	if ( size == -1 )
		size = (int)wcslen( wStr );

	if ( AllocateUnicodeBuffer( size + 1 ) == 0 )
		return 0;

	if ( wstr2unicodeBuff( unicodeBuffer, unicodeBufferSize, wStr, sizeInWideChars ) == 0 ) // , checkBigEndian ) == 0 )
		return 0;

	return unicodeBufferSize;
}

int cUniversalString::SetUnicode( const BYTE *utf16be, int sizeInBytes )
{
	if ( sizeInBytes < 0 )
	{ assert( false ); return 0; }

	if ( sizeInBytes & 1 )
	{ assert( false ); return 0; }

	int sizeInWideChars = sizeInBytes / 2;

	if ( AllocateUnicodeBuffer( sizeInWideChars ) == 0 )
	{ assert( sizeInWideChars == 0 ); return 0; }

	if ( unicodeBufferSize < sizeInBytes )
	{ assert( false ); return 0; }

	memcpy( unicodeBuffer, utf16be, sizeInBytes );

	return unicodeBufferSize;

	//return SetUnicode( ( const wchar_t *)buff, sizeInWideChars ); //, checkBigEndian );
}

BYTE *cUniversalString::AllocBuffer( int size )
{
	BYTE *newBuffer = (BYTE *)_myAlloc( size );
	return newBuffer;
}

cUniversalString::~cUniversalString()
{
	if ( buffer )
		FreeBuffer( (BYTE *)buffer );

	if ( wcBuffer )
		FreeBuffer( (BYTE *)wcBuffer );

	if ( unicodeBuffer )
		FreeBuffer( unicodeBuffer );
}

void cUniversalString::FreeBuffer( BYTE *buff )
{
    _myFree( buff );
}

int cUniversalString::GenerateUnicodeString()
{
	if ( unicodeBuffer )
	{
		memset( unicodeBuffer, 0, unicodeBufferSize );
		FreeBuffer( unicodeBuffer );
		unicodeBuffer = NULL;
		unicodeBufferSize = 0;
	}

	if ( ( wcBufferSize != 0 ) && ( wcBuffer[0] != 0 ) )
	{
		if ( wstr2unicodeBuff( wcBuffer ) == 0 )
			return 0;
	}
	else
	if ( bufferSize != 0 )
	{
		int pwdLen = (int)strlen( buffer ) + 1;

		unicodeBufferSize = ( pwdLen ) * 2;
		unicodeBuffer = (BYTE *)AllocBuffer( unicodeBufferSize );

		if ( unicodeBuffer == NULL )
			unicodeBufferSize = 0;
		else
		for ( int i = 0; i < pwdLen; i++ )
		{
			unicodeBuffer[ i * 2 ] = 0;
			unicodeBuffer[ i * 2 + 1 ] = buffer[ i ];
		}
	}

	return unicodeBufferSize;
}


//
//	Wide char string / unicode buffer conversions.
//
//	wstr2unicodeBuff 
//		unicode buffer size has to be equal wcslen( ws ) * 2 bytes
//		returns number of copied wide char symbols
//
//	PKCS 12 V1.0: PERSONAL INFORMATION EXCHANGE SYNTAX
//		B.1 Password formatting
//			- 'the password should consist of printable ASCII characters (values 32 to 126 inclusive)'
//			- In this specification however, all passwords are created from BMPStrings with a NULL
//				terminator.  This means that each character in the original  BMPString is encoded in 2
//				bytes in big-endian format (most-significant byte first).  There are no Unicode byte order
//				marks.  The 2 bytes produced from the last character in the BMPString are followed by
//				two additional bytes with the value 0x00.
//				To illustrate with a simple example, if a user enters the 6-character password "Beavis",
//				the string that PKCS #12 implementations should treat as the password is the following
//				string of 14 bytes:
//				0x00 0x42 0x00 0x65 0x00 0x61 0x00 0x76 0x00 0x69 0x00 0x73 0x00 0x00
//
// static
int cUniversalString::wstr2unicodeBuff( BYTE *unicodeBuff, int unicodeBuffSize, const wchar_t *wStr, int len /*= -1*/ ) //, bool checkBigEndian /* = false */ )

{
	//int len = (int)wcslen( ws );

	//for( int i = len; i--; )
	//{
	//	unicodeBuff[ i * 2 + 1 ] = BYTE( ws[ i ] );
	//	unicodeBuff[ i * 2 ] = BYTE( ws[ i ] >> 8 );
	//}

	//return len;

	//bool bigEndian = true;

	//if ( checkBigEndian )
	//{
	//	bigEndian = *(BYTE*)wStr == 0;
	//}

	//
	//	!!! Wide char in UNIX is 4 byte size !!!
	//
	//if ( bigEndian )
	//	 memcpy( unicodeBuffer, wStr, size * sizeof( wchar_t ) );
	//else
	//if ( size != 0 )
	//{
	//	memcpy( unicodeBuffer + 1, wStr, size * sizeof( wchar_t ) - 1 );
	//	unicodeBuffer[ 0 ] = 0;
	//}
	//
	//wchar_t *wPtr = (wchar_t *)unicodeBuffer;
	//wPtr[ size ] = 0;

	if ( len == -1 )
		len = (int)wcslen( wStr );

	//
	//	Buffer has to be prepared before.
	//
	if ( unicodeBuffSize / 2 < ( len + 1 ) )
	{
		assert( false );
		return 0;
	}

	const bool bigEndian = (*(BYTE*)L"A") == 0; 

	if ( bigEndian )
	{
		for( int i = 0; i < unicodeBuffSize / 2; i++ )
		{
			// !!! wcscpy cannot be used because wchar_t has 4 byte size !!!, but unicodeBuff has to be 2 byte size ( see comments above )
			unicodeBuff[ i * 2 ] = (BYTE)wStr[i];
			unicodeBuff[ i * 2 + 1 ] = (BYTE)( wStr[i] >> 8 );
		}
	}
	else
	{
		for( int i = 0; i < unicodeBuffSize / 2; i++ )
		{
			unicodeBuff[ i * 2 ] = BYTE( wStr[ i ] >> 8 );
			unicodeBuff[ i * 2 + 1 ] = BYTE( wStr[ i ] );
		}
	}

	return len;
}

//
//	unicodeBuff2wstr
//		size of wc buffer has to be one symbol more of unicodeBuffByteSize/2
//		for null termination symbol
//
int cUniversalString::unicodeBuff2wstr( wchar_t *ws, int wsBufferSize, BYTE *unicodeBuff, int unicodeBuffByteSize )
{
	int l = ( unicodeBuffByteSize / 2 );

	if ( l > (int)(wsBufferSize / sizeof( wchar_t )) )
		return 0;

	ws[l] = 0;

	for( ; l--; )
	{
		ws[l] = unicodeBuff[ l * 2 + 1 ] | ( wchar_t( unicodeBuff[ l * 2 ] ) << 8 );
	}

	return (int)wcslen( ws );
}
