#include "cPassword.h"
#include "../x509pkts/pkcs12_funcs.h"

void cPassword::InitBuffers()
{
	bufferSize = MaxLenght + 1;
	buffer = (char *)AllocBuffer( bufferSize );
	if ( buffer == NULL )
	{
		bufferSize = 0;
		return;
	}

	wcBufferSize = (MaxLenght + 1) * sizeof( wchar_t );
	wcBuffer = (wchar_t *)AllocBuffer( wcBufferSize );
	if ( wcBuffer == NULL )
	{
		wcBufferSize = 0;
		return;
	}

	unicodeBuffer = NULL;
	unicodeBufferSize = 0;

	AllocateUnicodeBuffer( MaxLenght + 1 );

	ClearBuffers();
}

cPassword::cPassword( const wchar_t *pwdStr )
{
	InitBuffers();

	if ( pwdStr != NULL )
	{
		wcsncpy( wcBuffer, pwdStr, MaxLenght );
	}
}

cPassword::cPassword( const char *pwdStr )
{
	InitBuffers();

	if ( pwdStr != NULL )
	{
		strncpy( buffer, pwdStr, MaxLenght );
	}
}

cPassword::~cPassword()
{
	ClearBuffers();
}

bool cPassword::CheckValidSize()
{
	return strlen( buffer ) >= MinimumLenght;
}

bool cPassword::wcCheckValidSize()
{
	return wcslen( wcBuffer ) >= MinimumLenght;
}

int cPassword::Compare( cPassword *pwd )
{
	return strncmp( pwd->buffer, buffer, MaxLenght );
}

int cPassword::wcCompare( cPassword *pwd )
{
	return wcsncmp( pwd->wcBuffer, wcBuffer, MaxLenght );
}

//
//	generateKeyFromPassword
//		Uses P12_producePseudoRandomBytes functions for key generation.
//
bool cPassword::generateKeyFromPassword(
		BYTE *outBuff,	int outBuffSize,
		BYTE hashAlgId, // = bchaSHA256,
		BYTE *salt,		int saltSize, // NULL
		int iteration, // == 1
		bool key // key or IV
	)
{
	//
	//	Function: P12_producePseudoRandomBytes
	//	See - pkcs #12, B.2 General method. (p.14)
	//
	if ( GenerateUnicodeString() == 0 )  // returns unicodeBufferSize in bytes, it is equal (pwd length + 1) * 2
		return false;

	int pwdLen = wcslen( (wchar_t *)unicodeBuffer );

	if ( pwdLen == 0 ) // < pwd->MinimumLenght )
	{
		assert( false );
		return false;
	}

	int res = P12_producePseudoRandomBytes(
					outBuff, outBuffSize, 
					salt, saltSize,
					iteration,
					unicodeBuffer, ( pwdLen + 1 ) * 2,
					(BYTE)( key ? P12_prgm4key : P12_prgm4IV ),
					hashAlgId
				);

	return res == x509e_ok;
}
