/**************************************************************************\
*
* Secure Memory module
*
* Intended for:
* 1) Automatic memory zeroizing before deallocation
* 2) For redirection of GNU MP 2.0 allocation and deallocation
*      It needs for checking allocation memory error during GMP operation.
*
\**************************************************************************/

#ifndef _SMEMORY_H_
#define _SMEMORY_H_

#include <stddef.h>
#include "../progressActor.h"

void *SMalloc( size_t size, bool fillByZero = false );
void *SCalloc( size_t num , size_t size );
void SFree( void *memblock );
void *SReAlloc( void *memblock , size_t size );
void *SMemCpy( void *dest , const void *src , size_t count );
size_t SMemCmp( const void *buf1 , const void *buf2 , size_t count );
void *SMemSet( void *dest , int c , size_t count );
void *SMemMove( void *dest , const void *src , size_t count );
int SMemCheck( void *memBlock, size_t **ppPtr = NULL, size_t *pSize = NULL );

/******************************************************************************************\
* GNU MP 2.0 extension...
*
* This functions required to replace standard memory allocation function from GNU MP 2.0
*
* To replace these functions use the GMP function before first initialisation
*  mp_set_memory_functions ( S_allocate_function, S_reallocate_function, S_deallocate_function );
*  This function defined in "gmh.h" file. It must be inluded befor using of if you 
* 
\******************************************************************************************/

void * S_allocate_function( size_t alloc_size );
void * S_reallocate_function(void *memblock, size_t old_size, size_t new_size);
void S_deallocate_function(void *memblock, size_t size);

extern int SMemoryRedirected; /* Used for detection was redirection or not */
extern int SMemoryError;		/* Not zero if was memory allocation error 
								 * if redirection is applied you need zeroed the flag 
								 * to normal allocate memory again.
								 */
/*
 * Procedure : SSetGMPMemoryFunction
 *	Used as define. So there are no compiler errors if GMP library does not used,
 *	"gmp.h" header file does not included, and the #define does not called.
 */
#define SSetGMPMemoryFunction()		\
{										\
	if ( SMemoryRedirected == 0)		\
	{	mp_set_memory_functions (		\
			S_allocate_function,		\
			S_reallocate_function,	\
			S_deallocate_function);	\
										\
		SMemoryRedirected = 1;		\
	}									\
}

#endif /*_SMEMORY_H_ */
