/////////////////////////////////////////////////////////////////
//
//	Copyright (c) 1994-2003 Jetico Inc., Finland
//	All rights reserved.
//
//	File: crtLibErrors.h
//	Revision: 
//	Created:
//	Description: Errors encountered during certificate 
//			library function processing ...
//
/////////////////////////////////////////////////////////////////

#ifndef __crtLibErrs__
#define __crtLibErrs__

enum {
	cae_ok = 0,
	sae_ok = 0,
	ppe_ok = 0,
	pkae_ok = 0,
	hae_ok = 0,
	x509e_ok = 0,
	RSA_OK = 0,

	
	pkae_bufferSizeTooSmall,
	ppe_bufferTooSmall = pkae_bufferSizeTooSmall,
	x509e_bufferTooSmall = pkae_bufferSizeTooSmall,
	RSA_INVALID_BLOCK_SIZE = pkae_bufferSizeTooSmall,

	//
	//	RSA algorithm errors
	//

	RSA_MALLOC_ERROR,
	RSA_INIT_ERROR,
	RSA_INVALID_KEY_BLOCK,
	RSA_INVALID_BLOCK_TYPE,

	//
	//	The error is generated if small public exponent is used
	//	and encoded value is too small, see description in RSA.cpp
	//	1. If signature generator encounters with same error during checking validity of 
	//	public/private process, it should make changes in hashed source
	//	or in signed value and generate new signature.
	//	2. Signature verification in application has to return error.
	//
	RSA_SIGNATURE_ATACK_SUSSPECTION,

	//
	//	x509 errors
	//
	
	x509e_memoryAllocation,
	x509e_internalError,
	x509e_invalidPktVersion,
	x509e_invalidPktSize,
	x509e_invalidPktTag,
	x509e_invalidPkt,
	x509e_undefinedOIDType,
	x509e_unexpectedOIDType,
	x509e_undefinedAlgorithm,
	x509e_emptyParameters,
	x509e_extensionIsNotPresent,
	x509e_featureIsNotImplemented,
	x509e_validityPeriodFinished,

	//
	//	The extractP7Content returns the result for pkcs-7-encryptedData type.
	//	It means that following value completely defined as encrypted algorithm and 
	//	no anther parsing required. ( It has been detected for SSL generated packets )
	//
	x509e_treatAsEncryptedAlgorithmContent,

	//
	//	Hash algorthm error enums
	//
	hae_memoryAllocation,
	hae_invalidAlgorithmID,
	hae_internalError,

	//
	//	Enums for Symmetric Algorithms' Errors
	//
	sae_internalError,
	sae_memoryAllocationError,
	sae_invalidBlockSize,
	sae_operationError,
	sae_invalidAlgorithmID,

	//
	//	Public key algorithm error enums
	//
	pkae_invalidInputData,
	pkae_internalError,
	pkae_memoryAllocation,
	pkae_invalidSignature,
	pkae_invalidAlgorithmID,
	pkae_cancelled,

	pkae_randomBufferRequired,		// use pka_setRandomBuffer to assign the required buffer...
	ppe_randomBufferRequired		// use cRNG::setPool( buffer, bufferSize ) to assign the required buffer...
		= pkae_randomBufferRequired,// use cRNG::clearPool() to clear the buffer

	ppe_invalidHeader,
	ppe_invalidPacketVersion,
	ppe_invalidPacketSize,
	ppe_invalidBlockType,
	ppe_invalidS2KBlockType,
	ppe_invalidBlock,

	ppe_invalidSignature,
	ppe_invalidSignatureType,
	ppe_invalidSignatureSubpacketType,
	ppe_signatureIssuerSubpacketNotFound,		// Signature subpackets do not contain a issuer key id
	ppe_signatureIssuerKeyRingDoesNotSpecified, // The key is not self signed but 
												// the PKT_VerifySignaturePacket was called without issuer key ring packet
	ppe_signatureIssuerNotFound, // PKT_VerifySignaturePacket has to be called with another issuer key ring

	ppe_invalidAlgorithmID,
	ppe_invalidKeySize,
	ppe_invalidMPIBuffer,
	ppe_invalidPassword,
	ppe_internalError,
	ppe_memoryAllocation,
	ppe_duplicatedKeyID,	// Specified key ring contains a keys with same ID
	ppe_keyNotFound,		// Specified key ring does not contain a key with ID
	ppe_messageTooLong,		// Encoding RSA message, mesasge has to be less then module size - 10 ( PKCS1 v.2 )

	ppe_last,

	// Compression algorithm errors

	cae_bufferTooSmall = pkae_bufferSizeTooSmall,
										// if there was not enough room in the output buffer
	cae_first = 0x10000,

	cae_memoryAllocation = cae_first,	// if there was not enough memory
	cae_inputDataIsCorrupted,			// compressd data block is damaged or corrupted
	cae_undefined,						// undefined compression error
	cae_internalError,
	cae_invalidAlgorithmID,

	cae_last = cae_invalidAlgorithmID
};

#endif // __crtLibErrs__
