/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 */

#ifndef BC_INTSIZE_H
#define BC_INTSIZE_H

#ifdef WIN32

#include <windows.h>

typedef char				bc_8;
typedef unsigned char		bc_u8;
typedef short				bc_16;
typedef wchar_t				bc_u16;
typedef long				bc_32;
typedef unsigned long 		bc_u32;
typedef __int64				bc_64;
typedef unsigned __int64	bc_u64;

#else

#include <stdint.h>

typedef int8_t		bc_8;
typedef uint8_t		bc_u8;
typedef int16_t		bc_16;
typedef uint16_t	bc_u16;
typedef int32_t		bc_32;
typedef uint32_t	bc_u32;
typedef int64_t		bc_64;
typedef uint64_t	bc_u64;
typedef uint64_t ULONGLONG;

#endif

#ifdef __cplusplus
static __inline bc_u32 u64tou32( bc_u64 u64, bc_u64 max = 0xFFFFffffLL )
{
	if ( u64 > max )
		return (bc_u32)max;
	else
		return (bc_u32)u64;
}
#endif

#if !(defined(_BIG_ENDIAN_DEFINED) && !defined(_LITTLE_ENDIAN_DEFINED))

	#if defined (__BYTE_ORDER__)
		#if (__ORDER_BIG_ENDIAN__==__BYTE_ORDER__)
			#define	_BIG_ENDIAN_DEFINED
		#elif (__ORDER_LITTLE_ENDIAN__==__BYTE_ORDER__)
			#define	_LITTLE_ENDIAN_DEFINED
		#elif (__ORDER_PDP_ENDIAN__==__BYTE_ORDER__)
			#define	_PDP_ENDIAN_DEFINED
			#error "pdp endian"
		#else
			#error "machine is undefined endian"
		#endif

	#else

		#define LITTLE_ENDIAN_ORDER	0x41424344UL
		#define BIG_ENDIAN_ORDER	0x44434241UL
		#define PDP_ENDIAN_ORDER	0x42414443UL
		#define ENDIAN_ORDER		('ABCD')

		#if ENDIAN_ORDER==LITTLE_ENDIAN_ORDER
			#define	_LITTLE_ENDIAN_DEFINED
		#elif ENDIAN_ORDER==BIG_ENDIAN_ORDER
			#define	_BIG_ENDIAN_DEFINED
		#elif ENDIAN_ORDER==PDP_ENDIAN_ORDER
			#define _PDP_ENDIAN_DEFINED
		#else
			#error "machine is undefined endian"
		#endif

	#endif

#endif // !(defined(_BIG_ENDIAN_DEFINED) || defined(_LITTLE_ENDIAN_DEFINED))

#endif

