/*******************************************************************
 *
 *	Copyright (c) 2007-2008 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: sha512.h
 *	Revision: 
 *	Created:
 *	Description: declaration of SHA-512 (SHA-2) Secure Hash Algorithm 
 *	             external procedures
 *
 *******************************************************************/

/*
 *	SHA-512 message-digest algorithm.
 */

#ifndef __SHA512_H__
#define __SHA512_H__

#include "./intsize.h"

#define SHA512_DIGEST_SIZE  64

#define SHA512_ERROR_NO               0x0
#define SHA512_ERROR_INTERNAL_ERROR   0x1

typedef struct SHA512Context_
{
     ULONGLONG digest[8];         /* Message digest */
     ULONGLONG countLo, countHi;  /* 128-bit byte count */
     ULONGLONG data[16];          /* SHA data buffer */
} SHA512Context;

#define SHA512_CONTEXT_SIZE  sizeof(SHA512Context)

int SHA512_Init(void *context);

int SHA512_Process(void *context, const unsigned char *data, DWORD length);

int SHA512_Update(void *context, unsigned char *data);

int SHA512_Final(void *context, unsigned char *digest); /* 'digest' array must be >= 32 bytes */

int SHA512_Allocate(void **context);

int SHA512_Free(void **context);

int SHA512_MakeDigest( unsigned char *message, DWORD messageLength, unsigned char *digest );

#endif /* __SHA512_H__ */
