/*****************************************************************\
*
*  Copyright (c) 2015 Jetico, Inc., Finland
*  All rights reserved.
*
*  File: skein512.h
*  Revision:
*  Created: 25.02.2015
*  Description:
*		Hash Skein with 512 bits.
*		Current implementation is optimized for 32 bit machines.
*
\*****************************************************************/

#include "Optimized_32bit/skein.h"
#include <string.h>
#ifdef _WIN32
#include <malloc.h>
#else
#include <stdlib.h>
#endif
#include "skein512.h"

//
//# Skein 1.3 512-512 Test data
//	From http://www.h2database.com/skein/
//
u08b_t msg[] = { 0xFF };
u08b_t MD[]  = {
	0x71, 0xB7, 0xBC, 0xE6, 0xFE, 0x64, 0x52, 0x22, 0x7B, 0x9C, 0xED, 0x60, 0x14, 0x24, 0x9E, 0x5B, 0xF9, 0xA9, 0x75, 0x4C, 0x3A, 0xD6, 0x18, 0xCC, 0xC4, 0xE0, 0xAA, 0xE1, 0x6B, 0x31, 0x6C, 0xC8, 
	0xCA, 0x69, 0x8D, 0x86, 0x43, 0x07, 0xED, 0x3E, 0x80, 0xB6, 0xEF, 0x15, 0x70, 0x81, 0x2A, 0xC5, 0x27, 0x2D, 0xC4, 0x09, 0xB5, 0xA0, 0x12, 0xDF, 0x2A, 0x57, 0x91, 0x02, 0xF3, 0x40, 0x61, 0x7A
};

int Skein512_AutoTest()
{
	u08b_t MDRes[ sizeof(MD) ];

	int res = Skein512_MakeDigest( msg, sizeof( msg ), MDRes );
	if ( res != SKEIN_SUCCESS )	return res;

	res = memcmp( MDRes, MD, sizeof(MD) );
	return res;
}

int Skein512_Allocate( void **ctx )
{
	if ( ctx == NULL )
		return SKEIN_FAIL;

	*ctx = malloc( sizeof( Skein_512_Ctxt_t ) );

	if ( *ctx == NULL )
		return SKEIN_FAIL;

	return SKEIN_SUCCESS;
}

int Skein512_Free( void **ctx)
{
	if ( ctx == NULL )
		return SKEIN_FAIL;

	if ( *ctx == NULL )
		return SKEIN_FAIL;

	memset( *ctx, 0, sizeof( Skein_512_Ctxt_t ) );

	free( *ctx );

	*ctx = NULL;

	return SKEIN_SUCCESS;
}

int Skein512_Init( void *ctx)
{
	if ( ctx == NULL )
		return SKEIN_FAIL;

	memset( ctx, 0, sizeof( Skein_512_Ctxt_t ) );

	return Skein_512_Init( (Skein_512_Ctxt_t *)ctx, 512 );
}

int Skein512_Process( void *ctx, const unsigned char *data, unsigned int length)
{
	return Skein_512_Update( (Skein_512_Ctxt_t *)ctx, data, length );
}

int Skein512_Final( void *ctx, unsigned char *digest )
{
	return Skein_512_Final( (Skein_512_Ctxt_t *)ctx, digest );
}

int Skein512_MakeDigest( const unsigned char *message, unsigned int  messageLength, unsigned char *digest )
{
	void *context = NULL;

	int res = Skein512_Allocate(&context);
	if ( res != SKEIN_SUCCESS ) return res;

	res = Skein512_Init(context);
	if ( res != SKEIN_SUCCESS ) return res;

	res = Skein512_Process(context, message, messageLength);
	if ( res != SKEIN_SUCCESS ) return res;

	res = Skein512_Final(context, digest);
	if ( res != SKEIN_SUCCESS ) return res;

	res = Skein512_Free(&context);
	return res;
}
