/*******************************************************************
 *
 *      Copyright (c) 2014 Jetico, Inc., Finland
 *      All rights reserved.
 *
 *      File: Whirlpool512.cpp
 *      Revision:
 *      Created:
 *      Description: implementation of Whirlpool 512 bit Secure Hash Algorithm
 *
 *******************************************************************/

#include "Whirlpool512.h"
#include "nessie.h"
#include <string.h>
#ifdef _WIN32
#include <malloc.h>
#else
#include <stdlib.h>
#endif

int Whirlpool512_AutoTest()
{
    u8 digest[DIGESTBYTES];

	//
	//https://en.wikipedia.org/wiki/Whirlpool_(cryptography)
	//

	const char *test = "";
	unsigned char res0[] = {
		0x19, 0xFA, 0x61, 0xD7, 0x55, 0x22, 0xA4, 0x66, 0x9B, 0x44, 0xE3, 0x9C, 0x1D, 0x2E, 0x17, 0x26, 0xC5, 0x30, 0x23, 0x21, 0x30, 0xD4, 0x07, 0xF8, 0x9A, 0xFE, 0xE0, 0x96, 0x49, 0x97, 0xF7, 0xA7,
		0x3E, 0x83, 0xBE, 0x69, 0x8B, 0x28, 0x8F, 0xEB, 0xCF, 0x88, 0xE3, 0xE0, 0x3C, 0x4F, 0x07, 0x57, 0xEA, 0x89, 0x64, 0xE5, 0x9B, 0x63, 0xD9, 0x37, 0x08, 0xB1, 0x38, 0xCC, 0x42, 0xA6, 0x6E, 0xB3
	};

	if ( Whirlpool512_MakeDigest( (const u8*)test, strlen(test), digest ) != Whirlpool512_ERROR_NO )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	bool ok = ( memcmp( digest, res0, sizeof( res0 ) ) == 0 );

	if ( !ok )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	test = "The quick brown fox jumps over the lazy dog";
	unsigned char res1[] = {
		0xB9, 0x7D, 0xE5, 0x12, 0xE9, 0x1E, 0x38, 0x28, 0xB4, 0x0D, 0x2B, 0x0F, 0xDC, 0xE9, 0xCE, 0xB3, 0xC4, 0xA7, 0x1F, 0x9B, 0xEA, 0x8D, 0x88, 0xE7, 0x5C, 0x4F, 0xA8, 0x54, 0xDF, 0x36, 0x72, 0x5F,
		0xD2, 0xB5, 0x2E, 0xB6, 0x54, 0x4E, 0xDC, 0xAC, 0xD6, 0xF8, 0xBE, 0xDD, 0xFE, 0xA4, 0x03, 0xCB, 0x55, 0xAE, 0x31, 0xF0, 0x3A, 0xD6, 0x2A, 0x5E, 0xF5, 0x4E, 0x42, 0xEE, 0x82, 0xC3, 0xFB, 0x35
	};

	if ( Whirlpool512_MakeDigest( (const u8*)test, strlen(test), digest ) != Whirlpool512_ERROR_NO )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	ok = ( memcmp( digest, res1, sizeof( res1 ) ) == 0 );

	if ( !ok )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	return Whirlpool512_ERROR_NO;

}

#ifndef NULL
#define NULL 0
#endif

int Whirlpool512_Allocate( void **ctx )
{
	if ( ctx == NULL )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	*ctx = malloc( sizeof( NESSIEstruct ) );

	if ( *ctx == NULL )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	return Whirlpool512_ERROR_NO;
}

int Whirlpool512_Free(void **ctx)
{
	if ( ctx == NULL )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	if ( *ctx == NULL )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	memset( *ctx, 0, sizeof( NESSIEstruct ) );

	free( *ctx );

	*ctx = NULL;

	return Whirlpool512_ERROR_NO;
}

int Whirlpool512_Init( void *ctx )
{
	if ( ctx == NULL )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

    NESSIEinit( (NESSIEstruct*)ctx );

	return Whirlpool512_ERROR_NO;
}

int Whirlpool512_Process( void *ctx, const unsigned char *msg, unsigned int size )
{
    NESSIEadd( msg, size * 8, (NESSIEstruct*)ctx );
	
	return Whirlpool512_ERROR_NO;
}

//int Whirlpool512_Update(void *ctx, unsigned char *data)
//{
//	return Whirlpool512_ERROR_INTERNAL_ERROR;
//}

int Whirlpool512_Final(void *ctx, unsigned char *digest)
{
    NESSIEfinalize( (NESSIEstruct*)ctx, digest );

	return Whirlpool512_ERROR_NO;
}

int Whirlpool512_MakeDigest( const unsigned char *message, unsigned int messageLength, unsigned char *digest )
{
	void *context = NULL;

    if ( Whirlpool512_Allocate(&context) != Whirlpool512_ERROR_NO)
		return Whirlpool512_ERROR_INTERNAL_ERROR;
  
    if ( Whirlpool512_Init(context) != Whirlpool512_ERROR_NO)
		return Whirlpool512_ERROR_INTERNAL_ERROR;

    if ( Whirlpool512_Process(context, message, messageLength) != Whirlpool512_ERROR_NO )
		return Whirlpool512_ERROR_INTERNAL_ERROR;

    if ( Whirlpool512_Final(context, digest) != Whirlpool512_ERROR_NO )
		return Whirlpool512_ERROR_INTERNAL_ERROR;
	
    if ( Whirlpool512_Free(&context) != Whirlpool512_ERROR_NO)
		return Whirlpool512_ERROR_INTERNAL_ERROR;

	return Whirlpool512_ERROR_NO;
}
