#ifndef __hashAlgs__
#define __hashAlgs__

#ifndef BYTE
typedef unsigned char BYTE;
#endif

#include "../crtLibErrs.h"

typedef class cHashAlg * HHA; // Handle to hash algorithm

//
//	9.4. Hash Algorithms
//
//	ID           Algorithm                              Text Name
//	--           ---------                              ---- ----
//	1          - MD5                                    "MD5"
//	2          - SHA-1                                  "SHA1"
//	3          - RIPE-MD/160                            "RIPEMD160"
//	4          - Reserved for double-width SHA (experimental)
//	5          - MD2                                    "MD2"
//	6          - Reserved for TIGER/192                 "TIGER192"
//	7          - Reserved for HAVAL (5 pass, 160-bit)	"HAVAL-5-160"
//	100 to 110 - Private/Experimental algorithm.
//

//enum enum_haID {
//	haID_MD5 = 1,
//	haID_SHA1 = 2,
//    haID_SHA256 = 128,
//	haID_Ripemd = 3,
//	haID_MD2 = 5
//};

const char *GetHashAlgName( BYTE algID );

int ha_getIDsOfAvailableAlgAlgorithms( int *pIDsCount, int **ppIDsTab );
const char *ha_getHashAlgName( BYTE algID );

int ha_createAlgorithm( HHA * pHHA, int algID );
int ha_getDigestSize( int *pSize, int algID );
void ha_free( HHA hHA );
int ha_init( HHA hHA );
int ha_getDigestSize( HHA hHA );
int ha_process( HHA hHA, const BYTE *data, int length );
int ha_final( HHA hHA, BYTE *digest );

int ha_makeDigest(
	HHA hHA,		// Handle of hash algorithm
	BYTE *digest,	// Output digest buffer.
					// Note, size of the buffer has not to be less then size
					// the ha_getDigestSize returns.
	BYTE *message,	// Input message buffer. 
	int messageSize	// Size of message buffer.
);

int ha_makeDigestEx(
	int algID,		// Hash algorithm ID, f.e. pgphaSHA1
	BYTE *digest,	// Output digest buffer.
					// Note, size of the buffer has not to be less then size
					// the ha_getDigestSize returns.
	BYTE *message,	// Input message buffer. 
	int messageSize	// Size of message buffer.
);

#endif // __hashAlgs__
