#include "../Power/Windefs.h"
#include "../Power/power.h"
#include "../../RNG/cRNG.h"
#include "Dsa.h"
#include "../../x509pkts/x509_structs.h"
#include "../../x509pkts/oidFuncs.h"
#include "../../PGPPkts/pktdefs.h"

//
// DSA private key packet
//
//	PrivateKeyInfo ::= SEQUENCE {
//	  version Version,
//	  privateKeyAlgorithm AgorithmIdentifier,
//	  privateKey PrivateKey,
//	  attributes [0] IMPLICIT Attributes OPTIONAL
//	}
//
//	Version ::= INTEGER { 0 }
//
//	AlgorithmIdentifier  ::=  SEQUENCE  {
//		algorithm               OBJECT IDENTIFIER,
//		parameters              ANY DEFINED BY algorithm OPTIONAL  }
//
//	DSA OID {1 2 840 10040 4 1)
//
//	Dss-Parms  ::=  SEQUENCE  {
//		p             INTEGER,
//		q             INTEGER,
//		g             INTEGER  }
//
//	PrivateKey ::= OCTET STRING encapsulates {
//		x			  INTEGER       }
//
//	Attributes ::= SET OF Attribute
//


/*
	Private Keys Samples

AlicePrivDSSSign =
   0 30  331: SEQUENCE {
   4 02    1:   INTEGER 0
   7 30  299:   SEQUENCE {
  11 06    7:     OBJECT IDENTIFIER dsa (1 2 840 10040 4 1)
            :       (ANSI X9.57 algorithm)
  20 30  286:     SEQUENCE {
  24 02  129:       INTEGER
            :         00 81 8D CD ED 83 EA 0A 9E 39 3E C2 48 28 A3 E4
            :         47 93 DD 0E D7 A8 0E EC 53 C5 AB 84 08 4F FF 94
            :         E1 73 48 7E 0C D6 F3 44 48 D1 FE 9F AF A4 A1 89
            :         2F E1 D9 30 C8 36 DE 3F 9B BF B7 4C DC 5F 69 8A
            :         E4 75 D0 37 0C 91 08 95 9B DE A7 5E F9 FC F4 9F
            :         2F DD 43 A8 8B 54 F1 3F B0 07 08 47 4D 5D 88 C3
            :         C3 B5 B3 E3 55 08 75 D5 39 76 10 C4 78 BD FF 9D
            :         B0 84 97 37 F2 E4 51 1B B5 E4 09 96 5C F3 7E 5B
            :         DB
 156 02   21:       INTEGER
            :         00 E2 47 A6 1A 45 66 B8 13 C6 DA 8F B8 37 21 2B
            :         62 8B F7 93 CD
 179 02  128:       INTEGER
            :         26 38 D0 14 89 32 AA 39 FB 3E 6D D9 4B 59 6A 4C
            :         76 23 39 04 02 35 5C F2 CB 1A 30 C3 1E 50 5D DD
            :         9B 59 E2 CD AA 05 3D 58 C0 7B A2 36 B8 6E 07 AF
            :         7D 8A 42 25 A7 F4 75 CF 4A 08 5E 4B 3E 90 F8 6D
            :         EA 9C C9 21 8A 3B 76 14 E9 CE 2E 5D A3 07 CD 23
            :         85 B8 2F 30 01 7C 6D 49 89 11 89 36 44 BD F8 C8
            :         95 4A 53 56 B5 E2 F9 73 EC 1A 61 36 1F 11 7F C2
            :         BD ED D1 50 FF 98 74 C2 D1 81 4A 60 39 BA 36 39
            :       }
            :     }
 310 04   23:   OCTET STRING, encapsulates {
 312 02   21:       INTEGER
            :         00 BB 44 46 D1 A5 C9 46 07 2E D0 FE 7A D6 92 07
            :         F0 9A 85 89 3F
            :       }
            :   }

AlicePrivRSASign =
   0 30  630: SEQUENCE {
   4 02    1:   INTEGER 0
   7 30   13:   SEQUENCE {
   9 06    9:     OBJECT IDENTIFIER rsaEncryption (1 2 840 113549 1 1 1)
            :       (PKCS #1)
  20 05    0:     NULL
            :     }
  22 04  608:   OCTET STRING, encapsulates {
  26 30  604:       SEQUENCE {
  30 02    1:         INTEGER 0
  33 02  129:         INTEGER
            :           00 E0 89 73 39 8D D8 F5 F5 E8 87 76 39 7F 4E B0
            :           05 BB 53 83 DE 0F B7 AB DC 7D C7 75 29 0D 05 2E
            :           6D 12 DF A6 86 26 D4 D2 6F AA 58 29 FC 97 EC FA
            :           82 51 0F 30 80 BE B1 50 9E 46 44 F1 2C BB D8 32
            :           CF C6 68 6F 07 D9 B0 60 AC BE EE 34 09 6A 13 F5
            :           F7 05 05 93 DF 5E BA 35 56 D9 61 FF 19 7F C9 81
            :           E6 F8 6C EA 87 40 70 EF AC 6D 2C 74 9F 2D FA 55
            :           3A B9 99 77 02 A6 48 52 8C 4E F3 57 38 57 74 57
            :           5F
 165 02    3:         INTEGER 65537
 170 02  128:         INTEGER
            :           00 A4 03 C3 27 47 76 34 34 6C A6 86 B5 79 49 01
            :           4B 2E 8A D2 C8 62 B2 C7 D7 48 09 6A 8B 91 F7 36
            :           F2 75 D6 E8 CD 15 90 60 27 31 47 35 64 4D 95 CD
            :           67 63 CE B4 9F 56 AC 2F 37 6E 1C EE 0E BF 28 2D
            :           F4 39 90 6F 34 D8 6E 08 5B D5 65 6A D8 41 F3 13
            :           D7 2D 39 5E FE 33 CB FF 29 E4 03 0B 3D 05 A2 8F
            :           B7 F1 8E A2 76 37 B0 79 57 D3 2F 2B DE 87 06 22
            :           7D 04 66 5E C9 1B AF 8B 1A C3 EC 91 44 AB 7F 21
 301 02   65:         INTEGER
            :           00 F6 D6 E0 22 21 4C 5F 0A 70 FF 27 FC E5 B3 50
            :           6A 9D E5 0F B5 85 96 C6 40 FA A8 0A B4 9B 9B 0C
            :           55 C2 01 1D F9 37 82 8A 14 C8 F2 93 0E 92 CD A5
            :           66 21 B9 3C D2 06 BF B4 55 31 C9 DC AD CA 98 2D
            :           D1
 368 02   65:         INTEGER
            :           00 E8 DE B0 11 25 09 D2 02 51 01 DE 8A E8 98 50
            :           F5 77 77 61 A4 45 93 6B 08 55 96 73 5D F4 C8 5B
            :           12 93 22 73 8B 7F D3 70 7F F5 A4 AA BB 74 FD 3C
            :           22 6A DA 38 91 2A 86 5B 6C 14 E8 AE 4C 9E FA 8E
            :           2F
 435 02   65:         INTEGER
            :           00 97 4C F0 87 9B 17 7F EE 1B 83 1B 14 B6 0B 6A
            :           90 5F 86 27 51 E1 B7 A0 7F F5 E4 88 E3 59 B9 F9
            :           1E 9B D3 29 77 38 22 48 D7 22 B1 25 98 BA 3D 59
            :           53 B7 FA 1E 20 B2 C8 51 16 23 75 93 51 E7 AB CD
            :           F1
 502 02   64:         INTEGER
            :           2C F0 24 5B FA A0 CD 85 22 EA D0 6E 4F FA 6C CD
            :           21 D3 C8 E4 F1 84 44 48 64 73 D7 29 8F 7E 46 8C
            :           EC 15 DE E4 51 B3 94 E7 2C 99 2D 55 65 7B 24 EA
            :           A3 62 1F 3E 6C 4D 67 41 11 3B E1 BE E9 83 02 83
 568 02   64:         INTEGER
            :           58 88 D9 A1 50 38 84 6A AB 03 BC BB DF 4B F4 9C
            :           6F B8 B4 2A 25 FB F6 E4 05 2F 6E E2 88 89 21 6F
            :           4B 25 9E D0 AB 50 93 CA BF 40 71 EC 21 25 C5 7F
            :           FB 02 E9 21 96 B8 33 CD E2 C6 95 EE 6F 8D 5F 28
            :         }
            :       }
            :   }

BobPrivDHEncrypt =
   0 30  355: SEQUENCE {
   4 02    1:   INTEGER 0
   7 30  312:   SEQUENCE {
  11 06    7:     OBJECT IDENTIFIER dhPublicNumber (1 2 840 10046 2 1)
            :       (ANSI X9.42 number-type)
  20 30  299:     SEQUENCE {
  24 02  129:       INTEGER
            :         00 EC 2C CD A4 EF 9A 26 2F 62 A7 BB 23 4D DF 2B
            :         25 C1 68 D2 9E A9 45 5B 36 F1 94 89 1A AF 7D 11
            :         24 9D 3D B9 3C 29 E8 D7 23 80 33 A6 9E 45 02 BB
            :         AA CC 9E 28 05 95 A0 B3 17 76 C1 F7 25 35 61 02
            :         41 92 27 0C 5E AE 48 E5 F3 6E 38 EF 91 D1 CF 37
            :         FE 9A 40 97 C8 2D 35 9E 9D 93 C6 F8 15 AF 3F DA
            :         74 3A B7 C4 93 B5 B9 BB 76 6C 1F A8 7E BC 3A AA
            :         43 0A 81 64 FC 63 F0 7B 71 98 FA C0 38 79 10 1A
            :         33
 156 02  129:       INTEGER
            :         00 BA 0B D7 74 3D E7 34 E5 4C 13 A7 95 96 BB F1
            :         E4 61 37 08 FB 12 C7 FB 9C 91 77 06 99 35 F0 48
            :         24 96 33 12 01 7E 8D EC 0B F6 B2 C0 63 A7 15 C5
            :         5E 95 86 A2 73 C5 49 46 37 79 60 FD 77 05 09 48
            :         9B 70 8D 3C 05 F6 CE 44 2C 7F 7D 1B 2B 15 DD F3
            :         05 2F BE 85 20 8F 8D F9 B4 A0 45 74 2B F4 3B 9D
            :         42 62 34 27 27 81 8E 6F 0F 5E 62 85 89 CC ED 21
            :         C3 91 70 06 54 EE 70 A8 92 55 5B 6E 19 22 4D 62
            :         A7
 288 02   33:       INTEGER
            :         00 C3 AB 4A 30 79 B3 D3 97 4E CA F5 A2 7D C7 70
            :         A3 45 F3 B3 A2 86 05 D2 3E 49 F9 9F D9 0A B3 BE
            :         BD
            :       }
            :     }
 323 04   34:   OCTET STRING, encapsulates {
 325 02   32:       INTEGER
            :         20 FC 67 82 EE CF 4A A6 C8 E5 83 D2 8C 3B 8A D2
            :         45 32 11 27 32 6C 86 EC 66 CA 71 AD F0 19 4D F7
            :       }
            :   }

*/

enum {
	DSAKeyType		= (UINT('D') << 24) | (UINT('S') << 16) | (UINT('A') << 8),
	DSAPublicKey	=  DSAKeyType | UINT('P'),
	DSASecretKey	=  DSAKeyType | UINT('S')
};

struct sDSAPublicKey
{
	UINT type;
	int sizeInBits;

	pw_int p;	// Public prime
	pw_int g;	// Public generator
	pw_int q;	// Public order of generator
	pw_int y;	// Public value y, y = g^x mod p,  0 < y < p

	sDSAPublicKey()
	{
		type = DSAPublicKey;
	}

	~sDSAPublicKey()
	{
		memset( this, 0, sizeof( *this ));
	}
};

struct sDSASecretKey : public sDSAPublicKey
{
	pw_int x;		// Private value x, discrete log of y

	sDSASecretKey()
	{
		type = DSASecretKey;
	}

	~sDSASecretKey()
	{
		memset( this, 0, sizeof( *this ));
	}
};

int DSA_GetModulusBitSize( DSA_KeyHandle hKey )
{
	sDSAPublicKey *pk = (sDSAPublicKey *)hKey;

	if ( ( pk == NULL ) || ((pk->type != DSAPublicKey) && (pk->type != DSASecretKey)))
		return 0;

	return pw_getSizeInBits( &pk->p );
}


//
//RFC 2440                 OpenPGP Message Format            November 1998
//Callas, et. al.             Standards Track                    [Page 35]
//
//       Algorithm Specific Fields for DSA public keys:
//
//         - MPI of DSA prime p;
//
//         - MPI of DSA group order q (q is a prime divisor of p-1);
//
//         - MPI of DSA group generator g;
//
//         - MPI of DSA public key value y (= g**x where x is secret).
//

static int readPublicPart( sDSAPublicKey *pk, const BYTE *mpiBuf, int mpiBufSize )
{
	int pos = 0;

	if (
		!pw_readMpiBuffer( &(pk->p), mpiBuf, mpiBufSize, &pos ) ||
		!pw_readMpiBuffer( &(pk->q), mpiBuf, mpiBufSize, &pos ) ||
		!pw_readMpiBuffer( &(pk->g), mpiBuf, mpiBufSize, &pos ) ||
		!pw_readMpiBuffer( &(pk->y), mpiBuf, mpiBufSize, &pos )
	)
			return 0;

	return pos;
}

//
// Signing 
// New data r first 160 bits  + s second 160 bits
//
static int sign( sDSASecretKey *sk, pw_int *r, pw_int *s, pw_int *digest )
{
	int res;

	//
	//	Generate random k,
	//	a randomly or pseudorandomly generated integer with 0 < k < q
	//
	pw_int k;

	BYTE buffer[ DSA_SMALL_PRIME_BYTE_SIZE ];

	res = cRNG::generate( buffer, sizeof( buffer ) );		if ( res != pw_OK ) return res;

	res = pw_bytes2int( &k, buffer, sizeof( buffer ) );		if ( res != pw_OK ) return res;

	if ( pw_cmpu( &k, 0 ) == 0 )
		k.setu( 0x123456789abcdefLL );

	res = pw_mod( &k, &k, &sk->q );							if ( res != pw_OK ) return res;

	//
	//	Calculate kInv = k exp -1 mod q
	//
	pw_int kInv;
	res = pw_invert( &kInv, &k, &sk->q );					if ( res != pw_OK ) return res;


	//
	//	 Prepare r value 
	//
	res = pw_powm( r , &sk->g , &k , &sk->p );		  		if ( res != pw_OK ) return res;
	res = pw_mod( r , r , &sk->q );							if ( res != pw_OK ) return res;

	//
	//	Prepare s value 
	//
	res = pw_mul( s , r , &sk->x );							if ( res != pw_OK ) return res;
	res = pw_add( s , digest , s );							if ( res != pw_OK ) return res;
	res = pw_mulm( s , s , &kInv, &sk->q );					if ( res != pw_OK ) return res;

	return pw_OK;
}

//
// Verifying 
//
static int verify( sDSAPublicKey *pk, pw_int *r, pw_int *s, pw_int *digest )
{
	pw_int w,u1,u2,v;
	int res;

	//
	//	1) Prepare w = s exp -1 mod q;
	//
	res = pw_invert( &w, s , &pk->q );				if ( res != pw_OK ) return res;

	//
	//	2) u1 = (H(m) * w) mod q
	//
	res = pw_mulm( &u1 , digest, &w, &pk->q );		if ( res != pw_OK ) return res;

	//
	//	3) u2= (r * w) mod q
	//
	res = pw_mulm( &u2 , &w , r , &pk->q );		if ( res != pw_OK ) return res;

	//
	//	4) v = (( g^u1 * y^u2) mod p) mod q;
	//
	res = pw_powm( &u1 , &pk->g , &u1 , &pk->p);	if ( res != pw_OK ) return res;
	res = pw_powm( &u2 , &pk->y , &u2 , &pk->p);	if ( res != pw_OK ) return res;
	res = pw_mulm( &v , &u1 , &u2, &pk->p );		if ( res != pw_OK ) return res;
	res = pw_mod( &v , &v , &pk->q );				if ( res != pw_OK ) return res;

	//
	//	5) compare v and r
	//
	if ( pw_cmp( r , &v ) != 0 )
		return pw_InvalidKeyBlock;

	return pw_OK;
}

///////////////////////////////////////////////////////////////
//
//	Interface functions.
//

DSA_KeyHandle DSA_ReadPublicKeyFromMpiBuffer( const BYTE *mpi, int mpiSize )
{
	sDSAPublicKey *pk = new sDSAPublicKey;

	if ( pk == NULL )
		return NULL;

	if ( readPublicPart( pk, mpi, mpiSize ) == 0 )
	{
		delete pk;
		return NULL;
	}

	return pk;
}

DSA_KeyHandle DSA_ReadSecretKeyFromMpiBuffer( const BYTE *mpi, int mpiSize, const BYTE *secretMpi, int secretMpiSize )
{
	sDSAPublicKey *sk = new sDSASecretKey;

	if ( sk == NULL )
		return NULL;

	if ( readPublicPart( sk, mpi, mpiSize ) == 0 )
	{
		delete sk;
		return NULL;
	}
	int pos = 0;

	if (!pw_readMpiBuffer( &(((sDSASecretKey *)sk)->x), secretMpi, secretMpiSize, &pos ) )
	{
		delete sk;
		return NULL;
	}

	return sk;
}

void DSA_FreeKey( DSA_KeyHandle hKey )
{
	sDSAPublicKey *pk = (sDSAPublicKey *)hKey;

	if ( pk->type == DSAPublicKey )
		delete pk;
	else
	if ( pk->type == DSASecretKey )
	{
		sDSASecretKey *sk = (sDSASecretKey *)hKey;
		delete sk;
	}
}

//
//	Signature interface functions
//   Algorithm Specific Fields for DSA signatures:
//     - MPI of DSA value r.
//     - MPI of DSA value s.
//
//
int DSA_Verify(
	DSA_KeyHandle	hPKey,
	const BYTE *	sigMpiBuf,
	int				sigMpiBufSize,
	const BYTE *	digestBuf,
	int				digestBufSize
	)
{
	sDSAPublicKey *sk = (sDSAPublicKey *)hPKey;

	//
	//	Check key handle...
	//
	if ( ( sk == NULL ) || ( sk->type != DSAPublicKey ) )
		return pw_InvalidKeyBlock;

	pw_int r, s, digest;

	int pos = 0;

	if ( !pw_readMpiBuffer( &r, sigMpiBuf, sigMpiBufSize, &pos ) ||
		 !pw_readMpiBuffer( &s, sigMpiBuf, sigMpiBufSize, &pos ) )
			return pw_InternalError;

	int res = pw_bytes2int( &digest, digestBuf, digestBufSize );
	if ( res != pw_OK )	return res;

	return verify( sk, &r, &s, &digest );
}

int DSA_Sign(
	DSA_KeyHandle	hSKey,
	BYTE *			sigMpiBuf,
	int *			pSigMpiBufSize,
	const BYTE *	digestBuf,
	int				digestBufSize
	)
{
	int sizeRequired = (DSA_GetModulusBitSize(hSKey) + 7) / 8  + 2; // One Mpi

	sizeRequired *=2; // Two mpis r and s

	if ( *pSigMpiBufSize < sizeRequired )
	{
		*pSigMpiBufSize = sizeRequired;
		return pw_BufferSizeTooSmall;
	}

	sDSASecretKey *sk = (sDSASecretKey *)hSKey;

	//
	//	Check key handle...
	//
	if ( ( sk == NULL ) || ( sk->type != DSAPublicKey ) )
		return pw_InvalidKeyBlock;

	pw_int r,s,digest;

	int res = pw_bytes2int( &digest, digestBuf, digestBufSize );
	if ( res != pw_OK )	return res;

	res = sign( sk, &r, &s, &digest );
	if ( res != pw_OK )	return res;

	int pos = 0;

	if ( !pw_writeMpiBuffer( sigMpiBuf, *pSigMpiBufSize, &pos, &r ) ||
		 !pw_writeMpiBuffer( sigMpiBuf, *pSigMpiBufSize, &pos, &s ) )
			return pw_InternalError;

	*pSigMpiBufSize = pos;

	return pw_OK;
}

//
//	X509 ASN.1 package funcs.
//

//
//
//	RFC 2459        Internet X.509 Public Key Infrastructure
//	7.2.2  DSA Signature AlgorithmtUnit
//
//	Dss-Sig-Value  ::=  SEQUENCE  {
//         r       INTEGER,
//         s       INTEGER  }
//
int DSA_Verify_X509( DSA_KeyHandle hPKey, const BYTE *sigBuf, int   sigBufSize, const BYTE *digestBuf, int digestBufSize )
{
	sDSAPublicKey *sk = (sDSAPublicKey *)hPKey;

	//
	//	Check key handle...
	//
	if ( ( sk == NULL ) || ( sk->type != DSAPublicKey ) )
		return pw_InvalidKeyBlock;

	pw_int r, s, digest;

	x509_struct seq;

	int res = seq.parse( sigBuf, sigBufSize );
	if ( res != x509e_ok ) return res;

	int pos = 0;

	if ( !pw_readASN1Buffer( &r, seq.B, seq.L, &pos ) ||
		 !pw_readASN1Buffer( &s, seq.B, seq.L, &pos ) )
			return pw_InternalError;

	res = pw_bytes2int( &digest, digestBuf, digestBufSize );
	if ( res != pw_OK )	return res;

	return verify( sk, &r, &s, &digest );
}

int DSA_Sign_X509( DSA_KeyHandle hSKey, BYTE *sigBuf, int *pSigBufSize, const BYTE *digestBuf, int digestBufSize )
{
	//
	//	Check key handle...
	//
	sDSASecretKey *sk = (sDSASecretKey *)hSKey;

	if ( ( sk == NULL ) || ( sk->type != DSASecretKey ) )
		return pw_InvalidKeyBlock;

	//
	// The s and r are moduled by sk->q
	//	So maximum space will be reseved here and size of the structure will be defined without signing.
	//		the pw_writeASN1Buffer( sSeq.B, sSeq.L, &pos, &r, qBodySize ) below will write additional space by zero.
	//

	int qSize = pw_getASN1Size( &(sk->q) );
	int qBodySize = pw_getASN1BodySize( &(sk->q) );

	x509_struct sSeqTest;
	if ( !sSeqTest.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
					qSize + // &r ) +
					qSize ) //  &s ) )
		)
		return x509e_internalError;

	if ( *pSigBufSize < (int)sSeqTest.TL )
	{
		*pSigBufSize = sSeqTest.TL;
		return pw_BufferSizeTooSmall;
	}

	pw_int r,s,digest;

	int res = pw_bytes2int( &digest, digestBuf, digestBufSize );
	if ( res != pw_OK )	return res;

	res = sign( sk, &r, &s, &digest );
	if ( res != pw_OK )	return res;

	//
	//	Convert to sequence
	//

	x509_struct sSeq;

	if ( !sSeq.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
					+ 2 * qSize )
//					pw_getASN1Size( &r ) +
//					pw_getASN1Size( &s ) )
		)
		return x509e_internalError;

	if ( sSeqTest.TL != sSeq.TL )
	{
		assert( false );
		return x509e_internalError;
	}

	if ( *pSigBufSize < (int)sSeq.TL )
	{
		*pSigBufSize = sSeq.TL;
		return pw_BufferSizeTooSmall;
	}

	if ( !sSeq.writeHeader( sigBuf, sSeq.TL ) )
		return x509e_internalError;

	int pos = 0;

	if ( !pw_writeASN1Buffer( sSeq.B, sSeq.L, &pos, &r, qBodySize ) ||
		 !pw_writeASN1Buffer( sSeq.B, sSeq.L, &pos, &s, qBodySize ) )
			return pw_InternalError;

	*pSigBufSize = (int)sSeq.TL;

	return pw_OK;
}

//       id-dsa ID ::= { iso(1) member-body(2) us(840) x9-57(10040) x9cm(4) 1 }
static const char *algDSA_OID = "1.2.840.10040.4.1";

//
//
//	RFC 2459        Internet X.509 Public Key Infrastructure
//	7.3.3  DSA Signature Keys
//
// SubjectPublicKeyInfo  ::=  SEQUENCE  {
//     algorithm            AlgorithmIdentifier,
//     subjectPublicKey     BIT STRING
// }
//
//	AlgorithmIdentifier  ::=  SEQUENCE  {
//		algorithm               OBJECT IDENTIFIER,
//		parameters              ANY DEFINED BY algorithm OPTIONAL  }
//
//	Dss-Parms  ::=  SEQUENCE  {
//		p             INTEGER,
//		q             INTEGER,
//		g             INTEGER  }
//
//	If the DSA algorithm parameters are absent from the
//	subjectPublicKeyInfo AlgorithmIdentifier and the CA signed the
//	subject certificate using DSA, then the certificate issuer's DSA
//	parameters apply to the subject's DSA key.  If the DSA algorithm
//	parameters are absent from the subjectPublicKeyInfo
//
//	DSAPublicKey ::= INTEGER -- public key, Y // It is included to subjectPublicKey BIT STRING
//
int DSA_ReadPublicKeyFromASN1Buffer( DSA_KeyHandle *hKey, x509_publicKeyInfo *pki )
{
	int res = x509e_ok;

	switch ( pki->algOID.params.T & asn1tag_TYPE )
	{
		case asn1tag_SEQUENCE: break;
		case asn1tag_NULL:
			return x509e_emptyParameters; // An issuer parameters are required.
		default:
			return x509e_invalidPktTag;
	}

	sDSAPublicKey *key = new sDSAPublicKey;

	if( key == NULL )
		return x509e_memoryAllocation;

	int parsed = 0;

	if (!pw_readASN1Buffer( (&key->p), pki->algOID.params.B, pki->algOID.params.L, &parsed ) ||
		!pw_readASN1Buffer( (&key->q), pki->algOID.params.B, pki->algOID.params.L, &parsed ) ||
		!pw_readASN1Buffer( (&key->g), pki->algOID.params.B, pki->algOID.params.L, &parsed ) ||
		( pki->algOID.params.L != (UINT)parsed )
		)
			res = x509e_invalidPkt;
	else
	{
		parsed = 0;

		if ( !pw_readASN1Buffer( &(key->y), pki->publicKey.B, pki->publicKey.L, &parsed ) ||
		   ( (UINT)parsed != pki->publicKey.L )
		)
			res = x509e_invalidPkt;
	}

	if ( res != x509e_ok )
	{
		delete key;
		return res;
	}

	*hKey = key;

	return x509e_ok;
}

int DSA_WritePublicKeyToASN1Buffer(	DSA_KeyHandle hKey, BYTE *buff, int *pBuffSize )
{
	sDSAPublicKey *key = (sDSAPublicKey *)hKey;

	if ( ( key == NULL ) || ((key->type != DSAPublicKey) && (key->type != DSASecretKey)))
		return x509e_internalError;

	x509_publicKeyInfo pki;

	//
	//	Forms public key info structure and calculates required size
	//
	//	Dss-Parms  ::=  SEQUENCE  {
	//		p             INTEGER,
	//		q             INTEGER,
	//		g             INTEGER  }
	//
	int required = 
		pki.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
			pki.algOID.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
				pki.algOID.oid.prepareForWritting( algDSA_OID )
				+
				pki.algOID.params.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
					pw_getASN1Size( &(key->p) )
					+
					pw_getASN1Size( &(key->q) )
					+
					pw_getASN1Size( &(key->g) )
				)
			)
			+
			pki.publicKey.prepareForWritting( pw_getASN1Size( &(key->y) )
			)
		);

	if ( required > *pBuffSize )
	{
		*pBuffSize = required;
		return x509e_bufferTooSmall;
	}

	*pBuffSize = required;

	//
	//	Fills buffers.
	//
	if ( pki.writeHeaders( buff, required ) == 0 )
		return x509e_internalError;

	//
	//	Fills data.
	//
	int oidLen = pki.algOID.oid.L;
	if ( (UINT)string2oid( pki.algOID.oid.B, &oidLen, algDSA_OID ) != pki.algOID.oid.L )
		return x509e_internalError;

	int pos = 0;
	if ( !pw_writeASN1Buffer( pki.algOID.params.B, pki.algOID.params.L, &pos , &(key->p) ) ||
		 !pw_writeASN1Buffer( pki.algOID.params.B, pki.algOID.params.L, &pos , &(key->q) ) ||
		 !pw_writeASN1Buffer( pki.algOID.params.B, pki.algOID.params.L, &pos , &(key->g) ) ||
		 ( (UINT)pos != pki.algOID.params.L )
		 )
			return x509e_internalError;

	pos = 0;
	if ( !pw_writeASN1Buffer( pki.publicKey.B, pki.publicKey.L, &pos , &(key->y) )  )
			return x509e_internalError;

	return x509e_ok;
}

int DSA_ReadSecretKeyFromASN1Buffer( DSA_KeyHandle *hKey, x509_privateKeyInfo *pki )
{
	*hKey = NULL;

	if ( pki->algOID.tu->pAlg != pgppkaDSA )
		 return x509e_internalError;
		 
	int res = x509e_internalError;

	switch ( pki->algOID.params.T & asn1tag_TYPE )
	{
		case asn1tag_SEQUENCE: break;
		case asn1tag_NULL:
			return x509e_emptyParameters; // An issuer parameters are required.
		default:
			return x509e_invalidPktTag;
	}

	sDSASecretKey *key = new sDSASecretKey;

	if( key == NULL )
		return x509e_memoryAllocation;

	int parsed = 0;

	if (!pw_readASN1Buffer( (&key->p), pki->algOID.params.B, pki->algOID.params.L, &parsed ) ||
		!pw_readASN1Buffer( (&key->q), pki->algOID.params.B, pki->algOID.params.L, &parsed ) ||
		!pw_readASN1Buffer( (&key->g), pki->algOID.params.B, pki->algOID.params.L, &parsed ) ||
		( pki->algOID.params.L != (UINT)parsed )
		)
			res = x509e_invalidPkt;
	else
	{
		parsed = 0;

		if ( !pw_readASN1Buffer( &(key->x), pki->privateKey.B, pki->privateKey.L, &parsed ) ||
			( (UINT)parsed != pki->privateKey.L )
		)

		res = x509e_invalidPkt;
	}

	if ( res != x509e_ok )
	{
		delete key;
		return res;
	}

	// Public value y, y = g^x mod p,  0 < y < p
	pw_powm( &(key->y), &(key->g), &(key->x), &(key->p) );

	*hKey = key;

	return x509e_ok;
}

int DSA_WriteSecretKeyToASN1Buffer(	DSA_KeyHandle hKey, BYTE *buff, int *pBuffSize )
{
	sDSASecretKey *key = (sDSASecretKey *)hKey;

	if ( ( key == NULL ) || ( key->type != DSASecretKey ) )
		return x509e_internalError;

	x509_privateKeyInfo pki;

	pw_int version;
	version.setu( 0 );

	//
	//	Forms private key info structure and calculates required size
	//
	int required = 
		pki.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
			pki.version.prepareForWritting( 0 )
			+
			pki.algOID.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
				pki.algOID.oid.prepareForWritting( algDSA_OID )
				+
				pki.algOID.params.prepareForWritting( asn1tag_CONSTRUCTED_SEQUENCE,
					pw_getASN1Size( &(key->p) )
					+
					pw_getASN1Size( &(key->q) )
					+
					pw_getASN1Size( &(key->g) )
				)
			)
			+
			pki.privateKey.prepareForWritting( asn1tag_OCTET_STRING, pw_getASN1Size( &(key->x) ) )
			// No attributes...
		);

	if ( required > *pBuffSize )
	{
		*pBuffSize = required;
		return x509e_bufferTooSmall;
	}

	*pBuffSize = required;

	//
	//	Fills headers.
	//
	if ( pki.writeHeaders( buff, required ) == 0 )
		return x509e_internalError;

	//
	//	Fills data.
	//
	if ( pki.algOID.oid.writeBody() != x509e_ok ) 
		return x509e_internalError;

	if ( pki.version.writeBody() != x509e_ok )
		return x509e_internalError;

	int pos = 0;
	if ( !pw_writeASN1Buffer( pki.algOID.params.B, pki.algOID.params.L, &pos , &(key->p) ) ||
		 !pw_writeASN1Buffer( pki.algOID.params.B, pki.algOID.params.L, &pos , &(key->q) ) ||
		 !pw_writeASN1Buffer( pki.algOID.params.B, pki.algOID.params.L, &pos , &(key->g) ) ||
		 ( (UINT)pos != pki.algOID.params.L )
		 )
			return x509e_internalError;

	pos = 0;
	if ( !pw_writeASN1Buffer( pki.privateKey.B, pki.privateKey.L, &pos , &(key->x) )  )
			return x509e_internalError;

	return x509e_ok;
}

int DSA_DerivePublicKey( DSA_KeyHandle hSKA, DSA_KeyHandle *phPKA )
{
	sDSASecretKey *privKey = ( sDSASecretKey *)hSKA;
	if ( ( privKey == NULL ) || ( privKey->type != DSASecretKey ) )
		return x509e_internalError;

	sDSAPublicKey *key = new sDSAPublicKey;

	if ( key == NULL ) return pkae_memoryAllocation;

	key->sizeInBits = privKey->sizeInBits;

	pw_set( &(key->p), &(privKey->p) );	// Public prime
	pw_set( &(key->g), &(privKey->g) );	// Public generator
	pw_set( &(key->q), &(privKey->q) );	// Public order of generator

	// Public value y, y = g^x mod p,  0 < y < p

	if ( ( privKey->y.data == NULL ) || ( pw_cmpu( &(privKey->y), 0 ) == 0 ) )
		pw_powm( &(privKey->y), &(privKey->g), &(privKey->x), &(privKey->p) );

	pw_set( &(key->y), &(privKey->y) );

	*phPKA = (DSA_KeyHandle)key;

	return pkae_ok;
}
