#ifndef __DSA_H__
#define __DSA_H__

#include "../../x509pkts/x509_structs.h"

#define DSA_SMALL_PRIME_BIT_LENGTH 160	/* Nist predefined small prime size	*/
#define DSA_SMALL_PRIME_BYTE_SIZE (DSA_SMALL_PRIME_BIT_LENGTH / 8)

typedef void * DSA_KeyHandle;
void DSA_FreeKey( DSA_KeyHandle hKey );
int  DSA_GetModulusBitSize( DSA_KeyHandle hKey );

//
//	Key management functions.
//
//	PGP MPI package funcs
//
DSA_KeyHandle DSA_ReadSecretKeyFromMpiBuffer( const BYTE *mpi, int mpiSize, const BYTE *secretMpi, int secretMpiSize );
DSA_KeyHandle DSA_ReadPublicKeyFromMpiBuffer( const BYTE *mpi, int mpiSize );

//
//	X509 ASN.1 package funcs.
//

int DSA_ReadSecretKeyFromASN1Buffer( DSA_KeyHandle *hKey, x509_privateKeyInfo *pki );
int DSA_ReadPublicKeyFromASN1Buffer( DSA_KeyHandle *hKey, x509_publicKeyInfo *pki );
int DSA_WritePublicKeyToASN1Buffer(	DSA_KeyHandle hKey, BYTE *buff, int *pBuffSize );
int DSA_WriteSecretKeyToASN1Buffer(	DSA_KeyHandle hKey, BYTE *buff, int *pBuffSize );
int DSA_DerivePublicKey( DSA_KeyHandle hSKA, DSA_KeyHandle *phPKA );


//
//	Signature functions.
//
int DSA_Verify( DSA_KeyHandle hPKey, const BYTE *sigMpiBuf, int   sigMpiBufSize, const BYTE *digestBuf, int digestBufSize );
int DSA_Sign  ( DSA_KeyHandle hSKey,       BYTE *sigMpiBuf, int *pSigMpiBufSize, const BYTE *digestBuf, int digestBufSize );

//
//
//	RFC 2459        Internet X.509 Public Key Infrastructure
//	7.2.2  DSA Signature AlgorithmtUnit
//
//	Dss-Sig-Value  ::=  SEQUENCE  {
//         r       INTEGER,
//         s       INTEGER  }
//

int DSA_Verify_X509( DSA_KeyHandle hPKey, const BYTE *sigBuf, int   sigBufSize, const BYTE *digestBuf, int digestBufSize );
int DSA_Sign_X509  ( DSA_KeyHandle hSKey,       BYTE *sigBuf, int *pSigBufSize, const BYTE *digestBuf, int digestBufSize );

#endif /* __DSA_H__ */
