#ifndef __ElGamal__
#define __ElGamal__

#include "../../x509pkts/x509_structs.h"
typedef void * ElGamal_KeyHandle;

//
//	Key management functions.
//


ElGamal_KeyHandle ElGamal_GenerateKey( int keyBitSize );
ElGamal_KeyHandle ElGamal_DerivePublicKey( ElGamal_KeyHandle hSecKey );

ElGamal_KeyHandle ElGamal_ReadSecretKey( BYTE *buffer, int bufferSize );
ElGamal_KeyHandle ElGamal_ReadPublicKey( BYTE *buffer, int bufferSize );

ElGamal_KeyHandle ElGamal_ReadSecretKeyFromMpiBuffer( const BYTE *mpi, int mpiSize, const BYTE *secretMpi, int secretMpiSize );
ElGamal_KeyHandle ElGamal_ReadPublicKeyFromMpiBuffer( const BYTE *mpi, int mpiSize );

void ElGamal_FreeKey( ElGamal_KeyHandle hKey );

int ElGamal_GetModulusBitSize( ElGamal_KeyHandle hKey );

//
//	Encryption functions.
//
int ElGamal_Encrypt( ElGamal_KeyHandle hPKey, BYTE *outMpiBuf, int *pOutMpiBufSize, const BYTE *inBuf, int inBufSize );
int ElGamal_Decrypt( ElGamal_KeyHandle hSKey, BYTE *outBuf, int *pOutBufSize, const BYTE *inMPIBuf, int inMpiBufSize );

//
//	Signature functions.
//
//	Function: ElGamal_Verify
//		Always returns an error code.
//		If signature is invalid returns pw_InvalidKeyBlock
//		If signature is valid return pw_OK
//
int ElGamal_Verify( ElGamal_KeyHandle hPKey, const BYTE *sigMpiBuf, int   sigMpiBufSize, const BYTE *digestBuf, int digestBufSize );
int ElGamal_Sign  ( ElGamal_KeyHandle hSKey,       BYTE *sigMpiBuf, int *pSigMpiBufSize, const BYTE *digestBuf, int digestBufSize );

//
//	X.509 specific functions
//

int ElGamal_ReadPublicKeyFromASN1Buffer( ElGamal_KeyHandle *hk, x509_publicKeyInfo *pki );
int ElGamal_ReadSecretKeyFromASN1Buffer( ElGamal_KeyHandle *hk, x509_privateKeyInfo *pki );
int ElGamal_WritePublicKeyToASN1Buffer(	ElGamal_KeyHandle hKey, BYTE *buff, int *pBuffSize );
int ElGamal_WriteSecretKeyToASN1Buffer(	ElGamal_KeyHandle hKey, BYTE *buff, int *pBuffSize );

int ElGamal_Encrypt_X509( ElGamal_KeyHandle hPKey, BYTE *outBuf, int *pOutBufSize, const BYTE *inBuf, int inBufSize );
int ElGamal_Decrypt_X509( ElGamal_KeyHandle hSKey, BYTE *outBuf, int *pOutBufSize, const BYTE *inBuf, int inBufSize );

int ElGamal_Verify_X509( ElGamal_KeyHandle hPKey, const BYTE *sigBuf, int   sigBufSize, const BYTE *digestBuf, int digestBufSize );
int ElGamal_Sign_X509  ( ElGamal_KeyHandle hSKey,       BYTE *sigBuf, int *pSigBufSize, const BYTE *digestBuf, int digestBufSize );

#endif // __ElGamal__
