#include "../../x509pkts/x509_structs.h"
#include "Windefs.h"
#include "power.h"
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef _ODS
#include "../../../Includes/ODS.h"
#endif

#ifdef _UseAsm_

#define DWLMul DWLMulAsm
#define pw_mul_def pw_mul_asm

#else

#define DWLMul DWLMulClassic
#define pw_mul_def pw_mul_classic

#endif // _UseAsm_
//
//	Standard pw_mul calls pw_mul_rec first...
//	pw_mul_rec calls pw_mul_classic when limb count acceed 
//	the OPTIMAL_CLASSIC_MUL_LIMB_COUNT
//
#define OPTIMAL_CLASSIC_MUL_LIMB_COUNT 0x10

class cBLETest
{
public:
	cBLETest()
	{
#ifdef _DEBUG
		ULONGLONG uli = 0x3333222211110000LL;
		assert( UINT( uli ) == 0x11110000L );
		assert( _HIUINT( uli ) == 0x33332222L );
		assert( LOUINT( uli ) == 0x11110000L );
		assert( HIUINT( uli ) == 0x33332222L );
#endif
	}
};
static cBLETest bigLettleEndianTest;

/*
static void pw_printf( FILE *f, const char *name, ULONGLONG x )
{
	fprintf( f, "%s %08X::%08X \r\n", name,
		HIUINT(x),
		LOUINT(x)
	);
}

static void pw_printf( FILE *f, const char *name, pw_int *x )
{
	fprintf( f, "%s %d\r\n", name, x->m_ullSize );
	for( int i = 0; i < x->m_ullSize; i++ )
#ifdef _WIN32
		fprintf( f, "(%d) %I6408X,", i, x->data[i] );
#else
		fprintf( f, "(%d) %08llX,", i, x->data[i] );
#endif
	fprintf( f, "\r\n" );
}
*/
//
//	MSVC Optimization is failed with shift ULONGLONG on 32
//
#ifdef _useHIUINTProc
UINT HIUINT( ULONGLONG X )
{
	return DWHi( &X );
}
#endif


//
// pw_getLastLimb returns zero based position of last nonzero limb
//
int pw_getLastLimb( const pw_int *i )
{
	int last = i->m_ullSize;

	if ( last == 0 )
		return 0;

	for ( ; --last; ) 
	{
		if ( i->data[ last ] != 0 )
			break;
	}

	return last;
}
/*
static WORD _dwlLastBit( ULONGLONG *x )
{
	UINT last = DWHi( x );

	WORD cnt = 0;

	if ( last != 0 )
		cnt = sizeof( UINT ) * 8;
	else
		last = DWLo( x ); // LOUINT( *x );

	for( ; last; )
	{
		cnt++;
		last /= 2;
	}

	return cnt;
}
*/
//static inline WORD __dwlLastBit( ULONGLONG *x )
//{
//	UINT last = DWHi( x );
//
//	WORD w, cnt = 0;
//
//	if ( last != 0 )
//	{
//		if ( last >= ( 1 << ( sizeof( WORD ) *  8 )) )
//		{
//			w = HIWORD( last );
//			cnt = ( sizeof( UINT ) + sizeof( WORD ) ) *  8 ;
//		}
//		else
//		{
//			w = LOWORD( last );
//			cnt = sizeof( UINT ) * 8;
//		}
//	}
//	else
//	{	last = LOUINT( *x );
//		
//		if ( last >= ( 1 << ( sizeof( WORD )  *  8 )) )
//		{
//			w = HIWORD( last );
//			cnt = sizeof( WORD ) *  8 ;
//		}
//		else
//		{
//			w = LOWORD( last );
//		}
//	}
//
//	for( ; w; )
//	{
//		cnt++;
//		w /= 2;
//	}
//
//	return cnt;
//}

static WORD dwlLastBit( ULONGLONG *x )
{
	UINT last = DWHi( x );

	register WORD w;
	register WORD cnt;

	if ( last != 0 )
	{
		w = HIWORD( last );

		if ( w )
		{
			if ( HIBYTE( w ) )
			{
				w = HIBYTE( w );
				cnt = ( sizeof( UINT ) + sizeof( WORD ) + sizeof( BYTE ) ) *  8 ;
			}
			else
				cnt = ( sizeof( UINT ) + sizeof( WORD ) ) *  8 ;
		}
		else
		{
			w = LOWORD( last );

			if ( HIBYTE( w ) )
			{
				w = HIBYTE( w );
				cnt = ( sizeof( UINT ) + sizeof( BYTE ) )* 8;
			}
			else
				cnt = sizeof( UINT ) * 8;
		}
	}
	else
	{
		last = DWLo( x ); // LOUINT( *x );
		w = HIWORD( last );
		
		if ( w )
		{
			if ( HIBYTE( w ) )
			{
				w = HIBYTE( w );
				cnt = ( sizeof( WORD ) + sizeof( BYTE ) ) *  8 ;
			}
			else
				cnt = sizeof( WORD ) *  8 ;
		}
		else
		{
			w = LOWORD( last );
			if ( HIBYTE( w ) )
			{
				w = HIBYTE( w );
				cnt = sizeof( BYTE ) *  8 ;
			}
			else
				cnt = 0;
		}
	}

	for( ; w; )
	{
		cnt++;
		w /= 2;
	}

	return cnt;
}

//
//	Function:
//		pw_getLastBit
//	returns one based position of last bit
//		0 if x == zero
//		1 if x == 1
//
int pw_getLastBit( const pw_int *x )
{
	if ( x->m_ullSize == 0 )
		return 0;

	int last = pw_getLastLimb( x );
	int lastBit = last * U64_BIT_Q;
/*
	for( ULONGLONG dwlLast = x->data[ last ]; dwlLast;  )
	{
		lastBit++;
		dwlLast /= 2;
	}

	return lastBit;
*/
	return lastBit + dwlLastBit( &x->data[ last ] );
}
int pw_getSizeInBits( const pw_int *x )
{
	return pw_getLastBit( x );
}

int pw_setBit( pw_int *t, int bitNum )
{
	int pos = bitNum / U64_BIT_Q;

	int bitOffset = ( bitNum % U64_BIT_Q );
	ULONGLONG mask = ((ULONGLONG)1) << bitOffset;

	if ( pos >= t->m_ullSize )
	{
		pw_int tmp;
		int res = tmp.set( ( pos + 1 )* sizeof( ULONGLONG ), NULL );
		if ( res != pw_OK )
			return res;

		memcpy( tmp.data,  t->data, t->m_ullSize * sizeof( ULONGLONG ) );
		tmp.data[ pos ] |= mask;
		return t->set( &tmp );
	}
	else
		t->data[ pos ] |= mask;

	return pw_OK;
}

int pw_clearBit( pw_int *t, int bitNum )
{
	int pos = bitNum / U64_BIT_Q;

	if ( pos >= t->m_ullSize )
		return pw_OK; // The bit is really zero.

	int bitOffset = ( bitNum % U64_BIT_Q );

	t->data[ pos ] &= ~(((ULONGLONG)1) << bitOffset);

	return pw_OK;
}

int pw_set( pw_int *x, pw_int *y )
{
	return x->set( y );
}

int pw_cmp( pw_int *x, pw_int *y )
{
	if ( x->data == NULL )
	{
		assert( 0 );
//		printf("\npw_cmp: X data NULL\n");
	}

	if ( y->data == NULL )
	{
		assert( 0 );
//		printf("\npw_cmp: Y data NULL\n");
	}


	int lastX = pw_getLastLimb( x );
	int lastY = pw_getLastLimb( y );

	if ( lastX != lastY )
		return lastX - lastY;

	for ( int last = lastX + 1; last--; )
	{
/*
		if ( x->data[ last ] < y->data[ last ] )
			return -1;

		if ( x->data[ last ] > y->data[ last ] )
			return 1;
*/
		if ( x->data[ last ] != y->data[ last ] )
			return x->data[ last ] < y->data[ last ] ? -1 : 1;
	}

	return 0;
}

int pw_cmpu( pw_int *x, ULONGLONG y )
{
	assert( x->data );

	int lastX = pw_getLastLimb( x );

	if ( lastX >= 1 )
		return 1;

	if ( x->data[ 0 ] == y )
		return 0;

	return x->data[ 0 ] < y ? -1 : 1;
}

static int DWLAddCarry( ULONGLONG *r, ULONGLONG &x, BOOL carryIn = 0, BOOL *pCarryOut = NULL)
{
	ULONGLONG res = x + carryIn;
	if ( pCarryOut )
	 *pCarryOut = res < x;

	*r = res;

	return pw_OK;
}

static int DWLAdd( ULONGLONG *r, ULONGLONG &x, ULONGLONG &y, BOOL carryIn = 0, BOOL *pCarryOut = NULL)
{
	 ULONGLONG res = x + y;
	 if ( pCarryOut )
		 *pCarryOut = ( res < x ) && ( res < y );

	 if ( carryIn )
	 {
		 res++;
		 if ( pCarryOut && !*pCarryOut )
			 *pCarryOut = ( res < 1 );
	 }

	 *r = res;

	 return pw_OK;
}

static ULONGLONG DWLSub( ULONGLONG x, ULONGLONG y, BOOL carryIn = 0, BOOL *pCarryOut = NULL)
{
	 ULONGLONG res = x - y;
	 if ( pCarryOut )
		 *pCarryOut = ( x < y );

	 if ( carryIn )
	 {
		 x = res;
		 res = x - 1;
		 if ( pCarryOut && !*pCarryOut )
			 *pCarryOut = ( x < 1 );
	 }

	 return res;
}

#ifndef _UseAsm_

static UINT DWAdd( UINT x, UINT y, BOOL carryIn = 0, BOOL *pCarryOut = NULL)
{
	 UINT res = x + y;
	 if ( pCarryOut )
		 *pCarryOut = ( res < x ) && ( res < y );

	 if ( carryIn )
	 {
		 x = res;
		 res = x + 1;
		 if ( pCarryOut )
			 *pCarryOut = ( res < x ) && ( res < 1 );
	 }

	 return res;
}

//
// The UInt32x32To64 function multiplies two unsigned 32-bit integers, returning an unsigned 64-bit integer result. 
//	The function performs optimally on 32-bit Windows.
//
static void DWLMulClassic( ULONGLONG *r0, ULONGLONG *r1, ULONGLONG &x, ULONGLONG &y )
{
	UL_INTEGER lx, ly, lr0, lr1, lMid;
	lx.QuadPart = x;
	ly.QuadPart = y;

	ULONGLONG m0, m1;

#ifdef _WIN32
// Makes low result
	lr0.QuadPart = UInt32x32To64( lx.Uint[bleLoPos], ly.Uint[bleLoPos] );
// Makes high result
	lr1.QuadPart = UInt32x32To64( lx.Uint[bleHiPos], ly.Uint[bleHiPos] );

// Makes miadle result. 

	m0 = UInt32x32To64( lx.Uint[bleLoPos], ly.Uint[bleHiPos] );
	m1 = UInt32x32To64( lx.Uint[bleHiPos], ly.Uint[bleLoPos] );

#else
// Makes low result
	lr0.QuadPart = (ULONGLONG)lx.Uint[bleLoPos] * ly.Uint[bleLoPos];
// Makes high result
	lr1.QuadPart = (ULONGLONG)lx.Uint[bleHiPos] * ly.Uint[bleHiPos];

// Makes miadle result. 

	m0 = (ULONGLONG)lx.Uint[bleLoPos]  * ly.Uint[bleHiPos];
	m1 = (ULONGLONG)lx.Uint[bleHiPos] * ly.Uint[bleLoPos];
#endif

	BOOL carry = 0;
	DWLAdd( &lMid.QuadPart,	m0,	m1,	0, &carry );

	if ( carry )
		lr1.Uint[bleHiPos] += 1;

// Adds middle low part
	lr0.Uint[bleHiPos] = DWAdd( lr0.Uint[bleHiPos], lMid.Uint[bleLoPos], 0, &carry );

	if ( carry )
		lr1.QuadPart += 1;

// Adds middle high part
	lr1.QuadPart += lMid.Uint[bleHiPos];

	*r0 = lr0.QuadPart;
	*r1 = lr1.QuadPart;
}

#else //  _UseAsm_

void mul_UINT_arrays( UINT *r, UINT *x, WORD sizeX, UINT *y, WORD sizeY )
{
	__asm{
		pushad
loopX:
		mov eax, x
		mov edi, DWORD PTR [ eax ] 
		mov	esi, y
		mov	ebx, r
		mov	cx, sizeY
		push ebp

		xor ebp, ebp
loopY:

		mov	eax, edi

		mul DWORD PTR [ esi ]

		add	DWORD PTR [ ebx ], eax
		adc	DWORD PTR [ ebx + 4], edx

		mov	eax, 0			; // Get C1 in al
		adc	al, al

		add	DWORD PTR [ ebx + 4], ebp	// Add C0

		adc al, ah			; // Get C1 total C1 in bl

		mov ebp, eax

		add ebx,4
		add esi,4

		dec cx
		jnz	loopY

		pop ebp

		add	x, 4
		add	r, 4

		dec sizeX
		jnz	loopX

		popad
	}
}
void inline DWMul( UINT *prh, UINT *prl, UINT *px, UINT *py )
{
	__asm {
		mov	eax, DWORD PTR [px]
		mul DWORD PTR [py]
		mov	DWORD PTR [prh], edx
		mov	DWORD PTR [prl], eax
	}
}
/*
#define DWMulDef( prh, prl, px, py )\
	__asm {\
		mov	eax, DWORD PTR [px]\
		mul DWORD PTR [py]\
		mov	DWORD PTR [prh], edx\
		mov	DWORD PTR [prl], eax\
	}
*/
void DWLMulAsm( ULONGLONG *r0, ULONGLONG *r1, ULONGLONG &x, ULONGLONG &y )
{
	UINT xh, xl, yh, yl, r1h, r1l, r0h, r0l, mh, ml;

	xh = _HIUINT( x ); // *((UINT*)(&x) + 1 );
	xl = (UINT)( x );

	yh = _HIUINT( y ); // *((UINT*)(&y) + 1 );
	yl = (UINT)( y );

	__asm {

// Makes low result
//	lr0.QuadPart = (ULONGLONG)lx.Uint[bleLoPos]  * ly.Uint[bleLoPos];

		mov	eax, xl
		mul yl
		mov r0h, edx
		mov	r0l, eax

// Makes high result
//	lr1.QuadPart = (ULONGLONG)lx.Uint[bleHiPos] * ly.Uint[bleHiPos];

		mov	eax, xh
		mul yh
		mov	r1h, edx
		mov	r1l, eax

// Makes miadle result. 

		mov	eax, xl
		mul yh
		mov	mh, edx
		mov	ml, eax
		mov	eax, xh
		mul yl
		add	ml, eax
		adc	mh, edx
		adc	r1h, 0

// Adds miadle low part

		mov eax, ml
		add	r0h, eax

// Adds miadle high part

		mov eax, mh
		adc	r1l, eax
		adc	r1h, 0
	}

//	*r0 = lr0.QuadPart;
//	*r1 = lr1.QuadPart;
	*r0 = (ULONGLONG( r0h ) << 32 )| r0l;
	*r1 = (ULONGLONG( r1h ) << 32 )| r1l;
}

/*
static div_asm( 
		UINT *q, // 2 UINT result
		UINT *r, // 2 UINT reminder
		UINT *x, // 4 UINT devision
		UINT *y  // 2 UINT devider
		)
{
//
//  x3 x2 x1 x0 | y1 y0
//	 q1*y1y0   	 ------
//   --------	| q1 q0
//     x2'x1'x0
//      q0*y1y0
//       ------
//		  r1 r0

	__asm{
		pushad

		mov esi, y

		mov eax, DWORD PTR [ esi + 4 ]	;// y1


		mov edi, x
		mov edx, DWORD PTR [ edi + 0Ch] ;// x3
		mov eax, DWORD PTR [ edi + 08h] ;// x2

		div DWORD PTR [ esi + 4 ]		;// y1

		// pre q1



		popad
	}
}
*/

/*

  AMS here is too slow

void add_UINT_array_asm( UINT *y, WORD sizeY, UINT *x, WORD sizeX )
{
	__asm{
		pushad

		mov	edi, x
		mov	esi, y
		xor ecx, ecx
		mov	cx, sizeX
		xor eax, eax		; // carry 1
		xor ebx, ebx		; // carry 0
loopX:
		add ebx,  DWORD PTR [ edi ]
		adc al, ah

		add	DWORD PTR [ esi ], ebx
		adc al, ah

		mov ebx, eax
		xor al, al

		add esi,4
		add edi,4

		loopnz loopX

		mov	cx, sizeY
		sub cx, sizeX
loopY:
		cmp bl, 0
		je	end

		add	DWORD PTR [ esi ], ebx
		adc al, ah

		mov ebx, eax
		xor al, al

		add esi,4

		loopnz loopY
end:
		popad
	}

}

int pw_add_asm( pw_int *r, pw_int *px, pw_int *py )
{
	assert( px->data && py->data );

	int dwSizeX = pw_getSizeInBits( px );
	int dwSizeY = pw_getSizeInBits( py );

	pw_int *x, *y; // lets x < y

	if ( dwSizeX > dwSizeY )
	{
		y = px;
		x = py;

		int size = dwSizeY;
		dwSizeY = dwSizeX;
		dwSizeX = size;
	}
	else
	{
		x = px;
		y = py;
	}


	if ( dwSizeX == 0 )	return r->set( py );
	if ( dwSizeY == 0 )	return r->set( px );

	dwSizeX = ( dwSizeX + 31 ) / 32;
	dwSizeY = ( dwSizeY + 31 ) / 32;

	pw_int tmp;
	int res = tmp.set( ( dwSizeY + 1 ) * sizeof( UINT ) );
	if ( res != pw_OK ) 	return res;
	res = tmp.set( y );	// Copy y -> tmp;
	if ( res != pw_OK ) 	return res;

	add_UINT_array_asm( (UINT *)tmp.data, dwSizeY + 1, (UINT *)x->data, dwSizeX );

#ifdef _DEBUG
	res = pw_add_classic( r, x, y );
	if ( res != pw_OK ) 	return res;

	if ( pw_cmp( r, &tmp ) != 0 )
		return pw_InternalError;

#endif // _DEBUG

	return r->set( &tmp );
}
*/

int pw_square_asm( pw_int *res, pw_int *x )
{
	int last = pw_getLastLimb( x );

	pw_int tmp;

	int result = tmp.set( ( 2 * last + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )	return result;

	WORD dwSize;

	dwSize = (WORD)( ( pw_getSizeInBits( x ) + 31 ) / 32 );

	if ( dwSize == 0 )
		return res->setu( 0 );

	mul_UINT_arrays( (UINT *)tmp.data, (UINT *)x->data, dwSize, (UINT *)x->data, dwSize );

#ifdef _DEBUG

	result = pw_mul_classic( res, x, x ); 
	if ( result != pw_OK )	return result;

	if ( pw_cmp( res, &tmp ) != 0 )
		return pw_InternalError;

#endif // _DEBUG

	return res->set( &tmp );
}

int pw_mul_asm( pw_int *res, pw_int *px, pw_int *py )
{

	int lastX = pw_getLastLimb( px );
	int lastY = pw_getLastLimb( py );

	pw_int *x,*y;

	//
	//	If order y > order x => we get optimisation for y loop.
	//	otherwise we swap y with x.
	//
	if ( lastX > lastY )
	{	x = py;
		y = px;
		int tmp = lastX;
		lastX = lastY;
		lastY = tmp;
	}
	else
	{
		x = px;
		y = py;
	}

	pw_int tmp;

	int result = tmp.set( ( lastX + lastY + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )	return result;

	WORD dwSizeX, dwSizeY;

	dwSizeX = WORD( ( pw_getSizeInBits( x ) + 31 ) / 32 );
	dwSizeY = WORD( ( pw_getSizeInBits( y ) + 31 ) / 32 );

	if (( dwSizeX == 0 ) || ( dwSizeY == 0 ))
		return res->setu( 0 );

	mul_UINT_arrays( (UINT *)tmp.data, (UINT *)x->data, dwSizeX, (UINT *)y->data, dwSizeY );

#ifdef _DEBUG

	result = pw_mul_classic( res, x, y ); 
	if ( result != pw_OK )	return result;

	if ( pw_cmp( res, &tmp ) != 0 )
		return pw_InternalError;

#endif // _DEBUG

	return res->set( &tmp );
}

#endif // _UseAsm_

int pw_sub( pw_int *res, pw_int *x, pw_int *y )
{
	if ( pw_cmp( x, y ) < 0 ) // The implementation based on X > Y
	{
		assert( false );
		return pw_InternalError;
	}

	pw_int tmp;

	int result = tmp.set( x->m_ullSize * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	int lastX = pw_getLastLimb( x );
	int lastY = pw_getLastLimb( y );

	int pos = 0;
	int carry = 0;

	for(; pos <= lastY; pos++ )
		tmp.data[ pos ] = DWLSub( x->data[ pos ], y->data[ pos ], carry, &carry );

	for(; pos <= lastX; pos++ )
		tmp.data[ pos ] = DWLSub( x->data[ pos ], 0, carry, &carry );

	return res->set( &tmp );
}

int pw_subu( pw_int *res, pw_int *x, ULONGLONG y )
{
	if ( pw_cmpu( x, y ) < 0 ) // The implementation based on X > Y
	{
		assert( false );
		return pw_InternalError;
	}

	pw_int tmp;

	int result = tmp.set( x->m_ullSize * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	int lastX = pw_getLastLimb( x );

	int carry = 0;

	tmp.data[ 0 ] = DWLSub( x->data[ 0 ], y, carry, &carry );

	int pos = 1;
	for( ; (pos <= lastX) ; pos++ )
		tmp.data[ pos ] = DWLSub( x->data[ pos ], 0, carry, &carry );

	return res->set( &tmp );
}


int pw_subm( pw_int *res, pw_int *x, pw_int *y, pw_int *mod )
{
	pw_int tmp;
	pw_int lx;
	pw_int ly;

	int result;

	result = pw_mod( &lx, x, mod );
	if ( result != pw_OK )
		return result;

	result = pw_mod( &ly, y, mod );
	if ( result != pw_OK )
		return result;

	int sig = pw_cmp( &lx, &ly );

	if ( sig == 0 )
		return res->setu( 0 );

	if ( sig < 0 )
	{
		result = pw_add( &lx, &lx, mod ); // lx now larger then ly.
		if ( result != pw_OK )
			return result;
	}

	result = pw_sub( &tmp, &lx, &ly );
	if ( result != pw_OK )
		return result;

	return pw_mod( res, &tmp, mod );
}

int pw_mul_exp2( pw_int *res, pw_int *x, int exp )
{
	if ( exp == 0 )
		return res->set( x );

	int lastBit = pw_getLastBit( x );
	lastBit += exp;
	if ( lastBit <= 0 )
	{
		res->clear();
		return pw_OK;
	}

	int newSize = lastBit / ( U64_BIT_Q );

	if ( lastBit % U64_BIT_Q )
		newSize++;

	pw_int tmp;

	int result = tmp.set( newSize * sizeof( ULONGLONG ));
	if ( result != pw_OK )
		return result;

	int offsetBit = exp % U64_BIT_Q;
	int offset = exp / U64_BIT_Q;

	int xSize = x->m_ullSize;
	int pos;

	//////////////////////////////////////////////
	// Sample with 8 bit limb.
	// exp = 11
	// offset = 1
	// offsetBit = 3

	for( pos = xSize; pos-- ; )
	{
	// 1) [0].01234567 -> [2].00000012
		if ( offsetBit != 0 ) // All data will be copied by next step
			tmp.data[ pos + offset + 1 ] |= 
					x->data[ pos ] >> ( U64_BIT_Q - offsetBit);
	// 2) [0].01234567 -> [1].34567000
		tmp.data[ pos + offset ] |= x->data[ pos ] << offsetBit;
	}


	result = res->set( &tmp );

	return result;
}

int pw_div_exp2( pw_int *res, pw_int *x, int exp )
{
	if ( exp == 0 )
		return res->set( x );

	int lastBit = pw_getLastBit( x );

	if ( lastBit <= exp  )
	{
		res->clear();
		return pw_OK;
	}

//	int xSize = ( lastBit + U64_BIT_Q - 1 ) / U64_BIT_Q;

	int newSize = ( lastBit - exp + U64_BIT_Q - 1 ) / U64_BIT_Q;

	pw_int tmp;

	int result = tmp.set( newSize * sizeof( ULONGLONG ));
	if ( result != pw_OK )
		return result;

	int offsetBit = exp % U64_BIT_Q;
	int offset = exp / U64_BIT_Q;

	int pos;

	//////////////////////////////////////////////
	// Sample with 8 bit limb.
	// exp = 11
	// offset = 1
	// offsetBit = 3

	for( pos = newSize; pos-- ; )
	{
	// 1) [2].01234567 -> [0].56700000
		if ( offsetBit != 0 ) // All data will be copied by next step
			tmp.data[ pos ] |= 
					x->data[ pos + offset + 1 ] << ( U64_BIT_Q - offsetBit);
	// 2) [2].01234567 -> [1].00001234
		tmp.data[ pos ] |= x->data[ pos + offset ] >> offsetBit;
	}


	result = res->set( &tmp );

	return result;
}

int pw_mod( pw_int *res, pw_int *x, pw_int *mod ) 
{
	return pw_div( NULL, res, x, mod );
}

int pw_modOld( pw_int *res, pw_int *x, pw_int *mod ) 
{
	int result;

//	if ( pw_cmp( x, mod ) == - 1 )
	if ( pw_cmp( x, mod ) < 0 )
		return	res->set( x );

	pw_int tmpX,maxM;


	result = tmpX.set( x );
	if ( result != pw_OK )
		return result;

	do
	{
		// Get maximal exponent 2 of module which is less then tmpX

		int exp = pw_getLastBit( &tmpX ) - pw_getLastBit( mod );

		if ( exp < 0 )
//			return res->set();
			return pw_InternalError;

		if ( exp != 0 )
			result = pw_mul_exp2( &maxM, mod, exp - 1 );
		else
			result = maxM.set( mod );

		if ( result != pw_OK )
			return result;

		// tmpX = tmpX - maxM; sub the maximal exponent of module from tmpX
		result = pw_sub( &tmpX, &tmpX, &maxM );
		if ( result != pw_OK )
			return result;
	}
	while( pw_cmp( &tmpX, mod ) >= 0 );

	return res->set( &tmpX );
}

#define CMP( x, y ) ((x) < (y) ? (-1) : (x) == (y) ? 0 : 1 )

//
//	SpecCMP -> r = x - y;
//
static inline int SpecCMP(
		UINT  x2, UINT  x1, UINT  x0,
		UINT  y2, UINT  y1, UINT  y0 ) 
{
	int res = CMP( x2, y2 );

	if ( res )
		return res;

	res = CMP( x1, y1 );

	if ( res )
		return res;

	return CMP( x0, y0 );
}

//
//	SpecSUB -> r = x - y;
//
static inline void SpecSUB(
		UINT *r2, UINT *r1, UINT *r0,
		UINT  x2, UINT  x1, UINT  x0,
		UINT  y2, UINT  y1, UINT  y0 ) 
{
	*r0 = x0 - y0;
	*r1 = x1 - y1;
	*r2 = x2 - y2;

	if ( x0 < y0 )
	{
		if ( *r1 == 0 )
			(*r2)--;
		(*r1)--;
	}

	if ( x1 < y1 )
		(*r2)--;

}

//
//	SpecMUL -> r = x * y;
//
static inline void SpecMUL( UINT *r2, UINT *r1, UINT *r0, UINT x, UINT y1, UINT y0 )
{
	*r2 = *r1 = *r0 = 0;

	UL_INTEGER xy0, xy1, R;

	xy0.QuadPart = ULONGLONG( x ) * y0;
	xy1.QuadPart = ULONGLONG( x ) * y1;

	*r2 = xy1.Uint[bleHiPos];
	xy1.Uint[bleHiPos] = xy1.Uint[bleLoPos];
	xy1.Uint[bleLoPos] = 0;

	R.QuadPart = xy1.QuadPart + xy0.QuadPart;

	if ( ( R.QuadPart < xy1.QuadPart ) && ( R.QuadPart < xy0.QuadPart ) )
		(*r2)++;

	*r1 = R.Uint[bleHiPos];
	*r0 = R.Uint[bleLoPos];
}

//
//	dw_GetFirstBitPos
//	return value in range [ 1..32 ]
//
static int dw_GetFirstBitPos( UINT x )
{
	register WORD w;
	register BYTE b;
	register WORD n;


	if ( HIWORD( x ) )
	{
		w = HIWORD( x );
		b = HIBYTE( w );

		if ( b )
		{	n = 3 * 8;
		}
		else
		{
			b = LOBYTE( w );
			n = 2 * 8;
		}
	}
	else
	{
		w = LOWORD( x );
		b = HIBYTE( w );

		if ( b )
		{	n = 8;
		}
		else
		{
			b = LOBYTE( w );
			n = 0;
		}
	}

	for ( ; b; b /= 2 )
		n++;

	return int(n);
}

//
//	SpecDIV -> q * y + r = x;
//
static int SpecDIV(
		UINT *q,						// result
		UINT *r1, UINT *r0,			// reminder
		UINT x2,  UINT x1,  UINT x0,	 
		UINT y1,  UINT y0 )
{

	if ( y1 == 0 )
	{
		if ( ( x2 != 0 ) || ( x1 >=  y0 ) || ( y0 == 0 ) ) // Overflow of one UINT q ...
		{
			return pw_InternalError;
		}

		UL_INTEGER tR;
		
		tR.Uint[bleHiPos] = x1;
		tR.Uint[bleLoPos]  = x0;

		*q = (UINT)(tR.QuadPart / y0);
	
		tR.QuadPart %= y0;

		*r1 = tR.Uint[bleHiPos];
		*r0 = tR.Uint[bleLoPos];

		return pw_OK;
	}

	//
	//	Use same theorem as for limb devision...
	//
	//	See pw_div_limb().	Divides V on D. Math explanation.
	//
	//	Find nearest devider q
	//

	int n = dw_GetFirstBitPos( y1 );

	const int dwBitQ = sizeof( UINT ) * 8;

	//
	//	Choose maximal filled UINT yM from y1, y0
	//
	UINT yM; // yM = (y1,y0) >> n; alder bit of yM is 1;

	UINT checkDWL;

	ULONGLONG tX = 0;

	if ( n == 32 )
	{
		yM = y1;
/*
		UL_INTEGER *ptX = (UL_INTEGER *)&tX;		// problem with struct size and strict aliasing here - NK

		ptX->Uint[bleHiPos] = x2;
		ptX->Uint[bleLoPos]  = x1;
*/
		tX =  ((ULONGLONG)x1) | (((ULONGLONG)x2) << dwBitQ ) ;
		checkDWL = 0;
	}
	else
	{

		yM = UINT((((ULONGLONG)y0) >> n) | ( ((ULONGLONG)y1) << ( dwBitQ - n )));

		tX = ( (ULONGLONG)x0 >> n ) | ((ULONGLONG)x1 << ( dwBitQ - n ) ) | ((ULONGLONG)x2 << ( 2 * dwBitQ - n ) );

		checkDWL = UINT( ((ULONGLONG)x2) >> n);
	}

	//
	//	Check overflow because y1 >> n == 0...
	//

	if ( ( checkDWL ) != 0 )
	{
/*
		_asm { int 3 }

printf("\nSpecDIV x2=%X", x2);
printf("\nSpecDIV y1=%08X, y0=%08X, yM=%08X", y1, y0, yM);
printf("\nSpecDIV n=%d", n  );
printf("\nSpecDIV x2>>n=%08X \n", checkDWL  );
*/
		assert( 0 );
		return pw_InternalError;
	}

	*q = (UINT)(tX / yM);

	//
	//	It can happens that low UINT tX/yM == 0, but high UINT == 1 
	//	( because we took only alder part of Y )
	//	Sample:
	//		X = 00000000 00000001 00000000 00000000 00000000 00000000
	//		Y =                   00000000 00000001 00000000 00000001
	//	So if HIUINT( tX / yM ) == 1 we have to set *q = 0xffffFFFFffffFFFF
	//	The value tX / yM cannot be more then 2^32,
	//	 it has to be decrieased on previose step
	//
	if ( *q == 0 )
	{
		if ( HIUINT( tX / yM ) == 1 )
			*q = (UINT)(tX / yM - 1);
		else
		{
			assert( HIUINT( tX / yM ) == 0 );
		}
	}
	else
	{
		assert( HIUINT( tX / yM ) == 0 );
	}

	//
	//	Reminder (r2,r1,r0) = (x2,x1,x0) - q * (y1,y0) 
	//	The reminder has to be less ore equal then twice devider r <= 2y
	//

	UINT r2 = 0;

	UINT qy2, qy1, qy0;

	SpecMUL( &qy2, &qy1, &qy0, *q, y1, y0 );

	//
	//	Check Res 1. (x2,x1,x3) >= q * (y1,y0) <==> positive reminder...
	//
	if ( SpecCMP( x2, x1, x0, qy2, qy1, qy0 ) >= 0 )
	{
		SpecSUB( &r2, r1, r0, x2, x1, x0, qy2, qy1, qy0 );

		assert( SpecCMP( r2, *r1, *r0, 0, y1, y0 ) < 0 );

		return pw_OK;
	}

	//
	//	Negative reminder...
	//		(x2,x1,x3) < q * (y1,y0)
	//
	//	1) Decrease q and qy
	//	qy2,qy1,qy0 = (q - 1) * (y1,y0)
	//

	SpecSUB( &qy2, &qy1, &qy0, qy2, qy1, qy0, 0, y1, y0 );
	(*q)--;

	if ( SpecCMP( x2, x1, x0, qy2, qy1, qy0 ) >= 0 )
	{
		SpecSUB( &r2, r1, r0, x2, x1, x0, qy2, qy1, qy0 );

		assert( SpecCMP( r2, *r1, *r0, 0, y1, y0 ) < 0 );

		return pw_OK;
	}

	//
	//	2) Decrease q and qy again and last, see the Math explanation...
	//	qy2,qy1,qy0 = (q - 1) * (y1,y0)
	//

	SpecSUB( &qy2, &qy1, &qy0, qy2, qy1, qy0, 0, y1, y0 );
	(*q)--;

	if ( SpecCMP( x2, x1, x0, qy2, qy1, qy0 ) >= 0 )
	{
		SpecSUB( &r2, r1, r0, x2, x1, x0, qy2, qy1, qy0 );

		assert( SpecCMP( r2, *r1, *r0, 0, y1, y0 ) < 0 );

		return pw_OK;
	}

	assert( 0 );

	return pw_InternalError;
}


#define AlderBit( x ) 	( ( (x) >> ( sizeof( x ) * 8 - 1 ) ) & 1 )

/*
x = x2 * 2^2k + x1 * 2^k + x0
y =             y1 * 2^k + y0

1. first devision x2 x1 / y1 result -> q , r

y1 * q + r = x2 * 2^k + x1, y1 * q = x2 * 2^k + x1 - r

q * ( y1 * 2^k + y0 ) = ( x2 * 2^k + x1 - r ) * 2^k - q * y0

R = x - q * y 
	= x2 * 2^2k + x1 * 2^k + x0 - (( x2 * 2^k + x1 - r ) * 2^k - q * y0 ) = 
	= x0 + q * y0 - r * 2^k
*/

static int DWLDiv( ULONGLONG *q, ULONGLONG *r, ULONGLONG &x1, ULONGLONG &x0, ULONGLONG &y )
{
	//
	//	Check overflows...
	// 
	// 1)
	if ( y == 0 )
	{
		return pw_InternalError;
	}

	//
	// 2) Previouse step does not substract y from x, because all devider was more
	//		then the virtual value. So if here we detect x1 == y, we can set teh maximum 
	//		values for q and r. The final result will be corrected ( see second call in pw_div_limb ).
	//		First call does not have the previouse step.
	//		Really the rest *r does not matter.
	//
	if ( x1 == y )
	{
		*q = (ULONGLONG)-1;
		*r = (ULONGLONG)-1;
		return pw_OK;
	}

	if ( x1 > y )
	{
		return pw_InternalError;
	}
/*
#ifdef _UseAsm_
	
	return div_asm( (UINT *)q, (UINT *)r, (UINT *)&x0, (UINT *)&y );

#endif // _UseAsm_
*/
	UINT
		*ql = &DWLo( q ),
		*qh = &DWHi( q ),

		yl = DWLo( &y ),
		yh = DWHi( &y ),

		x0l = DWLo( &x0 ),
		x0h = DWHi( &x0 ),
		x1l = DWLo( &x1 ),
		x1h = DWHi( &x1 ),

		*rl = &DWLo( r ),
		*rh = &DWHi( r );



	//
	//  x1.h x1.l x0.h x0.l | y.h y.l
	//	 q.h * y.h y.l   	----------
	//   --------------		| q.h q.l
	//       x1l' x0h' x0.l
	//        q.l * y.h y.l
	//       ---------------
	//			    rh   rl

	int res = SpecDIV( qh, &x1l, &x0h, x1h, x1l, x0h, yh, yl );

	if ( res != pw_OK )
	{
		return res;
	}

	res = SpecDIV( ql, rh, rl, x1l, x0h, x0l, yh, yl );

	if ( res != pw_OK )
	{
		return res;
	}

//	ULONGLONG tq = 0;
	
	return pw_OK;
}

static ULONGLONG pw_getDWL( pw_int *x, int pos )
{
	int limbNum = pos / U64_BIT_Q;

	if ( limbNum >= x->m_ullSize )
		return 0;

	int limbPos = pos % U64_BIT_Q;

	ULONGLONG res;
	
	if ( limbPos == 64 )
		res = 0;
	else
		res = x->data[ limbNum ] >> limbPos;

	if ( ( limbNum + 1 < x->m_ullSize ) && ( limbPos != 0 ) )
		res |= x->data[ limbNum + 1 ] << ( U64_BIT_Q - limbPos );

	return res;
}


//
//	pw_div_limb()
//	Divides V on D. 
//
//	Math explanation:
//	Required Q = V / D, R = V % D.
//
//	Note 1. Initially V has to be less then MAX_UINT_LONG * D, 
//  so division result can be placed to one ULONGLONG and returned.
//	Remander will less thev D is to be placed to V again.
//
//	Note 2. Choose d such MAX_UINT_LONG > d >= ( MAX_UINT_LONG / 2 ), alder bit is always set.
//
//	Let V = v * 2^k + vx, D = d * 2^k + dx, such 0 <= vx < 2^k, 0 < dx < 2^k, v >= d > 0;
//
//	Let q = v / d;
//									q * d == v	
//		r = V - q * D = v * 2^k + vx - q * d * 2^k - q * dx, q * d = v, ==>
//		r = vx - q * dx;
//
//		By means of conditions vx < 2^k and dx >= 0 ==>
//		r < 2^k - q * 0 and d >= 0 ==> D >= d * 2^k,
//
//		Res 1. r < D always;
//			So, if r is not negative so we find required result Q = q and R = r.
//			Check only if r >= 0 ( or if V >= q * D ), so D > V - q * D >= 0 <==>
//			 ( q + 1 ) * D > V >= q * D
//		
//	If r is negative r < 0 -> |r| == q * D - V;
//		Let proof that |r| < 2 * D
//
//		|r| = q * D - V = q * ( d * 2^k + dx ) - ( v * 2^k + vx ) = q * dx - vx , dx < 2^k , vx >= 0. ==>
//		Res 3. |r| < q * 2^k,
//
//		q <= MAX_UINT_LONG, d >= MAX_UINT_LONG / 2 ==>
//		q <= 2 * d, |r| < q * 2^k ( Res 3 ) ==> 
//		|r| < 2 * d * 2^k, D >= d *2^k ( D = d * 2^k + dx  ) ==>
//		Res 4. |r| < 2 * D. 
//		It allows us only one or twice substitute D and decrease Q to get required result.
//
static int pw_div_limb( ULONGLONG *q, pw_int *r, pw_int *v, pw_int *d )
{
	int res;

	ULONGLONG tq = 0;

	int lastBitD = pw_getLastBit( d );

	if ( lastBitD < (int)U64_BIT_Q )
	{
		assert( pw_getLastBit( v ) < 2 * (int)U64_BIT_Q );

		if ( v->m_ullSize == 0 ) // Not initialised...
		{
			*q = 0;
			return r->setu( 0 );
		}

		ULONGLONG tr;

		ULONGLONG v1 = v->m_ullSize > 1 ? v->data[1] : 0;

		res = DWLDiv( &tq, &tr, v1, v->data[0],  d->data[0] );
		if ( res != pw_OK )
		{
			return res;
		}

		*q = tq;

		return r->setu( tr );
	}

	//
	//	Try to find nearest result in q. 
	//

	ULONGLONG alderV0, alderV1, alderD;

	alderV1 = pw_getDWL( v, lastBitD );
	alderV0 = pw_getDWL( v, lastBitD - U64_BIT_Q );
	alderD  = pw_getDWL( d, lastBitD - U64_BIT_Q );

	assert( AlderBit( alderD ) );
	assert( pw_getLastBit( v ) <= int( lastBitD + U64_BIT_Q ) );

	//
	//	Check valid. See Note 1 above.
	//
	if ( alderD == 0 )
	{
		return pw_InternalError;
	}

	ULONGLONG tr;

	res = DWLDiv( &tq, &tr, alderV1, alderV0,  alderD );
	if ( res != pw_OK )
	{
		return res;
	}

	pw_int qd;

	res = pw_mulu( &qd, d, tq );

	if ( res != pw_OK )
	{
		return res;
	}

	//
	//	Check if reminder >= 0;
	//
	if ( pw_cmp( v, &qd ) >= 0 )
	{
		//
		//	Note 2. If reminder >= 0 it < divider;
		//

		res = pw_sub( r, v, &qd );

		assert( pw_cmp( r, d ) < 0 );

		*q = tq;

		return res;
	}

	//
	//	Reminder < 0,
	//	So |r| < 2 * d, see math explanation  Res 4.
	//	We have to maximum twice decrease qd to get qd <= v.
	//

	res = pw_sub( &qd, &qd, d );
	if ( res != pw_OK ) return res;
	
	tq -= 1;

	if ( pw_cmp( &qd, v ) > 0 )
	{
		res = pw_sub( &qd, &qd, d );
		if ( res != pw_OK )
		{
			return res;
		}

		tq -= 1;

		assert( pw_cmp( &qd, v ) <= 0 );
	}

	res = pw_sub( r, v, &qd );
	if ( res != pw_OK )
	{
		return res;
	}

	*q = tq;

	return pw_OK;
}

//
//	Limb division.
//
//			U2 U1 U0 | V1 V0  
//					  -----------------
//     Qn * V1 V0    | Qn Qn-1 result
//       --------
//	V1 V0 >	u2 u1
//          u2 u1 U0
//		Qn-1 * V1 V0
//          --------
//	   V1 V0 > u1 u0 == real reminder
//

int pw_div(
	pw_int *q,	// result
	pw_int *r,	// reminder
	pw_int *v,	// value 
	pw_int *d	// divider
	) 
{
	int res;

	int lastV, lastD; // Numbers of last value's and devider's limbs.

	lastV = pw_getLastLimb( v );
	lastD = pw_getLastLimb( d );

	if ( lastV < lastD )
	{
		if ( r )
		{
			res = r->set( v );
			if ( res != pw_OK )
			{
				return res;
			}
		}

		if ( q )
			return q->setu( 0 );

		return pw_OK;
	}
	
	//
	// Temporary result value.
	//
	pw_int tq;

	res = tq.set( ( lastV - lastD + 1 ) * sizeof( ULONGLONG ) );

	if ( res != pw_OK )
	{
		return res;
	}

	//
	// Temporary reminder values.
	//
	pw_int tr;

	res = tr.set( v );

	if ( res != pw_OK )
	{
		return res;
	}

	//
	//	Division loop..
	//

	//
	// Virtual reminder.
	//	It will use temporary reminder value -> tr.data .
	//
	pw_int vr;

	for ( int pos = lastV - lastD + 1; pos-- ; )
	{
		res = vr.setVirtual( &tr, pos );

		if ( res != pw_OK )
		{
			return res;
		}

		res = pw_div_limb( &(tq.data[ pos ]), &vr, &vr, d );

		if ( res != pw_OK )
		{
			return res;
		}

	}

	//
	//	Assign results.
	//
	if ( q )
	{
		res = q->set( &tq );
		if ( res != pw_OK )	return res;
	}

	if ( r )
		return r->set( &tr );

	return pw_OK;
}

//
//	See comments in pw_add_asm
//int pw_add_classic( pw_int *res, pw_int *x, pw_int *y )

int pw_add( pw_int *res, pw_int *x, pw_int *y )
{
	assert( x->data && y->data );

	int lastX = pw_getLastLimb( x );
	int lastY = pw_getLastLimb( y );

	int max = lastX > lastY ? lastX : lastY;
	int min = lastX < lastY ? lastX : lastY;

	pw_int tmp;

	int result = tmp.set( ( max + 2 ) * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	int pos;

	BOOL carry = 0;

	for( pos = 0; pos <= min; pos++ )
	{
		DWLAdd( &tmp.data[ pos ], x->data[ pos ], y->data[ pos ], carry, &carry );
	}

	for( ; pos <= lastX; pos++ )  // if x is larger
	{
		DWLAddCarry( &tmp.data[ pos ], x->data[ pos ], carry,  &carry );
	}

	for( ; pos <= lastY; pos++ )  // if Y is larger
	{
		DWLAddCarry( &tmp.data[ pos ], y->data[ pos ], carry, &carry );
	}

	if ( carry )
		tmp.data[ pos ] = 1;

	return res->set( &tmp );
}

/*
int pw_add( pw_int *r, pw_int *x, pw_int *y )
{
#ifdef _UseAsm_
	return pw_add_asm( r, x, y );
#else
	return pw_add_classic( r, x, y );
#endif
}
*/

int pw_addu( pw_int *res, pw_int *x, ULONGLONG y )
{
	assert( x->data );

	int lastX = pw_getLastLimb( x );

	pw_int tmp;

	int result = tmp.set( ( lastX + 2 ) * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;


	BOOL carry = 0;

	DWLAdd( &tmp.data[ 0 ], x->data[ 0 ], y, carry, &carry );

	int pos = 1;

	for( ; ( pos <= lastX ) ; pos++ )  // if x is larger
	{
		DWLAddCarry( &tmp.data[ pos ], x->data[ pos ], carry,  &carry );
	}

	if ( carry )
		tmp.data[ pos ] = 1;

	return res->set( &tmp );
}

int pw_addm( pw_int *res, pw_int *x, pw_int *y, pw_int *mod )
{
	int result = pw_add( res, x, y );
	if ( result != pw_OK )
		return result;

	return pw_mod( res, res, mod );  
}

int pw_mulu( pw_int *res, pw_int *x, ULONGLONG y )
{
	if ( y == 0 )
		return res->setu( 0 );

	int lastX = pw_getLastLimb( x );

	pw_int tmp0;

	int result = tmp0.set( ( lastX + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

#ifdef _UseAsm_

	WORD sizeX = WORD( (pw_getSizeInBits( x ) + 31 ) / 32 );

	mul_UINT_arrays( (UINT *)tmp0.data, (UINT *)x->data, sizeX, (UINT *)&y, 2 );

	return res->set( &tmp0 );

#else

	pw_int tmp1;

	result = tmp1.set( ( lastX + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	for( int posX = 0; posX <= lastX; posX += 2 )
	{
		DWLMul( 
			&(tmp0.data[ posX ]),
			&(tmp0.data[ posX + 1]),
			x->data[ posX ],
			y );

		if ( posX < lastX )
			DWLMul( 
				&(tmp1.data[ posX + 1 ]),
				&(tmp1.data[ posX + 2]),
				x->data[ posX + 1],
				y );
	}

	return pw_add( res, &tmp1, &tmp0 );

#endif // _UseAsm_
}
/*
int pw_mulEx( pw_int *res, pw_int *x, pw_int *y )
{
	int lastX = pw_getLastLimb( x );
	int lastY = pw_getLastLimb( y );

	pw_int tmp;

	int result = tmp.set( ( lastX + lastY + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	pw_int tmp1;

	result = tmp1.set( ( lastX + lastY + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	int tmpPos;

	for( int posX = 0; posX <= lastX; posX++ )
	{
		for( int posY = 0; posY <= lastY; posY++ )
		{
			tmpPos = posX + posY;

			tmp.clear();
			DWLMul( 
				&(tmp.data[ tmpPos ]),
				&(tmp.data[ tmpPos + 1]),
				x->data[ posX ],
				y->data[ posY ] );

			result = pw_add( &tmp1, &tmp1, &tmp );
			if ( result != pw_OK )
				return result;
		}
	}

	return res->set( &tmp1 );
}
*/
int pw_mul_classic( pw_int *res, pw_int *px, pw_int *py )
{

	int lastX = pw_getLastLimb( px );
	int lastY = pw_getLastLimb( py );

	pw_int *x,*y;

	//
	//	If order y > order x => we get optimisation for y loop.
	//	otherwise we swap y with x.
	//
	if ( lastX > lastY )
	{	x = py;
		y = px;
		int tmp = lastX;
		lastX = lastY;
		lastY = tmp;
	}
	else
	{
		x = px;
		y = py;
	}

	pw_int tmp0;

	int result = tmp0.set( ( lastX + lastY + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	pw_int tmp1;

	result = tmp1.set( ( lastX + lastY + 2 )  * sizeof( ULONGLONG ) );
	if ( result != pw_OK )
		return result;

	pw_int tmpRes;
	tmpRes.setu( 0 );

	int tmpPos;

	for( int posX = 0; posX <= lastX; posX++ )
	{
		tmp0.clear();
		tmp1.clear();

		for( int posY = 0; posY <= lastY; posY += 2 )
		{
			tmpPos = posX + posY;

			DWLMul( 
				&(tmp0.data[ tmpPos ]),
				&(tmp0.data[ tmpPos + 1]),
				x->data[ posX ],
				y->data[ posY ] );

			if ( posY < lastY )
				DWLMul( 
					&(tmp1.data[ tmpPos + 1 ]),
					&(tmp1.data[ tmpPos + 2]),
					x->data[ posX ],
					y->data[ posY + 1 ] );

		}

		result = pw_add( &tmpRes, &tmpRes, &tmp0 );
		if ( result != pw_OK )	return result;

		result = pw_add( &tmpRes, &tmpRes, &tmp1 );
		if ( result != pw_OK )	return result;
	}

	return res->set( &tmpRes );
}

int pw_mulm( pw_int *res, pw_int *x, pw_int *y, pw_int *mod )
{
	pw_int tmp;

	int result;
/*
	pw_int lx;
	pw_int ly;

	result = pw_mod( &lx, x, mod );
	if ( result != pw_OK )
		return result;

	result = pw_mod( &ly, y, mod );
	if ( result != pw_OK )
		return result;

	result = pw_mul( &tmp, &lx, &ly );
*/
	result = pw_mul( &tmp, x, y );
	if ( result != pw_OK )
		return result;

	return pw_mod( res, &tmp, mod );
}

int pw_getBit( pw_int *x, int bitNum )
{
	if ( x->m_ullSize < int( bitNum / U64_BIT_Q ))
		return 0;

	return (( x->data[ bitNum / U64_BIT_Q ] ) & ( (ULONGLONG)1<< (bitNum % U64_BIT_Q))) != 0;
}
/*
int pw_squaremEx( pw_int *res, pw_int *u, pw_int *mod)
{
	int q = pw_square( res, u );

	if ( q != pw_OK )
		return q;

	return pw_mod( res, res, mod );
}

int pw_squarem1( pw_int *res, pw_int *u, pw_int *mod)
{
    pw_int u0,u1,u0_2,u1_2,u1_u0_2,tmp;
    int result,last_bit,n;

    last_bit = pw_getLastBit(u);
    if (last_bit < 64)
    {
        tmp.set( 16 );
		DWLMul( &(tmp.data[0]), &(tmp.data[1]), u->data[0], u->data[0] );
		pw_mod( res, &tmp, mod );
        return pw_OK;
    }

    int byteQ = ( last_bit + 7 ) / 8;

	//
	//	n - number of alder bytes
	//	byteQ/2 - number of younger bytes.
	//

    n = byteQ - byteQ/2;
    
    BYTE *buff = new BYTE[ byteQ ];
    pw_int2bytes( buff, byteQ, u );

    pw_bytes2int(&u1,buff,byteQ/2);
    pw_bytes2int(&u0,buff+byteQ/2,n);

	memset( buff, 0, byteQ );
	delete buff;
    
    result = pw_squarem(&u1_2,&u1, mod );
    if (pw_OK != result)
        return result;
    
    result = pw_squarem(&u0_2,&u0, mod );
    if (pw_OK != result)
        return result;
    
    if (pw_cmp(&u1,&u0) > 0)
    {
        result = pw_subm(&u1_u0_2,&u1,&u0, mod );
        if (pw_OK != result)
            return result;
    }else
    {
        result = pw_subm(&u1_u0_2,&u0,&u1, mod );
        if (pw_OK != result)
            return result;
    }
    
    result = pw_squarem(&u1_u0_2,&u1_u0_2, mod );
    if (pw_OK != result)
        return result;

    result = pw_mul_exp2(&tmp,&u1_2,n*8*2);
    if (pw_OK != result)
        return result;

    result = pw_mod(&tmp,&tmp, mod );
    if (pw_OK != result)
        return result;
    
    result = pw_mul_exp2(&u1_2,&u1_2,n*8);
    if (pw_OK != result)
        return result;

    result = pw_mod(&u1_2,&u1_2, mod );
    if (pw_OK != result)
        return result;

    result = pw_addm(&u1_2,&u1_2,&tmp, mod );
    if (pw_OK != result)
        return result;

    result = pw_mul_exp2(&tmp,&u0_2,n*8);
    if (pw_OK != result)
        return result;

    result = pw_mod(&tmp,&tmp, mod );
    if (pw_OK != result)
        return result;

    result = pw_addm(&u0_2,&u0_2,&tmp, mod );
    if (pw_OK != result)
        return result;

    result = pw_addm(res,&u0_2,&u1_2, mod );
    if ( pw_OK != result )
        return result;

    result = pw_mul_exp2(&tmp,&u1_u0_2,n*8);
    if (pw_OK != result)
        return result;

    result = pw_mod(&tmp,&tmp, mod );
    if (pw_OK != result)
        return result;

	if ( pw_cmp( res, &tmp ) < 0 )
	{
		result = pw_add( res, res, mod );
	}

    return pw_sub(res,res,&tmp);

}
*/

int pw_square_rec( pw_int *res, pw_int *x )
{
	int last = pw_getLastLimb( x );
	pw_int t;

	int result;

	if ( last < OPTIMAL_CLASSIC_MUL_LIMB_COUNT )
	{
#ifdef _UseAsm_
		return pw_square_asm( res, x );
#else
		return pw_mul_def( res, x, x ); // Note, pw_mull calls pw_square_rec and pw_square correspondently.
#endif
	}

	int splitPos = ( last + 1 ) / 2;

	//
	//	n = splitPos * U64_BIT_Q
	//	x = u1*2^n + u0
	//
	//	res = x^2  = ( 2^2n + 2^n ) u1^2  - 2^n( u1 - u0 )^2 + ( 2^n + 1 ) u0 ^2
	//

	pw_int u1, u0;

	u1.setVirtual( x, splitPos );
	u0.setVirtual( x, 0, splitPos );

	pw_int u1u1;
	result = pw_square_rec( &u1u1, &u1 );
	if ( result != pw_OK ) return result;

	pw_int u0u0;
	result = pw_square_rec( &u0u0, &u0 );
	if ( result != pw_OK ) return result;

	pw_int sU10;
	int sig;

	sig = pw_cmp( &u1, &u0 ) < 0;

	if ( sig ) 	result = pw_sub( &sU10, &u0, &u1 );
	else		result = pw_sub( &sU10, &u1, &u0 );
	if ( result != pw_OK ) return result;


	pw_int sU10_2;
	result = pw_square_rec( &sU10_2, &sU10 );
	if ( result != pw_OK ) return result;

	//
	//	Accumulates the result. 
	//
	// Grows it for maximal size.
	//
	result = res->set( ( 2 * last + 2 ) * sizeof( ULONGLONG ) );
	if ( result != pw_OK ) return result;

	//
	//	res = x^2  = ( 2^2n + 2^n ) u1^2  - 2^n( u1 - u0 )^2 + ( 2^n + 1 ) u0^2
	//
	// 1. u0^2
	result = res->set( &u0u0 );
	if ( result != pw_OK ) return result;

	pw_int vRes;

	// 2.  2^n u0 u0
	vRes.setVirtual( res, splitPos );
	result = pw_add( &vRes, &vRes, &u0u0 );
	if ( result != pw_OK ) return result;

	// 3. 2^n u1 u1
//	vRes.setVirtual( res, splitPos );
	result = pw_add( &vRes, &vRes, &u1u1 );
	if ( result != pw_OK ) return result;

	// 4. 2^2n u1 u1
	vRes.setVirtual( res, splitPos * 2 );
	result = pw_add( &vRes, &vRes, &u1u1 );
	if ( result != pw_OK ) return result;

	//
	// 5. - 2^n( u1 - u0 )^2
	//
	vRes.setVirtual( res, splitPos );


	//
	//	result cannot be negative..
	//
	assert( pw_cmp( &vRes, &sU10_2 ) >= 0 );

	// So simple sub
	return pw_sub( &vRes, &vRes, &sU10_2 );
}
/*
int pw_squarem( pw_int *res, pw_int *u, pw_int *mod )
{
    pw_int u0,u1,u0_2,u1_2,u1_u0_2,tmp;
    int result,last_bit,n;

    last_bit = pw_getLastBit(u);
//    printf("last_bit = %d\n",last_bit);
    if (last_bit <= 64)
    {
        result = tmp.set( 16 );

		if ( result != pw_OK )
			return result;

		DWLMul( &(tmp.data[0]), &(tmp.data[1]), u->data[0], u->data[0] );
        return pw_mod( res, &tmp, mod );
    }

    int byteQ = ( last_bit + 7 ) / 8;

//  n = ((last_bit/64)+1)>>1;
	//
	//	n - number of alder bytes
	//	byteQ - n - number of younger bytes.
	//

    n = byteQ/2;
    
    BYTE *buff = new BYTE[ byteQ ];
    pw_int2bytes( buff, byteQ, u );

    pw_bytes2int(&u1,buff,byteQ - n);
    pw_bytes2int(&u0,buff+byteQ - n ,n);

	memset( buff, 0, byteQ );
	delete buff;
    
    result = pw_square(&u1_2,&u1);
    if (pw_OK != result)
        return result;
    
    result = pw_square(&u0_2,&u0);   
    if (pw_OK != result)
        return result;
    
    if (pw_cmp(&u1,&u0) > 0)
    {
        result = pw_sub(&u1_u0_2,&u1,&u0);
        if (pw_OK != result)
            return result;
    }else
    {
        result = pw_sub(&u1_u0_2,&u0,&u1);
        if (pw_OK != result)
            return result;
    }
    
    result = pw_square(&u1_u0_2,&u1_u0_2);   
    if (pw_OK != result)
        return result;

    result = pw_mul_exp2(&tmp,&u1_2,n*8*2);
    if (pw_OK != result)
        return result;
    
    result = pw_mul_exp2(&u1_2,&u1_2,n*8);
    if (pw_OK != result)
        return result;

    result = pw_add(&u1_2,&u1_2,&tmp);
    if (pw_OK != result)
        return result;

    result = pw_mul_exp2(&tmp,&u0_2,n*8);
    if (pw_OK != result)
        return result;

    result = pw_add(&u0_2,&u0_2,&tmp);
    if (pw_OK != result)
        return result;

    result = pw_add(res,&u0_2,&u1_2);
    if ( pw_OK != result )
        return result;

    result = pw_mul_exp2(&tmp,&u1_u0_2,n*8);
    if (pw_OK != result)
        return result;

    result = pw_sub(res,res,&tmp);
	if ( result != pw_OK )
		return result;

	return pw_mod( res, res, mod );

}
*/
int pw_powm_classic( pw_int *res, pw_int *x, pw_int *exp, pw_int *mod )
{
	pw_int tmp;
	int result;

	int bitExpNum = pw_getLastBit( exp );

	if ( bitExpNum == 0 ) // Zero exponent;
	{
		result = tmp.setu( 1 );
		if ( result != pw_OK )
			return result;
		return res->set( &tmp );
	}

	bitExpNum--; // One based result; if only first bit -> bitExpNum = 1

	result = tmp.set( x );
	if ( result != pw_OK )
		return result;

	for( ; bitExpNum--; )
	{
		result = pw_mulm( &tmp, &tmp, &tmp, mod );
//		result = pw_squarem( &tmp, &tmp, mod);
		if ( result != pw_OK )
		{
			return result;
		}

		BOOL currentBit = pw_getBit( exp, bitExpNum );
		if ( currentBit )
		{	result = pw_mulm( &tmp, &tmp, x, mod );
			if ( result != pw_OK )
			{
				return result;
			}
		}
	}

	return res->set( &tmp );
}

int pw_powm_windows( pw_int *res, pw_int *x, pw_int *exp, pw_int *mod )
{
	const int windowSize = 6; //6 is optimal for 2048 bit, 7 is optimal for 4096 bits

	//
	//	ptab is  table with x powers
	//	t[0] = 1 
	//	t[1] = x
	//	t[2] = x^2
	//	t[i] = x^i
	//	t[31] = x^31  1111
	//
	pw_int ptab[ 1 << windowSize ];

	int result = ptab[0].setu( 1 );
	if ( result != pw_OK )	return result;

	result = ptab[1].setVirtual( x, 0 );
	if ( result != pw_OK )	return result;

	for ( int i = 2; i < (int)(sizeof(ptab) / sizeof( pw_int )); i++ )
	{
		result = pw_mulm( &ptab[i], &ptab[i-1], x, mod );
		if ( result != pw_OK )	return result;
	}

	pw_int tmp;

	int bitExpNum = pw_getLastBit( exp );

	if ( bitExpNum == 0 ) // Zero exponent;
	{
		result = tmp.setu( 1 );
		if ( result != pw_OK )
			return result;
		return res->set( &tmp );
	}

	bitExpNum--; // One based result; if only first bit -> bitExpNum = 1

	result = tmp.set( x );
	if ( result != pw_OK )
		return result;

	for( ; bitExpNum > 0 ; )
	{
		int q = 0;
		int c = 0;
		for( ; (q < windowSize) && bitExpNum; q++)
		{
			bitExpNum--;

			result = pw_mulm( &tmp, &tmp, &tmp, mod );
			if ( result != pw_OK ) return result;

			BOOL currentBit = pw_getBit( exp, bitExpNum );
			c = ( c<<1 ) + currentBit;
		}

		result = pw_mulm( &tmp, &tmp, &ptab[c], mod );
		if ( result != pw_OK ) return result;
	}

	return res->set( &tmp );
}

//
//	17.09.2012
//
//	Speed test, processor i5 2320, 3Hz
//	Time result in clocks CLOCKS_PER_SEC == 1000
//
//---------------------------------------------------------
//			\	module		
//Function 	 \	size in bits    4096	4096	2048	2048 
//---------------------------------------------------------
//pw_powm_classic				376		272		31		46
//pw_powm_windows				297		285		47		32
//pw_powm_montgomery			217		224		16		31
//pw_powm_montgomery_windows	175		179		31		15
//---------------------------------------------------------
// windowSize	|				6       7       6		6


int pw_powm( pw_int *res, pw_int *x, pw_int *exp, pw_int *mod, bool classic )
{
	if ( classic )
		return pw_powm_classic( res, x, exp, mod );

	int bitQ = pw_getSizeInBits( mod );

	if ( bitQ < 32 )
		return pw_powm_classic( res, x, exp, mod );

#ifdef _ODSM
//#define _Test_
#endif

#ifdef _Test_
	pw_int resT[4];
	clock_t clockT[5];
	int i = 0;
	clockT[i] = clock();
	int iRes = pw_powm_classic( resT+i, x, exp, mod );
	pw_cmp( resT+i, resT+i ); // To get same time 
	clockT[++i] = clock();
	iRes = pw_powm_windows( resT+i, x, exp, mod );
	if ( pw_cmp( resT+i, resT ) != 0 )
		ODSF( L"%d. Assertion! %d/%d, %I64X/%I64X", i, resT[i].m_ullSize, resT->m_ullSize, resT[i].m_ullSize ? resT[i].data[0]:0, resT->m_ullSize ? resT->data[0]:0 );
	clockT[++i] = clock();
	iRes = pw_powm_montgomery( resT+i, x, exp, mod );
	if ( pw_cmp( resT+i, resT ) != 0 )
		ODSF( L"%d. Assertion! %d/%d, %I64X/%I64X", i, resT[i].m_ullSize, resT->m_ullSize, resT[i].m_ullSize ? resT[i].data[0]:0, resT->m_ullSize ? resT->data[0]:0 );
	clockT[++i] = clock();
	iRes = pw_powm_montgomery_windows( resT+i, x, exp, mod );
	if ( pw_cmp( resT+i, resT ) != 0 )
		ODSF( L"%d. Assertion! %d/%d, %I64X/%I64X", i, resT[i].m_ullSize, resT->m_ullSize, resT[i].m_ullSize ? resT[i].data[0]:0, resT->m_ullSize ? resT->data[0]:0 );
	clockT[++i] = clock();

	ODSF( L"Start: %d", clockT[0] );
	for( i = 0; i < 4; i++ ) 
		ODSF( L"Stop %d = %d", clockT[i+1], clockT[i+1] - clockT[i] );

	return res->set( resT );
#else
	// Due to test pw_powm_montgomery_windows wins with window size 6
	return pw_powm_montgomery_windows( res, x, exp, mod );
#endif
}


int pw_powmu( pw_int *res, pw_int *x, unsigned int pow, pw_int *mod )
{
	pw_int tmp;

	int result = tmp.setu( pow );
	if ( result != pw_OK )
		return result;

	return pw_powm( res, x, &tmp, mod );
}
/*
int pw_createPublicKey( 
		pw_int *exp,
		pw_int *mod,
		const BYTE *keyBlock,
		UINT *blockSize);

int pw_RSA_Update( 
		pw_int *exp,
		pw_int *mod,
		UINT dataSize,
		const BYTE *inBlock,
		BYTE *outBlock);

#define RSA_PublicKey 0

////////////////////////////////////////
// readUINT return Intel packed UINT.
//
UINT readUINT( const BYTE *ptr )
{
	return ptr[0] | (ptr[1] >> 8) | (ptr[2] >> 16) | (ptr[3] >> 24);
}

int pw_createPublicKey( 
		pw_int *exp,
		pw_int *mod,
		const BYTE *keyBlock,
		UINT *blockSize)
{
	UINT primeLengthInBytes;
	UINT byteNum = 0;

// Check block type...
	if ( readUINT(keyBlock + byteNum) != RSA_PublicKey )
		return pw_InvalidKeyBlock;

	byteNum = sizeof( UINT );

// Get size...
	int sizeInBites = readUINT( keyBlock + byteNum ); // *(UINT *)(keyBlock + byteNum);
	byteNum += sizeof( UINT );
	
// Get public exponent...
	int result = exp->setu( (ULONGLONG)readUINT( keyBlock + byteNum )); //(*(UINT *)(keyBlock + byteNum)));
	if ( result != pw_OK )
		return result;

	byteNum += sizeof( UINT );

// Get module n...
	primeLengthInBytes = (sizeInBites + 7) / 8 ;
	result = mod->set( primeLengthInBytes, (keyBlock + byteNum));
	if ( result != pw_OK )
		return result;

	byteNum += primeLengthInBytes;

// Check end block type...
	if ( readUINT(keyBlock + byteNum) != RSA_PublicKey )
		return pw_InvalidKeyBlock;

	*blockSize = primeLengthInBytes;

	return pw_OK;
}

int pw_RSA_Update( 
		pw_int *exp,
		pw_int *mod,
		UINT dataSize,
		const BYTE *inBlock,
		BYTE *outBlock
		)
{
	pw_int data;

	int result = data.set( dataSize, inBlock );
	if ( result != pw_OK )
		return result;

	result = pw_powm( &data, &data, exp, mod );
	if ( result != pw_OK )
		return result;

	return pw_int2bytes( outBlock, dataSize, &data );
}
*/

static BYTE getByte( pw_int *t, int byteNum )
{
	int pos = byteNum / sizeof( ULONGLONG );

	if ( pos >= t->m_ullSize )
		return 0;


	int bitOffset = (byteNum % sizeof( ULONGLONG  )) * 8;

	return BYTE((t->data[ pos ] >> bitOffset) & 0xFF);
}

int pw_setByte( pw_int *t, int byteNum, BYTE bt )
{
	int pos = byteNum / sizeof( ULONGLONG );

	if ( pos > t->m_ullSize )
		return pw_InternalError;

	int bitOffset = (byteNum % sizeof( ULONGLONG  )) * 8;

	t->data[ pos ] &= ~((ULONGLONG)0xFF << bitOffset);
	t->data[ pos ] |= ((ULONGLONG)bt) << bitOffset;

	return pw_OK;
}

//
//	pw_int2byte:
//		Prepare byte buffer from pw_integer.
//		First buffer byte is most significant byte of integer (def.PKCS#1)
//
int pw_int2bytes( BYTE *byteData, UINT length, const pw_int *intData )
{
	UINT i;
	pw_int tmp;

	int res = tmp.set( intData );
	if ( res != pw_OK ) return res;

	for ( i = length ; i-- ; )
	{
		byteData[ length - (i + 1) ] = getByte( &tmp, i); //( BYTE ) tmp.data[0];
//		res = pw_mul_exp2( &tmp , &tmp , -8 );
//		if ( res != pw_OK ) return res;
	}

	tmp.clear();

	return pw_OK;
}

//
//	pw_bytes2int:
//		Prepare pw_integer from byte buffer.
//		First buffer byte is a most significant byte of integer (def.PKCS#1)
//
int pw_bytes2int( pw_int *intData, const BYTE *byteData , UINT length )
{
	UINT i;
	int res = intData->set( length );
	if ( res != pw_OK ) return res;
	
//	pw_int tmp;
//	int res = tmp.set( length );
//	if ( res != pw_OK ) return res;

	for ( i = 0 ; i < length ; i++ )
	{
		pw_setByte( intData, i, byteData[ length - (i + 1) ] );

/*		res = pw_mul_exp2( intData , intData , 8 );
		if ( res != pw_OK ) return res;

		tmp.data[0] = ((ULONGLONG)(byteData[i])) & 0xFF;

		res = pw_add( intData , intData , &tmp );
		if ( res != pw_OK ) return res;
*/
	}

	return pw_OK;
}

int pw_int::set( const pw_int *x )
{
	if ( x == this )
		return pw_OK;

	int realSize = pw_getLastLimb( x ) + 1;

	if ( realSize > m_ullSize )
	{
		if ( virtualData )
			return pw_InternalError;

		if ( data )
			clear();

		m_ullSize = realSize;
		data = (ULONGLONG *)realloc( data, m_ullSize * sizeof( ULONGLONG ) );
	}

	if ( data == NULL )
		return pw_MemoryAllocationError;


	memset( data, 0, m_ullSize * sizeof( ULONGLONG ));

	memcpy( data, x->data, realSize * sizeof( ULONGLONG ) );

	return pw_OK;
}


int pw_int::setVirtual( pw_int *v, int pos, int newSize )
{
	if ( data && !virtualData )
	{
		clear();
		free( data );
		data = NULL;
		m_ullSize =0;
	}


	if ( pos >= v->m_ullSize )
	{
		virtualData = false;
		return setu( 0 );
//		return pw_InternalError;
	}

	data = v->data + pos;
	m_ullSize = v->m_ullSize - pos;

	if ( ( newSize != -1 ) && ( newSize < m_ullSize ) )
		m_ullSize = newSize;

	virtualData = true;

	return pw_OK;
}



int pw_int::set( int pDataSize, const BYTE *pData )
{
	if ( virtualData )
	{
		if ( int(m_ullSize * sizeof( ULONGLONG )) < pDataSize  )
			return pw_InternalError;

		clear();

		if ( pData != NULL )
			pw_bytes2int( this, pData , pDataSize );

		return pw_OK;
	}

	if ( data )
		memset( data, 0, m_ullSize * sizeof( ULONGLONG ));

	m_ullSize  = pDataSize / sizeof( ULONGLONG );
	m_ullSize += ( ( pDataSize % sizeof( ULONGLONG ) ) != 0 );

	if ( data == NULL )
	{
		if ( m_ullSize == 0 )
		{
			m_ullSize = 1;
			data = (ULONGLONG *)malloc( m_ullSize * sizeof( ULONGLONG ) );
			data[0] = 0;
		}
		else
			data = (ULONGLONG *)malloc( m_ullSize * sizeof( ULONGLONG ) );

	}
	else
	{
		data = (ULONGLONG *)realloc( data, m_ullSize * sizeof( ULONGLONG ));
	}

	if ( data == NULL )
		return pw_MemoryAllocationError;

	memset( data, 0, m_ullSize * sizeof( ULONGLONG ));

	if ( pData != NULL )
	//	memcpy( data, pData, pDataSize ); // Copy only real data form input byte buffer
		pw_bytes2int( this, pData , pDataSize );

	return pw_OK;
}

int pw_random( pw_int *x, int s )
{

	BYTE *buff = new BYTE[ s ];

	int i = 0;

	for( ; i < s; i++ )
	{
		buff[ i ] = (BYTE)rand();
	}

	int res = x->set( s, buff );

	memset( buff, 0, i );

	delete buff;

	return res;
}

static int possibly_prime( pw_int *n, pw_int *n_minus_1 )
{
    pw_int b,res;
    
	do 
	{
		pw_random (&b, 8);
		pw_mod(&b, &b, n);
	}
	while( pw_cmpu( &b, 1 ) < 0 );
    
    int result = pw_powm( &res, &b, n_minus_1, n );

	if ( result != pw_OK )
	{
#ifdef _ODS
	//	ODSF( L"Assertion! pw_powm failed %d", result );
#endif
	}

	assert( result == pw_OK );

    if ( pw_cmpu( &res, 1 ) == 0 )
        return 1;

    return 0;
}

//
//	Prime table...
//		Table af first prime...
//

static int	  primeQ = 0;
static pw_int *primeT = NULL;

//
//	scanPrimeTable
//		Finds divider in primeT ( reminder is not zero ).
//		Returns false if the divider was found, so the number is definitely not prime...
//
static bool scanPrimeTable( pw_int *number )
{
    pw_int res;

    for (int i=0; i < primeQ ; i++)
    {
		//
		//	Small acceleration for small numbers...
		//
		if ( ( number->m_ullSize == 1 ) && ( primeT[i].m_ullSize == 1  ) )
		{
			ULONGLONG dwlRes = pw_getLimb( number, 0 ) % pw_getLimb( &primeT[i], 0 );

			if ( dwlRes == 0 )
				return false;
		}
		else
		{
			pw_mod( &res, number, &primeT[i]);
        
			if ( pw_cmpu( &res, 0 ) == 0 )
				return false;
		}
    }

    return true;
}

int pw_primeTableInit(
	int primeQuantity,
	cProgressActor *actor
	)
{
	primeQuantity *= 4;

	//
	//	The prime table can be inited before so application can check the fact 
	//	and lets free it only in the procedure really allocates the table...
	//
	if ( primeT != NULL )
		return pw_InternalError;

	//
	//	Allocate required space...
	//
	primeT = (pw_int *) malloc( primeQuantity * sizeof( pw_int ) );

	if ( primeT == NULL )
		return pw_MemoryAllocationError;

	memset( primeT, 0, primeQuantity * sizeof( pw_int ) );

	//
	//	Initialization...
	//
	// 1) First prime is definitly 2..
	//
	primeQ = 0;

	if ( primeQuantity > 0 )
		primeT[ primeQ++ ].setu( 2 );

	//
	// 2) All others are even and start from 3.
	//
	pw_int number, n_minus_1;

	number.setu( 3 );

	int res;

    for ( ; primeQ < primeQuantity ; )
    {
		if ( pw_OK != pw_subu( &n_minus_1, &number, 1) )
		  return 0;

/*
		//
		//	Simple check if the number is not devided by all previouse prime...
		//
        if ( scanPrimeTable( &number ) )

		:( Using of the trick is slow then real test for prime because all values are small...
*/
		if ( possibly_prime ( &number , &n_minus_1 ) )
        {
            res = primeT[ primeQ++ ].set(&number);

			if ( res != pw_OK )
				return res;

			if ( actor != NULL )
			{
				int q = actor->func( NULL, res, primeQ );
				if ( q != 0 )
					return q;
			}
        }

        res = pw_addu( &number, &number, 2 );

		if ( res != pw_OK )
			return res;
    }

	return pw_OK;
}

int pw_primeTableFree( )
{
	if ( primeT != NULL )
		free( primeT );

	primeQ = 0;
	primeT = NULL;

	return pw_OK;
}

//
//	End of prime table section...
//	If reps == 0 it checks prime table dividers only.
//	Returns 1 if n is prime.
//
int pw_probab_prime ( pw_int *n, int reps, int *count )
{
	int is_prime = 1;

	//
	//	Find divider in table of primes...
	//	The pw_primeTableInit and pw_primeTableFree have to be used.
	//
	if ( primeQ && !scanPrimeTable( n ) )
	{
		if ( count )
			*count = -1; // n is not prime by prime table.

		is_prime = 0; // Divider is found in prime table

		return is_prime;
	}

	if ( reps == 0 )
		return is_prime;

	pw_int n_minus_1, b;

	//
	//	Check if even...
	//	
	if ( ( pw_getLimb( n, 0 ) & 1 ) == 0 )
	{
//	if ( 0 == pw_getBit(n,0))
		if ( count != NULL ) *count = -2; // Is even
		return 0;
	}

	if ( pw_OK != pw_subu( &n_minus_1, n, 1) )
	{
		if ( count != NULL ) *count = -3; // Error
		return 0;
	}

	int i = 0;

	for ( ; ( i < reps ) && is_prime; i++ )
	{
		is_prime = possibly_prime( n, &n_minus_1 );

		if ( !is_prime )
			break;
	}

	if ( count != NULL ) *count = i;

	return is_prime;
}


//
//	pw_gcd
//	Greatest common divisor...
//	It will use Euclid algorithm. 
//		gcd( u, v ) == gcd( v, u - q*v ) where q is any integer,
//	so if choose r == u mod v,
//		gcd( u, v ) == gcd( v, r );
//
// 1) Use recursion...
int pw_gcd_rec( pw_int *gcd, pw_int *u, pw_int *v )
{
	if ( pw_cmpu( v, 0 ) == 0 )
		return gcd->set( u );

	pw_int r;

	int res = pw_mod( &r, u, v );

	if ( res != pw_OK )
		return res;

	return pw_gcd( gcd, v, &r );
}

//
// 2) Use loop...
//
int pw_gcd( pw_int *gcd, pw_int *u, pw_int *v )
{
	pw_int r, tu, tv;

	tu.set( u );
	tv.set( v );

	pw_int
		*pt,
		*pr = &r,
		*pu = &tu,
		*pv = &tv;

	int res;

	for( int count = 0;; count++ )
	{
		if ( pw_cmpu( pv, 0 ) == 0 )
			return gcd->set( pu );

		res = pw_mod( pr, pu, pv );

		if ( res != pw_OK )
			return res;

		pt = pu;
		pu = pv;
		pv = pr;
		pr = pt;
	}

	//assert( 0 );

	//return pw_InternalError;
}

//
//	Function:
//
//		 pw_mpi2int.
//
//	Converts Multi-Precision Integers buffer to pw_int
//
//	RFC 2440                 OpenPGP Message Format            November 1998
//	3.2. Multi-Precision Integers
//
//	if successful returns number of processed bytes.
//
int pw_mpi2int( pw_int *x, const BYTE *buffer, int bufferSize )
{
	if ( bufferSize <= 2 )
	{
		if (( bufferSize == 2 ) && !buffer[0] && !buffer[1] )
		{
			x->setu( 0 );
			return 2;
		}

		return 0;
	}

	int bits  = int(( (UINT)buffer[0] << 8 ) | buffer[1]);
	int bytes = ( bits + 7 ) / 8;

	if ( bufferSize < (bytes + 2) )
		return 0;

	if ( pw_bytes2int( x, buffer + 2, bytes ) != pw_OK )
		return 0;

	return (bytes + 2);
}

bool pw_readASN1Buffer( pw_int *x, const BYTE *pkt, int pktSize, int *pPos )
{
	x509_struct t;

	int res = t.parse( ( BYTE *) pkt + *pPos, pktSize - *pPos );
	if ( res != x509e_ok ) return false;

	*pPos += t.TL;

	if ( *pPos > pktSize )
		return false;

	switch ( t.T )
	{
		case asn1tag_INTEGER:
		case asn1tag_BIT_STRING:
		case asn1tag_OCTET_STRING:
			return pw_bytes2int( x, t.B, t.L ) == pw_OK;
		default :
			return false;
	}
}

bool pw_readMpiBuffer( pw_int *x, const BYTE *mpi, int mpiSize, int *pPos )
{
	int num = pw_mpi2int( x, mpi + *pPos, mpiSize - *pPos );

	if ( num == 0 )
		return false;

	*pPos += num;
	
	if ( *pPos > mpiSize )
		return false;

	return true;
}

int pw_getMpiSize( pw_int *x )
{
	return ( pw_getSizeInBits( x ) + 7 ) / 8 + 2;
}

int pw_getASN1BodySize( const pw_int *x )
{
	// We means here non-negative integers only
	// the new value one byte more for integer with alder bit
	// the alder byte will be set 00, to detect positive integer
	int bodySize = ( pw_getSizeInBits( x ) / 8 ) + 1;

	return bodySize;
}

int pw_getASN1Size( unsigned int bodySize )
{
	return x509_struct::makeHeader( bodySize ) + bodySize;
}

int pw_getASN1Size( pw_int *x )
{
//	int bodySize = ( pw_getSizeInBits( x ) + 7 ) / 8;
	
	unsigned int bodySize = pw_getASN1BodySize( x );

	return pw_getASN1Size( bodySize );
}

//
//	pw_int2mpi
//
//	Places pw_int to MPI byte buffer.
//	Returns number of bytes busied into the buffer.
//	Otherwise return zero.
//
//	Note, the pw_getMpiSize function can be used to detect size of 
//	 the required mpi buffer to be allocated before.
//
int pw_int2mpi( BYTE *buffer, int bufferSize, pw_int *x )
{
	unsigned int bits  = pw_getSizeInBits( x );
	int bytes = (bits + 7)/8;

	if ( ( bytes + 2) > bufferSize )
		return 0;

	buffer[0] = BYTE( bits >> 8 );
	buffer[1] = BYTE( bits );

	if ( pw_int2bytes( buffer + 2, bytes, x ) != pw_OK )
		return 0;

	return bytes + 2;
}

bool pw_writeMpiBuffer( BYTE *mpi, int mpiSize, int *pPos, const pw_int *x )
{
	int num = pw_int2mpi( mpi + *pPos, mpiSize - *pPos, (pw_int *)x );

	if ( num == 0 )
		return false;

	*pPos += num;
	
	if ( *pPos > mpiSize )
		return false;

	return true;
}

bool pw_writeASN1Buffer( BYTE *buff, int buffSize, int *pPos, const pw_int *x, int extendedBodySize /*= 0*/ )
{
//	unsigned int bits  = pw_getSizeInBits( x );
//	int bytes = (bits + 7)/8;

	int bytes = pw_getASN1BodySize( x );

	if ( extendedBodySize > bytes )
		bytes = extendedBodySize;

	int hdrSize = x509_struct::makeHeader( bytes );

	if ( *pPos + hdrSize + bytes > buffSize )
		return false;

	*pPos += x509_struct::makeHeader( bytes, asn1tag_INTEGER, buff + *pPos );

	if ( pw_int2bytes( buff + *pPos, bytes, x ) != pw_OK )
		return false;

	*pPos += bytes;

	return true;
}

int pw_mul_rec( pw_int *res, pw_int *px, pw_int *py )
{
	int lastX = pw_getLastLimb( px );
	int lastY = pw_getLastLimb( py );

	pw_int *x,*y;
	pw_int t;

	//
	//	If order y > order x => we get optimisation for y loop.
	//	otherwise we swap y with x.
	//
	if ( lastX > lastY )
	{	x = py;
		y = px;
		int tmp = lastX;
		lastX = lastY;
		lastY = tmp;
	}
	else
	{
		x = px;
		y = py;
	}

	int result;

	if ( lastX < OPTIMAL_CLASSIC_MUL_LIMB_COUNT )
		return pw_mul_def( res, x, y );

	if ( lastY == 0 )
	{
		if ( ( x->m_ullSize == 0 ) || ( y->m_ullSize == 0 )  )
			return res->setu( 0 );

		result = res->set( 2 * sizeof( ULONGLONG ) );
		if ( result != pw_OK ) return result;

		DWLMul( &res->data[0], &res->data[1], x->data[0], y->data[0] );
		return pw_OK;
	}

	int splitPos = ( lastY + 1 ) / 2;

	// bool xIsShort = lastX < splitPos;

	//
	//	n = splitPos * U64_BIT_Q
	//	x = u1*2^n + u0
	//	y = v1*2^n + v0
	//
	//	res = x * y = ( 2^2n + 2^n ) u1 v1  + 2^n( u1 - u0 )( v0 - v1 ) + ( 2^n + 1 ) u0 v0.
	//

	pw_int u1, v1, u0, v0;

	u1.setVirtual( x, splitPos );
	u0.setVirtual( x, 0, splitPos );

	v1.setVirtual( y, splitPos );
	v0.setVirtual( y, 0, splitPos );

	pw_int v1u1;
	result = pw_mul_rec( &v1u1, &v1, &u1 );
	if ( result != pw_OK ) return result;

	pw_int v0u0;
	result = pw_mul_rec( &v0u0, &v0, &u0 );
	if ( result != pw_OK ) return result;

	pw_int sU10,sV01;
	int sigU10, sigV01;

	sigU10 = pw_cmp( &u1, &u0 ) < 0;
	sigV01 = pw_cmp( &v0, &v1 ) < 0 ;	

	if ( sigU10 ) 	result = pw_sub( &sU10, &u0, &u1 );
	else			result = pw_sub( &sU10, &u1, &u0 );
	if ( result != pw_OK ) return result;

	if ( sigV01 ) 	result = pw_sub( &sV01, &v1, &v0 );
	else			result = pw_sub( &sV01, &v0, &v1 );
	if ( result != pw_OK ) return result;

	int sigUV = sigU10 ^ sigV01;
	pw_int sUV;
	result = pw_mul_rec( &sUV, &sU10, &sV01 );
	if ( result != pw_OK ) return result;

	//
	//	Accumulates the result. 
	//
	// Grows it for maximal size.
	//
	result = res->set( ( lastY + lastX + 2 ) * sizeof( ULONGLONG ) );
	if ( result != pw_OK ) return result;

	//
	//	res = x * y = ( 2^2n + 2^n ) u1 v1  + 2^n( u1 - u0 )( v0 - v1 ) + ( 2^n + 1 ) u0 v0.
	//
	// 1. u0 v0
	result = res->set( &v0u0 );
	if ( result != pw_OK ) return result;

	pw_int vRes;

	// 2.  2^n u0 v0
	vRes.setVirtual( res, splitPos );
	result = pw_add( &vRes, &vRes, &v0u0 );
	if ( result != pw_OK ) return result;

	// 3. 2^n u1 v1
//	vRes.setVirtual( res, splitPos );
	result = pw_add( &vRes, &vRes, &v1u1 );
	if ( result != pw_OK ) return result;

	// 4. 2^2n u1 v1
	vRes.setVirtual( res, splitPos * 2 );
	result = pw_add( &vRes, &vRes, &v1u1 );
	if ( result != pw_OK ) return result;

	//
	// 5. 2^n( u1 - u0 )( v0 - v1 )
	//
	vRes.setVirtual( res, splitPos );

	if ( !sigUV )
	{
		return pw_add( &vRes, &vRes, &sUV );
	}
	//
	//	otherwise we have to sub
	//

	//
	//	result cannot be negative..
	//
	assert( pw_cmp( &vRes, &sUV ) >= 0 );

	// So simple sub
	return pw_sub( &vRes, &vRes, &sUV );

}

int pw_mul( pw_int *res, pw_int *px, pw_int *py )
{
	if ( px == py )
		return pw_square_rec( res, px );

	return pw_mul_rec( res, px, py );
}


/////////////////
//
//	findDivider
//		finds divider starting from d
//
int findDividerU( ULONGLONG *r, ULONGLONG p0, ULONGLONG n )
{
	ULONGLONG maxN = (ULONGLONG)0x10000 * 0x10000;
    for ( ; maxN >= n ; n +=2 )
	{
		ULONGLONG P = p0;

		int dec = 0;
		while( P <= p0 )
		{
			P -= n;
			dec++;
		}

		if ( ( n & 0xfffE ) == 0 )
#ifdef _WIN32
			printf( "\r%I64X\r", n );
#else
			printf( "\r%llX\r", (unsigned long long)n );
#endif

		ULONGLONG rest = P % n;

		if ( rest == 0 )
		{
			// *r = P / n + dec;
			*r = n;
			return pw_OK;
		}
	}

	return pw_BufferSizeTooSmall;
}

int findDivider( pw_int *d, pw_int *p )
{
	int isPrime = pw_probab_prime( p, 25, NULL );

	if ( isPrime )
		printf( "The value looks as prime\n" );

	int bitQ = pw_getSizeInBits( p );

	int maxBitQ = bitQ / 2;

	pw_int number, n_minus_1, r;

	if ( pw_cmpu( d, 0 ) == 0 )
		number.setu( 2 );
	else
		number.set( d );

	//
	// 1) First prime is definitly 2..
	//
	if ( pw_cmpu( &number, 2  ) == 0 )
	{
		if ( pw_getBit( p, 0 ) != 1 )
		{
			d->setu( 2 ); 
			return pw_OK;
		}
	}

	//
	// 2) All others primes are even and start from 3.
	//
	pw_setBit( &number, 0 );

	if ( ( bitQ == 0x41 ) && ( p->data[0] < 0x1000 ) && ( pw_getSizeInBits( &number ) < 0x40 ) )
	{
		d->setu( 0 );
		return findDividerU( &(d->data[0]), p->data[0], number.data[0] );
	}
	int res;
	int i = 0;

    for (; maxBitQ >= pw_getSizeInBits( &number ) ;  )
    {
		if ( pw_OK != pw_subu( &n_minus_1, &number, 1) )
		  return pw_InternalError;

		if ( possibly_prime ( &number , &n_minus_1 ) )
        {
			if ( ( ++i % 10  ) == 0 )
#ifdef _WIN32
				printf( "\r%I64X  \r", number.data[0] );
#else
				printf( "\r%llX  \r", (unsigned long long)(number.data[0]) );
#endif

 			pw_mod( &r, p, &number );
        
			if ( pw_cmpu( &r, 0 ) == 0 )
			{
				pw_set( d, &number );
				return pw_OK;
			}
        }

        res = pw_addu( &number, &number, 2 );

		if ( res != pw_OK )
			return res;
    }

	return pw_BufferSizeTooSmall;
}
