/*******************************************************************
 *
 *	Copyright (c) 1994-2008 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: power.h
 *	Revision: Oct 6, 2008
 *		The DWORD values where changed to UINT, x64 machine means sizeof DWORD ( unsigned long  ) = 8
 *		but the compilation works with 4 byte UINTs instead.
 *	Created:
 *	Description: Large interger mathematic functions for certificate library.
 *
 *******************************************************************/

#ifndef POWER_H
#define POWER_H

#include "Windefs.h"
#include <stdlib.h>
#include <memory.h>
#include "../../SMemory/smemory.h"

#define U64_BIT_Q ( sizeof( ULONGLONG ) * 8 )

enum {
	pw_OK,
	pw_BufferSizeTooSmall,
	pw_MemoryAllocationError,
	pw_InternalError,
	pw_InvalidKeyBlock,
	pw_InversionError
};



struct pw_int
{
	int m_ullSize; // size in ULONGLONG's;
	ULONGLONG *data;

private:
	int virtualData;
public:

	int setVirtual( pw_int *v, int pos, int newSize = -1  );

	pw_int()
	{
		memset( this, 0, sizeof( pw_int ) );
	}

	int set( const pw_int *x );

	//////////////////////////////////////////////
	// pData BYTE buffer;
	// pDataSize size of the data buffer in bytes.
	pw_int( int pDataSize, BYTE *pData = NULL )
	{
		memset( this, 0, sizeof( pw_int ) );
		set( pDataSize, pData );
	}

	int set( int pDataSize, const BYTE *pData = NULL );

	int setu( ULONGLONG dwData )
	{
		int result = set( sizeof( ULONGLONG ) );
		if ( result != pw_OK ) return result;

		data[0] = dwData;

		return pw_OK;
	}

	void freeData()
	{
		if ( data )
		{
			clear();
			free( data );
			data = NULL;
		}
	}

	~pw_int()
	{
		if ( virtualData )
			return;

		freeData();
	}

	void clear()
	{
		for( int pos = m_ullSize; pos--; )
			data[ pos ] = 0;
	}

};

int pw_add( pw_int *res, pw_int *x, pw_int *y );
int pw_addu( pw_int *res, pw_int *x, ULONGLONG y );
int pw_addm( pw_int *res, pw_int *x, pw_int *y, pw_int *mod );
int pw_mul( pw_int *res, pw_int *x, pw_int *y );
int pw_mul_classic( pw_int *res, pw_int *px, pw_int *py );
//int pw_mulEx( pw_int *res, pw_int *x, pw_int *y );
int pw_mulu( pw_int *res, pw_int *x, ULONGLONG y );
int pw_mulm( pw_int *res, pw_int *x, pw_int *y, pw_int *mod );
int pw_powm( pw_int *res, pw_int *x, pw_int *exp, pw_int *mod, bool classic = false );
int pw_powmu( pw_int *res, pw_int *x, unsigned int pow, pw_int *mod );
int pw_powm_montgomery( pw_int *res, pw_int *x, pw_int *exp, pw_int *mod );
int pw_powm_montgomery_windows( pw_int *r, pw_int *x, pw_int *exp, pw_int *mod );
int pw_powm_windows( pw_int *res, pw_int *x, pw_int *exp, pw_int *mod );
int pw_cmp( pw_int *x, pw_int *y );
int pw_cmpu( pw_int *x, ULONGLONG y );
int pw_sub( pw_int *res, pw_int *x, pw_int *y );
int pw_subu( pw_int *res, pw_int *x, ULONGLONG y );
int pw_subm( pw_int *res, pw_int *x, pw_int *y, pw_int *mod );
int pw_div_exp2( pw_int *res, pw_int *x, int exp );
int pw_mul_exp2( pw_int *res, pw_int *x, int exp );
int pw_mod( pw_int *res, pw_int *x, pw_int *mod );

//
//	pw_int2byte:
//		Prepare byte buffer from pw_integer.
//		First buffer byte is most significant byte of integer (def.PKCS#1)
//
int pw_int2bytes( BYTE *byteData, UINT length, const pw_int *intData );

//
//	pw_bytes2int:
//		Prepare pw_integer from byte buffer.
//		First buffer byte is a most significant byte of integer (def.PKCS#1)
//
int pw_bytes2int( pw_int *intData, const BYTE *byteData , UINT length );
int pw_div(	pw_int *q, pw_int *r, pw_int *v, pw_int *d );
int pw_gcd( pw_int *gcd, pw_int *u, pw_int *v );


//
//	All the bit functions means zero based bit position
//
int pw_clearBit( pw_int *t, int bitNum );
int pw_getBit( pw_int *x, int bitNum );
int pw_setBit( pw_int *t, int bitNum );

int pw_setByte( pw_int *t, int byteNum, BYTE bt );

//
// pw_getLastLimb returns zero based position of last nonzero limb
//
int pw_getLastLimb( const pw_int *i );

//
//		pw_getLastBit
//	returns one based position of last bit
//		0 if x == zero
//		1 if x == 1
//
int pw_getLastBit( const pw_int *x );
int pw_set( pw_int *x, pw_int *y );

//ULONGLONG pw_getLimb( pw_int *n, int limbNum );
#define pw_getLimb( n, limbNum ) ( (n)->m_ullSize <= (limbNum) ? 0 : (n)->data[ (limbNum) ] )

//
//	pw_invertEx
//		if pw_ok 
//			1. x * xInv - m * mInv = 1 -> positive xInv value 
//			2. m * mInv - x * xInv = 1 -> negative xInv value
//
int pw_invertEx( pw_int *xInv,  pw_int *mInv, pw_int *x, pw_int *m, bool *pNegative  );
int pw_invert( pw_int *inv, pw_int *x, pw_int *mod );
int pw_getSizeInBits( const pw_int *x );
#define pw_getSizeInBytes( x ) ( ( pw_getSizeInBits( x ) + 7 ) / 8 )
int pw_random( pw_int *x, int s );
int pw_probab_prime (pw_int *n, int reps, int *count=NULL);
int pw_primeTableInit( int primeQuantity, cProgressActor *actor = NULL );
int pw_primeTableFree( );

//int pw_square( pw_int *res, pw_int *u );
//int pw_squarem( pw_int *res, pw_int *u, pw_int *mod );


//
//	Function:
//
//		 pw_mpi2int.
//
//	Converts Multi-Precision Integers buffer to pw_int
//
//	RFC 2440                 OpenPGP Message Format            November 1998
//	3.2. Multi-Precision Integers
//
//	if successful returns number of processed bytes.
//
int pw_mpi2int( pw_int *x, const BYTE *buffer, int bufferSize );
bool pw_readMpiBuffer( pw_int *x, const BYTE *mpi, int mpiSize, int *pPos );
bool pw_readASN1Buffer( pw_int *x, const BYTE *pkt, int pktSize, int *pPos );

//
//	Function:
//
//		pw_int2mpi
//
//	Places pw_int to MPI byte buffer.
//	Returns number of bytes busied into the buffer.
//	Otherwise return zero.
//
//	Note, the pw_getMpiSize function can be used to detect size of 
//	 the required mpi buffer to be allocated before.
//
int pw_int2mpi( BYTE *buffer, int bufferSize, pw_int *x );
int pw_getMpiSize( pw_int *x );

int pw_getASN1Size( unsigned int bodySize );
int pw_getASN1Size( pw_int *x );
int pw_getASN1BodySize( const pw_int *x );

bool pw_writeMpiBuffer( BYTE *mpi, int mpiSize, int *pPos, const pw_int *x );

//
//	pw_writeASN1Buffer
//		if extended body size is more then real body size the extended body will be filled by zero.
//
bool pw_writeASN1Buffer( BYTE *buff, int buffSize, int *pPos, const pw_int *x, int extendedBodySize = 0 );

#endif // POWER_H
