#include "Windefs.h"
#include "power.h"

//
//	Structure for signed number operations..
//	It is used for pw_invert and euclidExt.
//
//
struct pws_int
{
	pw_int mod;
	bool isNegative;

	int set( pw_int *x, bool negative = false )
	{
		isNegative = negative;
		return mod.set( x );
	}

	int setu( ULONGLONG x, bool negative = false )
	{
		isNegative = negative;
		return mod.setu( x );
	}

	int sets( pws_int *x )
	{
		isNegative = x->isNegative;
		return mod.set( &x->mod );
	}
};

//static int pws_add( pws_int *r, pws_int *x, pws_int *y )
//{
//	if ( x->isNegative == y->isNegative )
//	{
//		r->isNegative = x->isNegative;
//		return pw_add( &r->mod, &x->mod, &y->mod );
//	}
//	else
//	{
//		if ( pw_cmp( &x->mod, &y->mod ) >= 0 )
//		{
//			r->isNegative = x->isNegative;
//			return pw_sub( &r->mod, &x->mod, &y->mod );
//		}
//		else
//		{
//			r->isNegative = y->isNegative;
//			return pw_sub( &r->mod, &y->mod, &x->mod );
//		}
//	}
//}

static int pws_sub( pws_int *r, pws_int *x, pws_int *y )
{
	if ( x->isNegative != y->isNegative )
	{
		r->isNegative = x->isNegative;
		return pw_add( &r->mod, &x->mod, &y->mod );
	}
	else
	{
		if ( pw_cmp( &x->mod, &y->mod ) >= 0 )
		{
			r->isNegative = x->isNegative;
			return pw_sub( &r->mod, &x->mod, &y->mod );
		}
		else
		{
			r->isNegative = !y->isNegative;
			return pw_sub( &r->mod, &y->mod, &x->mod );
		}
	}
}

static int pws_mul( pws_int *r, pws_int *x, pws_int *y )
{
	r->isNegative = x->isNegative != y->isNegative;
	return pw_mul( &r->mod, &x->mod, &y->mod );
}

static int pws_div( pws_int *r, pws_int *x, pws_int *y )
{
	r->isNegative = x->isNegative != y->isNegative;
	return pw_div( &r->mod, NULL, &x->mod, &y->mod );
}

//
//	The Extended Euclidean Algorithm...
//	See description in Knuth volume 2, subject 4.5.2, page 367.
//
//	It computes vector ( u3, u2, u1 ),  U*u1 + V*u2 == u3 == gcd( U, V )
//
static int pw_euclidExt( pws_int *u3, pws_int *u2, pws_int *u1, pw_int *U, pw_int *V )
{

	pws_int 
		t[3], *pT[3],
		v[3], *pV[3],
		*pU[3],
		*pS[3],
		q;

	//
	//	Initialisation...
	//

	int i;

	for ( i = 3; i--; )
		pT[i] = &t[i];

	for ( i = 3; i--; )
		pV[i] = &v[i];

	pU[2] = u3;
	pU[1] = u2;
	pU[0] = u1;

	//
	//	Initial settings
	//
	pU[0]->setu( 1 );
	pU[1]->setu( 0 );
	pU[2]->set ( U );

	pV[0]->setu( 0 );
	pV[1]->setu( 1 );
	pV[2]->set ( V );

	int res;

	while ( pw_cmpu( &pV[2]->mod, 0 ) != 0 )
	{
		res = pws_div( &q, pU[2], pV[2] );
		if ( res != pw_OK )
			return res;

		for ( i = 3; i--; )
		{
			//
			//	t(i) = u(i) - v(i) * q;
			//
			res = pws_mul( pT[i], pV[i], &q );
			if ( res != pw_OK )
				return res;

			res = pws_sub( pT[i], pU[i], pT[i] );
			if ( res != pw_OK )
				return res;

			//
			//	Swap values
			//		u(i)<--v(i)
			//		v(i)<--t(i)
			//
			//	Really just change pointers
			//

			pS[i] = pU[i];
			pU[i] = pV[i];
			pV[i] = pT[i];
			pT[i] = pS[i];
		}
	}

	u3->sets( pU[2] );
	u2->sets( pU[1] );
	u1->sets( pU[0] );

	return pw_OK;
}

//
//	pw_invertEx
//		if pw_ok 
//			xInv * x == 1 mod m
//			mInv * m == 1 mod x
//
int pw_invertEx( pw_int *xInv,  pw_int *mInv, pw_int *x, pw_int *m, bool *pNegative )
{
	pws_int u[3];

	int res;

	if ( pw_cmpu( m, 0 ) == 0 )
		return pw_InternalError;

	if ( pw_cmpu( x, 0 ) == 0 )
		return pw_InternalError;

	res = pw_euclidExt( &u[2], &u[1], &u[0], x, m );
	if ( res != pw_OK )	return res;

	//
	//	Check if gcd == 1.
	//	Otherwise return pw_InversionError.
	//
	if ( u[2].isNegative  || ( pw_cmpu( &u[2].mod, 1 ) != 0 )  )
		return pw_InversionError;


	if ( mInv != NULL )
	{
		res = pw_set( mInv, &u[1].mod );
		if ( res != pw_OK )	return res;
	}

	if ( pNegative != NULL )
		*pNegative = u[0].isNegative;

	return pw_set( xInv, &u[0].mod );
}

int pw_invert( pw_int *inv, pw_int *x, pw_int *mod )
{
	bool isNegative = false;

	int res = pw_invertEx( inv, NULL, x, mod, &isNegative );
	if ( res != pw_OK )	return res;

	//
	//	if inv is negative
	//	so real inv = m - inv.
	//
	if ( isNegative ) 
		return pw_sub( inv, mod, inv );

	return pw_OK;
}
