#ifndef __cAlgElgamal__
#define __cAlgElgamal__

#include "ElGamal/ElGamal.h"

class cPublicElgamal : public cPKA
{
public:
	ElGamal_KeyHandle key;
 	int moduleSize;

	cPublicElgamal()
	{
		key = NULL;
		moduleSize = 0;
	}

	~cPublicElgamal()
	{
		if ( key )
			ElGamal_FreeKey( &key );
	}

#ifdef _WIN32
#pragma warning( disable : 4100 )
#endif

	static class cPKA *createAlg( BYTE *mpiBf, int bfSize )
	{
		cPublicElgamal *alg = new cPublicElgamal;

		alg->key = ElGamal_ReadPublicKeyFromMpiBuffer( mpiBf, bfSize );

		if ( alg->key == NULL )
		{
			delete alg;
			return NULL;
		}

		alg->id = 20;
		alg->moduleSize = ( ElGamal_GetModulusBitSize( alg->key ) + 7 ) / 8;

		return alg;

	}

	static int createAlg_X509( cPKA **pPKA, x509_publicKeyInfo *pki )
	{
		cPublicElgamal *alg = new cPublicElgamal;

		int res = ElGamal_ReadPublicKeyFromASN1Buffer( &(alg->key), pki );

		if ( res != x509e_ok )
		{
			delete alg;
			return res;
		}

		alg->id = 20;
		alg->moduleSize = ( ElGamal_GetModulusBitSize( alg->key ) + 7 ) / 8;

		return x509e_ok;

	}

	int getModuleSize()
	{
		return moduleSize;
	}

	int encrypt(
			BYTE *encBuff,		// OUT - MPI buffer of encrypted message
			int  *pEncBuffSize,	// IN  - pointer to size of allocated buffer.
								// OUT - pointer to required or busied size.
			const BYTE *messageBuff,	// IN  - formatted message buffer. 
								// This value is padded as described in PKCS-1
			int messageBuffSize	// Size of the message buffer to be encrypted. 
								// It has to be equal to module size in bytes 
			)
	{
		int res = ElGamal_Encrypt( key, encBuff, pEncBuffSize, messageBuff, messageBuffSize );

		if ( res == pw_BufferSizeTooSmall )
			return pkae_bufferSizeTooSmall;

		if ( res == pkae_randomBufferRequired )
			return res;

		if ( res != pw_OK )
			return pkae_internalError;

		return pkae_ok;
	}

	int verify(
			const BYTE *sigBuff,		// IN - signature MPI buffer 
			int  sigBuffSize,	// IN  - size of allocated buffer.
			const BYTE *digestBuff,	// IN  - formatted digest buffer. 
								// This value is padded as described in PKCS-1
			int digestBuffSize	// Size of the formatted digest buffer to be verified with signature. 
			)
	{
		int res = ElGamal_Verify( key, sigBuff, sigBuffSize, digestBuff, digestBuffSize );

		if ( res == pw_BufferSizeTooSmall )
			return pkae_bufferSizeTooSmall;

		if ( res != 0 )
			return pkae_invalidSignature;

		return pkae_ok;
	}

	int encrypt_X509(
			BYTE *encBuff,		// OUT - ASN.1 buffer of encrypted message
			int  *pEncBuffSize,	// IN  - pointer to size of allocated buffer.
								// OUT - pointer to required or busied size.
			const BYTE *messageBuff,	// IN  - formatted message buffer. 
								// This value is padded as described in PKCS-1
			int messageBuffSize	// Size of the message buffer to be encrypted. 
								// It has to be equal to module size in bytes 
			)
	{
		int res = ElGamal_Encrypt_X509( key, encBuff, pEncBuffSize, messageBuff, messageBuffSize );

		if ( res == pw_BufferSizeTooSmall )
			return pkae_bufferSizeTooSmall;

		if ( res != pw_OK )
			return pkae_internalError;

		return pkae_ok;
	}

	int verify_X509(
			const BYTE *sigBuff,		// IN - signature ASN.1 buffer 
			int  sigBuffSize,	// IN  - size of allocated buffer.
			const BYTE *digestBuff,	// IN  - formatted digest buffer. 
								// This value is padded as described in PKCS-1
			int digestBuffSize	// Size of the formatted digest buffer to be verified with signature. 
			)
	{
		int res = ElGamal_Verify_X509( key, sigBuff, sigBuffSize, digestBuff, digestBuffSize );

		if ( res == pw_BufferSizeTooSmall )
			return pkae_bufferSizeTooSmall;

		if ( res != 0 )
			return pkae_invalidSignature;

		return pkae_ok;
	}

	int writeKeyToASN1Buffer(
		BYTE *buff,		// buffer for the key, x509_publicKeyInfo
		int  *pBuffSize	// IN  - pointer to size of buffer,
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
	)
	{
		return ElGamal_WritePublicKeyToASN1Buffer( key, buff, pBuffSize );
	}

#ifdef _WIN32
#pragma warning( default : 4100 )
#endif

};

class cPrivateElgamal : public cPKA
{
	ElGamal_KeyHandle key;

	int moduleSize;

 public:

	cPrivateElgamal()
	{
		key = NULL;
		moduleSize = 0;
	}

	~cPrivateElgamal()
	{
		if ( key )
			ElGamal_FreeKey( &key );
	}

#ifdef _WIN32
#pragma warning( disable : 4100 )
#endif

	static class cPKA *createAlg( BYTE *mpiBf, int bfSize, BYTE *mpiPrivateBf, int privateBfSize  )
	{
		cPrivateElgamal *alg = new cPrivateElgamal;

		alg->key = ElGamal_ReadSecretKeyFromMpiBuffer( mpiBf, bfSize, mpiPrivateBf, privateBfSize );

		if ( alg->key == NULL )
		{
			delete alg;
			return NULL;
		}
	
		alg->id = 20;

		alg->moduleSize = ( ElGamal_GetModulusBitSize( alg->key ) + 7 ) / 8;

		return alg;

	}

	static int createAlg_X509( cPKA **pPKA, x509_privateKeyInfo *pki )
	{
		cPrivateElgamal *alg = new cPrivateElgamal;

		int res = ElGamal_ReadSecretKeyFromASN1Buffer( &(alg->key), pki );

		if ( res != x509e_ok )
		{
			delete alg;
			return res;
		}

		alg->id = 20;

		alg->moduleSize = ( ElGamal_GetModulusBitSize( alg->key ) + 7 ) / 8;

		return x509e_ok;

	}

	int getModuleSize()
	{
		return moduleSize;
	}

	int decrypt(
		BYTE *decBuff,		// OUT - buffer for decrypted message
		int  *pDecBuffSize,	// IN  - pointer to size of allocated buffer. ( required size == module size in byte )
							// OUT - pointer to required or busied size.
		const BYTE *encBuff,// IN  - encrypted MPI buffer. 
		int encBuffSize	
	)
	{
		int res = ElGamal_Decrypt( key, decBuff, pDecBuffSize, encBuff, encBuffSize );

		if ( res == pw_BufferSizeTooSmall )
			return pkae_bufferSizeTooSmall;

		if ( res != pw_OK )
			return pkae_internalError;

		return pkae_ok;
	}

	int sign(
		BYTE *sigBuff,		// OUT - signature MPI buffer 
		int  *pSigBuffSize,	// IN  - pointer pointer size of allocated buffer.
							// OUT - ointer to required or really busied size.
		const BYTE *digestBuff,	// IN  - formatted digest buffer to be signed. 
							// This value is padded as described in PKCS-1
		int digestBuffSize	// Size of the formatted digest buffer. 
							// It has to be equal to module size in bytes 
	)
	{
		int res = ElGamal_Sign( key, sigBuff, pSigBuffSize, digestBuff, digestBuffSize );

		if ( res == pw_BufferSizeTooSmall )
			return pkae_bufferSizeTooSmall;

		if ( res != pw_OK )
			return pkae_internalError;

		return pkae_ok;
	}

	static int generateKey(
		int	 keyLength,				// Key size in byte,
		BYTE *publicMpiBuff,		// Public key MPI buffer,
		int  *pPublicMpiBuffSize,	// IN  - pointer to size of MPI buffer,
									// OUT - pointer to required or busied size.
									// the value is filled even if the error 
									// pkae_bufferSizeTooSmall returned
		BYTE *secretMpiBuff,		// Secret key MPI buffer,
		int  *pSecretMpiBuffSize	// IN  - pointer to size of MPI buffer,
									// OUT - pointer to required or busied size.
									// the value is filled even if the error 
									// pkae_bufferSizeTooSmall returned
		)
	{
		return pkae_internalError;
	}

	int derivePublicKeyAlgorithm(
		HPKA *pHPka	// OUT - pointer to the derived public key algorithm.
		)
	{
		cPublicElgamal *publicAlg = new cPublicElgamal();

		if ( publicAlg == NULL )
			return pkae_memoryAllocation;

		publicAlg->key = ElGamal_DerivePublicKey( key );

		if ( publicAlg->key == NULL )
		{
			delete publicAlg;
			return pkae_memoryAllocation;
		}

		publicAlg->moduleSize = ( ElGamal_GetModulusBitSize( key ) + 7 ) / 8;

		*pHPka = publicAlg;

		return pkae_ok;
	}

	int writeKeyToASN1Buffer(
		BYTE *buff,		// buffer for the key, x509_privateKeyInfo
		int  *pBuffSize	// IN  - pointer to size of buffer,
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
	)
	{
		return ElGamal_WriteSecretKeyToASN1Buffer( key, buff, pBuffSize );
	}
#ifdef _WIN32
#pragma warning( default : 4100 )
#endif
};

#endif // __cAlgElgamal__
