#ifndef __cPKA__
#define __cPKA__

#include "../crtLibErrs.h"

class cPKA
{

 public:

	int id; // It is filled just after creation

	// Days and time field are used for generation valid fingerprint and key id from key handle.
	UINT time;
	WORD days;

	cPKA()
	{
		id = 0;
		time = 0;
		days = 0;
	}

	int getAlgID()
	{
		return id;
	}

	virtual int getModuleSize()
	{
		return 0;
	};

	//
	//	Size of the block to be encrypted,signed or verified
	//
	virtual int getBlockSize()
	{
		return getModuleSize();
	}

#ifdef _WIN32
#pragma warning( disable : 4100 )
#endif

	//
	//	Public key funcs
	//
	virtual int encrypt(
			BYTE *encBuff,		// OUT - MPI buffer of encrypted message
			int  *encBuffSize,	// IN  - pointer to size of allocated buffer.
								// OUT - pointer to required or busied size.
			const BYTE *messageBuff,
								// IN  - formatted message buffer. 
			int messageBuffSize	// Size of the message buffer to be encrypted. 
								// It has to be equal to module size in bytes 
			)
	{
		return pkae_internalError;
	}

	virtual int encrypt_X509(
			BYTE *encBuff,		// OUT - ASN.1 buffer of encrypted message
			int  *encBuffSize,	// IN  - pointer to size of allocated buffer.
								// OUT - pointer to required or busied size.
			const BYTE *messageBuff,
								// IN  - formatted message buffer. 
			int messageBuffSize	// Size of the message buffer to be encrypted. 
								// It has to be equal to module size in bytes 
			)
	{
		return pkae_internalError;
	}

	virtual int verify(
			const BYTE *sigBuff,// IN - signature MPI buffer 
			int  sigBuffSize,	// IN  - size of allocated buffer.
			const BYTE *digestBuff,
								// IN  - formatted digest buffer. 
			int digestBuffSize	// Size of the formatted digest buffer to be verified with signature. 
			)
	{
		return pkae_internalError;
	}

	virtual int verify_X509(
			const BYTE *sigBuff,// IN - signature ASN.1 buffer 
			int  sigBuffSize,	// IN  - size of allocated buffer.
			const BYTE *digestBuff,
								// IN  - formatted digest buffer. 
			int digestBuffSize	// Size of the formatted digest buffer to be verified with signature. 
			)
	{
		return pkae_internalError;
	}

	//
	//	Secret key funcs
	//
	virtual int decrypt(
		BYTE *decBuff,		// OUT - buffer for decrypted message
		int  *decBuffSize,	// IN  - pointer to size of allocated buffer.
							// OUT - pointer to required or busied size.
		const BYTE *encBuff,// IN  - encrypted MPI buffer. 
		int encBuffSize	
	)
	{
		return pkae_internalError;
	}

	virtual int decrypt_X509(
		BYTE *decBuff,		// OUT - buffer for decrypted message
		int  *decBuffSize,	// IN  - pointer to size of allocated buffer.
							// OUT - pointer to required or busied size.
		const BYTE *encBuff,// IN  - encrypted ASN.1 buffer. 
		int encBuffSize	
	)
	{
		return pkae_internalError;
	}

	virtual int sign(
		BYTE *sigBuff,		// OUT - signature MPI buffer 
		int  *sigBuffSize,	// IN  - pointer pointer size of allocated buffer.
							// OUT - ointer to required or really busied size.
		const BYTE *digestBuff,
							// IN  - formatted digest buffer to be signed. 
		int digestBuffSize	// Size of the formatted digest buffer. 
							// It has to be equal to module size in bytes 
	)
	{
		return pkae_internalError;
	}

	virtual int sign_X509(
		BYTE *sigBuff,		// OUT - signature ASN1. buffer 
		int  *sigBuffSize,	// IN  - pointer pointer size of allocated buffer.
							// OUT - ointer to required or really busied size.
		const BYTE *digestBuff,
							// IN  - formatted digest buffer to be signed. 
		int digestBuffSize	// Size of the formatted digest buffer. 
							// It has to be equal to module size in bytes 
	)
	{
		return pkae_internalError;
	}

	virtual ~cPKA()
	{
		id = 0;
//		assert( 0 );
	}

	virtual int writeKeyToMpiBuffer(
		BYTE *mpiBuff,		// MPI buffer for the key,

		int  *pMpiBuffSize	// IN  - pointer to size of MPI buffer,
							// OUT - pointer to required or busied size.
							// the value is filled even if the error 
							// pkae_bufferSizeTooSmall returned
	)
	{
		return pkae_internalError;
	}

	virtual int writeKeyToASN1Buffer(
		BYTE *buff,		// buffer for the key, x509_publicKeyInfo or x509_privateKeyInfo
		int  *pBuffSize	// IN  - pointer to size of buffer,
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
	)
	{
		return pkae_internalError;
	}

	virtual int derivePublicKeyAlgorithm(
		cPKA **pHPka	// OUT - pointer to the derived public key algorithm.
		)
	{	return pkae_internalError; 	}

	virtual int decrypt_X509(
			const BYTE *encBuff,// IN - encrypted ASN.1 buffer 
			int  encBuffSize,	// IN - size of encrypted buffer.
			BYTE *decBuff,		// OUT decrypted buffer. 
			int *pDecBuffSize 	// OUT required size of decrypted buffer. size of allocated buffer
			)
	{	return pkae_internalError; 	}

	virtual int decrypt(
			const BYTE *encBuff,// IN - signature MPI buffer 
			int  encBuffSize,	// IN  - size of allocated buffer.
			BYTE *decBuff,		// OUT decrypted buffer. 
			int *pDecBuffSize 	// OUT required size of decrypted buffer. size of allocated buffer
			)
	{	return pkae_internalError; 	}
#ifdef _WIN32
#pragma warning( default : 4100 )
#endif
};


#endif // __cPKA__
