/////////////////////////////////////////////////////////////////
//
//	Copyright (c) 1994-2002 Jetico, Inc., Finland
//	All rights reserved.
//
//	File: publicKeyAlgs.h
//	Revision: 
//	Created:
//	Description: 
//
/////////////////////////////////////////////////////////////////

#ifndef __PublicKeyAlgs__
#define __PublicKeyAlgs__

#ifndef BYTE
typedef unsigned char BYTE;
#endif

#include "../crtLibErrs.h"
#include "../x509pkts/x509_structs.h"

// see the declaration in ..\x509pkts\x509_structs.h
// typedef class cPKA * HPKA; // Handle to public/private key algorithm


/////////////////////////
//
//	CreateAlgorthm functions
//
//	PGP compatible
//		pka_createAlgorithm 
//		ska_createAlgorithm
//
//	X.509 compatible
//		pka_createAlgorithm_X509
//		ska_createAlgorithm_X509
//

int pka_createAlgorithm(
	HPKA *hPka,
	int algID,		// Identificator of public key algorithm
	BYTE *keyBuff,	// Public key MPI buffer,
	int keyBuffSize );

int ska_createAlgorithm(
	HPKA *hSka,
	int algID,				// Identificator of public key algorithm
	BYTE *mpiBuff,			// Public key MPI buffer,
	int mpiBuffSize,
	BYTE *privateMpiBuff,	// Private key MPI buffer,
	int privateMpiBuffSize

);

int pka_createAlgorithm_X509( HPKA *hPka, x509_publicKeyInfo *pki );
int ska_createAlgorithm_X509( HPKA *hSka, x509_privateKeyInfo *pki );

int ska_getAlgID( HPKA pSka );
int pka_getAlgID( HPKA pPka );

///////////////////////
//
//	Miscellanious functions
//
int pka_getModuleSize( HPKA hpka );
int ska_getModuleSize( HPKA hpka );

int pka_getBlockSize( HPKA hpka );
int ska_getBlockSize( HPKA hpka );

////////////////////////////////////////////
//
//	Public key specific functions...
//

int pka_encrypt(
	HPKA hpka,			// Handle of public key algorithm
	BYTE *encBuff,		// OUT - MPI buffer of encrypted message
	int  *encBuffSize,	// IN  - pointer to size of allocated buffer.
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
	BYTE *messageBuff,	// IN  - formatted message buffer. 
						// This value is padded as described in PKCS-1
	int messageBuffSize	// Size of the message buffer to be encrypted. 
						// It has to be equal to block size in bytes 
);

int pka_encrypt_X509(
	HPKA hpka,			// Handle of public key algorithm
	BYTE *encBuff,		// OUT - ASN.1 buffer of encrypted message
	int  *encBuffSize,	// IN  - pointer to size of allocated buffer.
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
	BYTE *messageBuff,	// IN  - formatted message buffer. 
	int messageBuffSize	// Size of the message buffer to be encrypted. 
						// It has to be equal to block size in bytes 
);

//
//	Function:
//		pka_verify
//	Verifies signed MPI buffer with digest..
//
int pka_verify(
	HPKA hpka,			// Handle of public key algorithm
	BYTE *sigBuff,		// IN - signature MPI buffer 
	int  sigBuffSize,	// IN  - size of allocated buffer.
	BYTE *digestBuff,	// IN  - formatted digest buffer. 
						// This value is padded as described in PKCS-1
	int digestBuffSize	// Size of the formatted digest buffer to be verified with signature. 
						// It has to be equal to module size in bytes 
);

int pka_verify_X509(
	HPKA hpka,			// Handle of public key algorithm
	BYTE *sigBuff,		// IN - signature ASN.1 buffer 
	int  sigBuffSize,	// IN  - size of allocated buffer.
	BYTE *digestBuff,	// IN  - formatted digest buffer. 
	int digestBuffSize	// Size of the formatted digest buffer to be verified with signature. 
						// It has to be equal to module size in bytes 
);

int pka_decrypt_X509(
	class cPKA *pka,	// Public key algorithm
	BYTE *decBuff,		// OUT decrypted buffer. 
	int *pDecBuffSize, 	// OUT required size of decrypted buffer. size of allocated buffer
	const BYTE *encBuff,// IN - encrypted ASN.1 buffer 
	int  encBuffSize	// IN - size of encrypted buffer.
);

int pka_decrypt(
	class cPKA *pka,	// Public key algorithm
	BYTE *decBuff,		// OUT decrypted buffer. 
	int *pDecBuffSize, 	// OUT required size of decrypted buffer. size of allocated buffer
	const BYTE *encBuff,// IN - signature MPI buffer 
	int  encBuffSize	// IN  - size of allocated buffer.
);

void pka_free( HPKA hpka );


/////////////////////////////////////////////////////
//
//	Secret key specific functions.
//

void ska_free( HPKA hPka );

int ska_decrypt(
	HPKA hSka,			// Handle of secret key algorithm
	BYTE *decBuff,		// OUT - buffer for decrypted message
	int  *decBuffSize,	// IN  - pointer to size of allocated buffer.
						// OUT - pointer to required or busied size.
	BYTE *encBuff,		// IN  - encrypted MPI buffer. 
	int encBuffSize	
);

int ska_decrypt_X509(
	HPKA hSka,			// Handle of secret key algorithm
	BYTE *decBuff,		// OUT - buffer for decrypted message
	int  *decBuffSize,	// IN  - pointer to size of allocated buffer.
						// OUT - pointer to required or busied size.
	BYTE *encBuff,		// IN  - encrypted ASN.1 buffer. 
	int encBuffSize	
);

int ska_sign(
	HPKA hSka,			// Handle of secret key algorithm
	BYTE *sigBuff,		// OUT - signature MPI buffer 
	int  *sigBuffSize,	// IN  - pointer pointer size of allocated buffer.
						// OUT - ointer to required or really busied size.
	BYTE *digestBuff,	// IN  - formatted digest buffer to be signed. 
						// This value is padded as described in PKCS-1
	int digestBuffSize	// Size of the formatted digest buffer. 
						// It has to be equal to block size in bytes 
);

//
//	Generates signature to ASN.1 OCTET STRING ( PKCS-1 requirement )
//	See ska_sign_X509_BIT_STRING
//
int ska_sign_X509(
	HPKA hSka,			// Handle of secret key algorithm
	BYTE *sigBuff,		// OUT - signature ASN.1 buffer 
	int  *sigBuffSize,	// IN  - pointer pointer size of allocated buffer.
						// OUT - ointer to required or really busied size.
	BYTE *digestBuff,	// IN  - formatted digest buffer to be signed. 
	int digestBuffSize	// Size of the formatted digest buffer. 
						// It has to be equal to block size in bytes 
);

//
//	Generates signature to ASN.1 BIT STRING ( RFC-2459 requirement )
//	Uses ska_sign_X509 ( with OCTET STRING ) to generate signature. 
//	Then converts it to BIT STRING.
//	Verify function allows both OCTET and BIT strings.
//
int ska_sign_X509_BIT_STRING(
	class cPKA *ska,	// Handle of secret key algorithm
	BYTE *sigBuff,		// OUT - signature ASN.1 buffer 
	int  *pSigBuffSize,	// IN  - pointer pointer size of allocated buffer.
						// OUT - pointer to required or really busied size.
	BYTE *digestBuff,	// IN  - formatted digest buffer to be signed. 
	int digestBuffSize	// Size of the formatted digest buffer. 
						// It has to be equal to module size in bytes 
);

///////////////////////////////////////////////////////
//	
//	3. Random buffer is required for several public key operations
//

int pka_setRandomBuffer( BYTE *buffer, int bufferSize );

///////////////////////////////////////////////////////
//
//	4. Key pairs generation function...
//		Note, random buffer has to be assigned before,
//		use the pka_setRandomBuffer function.
//
//	Function:
//		ska_generateKeyAndCreateAlgorithm
//	Generates new secret key and create algorithm. 
//	Please use ska_derivePublicKeyAlgorithm to create public key algorithm from secret
//

class cProgressActor;
void ska_RSA_generateSafePrime( bool safe );

int ska_generateKeyAndCreateAlgorithm(
	HPKA *hPka,		// Pointer to handler of new algorithm.
	int  algID,		// Identificator of public key algorithm
	int	 keyLength,	// Key size in byte
	cProgressActor *actor = NULL
	);

int ska_derivePublicKeyAlgorithm(
	HPKA *pHPka,	// OUT - pointer to the derived public key algorithm.
	HPKA hSka		// IN  - secret key algorithm.
	);


/////////////////////////////////////////////////////
//
//	5. Save key functions
//
//	PGP compatible
//		pka_writeKeyToMpiBuffer
//		ska_writeKeyToMpiBuffer
//
//	X.509 compatible
//		pka_writeKeyToASN1Buffer
//		ska_writeKeyToASN1Buffer
//
int pka_writeKeyToMpiBuffer(
	HPKA hPka,			// Handler of public key algorithm
						// The handler can be derived from secret key algorithm
						// See ska_derivePublicKeyAlgorithm.

	BYTE *mpiBuff,		// Public key MPI buffer,

	int  *pMpiBuffSize	// IN  - pointer to size of MPI buffer,
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
	);

int ska_writeKeyToMpiBuffer(
	HPKA hSka,			// Handler of secret key algorithm
						// The handler can be generated by
						// ska_generateKeyAndCreateAlgorithm.

	BYTE *mpiBuff,		// Secret key MPI buffer,

	int  *pMpiBuffSize	// IN  - pointer to size of MPI buffer,
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
	);

int pka_writeKeyToASN1Buffer(
		HPKA hPka,
		BYTE *buff,		// publicKeyInfo, see  x509_publicKeyInfo 
		int *pBuffSize	// IN  - pointer to size of buffer,
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
		);

int ska_writeKeyToASN1Buffer(
		HPKA hSka,
		BYTE *buff,		// privateKeyInfo, see  x509_privateKeyInfo 
		int *pBuffSize	// IN  - pointer to size of buffer,
						// OUT - pointer to required or busied size.
						// the value is filled even if the error 
						// pkae_bufferSizeTooSmall returned
		);

//
//	pka_generateLocalKeyID
//	1. Makes ASN1 buffer with public key
//	2. Generates digest of the buffer to output buffer
//
int pka_generateLocalKeyID(
		HPKA hPka,		// Public key algorithm.
		BYTE *buff,		// Output buffer for local key ID 
		int *pBuffSize,	// Buffer size required
		int hashID = 0	// if hash algorithm is undefined = 0, so pgphaSHA1 identifier is used. 
);

//
//	ska_generateLocalKeyID
//	1. Derives public key from the secret key
//	2. Uses pka_generateLocalKeyID to generate local key ID to output buffer
//	3. Frees the derived public key.
//
int ska_generateLocalKeyID(
		HPKA hSka,		// Secret key algorithm.
		BYTE *buff,		// Buffer for local key ID 
		int *pBuffSize	// Buffer size required
		);

#endif // __PublicKeyAlgs__
