/*
		Shamir secret sharing scheme...
		Decryption - D.R.Stinson CRYPTOGRAPHY Theory and Practice CRC Press 1995 (#11)...

 */
#ifndef __Shamir__
#define __Shamir__

#include "includes.h"
#include "fscrypt.h"

#define __MPN

#include "gmp.h"

typedef struct {
	mpz_t x,y;
} intPair;

typedef struct {
	mpz_t prime;
	intPair *array;		/* Array[total] of integers pair */
	size_t arraySize;
} SSShares;

FSCRYPTRESULT SSSAllocateShares( SSShares **s , size_t total );
FSCRYPTRESULT SSSFree( SSShares ** s);

/*
	Function : SSSGenerateShares
	1) Generate prime p more the key and total shares number;
	2) Generate (recNum - 1) degree polinom P(x) = Summ( Ai*(x exp i)) , where 0 <= i <= t-1;
		A0 - is the key, others Ai are random 
	3) Fill shares array by Xi and Yi where P(Xi) == Yi;
 */

FSCRYPTRESULT SSSGenerateShares(
	SSShares *s,		/* Previously allocated shares */
	size_t recNum ,		/* Required number of participants those can compute key */
	byte *key,			/* Byte buffer of key to be shared */
	size_t keySize		/* size of key */
	);

FSCRYPTRESULT SSSRestoreKey( SSShares *s , byte *key , size_t size );

#endif /*__Shamir__*/