/////////////////////////////////////////////////////////////////
//
//	Copyright (c) 1994-2005 Jetico, Inc., Finland
//	All rights reserved.
//
//	File: shamir.h
//	Created:		17.01.2005
//	Revision:	
//	Description:
//
//		Shamir secret sharing scheme...
//		Decryption - D.R.Stinson CRYPTOGRAPHY Theory and Practice CRC Press 1995 (#11)...
//
/////////////////////////////////////////////////////////////////

#ifndef _shamir_h_
#define _shamir_h_

#include "../../publicKeyAlgs/Power/power.h"

struct intPair{
	pw_int x, y;
};

struct SSShares{
	pw_int prime;
	intPair *array;		// Array[total] of integers pair
	size_t arraySize;
	size_t curPos; // SSSAddShare uses the value
};

int SSSAllocateShares( SSShares **s , size_t total );

// The x values may be assigned after share allocation by random or by order
bool prepareOrderedX( SSShares *s );
// If x values were not set the following function is called automatically.
// bool prepareRandomX( SSShares *s , size_t size , cSimpleRNG *rng );

int SSSFree( SSShares ** s);

class cProgressBlock
{
public:

	// Progress test result enum
	enum { stepResult_False_ByTable = -1, stepResult_False_ByPrimeTest = 0, stepResult_True = 1 };
	
#ifdef _WIN32
#pragma warning( disable : 4100 )
#endif
	//
	// SetCurrentStepResult
	//	Reports result of every step, see enum above.
	//	Returns false to stop the process, otherwise returns non zero.
	//
	virtual bool SetCurrentStepResult( int res ){ return true; }

	virtual void SetText( const char *text ) {}
#ifdef _WIN32
#pragma warning( default : 4100 )
#endif
};

//
//	Function : SSSGenerateShares
//	1) Generate prime p more the key and total shares number;
//	2) Generate (reqNum - 1) degree polinom P(x) = Summ( Ai*(x exp i)) , where 0 <= i <= t-1;
//		A0 - is the key, others Ai are random 
//	3) If X values were not assigned, it uses prepareRandomX function to fill the values.
//		The prepareOrderedX maybe used before to generate small x values.
//	4) Fill shares Y array, where Yi == P(Xi);
//
int SSSGenerateShares(
	SSShares *s,		// Previously allocated shares
	size_t reqNum ,		// Required number of participants those can compute key
	BYTE *key,			// BYTE buffer of key to be shared
	size_t keySize,		// size of key
	BYTE *seed,			// Seed buffer for random generator
	int *pSeedSize,		// if seed buffer is to small, it returns ppe_randomBufferRequired
	cProgressBlock *progress = NULL
	);

//
// Restore functions 
//
//	1. SSShares has to be allocated by SSSAllocateShares and module prime has to be assigned
//	2. Shares has to be colleced by SSSAddShare to SSShares array up to required number first
//	3. And then key may be restored by SSSRestoreKey
//	4. The alocated SSShares has to be freed by SSSFree
//

//
//	SSSAddShare
//		Adds decrypted share info to share's array to current position (s->curPos) and incremets
//			the position.
//
//		Return values:
//			1. Note, all x values are to be different! So it checks if same x value alredy 
//				presents in the table. If same value is found so it returns pw_InvalidKeyBlock.
//			2. If s->curPos < s->arraySize, so required shares are not accumuated yet,
//				so it returns pw_BufferSizeTooSmall.
//			3. If all required shares are properly filles it returns pw_OK.
//
//			Otherwise returns pw_InternalError.
//
int SSSAddShare( SSShares *s, pw_int *x, pw_int *y );

//
//	SSSRestoreKey
//		Restores key by collected shares
//
int SSSRestoreKey( SSShares *s , BYTE *key , size_t size );

#endif /* _shamir_h_ */
