/***************************************************************************************
 * File: bc_rijn.h
 *  
 * Purpose: Declarations of the Rijndael functions (ECB, CBC and CFB mode)
 *          We use it with 16-bytes blocks and 256-bit key
 *
 *  Rijndael algorithm's authors:
 *		Joan Daemen
 *		Vincent Rijmen
 *
 * Code in aes_tab.h, aes.h, aes.c files are written by Dr B. R. Gladman <brg@gladman.uk.net>
 *
 ***************************************************************************************/

#ifndef __BC_RIJNDAEL
#define __BC_RIJNDAEL

#include "../ALG_DEF.H"

#define SERVICE_NAME	"BC_RIJN"
#define DISPLAY_NAME    "RIJNDAEL"
#define ALGORITHM_ID     0xF0

#define DRIVER_MAJOR_VERSION	0
#define DRIVER_MINOR_VERSION	1


// Key length in bits - required for external definitions of 
// BestCrypt Algorithm modules
#define KEY_LENGTH 256

// number of bits per block
#define	BLOCK_SIZE 128


// We have to reserve space both for encryption and decryption keys,
// The following structure is the same as defined in aes.c, 
// excluding the fields that are not used in our case, when we 
// use a definite block size (16 bytes)

#define BC_BLOCK_SIZE 16

typedef struct
{
    UINT    Nkey;       // the number of 32-bit words in the key input block
    UINT    Nrnd;       // the number of cipher rounds
    UINT    e_key[4 * BC_BLOCK_SIZE];  // the encryption key schedule
    UINT    d_key[4 * BC_BLOCK_SIZE];  // the decryption key schedule
    BYTE     mode;       // encrypt, decrypt or both
} BC_AES_STRUCT;

// the value is in bits
#define KEY_LENGTH_EXTENDED (sizeof(BC_AES_STRUCT) * 8)


/******* Exported functions *********/

#define IN

BOOL
RIJN_KeyExtend(
		PUCHAR KeySource,
		PUINT KeyDestination );

/* 
   Encryption and decryption in Cipher Block Chaining (CBC) mode
*/

VOID
RIJN_Encrypt( UINT *IVector, 
         UINT *KeyAdress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes

VOID
RIJN_Decrypt( UINT *IVector, 
         UINT *KeyAdress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes

/* 
   Encryption and decryption in 32-bit Cipher-Feedback (CFB) mode
*/
VOID
RIJN_encrypt_CFB( UINT *IVector, 
             UINT *KeyAddress,
             UINT *SrcBuffer,
             UINT *DstBuffer,
             UINT Length );    /* in bytes */

VOID
RIJN_decrypt_CFB( UINT *IVector, 
             UINT *KeyAddress,
             UINT *SrcBuffer,
             UINT *DstBuffer,
             UINT Length );    /* in bytes */

//
//	ECB one block encryption/decryption
//
VOID
RIJN_encrypt_ECB( BYTE *KeyAddress,
             const BYTE *inBlock,
             BYTE *outBlock );

VOID
RIJN_decrypt_ECB( BYTE *KeyAddress,
             const BYTE *inBlock,
             BYTE *outBlock );

#endif
