#ifndef __BC_TWOFISH
#define __BC_TWOFISH

#include "../ALG_DEF.H"

#define SERVICE_NAME	"BC_TFISH"
#define DISPLAY_NAME    "TWOFISH"
#define ALGORITHM_ID     0xA0

#define DRIVER_MAJOR_VERSION	0
#define DRIVER_MINOR_VERSION	1

/* define LittleEndian for x86 platform */
#define LittleEndian 1

/* Twofish definitions */

#define		BLOCK_SIZE		128	/* number of bits per block */
#define		MAX_ROUNDS		16
#define		MAX_KEY_BITS	256	/* max number of bits of key */

#define		INPUT_WHITEN	(BLOCK_SIZE/32)
#define		OUTPUT_WHITEN	(BLOCK_SIZE/32)
#define		ROUND_SUBKEYS	(INPUT_WHITEN + OUTPUT_WHITEN)
#define		TOTAL_SUBKEYS	(ROUND_SUBKEYS + 2*MAX_ROUNDS)

typedef struct KeyInstance_
{
	UINT sboxKeys[MAX_KEY_BITS/64];/* key bits used for S-boxes */
	UINT subKeys[TOTAL_SUBKEYS];	/* round subkeys, input/output whitening bits */
	UINT sBox8x32[4][256];			/* fully expanded S-box */
} KeyInstance;


// Key length in bits - required for external definitions of 
// BestCrypt Algorithm modules
#define KEY_LENGTH          (MAX_KEY_BITS)
// We need to reserve space both for encryption and decryption keys (the data is in bits)
#define KEY_LENGTH_EXTENDED (sizeof(KeyInstance)*8)


/******* Exported functions *********/

#define IN

BOOL
TFish_KeyExtend( PUCHAR KeySource,
           PUINT KeyDestination );

/* 
   Encryption and decryption in Cipher Block Chaining (CBC) mode
*/

VOID
TFish_Encrypt( UINT *IVector, 
         UINT *KeyAdress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes

VOID
TFish_Decrypt( UINT *IVector, 
         UINT *KeyAdress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes

/* 
   Declare Twofish encrypt and decrypt functions in Electronic 
   Code Book (ECB) mode for testing the implementation - not for using
   in the BestCrypt Encryption Algorithm Module
 */

BOOL
TFish_blockEncrypt( UINT       *Block,
              KeyInstance *key );

BOOL
TFish_blockDecrypt( UINT       *Block,
              KeyInstance *key );


/* 
   Encryption and decryption in 32-bit Cipher-Feedback (CFB) mode
*/
VOID
TFish_encrypt_CFB( UINT *IVector, 
             UINT *KeyAddress,
             UINT *SrcBuffer,
             UINT *DstBuffer,
             UINT Length );    /* in bytes */

VOID
TFish_decrypt_CFB( UINT *IVector, 
             UINT *KeyAddress,
             UINT *SrcBuffer,
             UINT *DstBuffer,
             UINT Length );    /* in bytes */

#endif
