#ifndef __BC_BFISH
#define __BC_BFISH

#include "../ALG_DEF.H"

#define SERVICE_NAME	"BC_BFish"
#define DISPLAY_NAME    "BLOWFISH"
#define ALGORITHM_ID     0x80

#define BLOCK_SIZE 8

// Key length in bits
#define MIN_KEY_LENGTH 40
#define MAX_KEY_LENGTH 448

#define KEY_LENGTH          (256)
#define KEY_LENGTH_EXTENDED (4168*8)

#define DRIVER_MAJOR_VERSION	1
#define DRIVER_MINOR_VERSION	1


/******* Hardware dependences *******/

// Currently we choose Intel processor
#define ORDER_DCBA 

/* DCBA - little endian - Intel */
#ifdef ORDER_DCBA
union unionUINT 
 { UINT Uint;
   struct
	{ BYTE Byte3;
      BYTE Byte2;
      BYTE Byte1;
      BYTE Byte0;
    } Byte;
 };
#endif	/* ORDER_DCBA */

/* ABCD - big endian - Motorola */
#ifdef ORDER_ABCD
union unionUINT 
 { UINT Uint;
   struct
	{ BYTE Byte0;
      BYTE Byte1;
      BYTE Byte2;
      BYTE Byte3;
    } Byte;
 };
#endif	/* ORDER_ABCD */

/* BADC - VAX */
#ifdef ORDER_BADC
union unionUINT 
 { UINT Uint;
   struct
	{ BYTE Byte1;
      BYTE Byte0;
      BYTE Byte3;
      BYTE Byte2;
    } Byte;
 };
#endif	/* ORDER_BADC */


/******* Exported functions *********/

#define IN

BOOL
BFISH_KeyExtend( int    KeyLength,
		   PUCHAR KeySource,          // 32       8-bit uchars
           PUINT KeyDestination );   // (4168/4) 32-bit words

VOID
BFISH_EncryptCBC( UINT *IVector, 
         UINT *KeyAdress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes

VOID
BFISH_DecryptCBC( UINT *IVector, 
         UINT *KeyAdress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes

/************/

typedef union _BKEY { UINT Key[1042];
                      struct { UINT pArray[18];
                               UINT sArray[4][256];
                             } Array;
                    } BKEY, *PBKEY;

VOID
Blowfish_ECB_Encrypt( PUINT Left,
                      PUINT Right,
		              PBKEY  KeyAddress );

VOID
Blowfish_ECB_Decrypt( PUINT Left,
                      PUINT Right,
		              PBKEY  KeyAddress );

#endif
