#include "Bc_cast.h"
#include "BC_CASTI.H"
#ifdef _WIN32
#include <memory.h>
#else
#include <string.h>
#endif


/**********************************************************************
 *
 * Useful definitions
 *
 **********************************************************************/

#define ROL( X, n )	( ( ( X ) << ( n ) ) | ( ( X ) >> ( 32 - ( n ) ) ) )

/*  Macroses to transform array of 4 bytes to 32-bit words
 *  (and reverse) without depending on the Little-Endian or 
 *  Big-Endian processor's architecture 
 */
#define BYTES_TO_UINT(b,d) (d =((UINT)(*((b)++)))<<24, \
                               d|=((UINT)(*((b)++)))<<16, \
                               d|=((UINT)(*((b)++)))<<8,  \
                               d|=((UINT)(*((b)++))))

#define UINT_TO_4BYTES(d,b) (*((b)++)=(BYTE)(((d)>>24)&0xff), \
                                *((b)++)=(BYTE)(((d)>>16)&0xff), \
                                *((b)++)=(BYTE)(((d)>> 8)&0xff), \
                                *((b)++)=(BYTE)(((d)    )&0xff))

/**********************************************************************
 *
 * CAST core implementation
 *
 **********************************************************************/

 /*
  *  PRE_SUBKEY, FIND_SUBKEY and FIND_SUBKEYS macroses help to clarify the 
  *  procedures of the subkeys generation.
  *
  *  PRE_SUBKEY macros helps to calculate temporary variables for subkeys 
  *  generation 
  */
#define PRE_SUBKEY( Result, Parameter, S, I0, I1, I2, I3, I4) \
  ( Result = Parameter ^ cast_Sbox[ 4 ][ I0 ] ^ \
                         cast_Sbox[ 5 ][ I1 ] ^ \
                         cast_Sbox[ 6 ][ I2 ] ^ \
                         cast_Sbox[ 7 ][ I3 ] ^ \
                         cast_Sbox[ S ][ I4 ] )

#define FIND_SUBKEY( Subkey, S, I0, I1, I2, I3, I4) \
    Subkey = cast_Sbox[ 4 ][ I0 ] ^ \
             cast_Sbox[ 5 ][ I1 ] ^ \
             cast_Sbox[ 6 ][ I2 ] ^ \
             cast_Sbox[ 7 ][ I3 ] ^ \
             cast_Sbox[ S ][ I4 ]

#define FIND_SUBKEYS1( skIndex, bPtr ) \
    FIND_SUBKEY( subKeys[skIndex  ], 4, bPtr[0x8], bPtr[0x9], bPtr[0x7], bPtr[0x6], bPtr[0x2] ); \
    FIND_SUBKEY( subKeys[skIndex+1], 5, bPtr[0xA], bPtr[0xB], bPtr[0x5], bPtr[0x4], bPtr[0x6] ); \
    FIND_SUBKEY( subKeys[skIndex+2], 6, bPtr[0xC], bPtr[0xD], bPtr[0x3], bPtr[0x2], bPtr[0x9] ); \
    FIND_SUBKEY( subKeys[skIndex+3], 7, bPtr[0xE], bPtr[0xF], bPtr[0x1], bPtr[0x0], bPtr[0xC] )

#define FIND_SUBKEYS2( skIndex, bPtr ) \
    FIND_SUBKEY( subKeys[skIndex  ], 4, bPtr[0x3], bPtr[0x2], bPtr[0xC], bPtr[0xD], bPtr[0x8] ); \
    FIND_SUBKEY( subKeys[skIndex+1], 5, bPtr[0x1], bPtr[0x0], bPtr[0xE], bPtr[0xF], bPtr[0xD] ); \
    FIND_SUBKEY( subKeys[skIndex+2], 6, bPtr[0x7], bPtr[0x6], bPtr[0x8], bPtr[0x9], bPtr[0x3] ); \
    FIND_SUBKEY( subKeys[skIndex+3], 7, bPtr[0x5], bPtr[0x4], bPtr[0xA], bPtr[0xB], bPtr[0x7] )

#define FIND_SUBKEYS3( skIndex, bPtr ) \
    FIND_SUBKEY( subKeys[skIndex  ], 4, bPtr[0x3], bPtr[0x2], bPtr[0xC], bPtr[0xD], bPtr[0x9] ); \
    FIND_SUBKEY( subKeys[skIndex+1], 5, bPtr[0x1], bPtr[0x0], bPtr[0xE], bPtr[0xF], bPtr[0xC] ); \
    FIND_SUBKEY( subKeys[skIndex+2], 6, bPtr[0x7], bPtr[0x6], bPtr[0x8], bPtr[0x9], bPtr[0x2] ); \
    FIND_SUBKEY( subKeys[skIndex+3], 7, bPtr[0x5], bPtr[0x4], bPtr[0xA], bPtr[0xB], bPtr[0x6] )

#define FIND_SUBKEYS4( skIndex, bPtr ) \
    FIND_SUBKEY( subKeys[skIndex  ], 4, bPtr[0x8], bPtr[0x9], bPtr[0x7], bPtr[0x6], bPtr[0x3] ); \
    FIND_SUBKEY( subKeys[skIndex+1], 5, bPtr[0xA], bPtr[0xB], bPtr[0x5], bPtr[0x4], bPtr[0x7] ); \
    FIND_SUBKEY( subKeys[skIndex+2], 6, bPtr[0xC], bPtr[0xD], bPtr[0x3], bPtr[0x2], bPtr[0x8] ); \
    FIND_SUBKEY( subKeys[skIndex+3], 7, bPtr[0xE], bPtr[0xF], bPtr[0x1], bPtr[0x0], bPtr[0xD] )


/* One round of CAST encryption. There are 3 types of calculations
 * depending on the round number:
 * - Rounds 1, 4, 7, 10, 13, and 16 use f function Type 1.
 * - Rounds 2, 5, 8, 11, and 14 use f function Type 2.
 * - Rounds 3, 6, 9, 12, and 15 use f function Type 3.
 */
#define ROUND_TYPE1(Left, Right, Ks, Num, Tmp )   \
    Tmp   = ROL((Ks[Num] + Right), (Ks[Num+16])); \
    Left ^= ((cast_Sbox[ 0 ][ (Tmp>>24)&0xff ]  ^ \
              cast_Sbox[ 1 ][ (Tmp>>16)&0xff ]) - \
              cast_Sbox[ 2 ][ (Tmp>>8 )&0xff ]) + \
              cast_Sbox[ 3 ][ (Tmp    )&0xff ]

#define ROUND_TYPE2(Left, Right, Ks, Num, Tmp )    \
    Tmp   = ROL((Ks[Num] ^ Right), (Ks[Num+16]));  \
    Left ^= ((cast_Sbox[ 0 ][ (Tmp>>24)&0xff ]  -  \
              cast_Sbox[ 1 ][ (Tmp>>16)&0xff ]) +  \
              cast_Sbox[ 2 ][ (Tmp>>8 )&0xff ]) ^  \
              cast_Sbox[ 3 ][ (Tmp    )&0xff ]

#define ROUND_TYPE3(Left, Right, Ks, Num, Tmp )    \
    Tmp   = ROL((Ks[Num] - Right), (Ks[Num+16]));  \
    Left ^= ((cast_Sbox[ 0 ][ (Tmp>>24)&0xff ]  +  \
              cast_Sbox[ 1 ][ (Tmp>>16)&0xff ]) ^  \
              cast_Sbox[ 2 ][ (Tmp>>8 )&0xff ]) -  \
              cast_Sbox[ 3 ][ (Tmp    )&0xff ]

/**********************************************************************
 *
 *	CAST_KeyExtend() calculates extended key from a given key
 *
 **********************************************************************/

BOOL
CAST_KeyExtend( int   KeyLength,          // in bytes
			    BYTE  *KeySource,         // 8-bit uchars
                UINT *KeyDestination )   // 32-bit words
{
  /* keyByteStream and key32bStream arrays are used to avoid big-endian and
     little-endian problems */
  BYTE      keyByteStream[CAST_MAXIMUM_KEYSIZE_BYTES], *keyBytePtr, *kb, 
            tmpByteStream[CAST_MAXIMUM_KEYSIZE_BYTES], *tmpBytePtr, *tb;
  UINT     key32bStream[CAST_MAXIMUM_KEYSIZE_UINT],
            tmp32bStream[CAST_MAXIMUM_KEYSIZE_UINT];
  CastData  *castData;
  BYTE      *key;
  UINT     *subKeys;
  int       i, subKeyNum;

  /* Get addresses of an encryption key and an extended key
     that will be calculated from the encryption key */
  key = KeySource;
  castData = (CastData *)KeyDestination;
  subKeys   = castData->subKeys;

  /* Padding by zeros of extra bytes of a key if we use the key 
     which size is less than CAST_MAXIMUM_KEYSIZE_BYTES */
  memset(keyByteStream, 0, CAST_MAXIMUM_KEYSIZE_BYTES);
  memcpy(keyByteStream, key, KeyLength);

  /* Depending on the keyData->keySize value we will make
     12 or 16 rounds in the encrypt/decrypt operations 
   */
  if (KeyLength > MAX_KEYSIZE_FOR_12_ROUNDS)
      castData->make16rounds = TRUE;
  else
      castData->make16rounds = FALSE;

  /* Calculating of the CAST subkeys */
  kb = keyByteStream;
  tb = tmpByteStream;
  keyBytePtr = keyByteStream;
  tmpBytePtr = tmpByteStream;

  for ( i=0; i<(int)CAST_MAXIMUM_KEYSIZE_UINT; i++ )
  { BYTES_TO_UINT( keyBytePtr, key32bStream[i] );
  }

  subKeyNum = 0;

  for (i=0; i<2; i++)
  {
   /* Define 0-3 subkeys (16-19 on the second round) */
   keyBytePtr = keyByteStream;
   tmpBytePtr = tmpByteStream;

   PRE_SUBKEY( tmp32bStream[0],key32bStream[0],6,kb[0xD],kb[0xF],kb[0xC],kb[0xE],kb[0x8]);
   UINT_TO_4BYTES(tmp32bStream[0], tmpBytePtr); 

   PRE_SUBKEY( tmp32bStream[1],key32bStream[2],7,tb[0x0],tb[0x2],tb[0x1],tb[0x3],kb[0xA]);
   UINT_TO_4BYTES(tmp32bStream[1], tmpBytePtr);

   PRE_SUBKEY( tmp32bStream[2],key32bStream[3],4,tb[0x7],tb[0x6],tb[0x5],tb[0x4],kb[0x9]);
   UINT_TO_4BYTES(tmp32bStream[2], tmpBytePtr);

   PRE_SUBKEY( tmp32bStream[3],key32bStream[1],5,tb[0xA],tb[0x9],tb[0xB],tb[0x8],kb[0xB]);
   UINT_TO_4BYTES(tmp32bStream[3], tmpBytePtr);

   FIND_SUBKEYS1(subKeyNum, tb);
   subKeyNum += 4;
  
   /* Define 4-7 subkeys (20-23 on the second round) */
   keyBytePtr = keyByteStream;
   tmpBytePtr = tmpByteStream;

   PRE_SUBKEY( key32bStream[0],tmp32bStream[2],6,tb[0x5],tb[0x7],tb[0x4],tb[0x6],tb[0x0]);
   UINT_TO_4BYTES(key32bStream[0], keyBytePtr); 

   PRE_SUBKEY( key32bStream[1],tmp32bStream[0],7,kb[0x0],kb[0x2],kb[0x1],kb[0x3],tb[0x2]);
   UINT_TO_4BYTES(key32bStream[1], keyBytePtr);

   PRE_SUBKEY( key32bStream[2],tmp32bStream[1],4,kb[0x7],kb[0x6],kb[0x5],kb[0x4],tb[0x1]);
   UINT_TO_4BYTES(key32bStream[2], keyBytePtr);

   PRE_SUBKEY( key32bStream[3],tmp32bStream[3],5,kb[0xA],kb[0x9],kb[0xB],kb[0x8],tb[0x3]);
   UINT_TO_4BYTES(key32bStream[3], keyBytePtr);

   FIND_SUBKEYS2(subKeyNum, kb);
   subKeyNum += 4;
 
   /* Define 8-11 subkeys (24-27 on the second round) */
   keyBytePtr = keyByteStream;
   tmpBytePtr = tmpByteStream;

   PRE_SUBKEY( tmp32bStream[0],key32bStream[0],6,kb[0xD],kb[0xF],kb[0xC],kb[0xE],kb[0x8]);
   UINT_TO_4BYTES(tmp32bStream[0], tmpBytePtr);

   PRE_SUBKEY( tmp32bStream[1],key32bStream[2],7,tb[0x0],tb[0x2],tb[0x1],tb[0x3],kb[0xA]);
   UINT_TO_4BYTES(tmp32bStream[1], tmpBytePtr);

   PRE_SUBKEY( tmp32bStream[2],key32bStream[3],4,tb[0x7],tb[0x6],tb[0x5],tb[0x4],kb[0x9]);
   UINT_TO_4BYTES(tmp32bStream[2], tmpBytePtr);

   PRE_SUBKEY( tmp32bStream[3],key32bStream[1],5,tb[0xA],tb[0x9],tb[0xB],tb[0x8],kb[0xB]);
   UINT_TO_4BYTES(tmp32bStream[3], tmpBytePtr);

   FIND_SUBKEYS3(subKeyNum, tb);
   subKeyNum += 4;
 
   /* Define 12-15 subkeys (28-31 on the second round) */
   keyBytePtr = keyByteStream;
   tmpBytePtr = tmpByteStream;

   PRE_SUBKEY( key32bStream[0],tmp32bStream[2],6,tb[0x5],tb[0x7],tb[0x4],tb[0x6],tb[0x0]);
   UINT_TO_4BYTES(key32bStream[0], keyBytePtr);

   PRE_SUBKEY( key32bStream[1],tmp32bStream[0],7,kb[0x0],kb[0x2],kb[0x1],kb[0x3],tb[0x2]);
   UINT_TO_4BYTES(key32bStream[1], keyBytePtr);

   PRE_SUBKEY( key32bStream[2],tmp32bStream[1],4,kb[0x7],kb[0x6],kb[0x5],kb[0x4],tb[0x1]);
   UINT_TO_4BYTES(key32bStream[2], keyBytePtr);

   PRE_SUBKEY( key32bStream[3],tmp32bStream[3],5,kb[0xA],kb[0x9],kb[0xB],kb[0x8],tb[0x3]);
   UINT_TO_4BYTES(key32bStream[3], keyBytePtr);

   FIND_SUBKEYS4(subKeyNum, kb);
   subKeyNum += 4;
  } 

  /* 32-bit subkeys beginning from the 16-th subkey are used in CAST for 
     rotational operation. This operation performed on 32-bit numbers, 
	 hence, the maximum bits we need is 5 bits. Let's mask all other bits
	 in the 16 - 31 subkeys
   */
  for ( i=16; i<CAST_MAX_NUMBER_OF_ROUNDS*2; i++ )
        subKeys[i] &= 0x1f;
  
  return TRUE;
}


/****************************************************************
 *
 *	CastECB() does encrypting (encrypt==TRUE) or
 *	decrypting (encrypt==FALSE) for a block of data the size of
 *	cipher block (BLOCK_SIZE, given in bytes).
 *
 ****************************************************************/


BOOL 
CastECB( const BYTE  *InBlock, 
         BYTE  *OutBlock,
         UINT *ExtKey,
         BOOL  Encrypt)
{
  UINT  left, righ;
  const BYTE *dataPtr;
  CastData *castData;
  UINT  *subKeys, temp;

  dataPtr  = InBlock;
  castData = (CastData *)ExtKey;
  subKeys  = castData->subKeys;

  BYTES_TO_UINT(dataPtr, left);
  BYTES_TO_UINT(dataPtr, righ);

  if (Encrypt)
  { 
 	ROUND_TYPE1(left, righ, subKeys, 0,  temp );   /* 1  round */
    ROUND_TYPE2(righ, left, subKeys, 1,  temp );   /* 2  round */
    ROUND_TYPE3(left, righ, subKeys, 2,  temp );   /* 3  round */
    ROUND_TYPE1(righ, left, subKeys, 3,  temp );   /* 4  round */
    ROUND_TYPE2(left, righ, subKeys, 4,  temp );   /* 5  round */
    ROUND_TYPE3(righ, left, subKeys, 5,  temp );   /* 6  round */
    ROUND_TYPE1(left, righ, subKeys, 6,  temp );   /* 7  round */
    ROUND_TYPE2(righ, left, subKeys, 7,  temp );   /* 8  round */
    ROUND_TYPE3(left, righ, subKeys, 8,  temp );   /* 9  round */
    ROUND_TYPE1(righ, left, subKeys, 9,  temp );   /* 10 round */
    ROUND_TYPE2(left, righ, subKeys, 10, temp );   /* 11 round */
    ROUND_TYPE3(righ, left, subKeys, 11, temp );   /* 12 round */

    if (castData->make16rounds)
	{
      ROUND_TYPE1(left, righ, subKeys, 12, temp ); /* 13 round */
      ROUND_TYPE2(righ, left, subKeys, 13, temp ); /* 14 round */
      ROUND_TYPE3(left, righ, subKeys, 14, temp ); /* 15 round */
      ROUND_TYPE1(righ, left, subKeys, 15, temp ); /* 16 round */
	}
	temp = left; 
	left = righ;
	righ = temp;
  }
  else
  { 
	temp = left; 
	left = righ;
	righ = temp;
    if (castData->make16rounds)
	{
      ROUND_TYPE1(righ, left, subKeys, 15, temp ); /* 16 round */
      ROUND_TYPE3(left, righ, subKeys, 14, temp ); /* 15 round */
      ROUND_TYPE2(righ, left, subKeys, 13, temp ); /* 14 round */
      ROUND_TYPE1(left, righ, subKeys, 12, temp ); /* 13 round */
    }
	ROUND_TYPE3(righ, left, subKeys, 11, temp );   /* 12 round */
    ROUND_TYPE2(left, righ, subKeys, 10, temp );   /* 11 round */
    ROUND_TYPE1(righ, left, subKeys, 9,  temp );   /* 10 round */
    ROUND_TYPE3(left, righ, subKeys, 8,  temp );   /* 9  round */
    ROUND_TYPE2(righ, left, subKeys, 7,  temp );   /* 8  round */
    ROUND_TYPE1(left, righ, subKeys, 6,  temp );   /* 7  round */
    ROUND_TYPE3(righ, left, subKeys, 5,  temp );   /* 6  round */
    ROUND_TYPE2(left, righ, subKeys, 4,  temp );   /* 5  round */
    ROUND_TYPE1(righ, left, subKeys, 3,  temp );   /* 4  round */
    ROUND_TYPE3(left, righ, subKeys, 2,  temp );   /* 3  round */
    ROUND_TYPE2(righ, left, subKeys, 1,  temp );   /* 2  round */
    ROUND_TYPE1(left, righ, subKeys, 0,  temp );   /* 1  round */
  }

  BYTE *ptr  = OutBlock;
  UINT_TO_4BYTES(left, ptr);
  UINT_TO_4BYTES(righ, ptr);
  
  return TRUE;
}


//************ CAST CBC mode encryption **************

VOID
CAST_EncryptCBC( UINT *IVector, 
         UINT *KeyAddress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length )    // in bytes
{ UINT i;
  UINT left, right;
  UCHAR buf[8], *bytePtr;

  left  = IVector[0];
  right = IVector[1];

  for(i = 0; i < (Length >> 2); i = i+2)
    { 
	  // do EBC encryption of (Initial_Vector XOR Data)
      left  = SrcBuffer[i] ^ left;
      right = SrcBuffer[i+1] ^ right;

      bytePtr = buf;       UINT_TO_4BYTES(left, bytePtr);
      bytePtr = &(buf[4]); UINT_TO_4BYTES(right,bytePtr);

      CastECB( buf, buf, KeyAddress, TRUE );
            
      bytePtr = buf;       BYTES_TO_UINT(bytePtr,left);
      bytePtr = &(buf[4]); BYTES_TO_UINT(bytePtr,right);

      DstBuffer[i]   = left;
      DstBuffer[i+1] = right;	  
    }
}


//************ CAST CBC mode decryption **************

VOID
CAST_DecryptCBC( UINT *IVector, 
         UINT *KeyAddress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length )    // in bytes
{ UINT i;
  UINT left, right, ivectorL, ivectorR, oldSrcL, oldSrcR;
  UCHAR buf[8], *bytePtr;

  ivectorL = IVector[0];
  ivectorR = IVector[1];

  for(i = 0; i < (Length >> 2); i = i+2)
    { 
      left  = oldSrcL = SrcBuffer[i];
      right = oldSrcR = SrcBuffer[i+1];

	  // Encrypted Data -> new IV, 
	  // then do EBC decryption of Encrypted Data, 
      // then XOR decrypted data with old IV

      bytePtr = buf;       UINT_TO_4BYTES(left, bytePtr);
      bytePtr = &(buf[4]); UINT_TO_4BYTES(right,bytePtr);

      CastECB( buf, buf, KeyAddress, FALSE );

      bytePtr = buf;       BYTES_TO_UINT(bytePtr,left);
      bytePtr = &(buf[4]); BYTES_TO_UINT(bytePtr,right);

      DstBuffer[i]   = left  ^ ivectorL;
      DstBuffer[i+1] = right ^ ivectorR;	  
      
      ivectorL = oldSrcL;
      ivectorR = oldSrcR;
    }
}

