#ifndef __CAST_H__
#define __CAST_H__

#include "../ALG_DEF.H"

#define SERVICE_NAME	"BC_CAST"
#define DISPLAY_NAME    "CAST"
#define ALGORITHM_ID     0xE0

#define DRIVER_MAJOR_VERSION	0
#define DRIVER_MINOR_VERSION	1

#define BLOCK_SIZE 8

/* The CAST algorithm allows to define a variable key size.
   The following key sizes (in bytes) are possible:
   5,6,7,8,9,10,11,12,13,14,15,16
 */
#define CAST_DEFAULT_KEYSIZE_BYTES   16

#define CAST_MINIMUM_KEYSIZE_BYTES   5
#define CAST_MAXIMUM_KEYSIZE_BYTES   16

#define CAST_MAXIMUM_KEYSIZE_UINT (CAST_MAXIMUM_KEYSIZE_BYTES / sizeof(UINT))

/* The CAST algorithm uses 12 or 16 rounds depending on the key size used */
#define CAST_MAX_NUMBER_OF_ROUNDS    16

/* 1) For key sizes up to and including 10 bytes (i.e., 5, 6, 7, 8,
      9, and 10 bytes), the algorithm is exactly as specified in the 
	  algorithm description, but uses 12 rounds instead of 16;

   2) For key sizes greater than 10 bytes, the algorithm uses the full 16
      rounds;

   The following KEYSIZE values are in bytes.
*/
#define MAX_KEYSIZE_FOR_12_ROUNDS    10
#define MIN_KEYSIZE_FOR_16_ROUNDS    11

/* CastData structure describes a place in memory for expanded CAST key.
   The make16rounds field is necessary to define a correct number of rounds
   (12 or 16) during encrypt/decrypt operations.
 */
typedef struct CastData_ {
    UINT subKeys[CAST_MAX_NUMBER_OF_ROUNDS*2];
    BOOL make16rounds;
} CastData;


// Key length in bites - required for external definitions of 
// BestCrypt Algorithm modules
#define KEY_LENGTH          (CAST_MAXIMUM_KEYSIZE_BYTES * 8)

// We need to reserve space for extended key
#define KEY_LENGTH_EXTENDED (sizeof(CastData) * 8)


// Exported functions

BOOL
CAST_KeyExtend( int   KeyLength,          // in bytes
			    BYTE  *KeySource,         // 8-bit uchars
                UINT *KeyDestination );  // 32-bit words

//	CastECB() does encrypting (encrypt==TRUE) or
//	decrypting (encrypt==FALSE) for a block of data the size of
//	cipher block (BLOCK_SIZE, given in bytes).
BOOL 
CastECB( const BYTE  *InBlock, 
         BYTE  *OutBlock,
         UINT *ExtKey,
         BOOL  Encrypt);

VOID
CAST_EncryptCBC( UINT *IVector, 
         UINT *KeyAddress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes

VOID
CAST_DecryptCBC( UINT *IVector, 
         UINT *KeyAddress,
         UINT *SrcBuffer,
         UINT *DstBuffer,
         UINT Length );    // in bytes


#endif /* __CAST_H__ */

