/********************************************************************************************
 * File: bc_serp.c
 *
 * Purpose: Use the Serpent algorithm in CBC mode with 16 byte blocks and a 256-bit key.
 *
 * The Serpent algorithm's authors: Ross Anderson, Eli Biham and Lars Knudsen
 * More information about Serpent at: http://www.cl.cam.ac.uk/~rja14/serpent.html
 *
 * Dag Arne Osvik found a way to express Serpent S-Boxes as boolean functions and
 * Dr B. R. Gladman used this to implement the algorithm. This code is based on his
 * implementation.
 *
 * BestCrypt module by Michael Oestergaard Pedersen <michael@daimi.au.dk>
 *
 ********************************************************************************************/

//#include <windows.h>   /* the string MUST be commented if */
//                       /* we compile Windows 95 or NT drivers */

//#ifndef __WIN95__
//#include <windef.h>
//#else
//#include <basedef.h>
//#endif

#include "bc_serp.h"


void ECB_Encrypt(UINT *InBuffer, UINT *OutBuffer, UINT *KeyAddress)
{
	UINT a,b,c,d,e,f,g,h;
    UINT t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15;
    
    a = InBuffer[0]; b = InBuffer[1]; c = InBuffer[2]; d = InBuffer[3];

    k_xor( 0,a,b,c,d); sb0(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor( 1,e,f,g,h); sb1(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor( 2,a,b,c,d); sb2(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor( 3,e,f,g,h); sb3(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor( 4,a,b,c,d); sb4(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor( 5,e,f,g,h); sb5(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor( 6,a,b,c,d); sb6(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor( 7,e,f,g,h); sb7(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor( 8,a,b,c,d); sb0(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor( 9,e,f,g,h); sb1(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(10,a,b,c,d); sb2(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(11,e,f,g,h); sb3(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(12,a,b,c,d); sb4(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(13,e,f,g,h); sb5(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(14,a,b,c,d); sb6(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(15,e,f,g,h); sb7(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(16,a,b,c,d); sb0(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(17,e,f,g,h); sb1(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(18,a,b,c,d); sb2(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(19,e,f,g,h); sb3(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(20,a,b,c,d); sb4(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(21,e,f,g,h); sb5(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(22,a,b,c,d); sb6(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(23,e,f,g,h); sb7(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(24,a,b,c,d); sb0(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(25,e,f,g,h); sb1(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(26,a,b,c,d); sb2(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(27,e,f,g,h); sb3(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(28,a,b,c,d); sb4(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(29,e,f,g,h); sb5(e,f,g,h,a,b,c,d); rot(a,b,c,d); 
    k_xor(30,a,b,c,d); sb6(a,b,c,d,e,f,g,h); rot(e,f,g,h); 
    k_xor(31,e,f,g,h); sb7(e,f,g,h,a,b,c,d); k_xor(32,a,b,c,d); 
    
    OutBuffer[0] = a; OutBuffer[1] = b; OutBuffer[2] = c; OutBuffer[3] = d;
}


void ECB_Decrypt(UINT *InBuffer, UINT *OutBuffer, UINT *KeyAddress)
{
	UINT a,b,c,d,e,f,g,h;
    UINT t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16;
    
    a = InBuffer[0]; b = InBuffer[1]; c = InBuffer[2]; d = InBuffer[3];

    k_xor(32,a,b,c,d); ib7(a,b,c,d,e,f,g,h); k_xor(31,e,f,g,h);
    irot(e,f,g,h); ib6(e,f,g,h,a,b,c,d); k_xor(30,a,b,c,d);
    irot(a,b,c,d); ib5(a,b,c,d,e,f,g,h); k_xor(29,e,f,g,h);
    irot(e,f,g,h); ib4(e,f,g,h,a,b,c,d); k_xor(28,a,b,c,d);
    irot(a,b,c,d); ib3(a,b,c,d,e,f,g,h); k_xor(27,e,f,g,h);
    irot(e,f,g,h); ib2(e,f,g,h,a,b,c,d); k_xor(26,a,b,c,d);
    irot(a,b,c,d); ib1(a,b,c,d,e,f,g,h); k_xor(25,e,f,g,h);
    irot(e,f,g,h); ib0(e,f,g,h,a,b,c,d); k_xor(24,a,b,c,d);
    irot(a,b,c,d); ib7(a,b,c,d,e,f,g,h); k_xor(23,e,f,g,h);
    irot(e,f,g,h); ib6(e,f,g,h,a,b,c,d); k_xor(22,a,b,c,d);
    irot(a,b,c,d); ib5(a,b,c,d,e,f,g,h); k_xor(21,e,f,g,h);
    irot(e,f,g,h); ib4(e,f,g,h,a,b,c,d); k_xor(20,a,b,c,d);
    irot(a,b,c,d); ib3(a,b,c,d,e,f,g,h); k_xor(19,e,f,g,h);
    irot(e,f,g,h); ib2(e,f,g,h,a,b,c,d); k_xor(18,a,b,c,d);
    irot(a,b,c,d); ib1(a,b,c,d,e,f,g,h); k_xor(17,e,f,g,h);
    irot(e,f,g,h); ib0(e,f,g,h,a,b,c,d); k_xor(16,a,b,c,d);
    irot(a,b,c,d); ib7(a,b,c,d,e,f,g,h); k_xor(15,e,f,g,h);
    irot(e,f,g,h); ib6(e,f,g,h,a,b,c,d); k_xor(14,a,b,c,d);
    irot(a,b,c,d); ib5(a,b,c,d,e,f,g,h); k_xor(13,e,f,g,h);
    irot(e,f,g,h); ib4(e,f,g,h,a,b,c,d); k_xor(12,a,b,c,d);
    irot(a,b,c,d); ib3(a,b,c,d,e,f,g,h); k_xor(11,e,f,g,h);
    irot(e,f,g,h); ib2(e,f,g,h,a,b,c,d); k_xor(10,a,b,c,d);
    irot(a,b,c,d); ib1(a,b,c,d,e,f,g,h); k_xor( 9,e,f,g,h);
    irot(e,f,g,h); ib0(e,f,g,h,a,b,c,d); k_xor( 8,a,b,c,d);
    irot(a,b,c,d); ib7(a,b,c,d,e,f,g,h); k_xor( 7,e,f,g,h);
    irot(e,f,g,h); ib6(e,f,g,h,a,b,c,d); k_xor( 6,a,b,c,d);
    irot(a,b,c,d); ib5(a,b,c,d,e,f,g,h); k_xor( 5,e,f,g,h);
    irot(e,f,g,h); ib4(e,f,g,h,a,b,c,d); k_xor( 4,a,b,c,d);
    irot(a,b,c,d); ib3(a,b,c,d,e,f,g,h); k_xor( 3,e,f,g,h);
    irot(e,f,g,h); ib2(e,f,g,h,a,b,c,d); k_xor( 2,a,b,c,d);
    irot(a,b,c,d); ib1(a,b,c,d,e,f,g,h); k_xor( 1,e,f,g,h);
    irot(e,f,g,h); ib0(e,f,g,h,a,b,c,d); k_xor( 0,a,b,c,d);
    
    OutBuffer[0] = a; OutBuffer[1] = b; OutBuffer[2] = c; OutBuffer[3] = d;
}


void SERP_Encrypt_ECB( BYTE  *Input,        /* 8 bytes of input data              */
                  BYTE  *Output,       /* 8 bytes of output data             */
                  BYTE  *ExtendedKey)
{
	ECB_Encrypt( (UINT *)Input, (UINT *)Output, (UINT *)ExtendedKey );
}


void SERP_Decrypt_ECB( BYTE  *Input,        /* 8 bytes of input data              */
                  BYTE  *Output,       /* 8 bytes of output data             */
                  BYTE  *ExtendedKey)
{
	ECB_Decrypt( (UINT *)Input, (UINT *)Output, (UINT *)ExtendedKey );
}



/******************* Serpent encryption *******************/

VOID
Encrypt(UINT *IVector,
		UINT *KeyAddress,
		UINT *SrcBuffer,
		UINT *DstBuffer,
		UINT Length )    // in bytes
{
	UINT i,j;
	UINT ivLocal[4];

	for(i=0; i<4; i++) ivLocal[i] = IVector[i&1];
	for(i=0; Length > 0; Length -= 16, i=i+4)
	{
		// Use CFB mode to encrypt data of length less than a full block
		if (Length < 16)
		{
			ECB_Encrypt(ivLocal, ivLocal, KeyAddress);
			for (j = 0; j < Length; j++)
				((BYTE *)DstBuffer)[(i*4)+j] = ((BYTE *)SrcBuffer)[(i*4)+j] ^ ((BYTE *)ivLocal)[j];
			return;
		}

		ivLocal[0] ^= SrcBuffer[i  ];
		ivLocal[1] ^= SrcBuffer[i+1];
		ivLocal[2] ^= SrcBuffer[i+2];
		ivLocal[3] ^= SrcBuffer[i+3];

		// Encrypt data with Serpent
		ECB_Encrypt(ivLocal, ivLocal, KeyAddress);

		DstBuffer[i  ] = ivLocal[0];
		DstBuffer[i+1] = ivLocal[1];
		DstBuffer[i+2] = ivLocal[2];
		DstBuffer[i+3] = ivLocal[3];
	}
}

/******************* Serpent decryption *******************/

VOID
Decrypt(UINT *IVector,
		UINT *KeyAddress,
		UINT *SrcBuffer,
		UINT *DstBuffer,
		UINT Length )    // in bytes
{
	UINT i,j;
	UINT ivLocal[4], ivNew[4], buf[4];

	for(i=0; i<4; i++) ivLocal[i] = IVector[i&1];
	for(i=0; Length > 0; Length -= 16, i=i+4)
	{
		// Use CFB mode to decrypt data of length less than a full block
		if (Length < 16)
		{
			ECB_Encrypt(ivLocal, ivLocal, KeyAddress);
			for (j = 0; j < Length; j++) {
				((BYTE *)DstBuffer)[(i*4)+j] = ((BYTE *)SrcBuffer)[(i*4)+j] ^ ((BYTE *)ivLocal)[j];
			}
			return;
		}

		for(j=0; j<4; j++) buf[j] = ivNew[j] = SrcBuffer[i+j];
		ECB_Decrypt(buf, buf, KeyAddress);
		for(j=0; j<4; j++)
		{
			DstBuffer[i+j] = ivLocal[j] ^ buf[j];
			ivLocal[j] = ivNew[j];
		}
	}
}

/***************** Serpent Key Extension ******************/

BOOL
SERP_KeyExtend( PUCHAR KeySource,		// 32 bytes - 256-bit key
           PUINT KeyDestination )
{
	UINT i,lk,a,b,c,d,e,f,g,h;
    UINT t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15;

#if( KEY_LENGTH < 0 || KEY_LENGTH > 256 )
	return false;
#endif

    i = 0; lk = (KEY_LENGTH + 31) / 32;
    
    while(i < lk)
    {
        KeyDestination[i] = ((UINT *)KeySource)[i];
        i++;
    }

#if(KEY_LENGTH < 256)
    while(i < 8)
	{
        KeyDestination[i++] = 0;
	}

    i = KEY_LENGTH / 32; lk = 1 << KEY_LENGTH % 32; 

    KeyDestination[i] = (KeyDestination[i] & (lk - 1)) | lk;
#endif

    for(i = 0; i < 132; ++i)
    {
        lk = KeyDestination[i] ^ KeyDestination[i + 3] ^ KeyDestination[i + 5] 
                               ^ KeyDestination[i + 7] ^ 0x9e3779b9 ^ i;

        KeyDestination[i + 8] = (lk << 11) | (lk >> 21); 
    }

    k_set( 0,a,b,c,d);sb3(a,b,c,d,e,f,g,h);k_get( 0,e,f,g,h);
    k_set( 1,a,b,c,d);sb2(a,b,c,d,e,f,g,h);k_get( 1,e,f,g,h);
    k_set( 2,a,b,c,d);sb1(a,b,c,d,e,f,g,h);k_get( 2,e,f,g,h);
    k_set( 3,a,b,c,d);sb0(a,b,c,d,e,f,g,h);k_get( 3,e,f,g,h);
    k_set( 4,a,b,c,d);sb7(a,b,c,d,e,f,g,h);k_get( 4,e,f,g,h);
    k_set( 5,a,b,c,d);sb6(a,b,c,d,e,f,g,h);k_get( 5,e,f,g,h);
    k_set( 6,a,b,c,d);sb5(a,b,c,d,e,f,g,h);k_get( 6,e,f,g,h);
    k_set( 7,a,b,c,d);sb4(a,b,c,d,e,f,g,h);k_get( 7,e,f,g,h);
    k_set( 8,a,b,c,d);sb3(a,b,c,d,e,f,g,h);k_get( 8,e,f,g,h);
    k_set( 9,a,b,c,d);sb2(a,b,c,d,e,f,g,h);k_get( 9,e,f,g,h);
    k_set(10,a,b,c,d);sb1(a,b,c,d,e,f,g,h);k_get(10,e,f,g,h);
    k_set(11,a,b,c,d);sb0(a,b,c,d,e,f,g,h);k_get(11,e,f,g,h);
    k_set(12,a,b,c,d);sb7(a,b,c,d,e,f,g,h);k_get(12,e,f,g,h);
    k_set(13,a,b,c,d);sb6(a,b,c,d,e,f,g,h);k_get(13,e,f,g,h);
    k_set(14,a,b,c,d);sb5(a,b,c,d,e,f,g,h);k_get(14,e,f,g,h);
    k_set(15,a,b,c,d);sb4(a,b,c,d,e,f,g,h);k_get(15,e,f,g,h);
    k_set(16,a,b,c,d);sb3(a,b,c,d,e,f,g,h);k_get(16,e,f,g,h);
    k_set(17,a,b,c,d);sb2(a,b,c,d,e,f,g,h);k_get(17,e,f,g,h);
    k_set(18,a,b,c,d);sb1(a,b,c,d,e,f,g,h);k_get(18,e,f,g,h);
    k_set(19,a,b,c,d);sb0(a,b,c,d,e,f,g,h);k_get(19,e,f,g,h);
    k_set(20,a,b,c,d);sb7(a,b,c,d,e,f,g,h);k_get(20,e,f,g,h);
    k_set(21,a,b,c,d);sb6(a,b,c,d,e,f,g,h);k_get(21,e,f,g,h);
    k_set(22,a,b,c,d);sb5(a,b,c,d,e,f,g,h);k_get(22,e,f,g,h);
    k_set(23,a,b,c,d);sb4(a,b,c,d,e,f,g,h);k_get(23,e,f,g,h);
    k_set(24,a,b,c,d);sb3(a,b,c,d,e,f,g,h);k_get(24,e,f,g,h);
    k_set(25,a,b,c,d);sb2(a,b,c,d,e,f,g,h);k_get(25,e,f,g,h);
    k_set(26,a,b,c,d);sb1(a,b,c,d,e,f,g,h);k_get(26,e,f,g,h);
    k_set(27,a,b,c,d);sb0(a,b,c,d,e,f,g,h);k_get(27,e,f,g,h);
    k_set(28,a,b,c,d);sb7(a,b,c,d,e,f,g,h);k_get(28,e,f,g,h);
    k_set(29,a,b,c,d);sb6(a,b,c,d,e,f,g,h);k_get(29,e,f,g,h);
    k_set(30,a,b,c,d);sb5(a,b,c,d,e,f,g,h);k_get(30,e,f,g,h);
    k_set(31,a,b,c,d);sb4(a,b,c,d,e,f,g,h);k_get(31,e,f,g,h);
    k_set(32,a,b,c,d);sb3(a,b,c,d,e,f,g,h);k_get(32,e,f,g,h);

    return TRUE;
}

