/*
	Copyright (c) 2000, Jetico, Inc.
	All rights reserved.

	File: test_des.c
	Revision: $Revision
	Created:
	Description: Test procedure for the DES and Triple-DES encryption algorithms.
*/

//#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stddef.h>
#include <conio.h>

#include "../cAlgDes.h"
#include "../cAlg3des.h"


#define CHECK_SUCCESS             0
#define CHECK_ERROR_FILE_OPEN     1
#define CHECK_ERROR_INIT_KEY      2
#define CHECK_ERROR_IN_OPERATION  3
#define CHECK_ERROR_IN_ENCRYPTION 4
#define CHECK_ERROR_IN_DECRYPTION 5


/* Extracts 1 byte from ASCII string '0123456789ABCDEF' */
static BYTE *getbyte(unsigned char *output, unsigned char *input)
{ unsigned char c;
  int ch;

  while((c=*input++)==' ') ;
  ch=toupper((int)c);
  ch-='0';
  if(ch>9) ch-=7;
  *output=ch*16;
  ch=toupper((int)*input++);
  ch-='0';
  if(ch>9) ch-=7;
  *output += ch;
  return(input);
}


/* Extracts 8 bytes from ASCII string '0123456789ABCDEF' */
static BYTE *parse(unsigned char *text, unsigned char *output, int byteLength)
{ int i;

  for(i=0;i<byteLength;++i) text = getbyte(output+i,text);
  return(text);
}

/* 
 * Function 'checkDesImplementation()' reads the file 
 * 'TVectors_des.txt' with the test vectors for DES algorithms,
 * calls key extension and encrypt/decrypt DES functions,
 * and verifies results
 */

static int checkDesImplementation()
{
  unsigned char   string[128];
  unsigned char   plainText[8];  /* stores a plaintext  from Test Vectors */
  unsigned char   cipherText[8]; /* stores a cryptotext from Test Vectors */
  unsigned char   output[8];     /* result we get when our implementation works */
  unsigned char   keybuf[8];
  int             keyLength;
  FILE            *fp;
  unsigned char   *cp;
  int             linecount=0, testcount=0, errors=0;

  fp = fopen("tvectors_des.txt","rt");
   
  if(!fp) return CHECK_ERROR_FILE_OPEN;

  while(fgets((char *)string,127,fp))
  {
     ++linecount;
     if(string[0]==';' || string[0]=='#') continue;
     ++testcount;
     cp=string;
     cp=parse(cp,keybuf,8);
     keyLength = 8;
     cp=parse(cp,plainText,8);
     cp=parse(cp,cipherText,8);

     cAlgDes *alg = (cAlgDes *)cAlgDes::createAlg( keybuf, sizeof(keybuf) );
     if (!alg) return CHECK_ERROR_INIT_KEY;

     /* Encryption */
     if (alg->encryptECB( plainText, output, 8 ) != csae_ok )
     {
	     delete alg;
	     return CHECK_ERROR_IN_ENCRYPTION;
     }

     /* Compare result with the correspondent test vector */
     if( memcmp( cipherText, output, 8 ) != 0)
     {
	     delete alg;
         return CHECK_ERROR_IN_ENCRYPTION;
	 }

     /* Decryption */
     if (alg->decryptECB( cipherText, output, 8 )  != csae_ok )
     {
	     delete alg;
	     return CHECK_ERROR_IN_DECRYPTION;
     }

     /* Compare result with the correspondent test vector */
     if( memcmp( plainText, output, 8 ) != 0)
     {
	     delete alg;
         return CHECK_ERROR_IN_DECRYPTION;
	 }
 
	  delete alg;
  }
  fclose(fp);
  return CHECK_SUCCESS;
}


/* 
 * Function 'check3DesImplementation()' reads the file 
 * 'TVectors_3des.txt' with the test vectors for DES algorithms,
 * calls key extension and encrypt/decrypt DES functions,
 * and verifies results
 */


static int check3DesImplementation()
{
  unsigned char   string[128];
  unsigned char   plainText[8];  /* stores a plaintext  from Test Vectors */
  unsigned char   cipherText[8]; /* stores a cryptotext from Test Vectors */
  unsigned char   output[8];     /* result we get when our implementation works */
  unsigned char   keybuf[24];
  int             keyLength;
  FILE            *fp;
  unsigned char   *cp;
  int             linecount=0, testcount=0, errors=0;

  fp = fopen("tvectors_3des.txt","rt");
   
  if(!fp) return CHECK_ERROR_FILE_OPEN;

  while(fgets((char *)string,127,fp))
  {
     ++linecount;
     if(string[0]==';' || string[0]=='#') continue;
     ++testcount;
     cp=string;
     cp=parse(cp,keybuf,24);
     keyLength = 8;
     cp=parse(cp,plainText,8);
     cp=parse(cp,cipherText,8);

     cAlg3des *alg = (cAlg3des *)cAlg3des::createAlg( keybuf, sizeof(keybuf) );
     if (!alg) return CHECK_ERROR_INIT_KEY;

     /* Encryption */
     if (alg->encryptECB( plainText, output, 8 ) != csae_ok )
     {
	     delete alg;
	     return CHECK_ERROR_IN_ENCRYPTION;
     }

     /* Compare result with the correspondent test vector */
     if( memcmp( cipherText, output, 8 ) != 0)
     {
	     delete alg;
         return CHECK_ERROR_IN_ENCRYPTION;
	 }

     /* Decryption */
     if (alg->decryptECB( cipherText, output, 8 ) != csae_ok )
     {
	     delete alg;
	     return CHECK_ERROR_IN_DECRYPTION;
     }

     /* Compare result with the correspondent test vector */
     if( memcmp( plainText, output, 8 ) != 0)
     {
	     delete alg;
         return CHECK_ERROR_IN_DECRYPTION;
	 }
 
	  delete alg;
  }

  fclose(fp);
  return CHECK_SUCCESS;
}


static int check3DesCBCimplementation()
{
  unsigned char   plainText[] = "0123456789abcdef";
  unsigned char   iv[] = "01234567"; /* stores Initial Vector */
  unsigned char   output[16];        /* result we get when our implementation works */
  unsigned char   keybuf[] = "012345670123456701234567";

  cAlg3des *alg = (cAlg3des *)cAlg3des::createAlg( keybuf, sizeof(keybuf) );
  if (!alg) return CHECK_ERROR_INIT_KEY;

  /* Encryption */
  if (alg->initCFB( iv ) != csae_ok )
  {
	  delete alg;
	  return CHECK_ERROR_INIT_KEY;
  }

  if (alg->encryptCFB( plainText, output, 16 ) != csae_ok )
  {
	  delete alg;
	  return CHECK_ERROR_IN_ENCRYPTION;
  }

  /* Decryption */
  if (alg->initCFB( iv ) != csae_ok )
  {
	  delete alg;
	  return CHECK_ERROR_INIT_KEY;
  }

  if (alg->decryptCFB( output, output, 16 ) != csae_ok )
  {
	  delete alg;
	  return CHECK_ERROR_IN_DECRYPTION;
  }

  /* Compare result with initial data */
  if( memcmp( plainText, output, 8 ) != 0)
  {
	  delete alg;
      return CHECK_ERROR_IN_DECRYPTION;
  }

  delete alg;
  return CHECK_SUCCESS;
}


int Test_3DES()
{
  int iRes;
  char ch;
  
  /******** Testing DES implementation **********/
  
  iRes = checkDesImplementation(); 

  switch( iRes )
  {
    case CHECK_SUCCESS:
		printf( "Checking DES implementation: Successfully Passed\n" );
		break;
    case CHECK_ERROR_FILE_OPEN:
		printf( "Checking DES implementation: ERROR IN OPENING FILE\n" );
		break;
    case CHECK_ERROR_INIT_KEY:
		printf( "Checking DES implementation: ERROR IN KEY INITIALIZATION\n" );
		break;
    case CHECK_ERROR_IN_OPERATION:
		printf( "Checking DES implementation: ERROR IN OPERATION\n" );
		break;
    case CHECK_ERROR_IN_ENCRYPTION:
		printf( "Checking DES implementation: ERROR IN ENCRYPTION\n" );
		break;
    case CHECK_ERROR_IN_DECRYPTION:
		printf( "Checking DES implementation: ERROR IN DECRYPTION\n" );
		break;
	default:
		printf( "Checking DES implementation: Undefined error.\n" );
		break;
  }

  /******** Testing Triple-DES implementation **********/

  iRes = check3DesImplementation(); 

  switch( iRes )
  {
    case CHECK_SUCCESS:
		printf( "Checking 3-DES implementation: Successfully Passed\n" );
		break;
    case CHECK_ERROR_FILE_OPEN:
		printf( "Checking 3-DES implementation: ERROR IN OPENING FILE\n" );
		break;
    case CHECK_ERROR_INIT_KEY:
		printf( "Checking 3-DES implementation: ERROR IN KEY INITIALIZATION\n" );
		break;
    case CHECK_ERROR_IN_OPERATION:
		printf( "Checking 3-DES implementation: ERROR IN OPERATION\n" );
		break;
    case CHECK_ERROR_IN_ENCRYPTION:
		printf( "Checking 3-DES implementation: ERROR IN ENCRYPTION\n" );
		break;
    case CHECK_ERROR_IN_DECRYPTION:
		printf( "Checking 3-DES implementation: ERROR IN DECRYPTION\n" );
		break;
	default:
		printf( "Checking 3-DES implementation: Undefined error.\n" );
		break;
  }

  /******** Testing Triple-DES in CBC mode implementation **********/

  iRes = check3DesCBCimplementation();

  switch( iRes )
  {
    case CHECK_SUCCESS:
		printf( "Checking 3-DES implementation: Successfully Passed\n" );
		break;
    case CHECK_ERROR_FILE_OPEN:
		printf( "Checking 3-DES implementation: ERROR IN OPENING FILE\n" );
		break;
    case CHECK_ERROR_INIT_KEY:
		printf( "Checking 3-DES implementation: ERROR IN KEY INITIALIZATION\n" );
		break;
    case CHECK_ERROR_IN_OPERATION:
		printf( "Checking 3-DES implementation: ERROR IN OPERATION\n" );
		break;
    case CHECK_ERROR_IN_ENCRYPTION:
		printf( "Checking 3-DES implementation: ERROR IN ENCRYPTION\n" );
		break;
    case CHECK_ERROR_IN_DECRYPTION:
		printf( "Checking 3-DES implementation: ERROR IN DECRYPTION\n" );
		break;
	default:
		printf( "Checking 3-DES implementation: Undefined error.\n" );
		break;
  }

  printf("Press any key to continue...\n");
  ch = getch();

  return 1;
}

