#include <stdio.h>
#include <string.h>
#include <conio.h>

//#include "../Blowfish/bc_bfish.h"
#include "../cAlgBfish.h"


/* Test vectors for the Blowfish algorithm */

static struct 
{	BYTE    plain[8];       /* Plain data */
	BYTE    cipher[8];      /* Ciphered data */
	int     dataLength;   /* Length of data */
	BYTE    key[56];      /* Encryption key */
	int     keyLength;    /* Blowfish key length may have up to 448/8 bytes key */
} testVectors[] = 

/* First 5 vectors are from Eric Young's test vectors for Blowfish
   Last 2 are commonly used for testing Blowfish */
{
	{ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x4e, 0xf9, 0x97, 0x45, 0x61, 0x98, 0xdd, 0x78 }, 8,
      { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, 8*8
	},

	{ { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff }, 
      { 0x51, 0x86, 0x6f, 0xd5, 0xb8, 0x5e, 0xcb, 0x8a }, 8,
	  { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff }, 8*8
	},

    { { 0x43, 0x75, 0x40, 0xc8, 0x69, 0x8f, 0x3c, 0xfa }, 
	  { 0x53, 0xc5, 0x5f, 0x9c, 0xb4, 0x9f, 0xc0, 0x19 }, 8,
      { 0x49, 0x79, 0x3e, 0xbc, 0x79, 0xb3, 0x25, 0x8f }, 8*8
	},

	{ { 0x42, 0xfd, 0x44, 0x30, 0x59, 0x57, 0x7f, 0xa2 }, 
	  { 0x35, 0x38, 0x82, 0xb1, 0x09, 0xce, 0x8f, 0x1a }, 8,
      { 0x04, 0xb9, 0x15, 0xba, 0x43, 0xfe, 0xb5, 0xb6 }, 8*8
	},

    { { 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11 }, 
	  { 0x61, 0xf9, 0xc3, 0x80, 0x22, 0x81, 0xb0, 0x96 }, 8,
      { 0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef }, 8*8
	},

	{ { 'B',  'L',  'O',  'W',  'F',  'I',  'S',  'H' },
	  { 0x32, 0x4E, 0xD0, 0xFE, 0xF4, 0x13, 0xA2, 0x03 }, 8, 
	  "abcdefghijklmnopqrstuvwxyz", 26*8
	},
	

	{ { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
	  { 0xCC, 0x91, 0x73, 0x2B, 0x80, 0x22, 0xF6, 0x84 }, 8,
      "Who is John Galt?", 17*8
	},

	{ 0, 0, 0, 0, 0 
	}
};



int Test_Blowfish()
{
  int   i;
  BOOL  result;
  UCHAR outputData[8];

  result = TRUE; // Assume everything will be Ok

  printf("\n");

  for (i=0; testVectors[i].dataLength != 0; i++)
  {

    cAlgBfish *alg = (cAlgBfish *)cAlgBfish::createAlg( testVectors[i].key, testVectors[i].keyLength / 8 );
    if (!alg)
	{
		printf("\n(BLOWFISH ERROR) Test Vector N %d: error to extend encryption key!\n",i);
		return 0;
    }		
	  
	// Test if Blowfish encryption works correctly...

	if (alg->encryptECB( testVectors[i].plain, outputData, 8 ) != csae_ok )
	{
		delete alg;
		printf("\n(BLOWFISH ERROR) Test Vector N %d: internal error (encryption)!\n",i);
		return 0;
	}

    if( memcmp( outputData, testVectors[i].cipher, testVectors[i].dataLength ) != 0)
	{
		delete alg;
		printf("\n(BLOWFISH ERROR) Test Vector N %d: the BLOWFISH implementation failed the test (encryption)!\n",i);
		break;
	}

	// Test if Blowfish decryption works correctly...

	if (alg->decryptECB( testVectors[i].cipher, outputData, 8 ) != csae_ok )
	{
		delete alg;
		printf("\n(BLOWFISH ERROR) Test Vector N %d: internal error (decryption)!\n",i);
		return 0;
	}

    if( memcmp( outputData, testVectors[i].plain, testVectors[i].dataLength ) != 0)
	{
		delete alg;
		printf("\n(BLOWFISH ERROR) Test Vector N %d: the BLOWFISH implementation failed the test (decryption)!\n",i);
		return 0;
	}

	delete alg;

    printf("Test Vector N %2d: the Blowfish implementation passes the test successfully.\n",i);
  }

  printf("Press any key to continue...\n");
  char ch = getch();

  return 0;
}

