#include <stdio.h>
#include <string.h>
#include <conio.h>

#include "../cAlgCast.h"
#include "../Cast/bc_cast.h"

struct 
    {    UCHAR key[ CAST_MAXIMUM_KEYSIZE_BYTES ];
         UCHAR plain[8];
         UCHAR cipher[8];
		 DWORD dataLength;
	} testVectorsCast[] =
{
	{ { 0x01, 0x23, 0x45, 0x67, 0x12, 0x34, 0x56, 0x78,
		0x23, 0x45, 0x67, 0x89, 0x34, 0x56, 0x78, 0x9A },
	  { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
	  { 0x23, 0x8B, 0x4F, 0xE5, 0x84, 0x7E, 0x44, 0xB2 }, 8 
	},
	{ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, 0 
	}
};



int Test_Cast()
{
  int   i;
  UCHAR outputData[8];
  BOOL  result;

  result = TRUE; // Assume everything will be Ok

  printf("\n");

  for (i=0; testVectorsCast[i].dataLength != 0; i++)
  {
    cAlgCast *alg = (cAlgCast *)cAlgCast::createAlg( testVectorsCast[i].key, CAST_MAXIMUM_KEYSIZE_BYTES );
    if (!alg)
	{
		printf("\n(CAST ERROR) Test Vector N %d: error to extend encryption key!\n",i);
		return 0;
    }		
	  
	// Test if CAST encryption works correctly...
	if (alg->encryptECB( testVectorsCast[i].plain, outputData, 8 ) != csae_ok )
	{
		delete alg;
		printf("\n(CAST ERROR) Test Vector N %d: internal error (encryption)!\n",i);
		return 0;
	}

    if( memcmp( outputData, testVectorsCast[i].cipher, testVectorsCast[i].dataLength ) != 0)
	{
		delete alg;
		printf("\n(CAST ERROR) Test Vector N %d: the CAST implementation failed the test (encryption)!\n",i);
		return 0;
	}

    // Test if CAST decryption works correctly...

	if (alg->decryptECB( testVectorsCast[i].cipher, outputData, 8 ) != csae_ok )
	{
		delete alg;
		printf("\n(CAST ERROR) Test Vector N %d: internal error (decryption)!\n",i);
		return 0;
	}

    if( memcmp( outputData, testVectorsCast[i].plain, testVectorsCast[i].dataLength ) != 0)
	{
		delete alg;
		printf("\n(CAST ERROR) Test Vector N %d: the CAST implementation failed the test (decryption)!\n",i);
		return 0;
	}

	delete alg;
    printf("Test Vector N %2d: the CAST implementation passed the test successfully.\n",i);
  }

  printf("Press any key to continue...\n");
  char ch = getch();

  return 0;
}


