#include "cAlg3des.h"
#include "TripleDes/Bc_3des.h"

const char *cAlg3des::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlg3des::cAlg3des()
{
	m_extKey = NULL;
}


cAlg3des::~cAlg3des()
{
	safeFree( m_extKey );
}


BOOL cAlg3des::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE;
	*keySize   = KEY_LENGTH / 8;
	return TRUE;
}



int  cAlg3des::getBlockSize()
{
	return BLOCK_SIZE;
}

int cAlg3des::getKeySize( int *pMax )
{
	if (pMax) *pMax = KEY_LENGTH / 8;

	return KEY_LENGTH / 8;
}


cSymAlg * cAlg3des::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlg3des *alg = new cAlg3des();

	if (keySize < (KEY_LENGTH / 8))
	{
		lKey = safeAlloc( KEY_LENGTH / 8 );
		if (!lKey) return NULL;
		memset( lKey, 0, KEY_LENGTH / 8 );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!DES3_KeyExtend( lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}




int cAlg3des::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		DES3_ECB_mode( &(inBf[i]), &(outBf[i]), (UINT *)m_extKey, DES_ECB_ENCRYPT );
	}

	return sae_ok;
}



int cAlg3des::decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		DES3_ECB_mode( &(inBf[i]), &(outBf[i]), (UINT *)m_extKey, DES_ECB_DECRYPT );
	}

	return sae_ok;
}

