#include "cAlgBfish.h"
#include "Blowfish/Bc_bfish.h"

const char *cAlgBfish::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlgBfish::cAlgBfish()
{
	m_extKey = NULL;
	m_keySize = 0;
}


cAlgBfish::~cAlgBfish()
{
	safeFree( m_extKey );
}


BOOL cAlgBfish::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE;
	*keySize   = m_keySize;
	return TRUE;
}


int  cAlgBfish::getBlockSize()
{
	return BLOCK_SIZE;
}

int cAlgBfish::getKeySize( int *pMax )
{
	if (pMax) *pMax = MAX_KEY_LENGTH / 8;

	return PGP_DEFINED_KEY_LENGTH / 8;
}

int cAlgBfish448::getKeySize( int *pMax )
{
	if (pMax) *pMax = MAX_KEY_LENGTH / 8;

	return MAX_KEY_LENGTH / 8;
}

int cAlgBfish256::getKeySize( int *pMax )
{
	if (pMax) *pMax = MAX_KEY_LENGTH / 8;

	return 256 / 8;
}

cSymAlg * cAlgBfish::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgBfish *alg = new cAlgBfish();

	if (keySize < (MIN_KEY_LENGTH / 8))
	{
		alg->m_keySize = MIN_KEY_LENGTH / 8;
		lKey = safeAlloc( MIN_KEY_LENGTH / 8 );
		if (!lKey) return NULL;
		memset( lKey, 0, MIN_KEY_LENGTH / 8 );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		alg->m_keySize = (keySize > (MAX_KEY_LENGTH / 8)) ? (MAX_KEY_LENGTH / 8) : keySize;
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!BFISH_KeyExtend( alg->m_keySize * 8, lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}


/*  Macroses to transform array of 4 bytes to 32-bit words
 *  (and reverse) without depending on the Little-Endian or 
 *  Big-Endian processor's architecture 
 */
/*
#define BYTES_TO_UINT(b,d) (d =((UINT)(*((b)++))), \
                             d|=((UINT)(*((b)++)))<< 8, \
                             d|=((UINT)(*((b)++)))<<16, \
                             d|=((UINT)(*((b)++)))<<24)

#define UINT_TO_4BYTES(d,b) (*((b)++)=(BYTE)(((d)    )&0xff), \
                              *((b)++)=(BYTE)(((d)>> 8)&0xff), \
                              *((b)++)=(BYTE)(((d)>>16)&0xff), \
                              *((b)++)=(BYTE)(((d)>>24)&0xff))
*/


#define BYTES_TO_UINT_LE(i,d) (d =(UINT)(inBf[i]), \
                             d|=(UINT)(inBf[i+1]) << 8, \
                             d|=(UINT)(inBf[i+2]) <<16, \
                             d|=(UINT)(inBf[i+3]) <<24)

#define UINT_TO_4BYTES_LE(d,i) (outBf[i]=(BYTE)(((d)    )&0xff), \
                              outBf[i+1]=(BYTE)(((d)>> 8)&0xff), \
                              outBf[i+2]=(BYTE)(((d)>>16)&0xff), \
                              outBf[i+3]=(BYTE)(((d)>>24)&0xff))

#define BYTES_TO_UINT_BE(i,d) (d =(UINT)(inBf[i+3]), \
                             d|=(UINT)(inBf[i+2]) << 8, \
                             d|=(UINT)(inBf[i+1]) <<16, \
                             d|=(UINT)(inBf[i])   <<24)

#define UINT_TO_4BYTES_BE(d,i) (outBf[i+3]=(BYTE)(((d)    )&0xff), \
                              outBf[i+2]=(BYTE)(((d)>> 8)&0xff), \
                              outBf[i+1]=(BYTE)(((d)>>16)&0xff), \
                              outBf[i]  =(BYTE)(((d)>>24)&0xff))


typedef VOID type_Blowfish_ECB_Update( PUINT Left, PUINT Right, PBKEY  KeyAddress );

static int bFish_ECB_Update_BE( type_Blowfish_ECB_Update *updateFunc, const BYTE *inBf, BYTE *outBf, int  bfSize, BKEY *extKey )
{
	int i;
	UINT left, right;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		BYTES_TO_UINT_BE( i, left );
		BYTES_TO_UINT_BE( i+sizeof(UINT), right );

		updateFunc( &left, &right, extKey );

		UINT_TO_4BYTES_BE( left,  i);
		UINT_TO_4BYTES_BE( right, i+sizeof(UINT) );
	}

	return sae_ok;
}

static int bFish_ECB_Update_LE( type_Blowfish_ECB_Update *updateFunc, const BYTE *inBf, BYTE *outBf, int  bfSize, BKEY *extKey )
{
	int i;
	UINT left, right;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		BYTES_TO_UINT_LE( i, left );
		BYTES_TO_UINT_LE( i+sizeof(UINT), right );

		updateFunc( &left, &right, extKey );

		UINT_TO_4BYTES_LE( left,  i);
		UINT_TO_4BYTES_LE( right, i+sizeof(UINT) );
	}

	return sae_ok;
}

//
//	12.12.2012
//	Windows driver simple converts byte pointer to UINT pointer.
//	CertificateLib BlowFish realisation tried to convert BYTE bufer a Big-Endian UINT, but it brings a error in container header decryption.
//	This flag allows to use same method for BlowFish updating for certificate.lib as well as for MAC/Linux drivers do.
//
bool crtLib_bWindowsDriverCompatible = false;

int cAlgBfish::encryptECB( const BYTE *inBf, BYTE *outBf, int  bfSize )
{
	if ( crtLib_bWindowsDriverCompatible )
		return bFish_ECB_Update_LE( Blowfish_ECB_Encrypt, inBf, outBf, bfSize, (BKEY *)m_extKey );
	else
		return bFish_ECB_Update_BE( Blowfish_ECB_Encrypt, inBf, outBf, bfSize, (BKEY *)m_extKey );
}

int cAlgBfish::decryptECB( const BYTE *inBf, BYTE *outBf, int  bfSize )
{
	if ( crtLib_bWindowsDriverCompatible )
		return bFish_ECB_Update_LE( Blowfish_ECB_Decrypt, inBf, outBf, bfSize, (BKEY *)m_extKey );
	else
		return bFish_ECB_Update_BE( Blowfish_ECB_Decrypt, inBf, outBf, bfSize, (BKEY *)m_extKey );
}

//int cAlgBfish::encryptECB(
//		const BYTE *inBf,		// Input plaintext buffer. 
//		BYTE *outBf,	// Output encrypted buffer
//		int  bfSize		// In/out buffers' size.
//		)
//{
//	int i;
//	UINT left, right;
//
//	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;
//
//	for( i = 0; i < bfSize; i += BLOCK_SIZE)
//	{
//		BYTES_TO_UINT_BE( i, left );
//		BYTES_TO_UINT_BE( i+sizeof(UINT), right );
//
//		Blowfish_ECB_Encrypt( &left, &right, (BKEY *)m_extKey );
//
//		UINT_TO_4BYTES_BE( left,  i);
//		UINT_TO_4BYTES_BE( right, i+sizeof(UINT) );
//	}
//
//	return sae_ok;
//}
//
//
//
//int cAlgBfish::decryptECB(
//		const BYTE *inBf,		// Input encrypted buffer.
//		BYTE *outBf,	// Output plaintext buffer.
//		int  bfSize		// In/out buffers' size.
//		)
//{
//	int i;
//
//	UINT left, right;
//
//	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;
//
//	for( i = 0; i < bfSize; i += BLOCK_SIZE)
//	{
//		BYTES_TO_UINT_BE( i, left );
//		BYTES_TO_UINT_BE( i+sizeof(UINT), right );
//
//		Blowfish_ECB_Decrypt( &left, &right, (BKEY *)m_extKey );
//
//		UINT_TO_4BYTES_BE( left,  i);
//		UINT_TO_4BYTES_BE( right, i+sizeof(UINT) );
//	}
//
//	return sae_ok;
//}
