/////////////////////////////////////////////////////////////////
//
//      Copyright (c) 1994-2008 Jetico, Inc., Finland
//      All rights reserved.
//
//      File: cAlgBfish.h
//      Revision:
//      Created:
//      Description:
//
/////////////////////////////////////////////////////////////////

#ifndef __cAlgBfish__
#define __cAlgBfish__

#include "ALG_DEF.H"
#include "cSymAlg.h"

#define PGP_DEFINED_KEY_LENGTH 128

class cAlgBfish: public cSymAlg
{
 private:
	BYTE *m_extKey;
	int  m_keySize; // Blowfish can be used with key from 40 to 448 bits

	BOOL getAlgConstants( int *blockSize, int *keySize );

 public:

     cAlgBfish();
	 ~cAlgBfish();

	static int getBlockSize();
	static const char *getDisplayName();

	// returns minimum key size in bytes, 
	// if algorithm supports other key length, pMax contains maximum key size
	// if algorithm supports fixed key length, pMax is equal to returned value
	static int getKeySize( int *pMax = NULL );

	static cSymAlg *createAlg( BYTE *key, int keySize );  // keySize in bytes

	// Encryption and decription in Electronic Code Book mode

	virtual int encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		);

	virtual int decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		);

/* implementation of the following procedures is in cSymAlg.cpp

	// Encryption and decription in Cipher Feedback mode

	int initCFB( BYTE *iv );
	int syncCFB(); // see the NOTE below

	virtual int encryptCFB(
		BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		);

	virtual int decryptCFB(
		BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		);
*/

};

class cAlgBfish448: public cAlgBfish
{
public:
	static int getKeySize( int *pMax = NULL );
};

class cAlgBfish256: public cAlgBfish
{
public:
	static int getKeySize( int *pMax = NULL );
};


#endif // __cAlgBfish__

