#include "cAlgCast.h"
#include "Cast/Bc_cast.h"

const char *cAlgCast::getDisplayName()
{
	return DISPLAY_NAME;
}


cAlgCast::cAlgCast()
{
	m_extKey = NULL;
}


cAlgCast::~cAlgCast()
{
	safeFree( m_extKey );
}


BOOL cAlgCast::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE;
	*keySize   = CAST_MAXIMUM_KEYSIZE_BYTES;
	return TRUE;
}


int  cAlgCast::getBlockSize()
{
	return BLOCK_SIZE;
}


int cAlgCast::getKeySize( int *pMax )
{
	if (pMax) *pMax = CAST_MAXIMUM_KEYSIZE_BYTES;

	return CAST_MAXIMUM_KEYSIZE_BYTES;
}




cSymAlg * cAlgCast::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgCast *alg = new cAlgCast();

	if (keySize < CAST_MAXIMUM_KEYSIZE_BYTES)
	{
		lKey = safeAlloc( CAST_MAXIMUM_KEYSIZE_BYTES );
		if (!lKey) return NULL;
		memset( lKey, 0, CAST_MAXIMUM_KEYSIZE_BYTES );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!CAST_KeyExtend( CAST_MAXIMUM_KEYSIZE_BYTES, lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}

int cAlgCast::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		CastECB( &(inBf[i]), &(outBf[i]), (UINT *)m_extKey, TRUE );
	}

	return sae_ok;
}

int cAlgCast::decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		CastECB( &(inBf[i]), &(outBf[i]), (UINT *)m_extKey, FALSE );
	}

	return sae_ok;
}

