#include "cAlgDes.h"
#include "TripleDes/Bc_des.h"

const char *cAlgDes::getDisplayName()
{
	return "DES";
}

cAlgDes::cAlgDes()
{
	m_extKey = NULL;
}


cAlgDes::~cAlgDes()
{
	safeFree( m_extKey );
}


BOOL cAlgDes::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE;
	*keySize   = KEY_LENGTH_DES / 8;
	return TRUE;
}


int  cAlgDes::getBlockSize()
{
	return BLOCK_SIZE;
}

int cAlgDes::getKeySize( int *pMax )
{
	if (pMax) *pMax = KEY_LENGTH_DES / 8;

	return KEY_LENGTH_DES / 8;
}




cSymAlg * cAlgDes::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgDes *alg = new cAlgDes();

	if (keySize < (KEY_LENGTH_DES / 8))
	{
		lKey = safeAlloc( KEY_LENGTH_DES / 8 );
		if (!lKey) return NULL;
		memset( lKey, 0, KEY_LENGTH_DES / 8 );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED_DES / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!DES_KeyExtend( lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}



int cAlgDes::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		DES_ECB_mode( &(inBf[i]), &(outBf[i]), (UINT *)m_extKey, DES_ECB_ENCRYPT );
	}

	return sae_ok;
}



int cAlgDes::decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		DES_ECB_mode( &(inBf[i]), &(outBf[i]), (UINT *)m_extKey, DES_ECB_DECRYPT );
	}

	return sae_ok;
}

