#include "cAlgGOST.h"
#include "BC_GOST/BC_GOST.H"

#define GOST_MAXIMUM_KEYSIZE_BYTES ( KEY_LENGTH / 8 )

const char *cAlgGOST::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlgGOST::cAlgGOST()
{
	m_extKey = NULL;
	m_keySize = 0;
}


cAlgGOST::~cAlgGOST()
{
	safeFree( m_extKey );
}


BOOL cAlgGOST::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE;
	*keySize   = m_keySize;
	return TRUE;
}


int  cAlgGOST::getBlockSize()
{
	return BLOCK_SIZE;
}

int cAlgGOST::getKeySize( int *pMax )
{
	if (pMax) *pMax = KEY_LENGTH / 8;

	return KEY_LENGTH / 8;
}

cSymAlg * cAlgGOST::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgGOST *alg = new cAlgGOST();

	if (keySize < GOST_MAXIMUM_KEYSIZE_BYTES)
	{
		lKey = safeAlloc( GOST_MAXIMUM_KEYSIZE_BYTES );
		if (!lKey) return NULL;
		memset( lKey, 0, GOST_MAXIMUM_KEYSIZE_BYTES );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!GOST_KeyExtend( lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}


int cAlgGOST::encryptECB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize			// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		memcpy( &(outBf[i]), &(inBf[i]), BLOCK_SIZE );

		UINT *N1 = (UINT *)&(outBf[i]);
		UINT *N2 = (UINT *)&(outBf[i + sizeof( UINT )]);

		GOST_ECB_Encrypt( N1, N2, (UINT *)m_extKey );
	}

	return sae_ok;
}

int cAlgGOST::decryptECB(
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize			// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		memcpy( &(outBf[i]), &(inBf[i]), BLOCK_SIZE );

		UINT *N1 = (UINT *)&(outBf[i]);
		UINT *N2 = (UINT *)&(outBf[i + sizeof( UINT )]);

		GOST_ECB_Decrypt( N1, N2, (UINT *)m_extKey );
	}

	return sae_ok;
}

