#include "cAlgIdea.h"
#include "Idea/Bc_idea.h"

const char *cAlgIdea::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlgIdea::cAlgIdea()
{
	m_extKey = NULL;
}


cAlgIdea::~cAlgIdea()
{
	safeFree( m_extKey );
}


BOOL cAlgIdea::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE;
	*keySize   = IDEA_KEY_LENGTH_BYTES;
	return TRUE;
}


int  cAlgIdea::getBlockSize()
{
	return BLOCK_SIZE;
}



int cAlgIdea::getKeySize( int *pMax )
{
	if (pMax) *pMax = IDEA_KEY_LENGTH_BYTES;

	return IDEA_KEY_LENGTH_BYTES;
}




cSymAlg * cAlgIdea::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgIdea *alg = new cAlgIdea();

	if (keySize < IDEA_KEY_LENGTH_BYTES)
	{
		lKey = safeAlloc( IDEA_KEY_LENGTH_BYTES );
		if (!lKey) return NULL;
		memset( lKey, 0, IDEA_KEY_LENGTH_BYTES );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!IDEA_KeyExtend( lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}



int cAlgIdea::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		IdeaECB( &(inBf[i]), &(outBf[i]), (IdeaExtKey *)m_extKey, TRUE );
	}

	return sae_ok;
}



int cAlgIdea::decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		IdeaECB( &(inBf[i]), &(outBf[i]), (IdeaExtKey *)m_extKey, FALSE );
	}

	return sae_ok;
}

