#include "cAlgRIJN.h"
#include "BC_RIJN/BC_RIJN.H"

#define RIJN_MAXIMUM_KEYSIZE_BYTES ( KEY_LENGTH / 8 )

const char *cAlgRIJN::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlgRIJN::cAlgRIJN()
{
	m_extKey = NULL;
}

cAlgRIJN::~cAlgRIJN()
{
	safeFree( m_extKey );
}

BOOL cAlgRIJN::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BC_BLOCK_SIZE;
	*keySize   = RIJN_MAXIMUM_KEYSIZE_BYTES;
	return TRUE;
}

int  cAlgRIJN::getBlockSize()
{
	return BC_BLOCK_SIZE;
}

int cAlgRIJN::getKeySize( int *pMax )
{
	if (pMax) *pMax = RIJN_MAXIMUM_KEYSIZE_BYTES;

	return RIJN_MAXIMUM_KEYSIZE_BYTES;
}

cSymAlg * cAlgRIJN::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgRIJN *alg = new cAlgRIJN();

	if (keySize < RIJN_MAXIMUM_KEYSIZE_BYTES)
	{
		lKey = safeAlloc( RIJN_MAXIMUM_KEYSIZE_BYTES );
		if (!lKey) return NULL;
		memset( lKey, 0, RIJN_MAXIMUM_KEYSIZE_BYTES );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!RIJN_KeyExtend( lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}

int cAlgRIJN::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BC_BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BC_BLOCK_SIZE)
	{
		RIJN_encrypt_ECB( m_extKey, &(inBf[i]), &(outBf[i]) );
	}

	return sae_ok;
}

int cAlgRIJN::decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BC_BLOCK_SIZE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BC_BLOCK_SIZE )
	{
		RIJN_decrypt_ECB( m_extKey, &(inBf[i]), &(outBf[i]) );
	}

	return sae_ok;
}

