#include "cAlgSerpent.h"
#include "Serpent/bc_serp.h"

#define MAXIMUM_KEYSIZE_BYTES ( KEY_LENGTH / 8 )
#define BLOCK_SIZE_BYTE ( BLOCK_SIZE / 8 )

const char *cAlgSerpent::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlgSerpent::cAlgSerpent()
{
	m_extKey = NULL;
	m_keySize = 0;
}


cAlgSerpent::~cAlgSerpent()
{
	safeFree( m_extKey );
}


BOOL cAlgSerpent::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE_BYTE;
	*keySize   = m_keySize;
	return TRUE;
}


int cAlgSerpent::getBlockSize()
{
	return BLOCK_SIZE_BYTE;
}

int cAlgSerpent::getKeySize( int *pMax )
{
	if (pMax) *pMax = MAXIMUM_KEYSIZE_BYTES;

	return MAXIMUM_KEYSIZE_BYTES;
}

cSymAlg * cAlgSerpent::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgSerpent *alg = new cAlgSerpent();

	if (keySize < MAXIMUM_KEYSIZE_BYTES)
	{
		lKey = safeAlloc( MAXIMUM_KEYSIZE_BYTES );
		if (!lKey) return NULL;
		memset( lKey, 0, MAXIMUM_KEYSIZE_BYTES );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!SERP_KeyExtend( lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}

int cAlgSerpent::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE_BYTE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE_BYTE)
	{
		SERP_Encrypt_ECB( (BYTE*)&(inBf[i]), &(outBf[i]), m_extKey );
	}

	return sae_ok;
}

int cAlgSerpent::decryptECB(
		const BYTE *inBf,// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE_BYTE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE_BYTE)
	{
		SERP_Decrypt_ECB( (BYTE*)&(inBf[i]), &(outBf[i]), m_extKey );
	}

	return sae_ok;
}

