#include "cAlgTFish.h"
#include "BC_TFISH/bc_tfish.h"

#define TFish_MAXIMUM_KEYSIZE_BYTES ( KEY_LENGTH / 8 )

#define BLOCK_SIZE_BYTE ( BLOCK_SIZE / 8 )

const char *cAlgTFish::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlgTFish::cAlgTFish()
{
	m_extKey = NULL;
	m_keySize = 0;
}


cAlgTFish::~cAlgTFish()
{
	safeFree( m_extKey );
}


BOOL cAlgTFish::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE_BYTE;
	*keySize   = m_keySize;
	return TRUE;
}


int cAlgTFish::getBlockSize()
{
	return BLOCK_SIZE_BYTE;
}

int cAlgTFish::getKeySize( int *pMax )
{
	if (pMax) *pMax = TFish_MAXIMUM_KEYSIZE_BYTES;

	return TFish_MAXIMUM_KEYSIZE_BYTES;
}

cSymAlg * cAlgTFish::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgTFish *alg = new cAlgTFish();

	if (keySize < TFish_MAXIMUM_KEYSIZE_BYTES)
	{
		lKey = safeAlloc( TFish_MAXIMUM_KEYSIZE_BYTES );
		if (!lKey) return NULL;
		memset( lKey, 0, TFish_MAXIMUM_KEYSIZE_BYTES );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!TFish_KeyExtend( lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}


int cAlgTFish::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE_BYTE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE_BYTE)
	{
		memcpy( &(outBf[i]), &(inBf[i]), BLOCK_SIZE_BYTE );
		TFish_blockEncrypt( (UINT *)&(outBf[i]), (KeyInstance *)m_extKey );
	}

	return sae_ok;
}

int cAlgTFish::decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	if ((bfSize % BLOCK_SIZE_BYTE) != 0) return sae_invalidBlockSize;

	for( i = 0; i < bfSize; i += BLOCK_SIZE_BYTE)
	{
		memcpy( &(outBf[i]), &(inBf[i]), BLOCK_SIZE_BYTE );
		TFish_blockDecrypt( (UINT *)&(outBf[i]), (KeyInstance *)m_extKey );
	}

	return sae_ok;
}

