/////////////////////////////////////////////////////////////////
//
//      Copyright (c) 1994-2008 Jetico, Inc., Finland
//      All rights reserved.
//
//      File: base64funcs.h
//      Revision:
//      Created:
//      Description:
//
/////////////////////////////////////////////////////////////////

#ifndef __base64funcs__
#define __base64funcs__

#ifdef WIN32
#include <windows.h>
#else
#include "../publicKeyAlgs/Power/Windefs.h"
#endif

#include "../crtLibErrs.h"

//
//	Uses RFC 1991 - ASCII Armor 
//

int findEncodedBlock(
		const BYTE **pBlock,		// OUT - Pointer to byte position of encoded block
		int *pBlockSize,	// OUT - Pointer to finded size of encoded block
		const BYTE *pkt,			// IN - the packet with encoded block to be find.
		int pktSize,		// IN - the packet size
		const char *szBlockStart,
		const char *szBlockEnd,
		const char **pArmorHeaders = NULL // OUT pointer to position of Armor headers
									// the headers are separated by single 0x0A symbol
									// and are terminated by double 0x0A symbols ( a blank line )
		);

int base64Decode(
		BYTE *decBlock,		// OUT - buffer for decoded data.
		int *pDecBlockSize,	// IN - pointer for decoded size, if size of 
							// the output block is too small return ppe_bufferTooSmall
							// OUT - required or really filled size 
		const BYTE *encBlock,		// IN - encoded buffer
		int encBlockSize );

//
//	v. 1.03.1 - 27.08.09.
//	The base64Encode function generates text finished by '\r\n' always.
//	Prev version generated without the symbols and returned error because the size was two bytes less.
//
int base64Encode(
		BYTE *encBlock,		// OUT - buffer for encoded data.
		int *pEncBlockSize,	// IN - pointer for encoded size, if size of 
							// the output block is too small return ppe_bufferTooSmall
							// OUT - required or really filled size 
		const BYTE *decBlock,		// IN - plain text buffer
		int decBlockSize,

		const char *szBlockStart = NULL,	// If the strings are not NULL,
		const char *szBlockEnd = NULL,		// so the encoded block will be encapsulated by the strings

		const char *szVerInfo = NULL		// PGP allows version information string before real armored data
	);

//
//	Extended functions
//	Are released at base64funcsEx.cpp
//
class cPlainTextBlock;

//
//	createArmoredBlock
//		Fills ptb->buffer by text
//			1. szStart line
//			2. base64 encoded buffer
//			3. szEnd line
//
bool createArmoredBlock( cPlainTextBlock *ptb, const BYTE *body, int bodySize, const char *szStart, const char *szEnd, const char *szVerInfo = NULL );

//
//	readArmoredBlock
//		1. Extracts the body is concluded by szStart and szEnd strings
//		2. Decodes base64 bode to ptb->buffer.
//
bool readArmoredBlock( cPlainTextBlock *ptb, const char *buff, int buffSize, const char *szStart, const char *szEnd );
int readArmoredBlock( BYTE *bf, int *bfSize, const char *text, int textSize, const char *szStart, const char *szEnd );

bool checkBase64Encoding(
		const BYTE *encBlock, 	int encBlockSize,
		const BYTE *decBlock,	int decBlockSize,
		const char *szBlockStart,
		const char *szBlockEnd
	);


#endif // __base64funcs__
