#include "base64funcs.h"
#include "../SMemory/SBlocks.h"
#include <assert.h>

bool createArmoredBlock( cPlainTextBlock *ptb, const BYTE *body, int bodySize, const char *szStart, const char *szEnd, const char *szVerInfo )
{
	int b64size = 0;

	int res = base64Encode( NULL, &b64size, body, bodySize, szStart, szEnd, szVerInfo );
	if ( res != ppe_bufferTooSmall )
		return false;

	int pos = ptb->bufferSize;

	if ( ptb->WriteBytes( NULL,  b64size ) != b64size )
		return false;
	 
	res = base64Encode( ptb->buffer + pos, &b64size, body, bodySize, szStart, szEnd, szVerInfo );
	if ( res != ppe_ok )
		return false;

	if ( !checkBase64Encoding( ptb->buffer + pos, b64size, body, bodySize, szStart, szEnd ) )
	{
		assert( false );
		return false;
	}

	return true;
}

bool readArmoredBlock( cPlainTextBlock *ptb, const char *buff, int buffSize, const char *szStart, const char *szEnd )
{
	const BYTE *pos;
	int size;

	int res = findEncodedBlock( &pos, &size, (const BYTE *)buff, buffSize, szStart, szEnd  );
	if ( res != pkae_ok )
		return false;

	int decSize = 0;

	res = base64Decode( NULL, &decSize, (const BYTE *)pos, size );

	if ( res != pkae_bufferSizeTooSmall )
		return false;

	if ( ptb->Allocate( decSize ) == NULL )
		return false;

	res = base64Decode( ptb->buffer, &decSize,(const BYTE *)pos, size );
	if ( res != ppe_ok )
		return false;

	return true;
}

int readArmoredBlock( BYTE *bf, int *bfSize, const char *text, int textSize, const char *szStart, const char *szEnd )
{
	const BYTE *pos;
	int size;

	int res = findEncodedBlock( &pos, &size, (const BYTE *)text, textSize, szStart, szEnd  );
	if ( res != pkae_ok )
		return res;

	return base64Decode( bf, bfSize, (const BYTE *)pos, size );
}

bool checkBase64Encoding(
		const BYTE *encBlock, 	int encBlockSize,
		const BYTE *decBlock,	int decBlockSize,
		const char *szBlockStart,
		const char *szBlockEnd
	)
{

	const BYTE *internalBlock;
	int internalBlockSize;

	int res = findEncodedBlock( 
		&internalBlock, &internalBlockSize, 
		encBlock, encBlockSize,
		szBlockStart, szBlockEnd );

	if ( res != ppe_ok )
		return false;

	int decSize = 0;

	res = base64Decode( NULL, &decSize, internalBlock, internalBlockSize );

	if ( res != pkae_bufferSizeTooSmall )
		return false;

	if ( decSize != decBlockSize )
		return false;

	cPlainTextBlock ptb;

	if ( ptb.Allocate( decSize ) == NULL )
		return false;

	res = base64Decode( ptb.buffer, &decSize, internalBlock, internalBlockSize );
	if ( res != ppe_ok ) return false;

	bool ok = memcmp( ptb.buffer, decBlock, decBlockSize ) == 0;

	return ok;
}
