/////////////////////////////////////////////////////////////////
//
//      Copyright (c) 1994-2008 Jetico, Inc., Finland
//      All rights reserved.
//
//      File: oidFuncs.h
//      Revision:
//      Created:
//      Description:
//
/////////////////////////////////////////////////////////////////

#ifndef __oidFuncs__
#define __oidFuncs__

#include "oidStrings.h"

//
//	Function: oid2string
//
//	converts a DER encoding ASN.1 object-identifier into dot-string format
// 
const int maxOIDStrSize = 256;
char *oid2string( char *buf, unsigned char *oidVal, unsigned int oidSize );

//
//	Function: string2oid
//		Converts OID string to unsigned char buffer.
//		Returns -1 if *pOidSize too small, otherwise return filled size.
//	if oid == NULL returns required size.
//
int string2oid( unsigned char *oid, int *pOidSize, const char *str );

//
//	Function: generateOIDName
//		Parses the oid string and collects name from table.name.
//		If succeful returns length of generated name.
//		If specified oid is not found returns 0;
//
int generateOIDName( 
		char *name,	// Can be NULL, if required only ultimate unit.
		const char *oidString,
		tUnit **pUnit = 0 // ultimate unit, it contains an additional information see tUnit
		);

//
//	Function: checkOIDName
//		Parses the oid string, gets name from table.name and checks 
//			if the names are compatible with corresponded names from nameT.
//		If sucessful returns x509e_ok.
//		Otherwise returns x509e_undefinedOIDType
//
int checkOIDName( 
		const char *oidString,
		const char **nameT, // NULL terminated or nameTSize defined table with szNames.
		int nameTSize = 0	// If nameTSize == 0, nameT has to be NULL terminated and the 
							// function will sucessed only if whole OID name checked.
							// otherwise checking successfully stops if the nameTSize is achieved
	);


//
//	Function: getSignatureAlgOidStr
//		returns specified algorithm oid string if successful, otherwise return NULL
//
const char *getSignatureAlgOidStr(
		unsigned char hAlg,	// Hash alg id				f.e. pgphaSHA1
		unsigned char pAlg	// Public key alg id		f.e. pgppkaRSA
	);

//
//	Function: getDigestAlgOidStr
//		returns specified algorithm oid string
//
const char *getDigestAlgOidStr(
		unsigned char hAlg	// Hash alg id				f.e. pgphaSHA1
	);

#endif // __oidFuncs__
