#include "oidStrings.h"
#include "../PGPPkts/pktdefs.h"

// Unused field
#define UDF (BYTE(-1))

#define tObj( T, oid, name, desc ) { T, sizeof( T ) / sizeof( *T ), oid, name, desc, UDF, UDF, UDF, UDF }
#define uObj( oid, name, desc ) { 0, 0, oid, name, desc, UDF, UDF, UDF, UDF }

//
//	Algorithm specific OIDs
//	http://www.alvestrand.no/objectid

//
// Table unit declarations:
//
#define uObjP( oid, name, desc, pA ) { 0, 0, oid, name, desc, UDF,UDF, BYTE(pA), UDF } // Public key algorithm unit
#define uObjS( oid, name, desc, sA, mode ) { 0, 0, oid, name, desc, UDF, BYTE(sA), UDF, BYTE(mode) } // Symmetric algorithm unit
#define uObjH( oid, name, desc, hA ) { 0, 0, oid, name, desc, BYTE(hA), UDF, UDF, UDF } // Hash algorithm unit

#define uObjHP( oid, name, desc, hA, pA ) { 0, 0, oid, name, desc, BYTE(hA), UDF, BYTE(pA), UDF } // Hash nad Public key algorithms - signature unit.
#define uObjHS( oid, name, desc, hA, sA, mode ) { 0, 0, oid, name, desc, BYTE(hA), BYTE(sA), UDF, BYTE(mode) } // Hash and Symmetric algorithms - password based encryption unit.


//////////
//
// pkcs OID
//
// The DER for this in hexadecimal is:
// 06 08 
//    2A 86 48 86 F7 0D 01
//
// pkcs OBJECT IDENTIFIER ::= 
//	{ iso(1) member-body(2) us(840) rsadsi(113549) pkcs(1) }
//
//	{ 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01 };
//

//
// pkcs #1 OIDs
//
tUnit pkcs1_T[] =
{
	uObjP( "1", NULL, "rsaEncryption", pgppkaRSA ),
	uObjHP( "2", NULL, "md2WithRSAEncryption", pgphaMD2, pgppkaRSA ),
	uObjHP( "3", NULL, "md4WithRSAEncryption", pgphaMD4, pgppkaRSA ),
	uObjHP( "4", NULL, "md5WithRSAEncryption", pgphaMD5, pgppkaRSA ),
	uObjHP( "5", NULL, "sha1WithRSAEncryption", pgphaSHA1, pgppkaRSA ), // 1.2.840.113549.1.1.5
	uObj( "6", NULL, "rsaOAEPEncryptionSET" ),
	uObj( "7", NULL, "id-RSAES-OAEP" ),
	uObj( "8", NULL, "id-mgf1" ),
	uObj( "9", NULL, "id-pSpecified" ),
	uObj( "10", NULL, "id-RSASSA-PSS" ), // id-RSASSA-PSS(10) 
	uObjHP( "11", NULL, "sha256WithRSAEncryption", pgphaSHA256, pgppkaRSA ), // 1.2.840.113549.1.1.11 - sha256WithRSAEncryption(11) 
	uObjHP( "12", NULL, "sha384WithRSAEncryption", pgphaSHA384, pgppkaRSA ), // 1.2.840.113549.1.1.12 - sha384WithRSAEncryption(12) 
	uObjHP( "13", NULL, "sha512WithRSAEncryption", pgphaSHA512, pgppkaRSA ), // 1.2.840.113549.1.1.13 - sha512WithRSAEncryption(13) 
	uObjHP( "14", NULL, "sha224WithRSAEncryption", pgphaSHA224, pgppkaRSA ), // 1.2.840.113549.1.1.14 - sha224WithRSAEncryption(14)  
};

// The DER for this in hexadecimal is:
// 06 08 
//    2A 86 48 86 F7 0D 02
//
// pkcs OBJECT IDENTIFIER ::= 
//	{ iso(1) member-body(2) us(840) rsadsi(113549) pkcs(2) }
//
//	{ 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x02 };

//
// pkcs #2 OIDs
//
tUnit pkcs2_T[] =
{
	uObjH( "2", NULL, "digestAlgorithm-md2", pgphaMD2 ),
	uObjH( "4", NULL, "digestAlgorithm-md4", pgphaMD4 ),
	uObjH( "5", NULL, "digestAlgorithm-md5", pgphaMD5 ),
	uObjH( "7", NULL, "HMAC-SHA-1 pseudorandom function", pgphaSHA1 )
};

//
// pkcs #5 OIDs
//
tUnit pkcs5_T[] =
{
	uObjHS( "1", NULL, "pbeWithMD2AndDES-CBC", pgphaMD2, pgpsa2KeyTripleDES, pgpsamodeCBC ),
	uObjHS( "3", NULL, "pbeWithMD5AndDES-CBC", pgphaMD5, pgpsa2KeyTripleDES, pgpsamodeCBC ),
	uObjHS( "10", NULL, "pbeWithSHA1AndDES-CBC", pgphaSHA1, pgpsa2KeyTripleDES, pgpsamodeCBC )
};

//
// pkcs #7 OIDs
//

tUnit pkcs7_T[] =
{
	uObj( "1", NULL, "data"  ),
	uObj( "2", NULL, "signedData"  ),
	uObj( "3", NULL, "envelopedData"  ),
	uObj( "7", NULL, "signedAndEnvelopedData"  ),
	uObj( "5", NULL, "digestedData"  ),
	uObj( "6", NULL, "encryptedData" )
};

//
// pkcs9 OID
//

//
// pkcs9 attributes branch OIDs
// pkcs-9-at { pkcs-9 25 }
//
//{ 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x09, 0x19 };
//
tUnit pkcs9at_T[] =
{
	uObj( "1", NULL, "pkcs15Token" ),
	uObj( "2", NULL, "encryptedPrivateKeyInfo" ),
	uObj( "3", NULL, "randomNonce" ),
	uObj( "4", NULL, "sequenceNumber" ),
	uObj( "5", NULL, "pkcs7PDU" )
};

tUnit pkcs9_T[] =
{
	uObj( "1", "E-mail", "email address" ),
	uObj( "2", NULL, "unstructured name"  ),
	uObj( "3", NULL, "content type"  ),
	uObj( "4", NULL, "message digest"  ),
	uObj( "5", NULL, "signing time"  ),
	uObj( "6", NULL, "counter signature"  ),
	uObj( "7", NULL, "challenge rassword"  ),
	uObj( "8", NULL, "unstructured address"  ),
	uObj( "9", NULL, "extCertAttributes"  ),
//-- Obsolete (?) attribute identifiers, purportedly from "tentative PKCS #9 draft"
		uObj( "10", NULL, "issuerAndSerialNumber" ),
		uObj( "11", NULL, "passwordCheck" ),
		uObj( "12", NULL, "publicKey" ),

	uObj( "13", NULL, "signingDescription" ),
	uObj( "14", NULL, "extensionRequest" ),
	uObj( "15", NULL, "smimeCapabilities" ),

//-- Unused (?)
//-- pkcs-9-at-? 	OBJECT IDENTIFIER ::= {pkcs-9 17}
//-- pkcs-9-at-? 	OBJECT IDENTIFIER ::= {pkcs-9 18}
//-- pkcs-9-at-? 	OBJECT IDENTIFIER ::= {pkcs-9 19}

	uObj( "20", NULL, "friendlyName" ),
	uObj( "21", NULL, "localKeyId" ),

	tObj( pkcs9at_T, "25", "at", "pkcs9 attributes branch" )

};

//
// pkcs12 Password based encrypted OID
//
// { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x0C, 0x01 };
//
// 
tUnit pkcs12_pbe_T[] =
{
	uObjHS( "1", NULL, "pbeWithSHAAnd128BitRC4", pgphaSHA1, pgpsa128BitRC4, pgpsamodeCBC ),
	uObjHS( "2", NULL, "pbeWithSHAAnd40BitRC4", pgphaSHA1, pgpsa40BitRC4, pgpsamodeCBC ),
	uObjHS( "3", NULL, "pbeWithSHAAnd3-KeyTripleDES-CBC", pgphaSHA1, pgpsaTripleDES, pgpsamodeCBC ),
	uObjHS( "4", NULL, "pbeWithSHAAnd2-KeyTripleDES-CBC", pgphaSHA1, pgpsa2KeyTripleDES, pgpsamodeCBC ),
	uObjHS( "5", NULL, "pbeWithSHAAnd128BitRC2-CBC", pgphaSHA1, pgpsa128BitRC2, pgpsamodeCBC ),
	uObjHS( "6", NULL, "pbewithSHAAnd40BitRC2-CBC", pgphaSHA1, pgpsa40BitRC2, pgpsamodeCBC ),
};

//
// pkcs12 bagtypes OBJECT IDENTIFIER ::= {pkcs-12 10 1}
//
//
// { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x0C, 0x0A, 0x01 };
//
tUnit pkcs12_BT_T[] =
{
	uObj( "1", NULL, "KeyBag" ),
	uObj( "2", NULL, "PKCS8ShroudedKeyBag" ),
	uObj( "3", NULL, "CertBag" ),
	uObj( "4", NULL, "CRLBag" ),
	uObj( "5", NULL, "SecretBag" ),
	uObj( "6", NULL, "SafeContents" )
};

tUnit pkcs12_T[] =
{
	tObj( pkcs12_pbe_T, "1", "pbe", "pkcs12 - Password based encrypted" ),
	tObj( pkcs12_BT_T, "10.1", "bagType", "pkcs12 - Bag types" )
};

// 1.2.840.113549 - pkcs
tUnit pkcs_T[] =
{
	tObj( pkcs1_T, "1", "1", "pkcs#1" ),
	tObj( pkcs2_T, "2", "2", "pkcs#2" ),
	uObjP( "3.1", "dhKeyAgreement", "pkcs#3-dhKeyAgreement", pgppkaElgamal ),
	tObj( pkcs5_T, "5", "5", "pkcs#5" ),
	tObj( pkcs7_T, "7", "7", "pkcs#7" ),
	tObj( pkcs9_T, "9", "9", "pkcs#9" ),
	tObj( pkcs12_T, "12", "12", "pkcs#12" )
};

//
//	End of pkcs
//
//////////

//////////
//
//
// OIW Security Special Interest Group OIDs. http://www.alvestrand.no/objectid/1.3.14.3.2.html
//
// id-sha1                OBJECT IDENTIFIER ::= { 
//   iso(1) identified-organization(3) oiw(14) secsig(3)
//   algorithms(2) 26 }
// { 1.3.14.3.2.26 }
//
// BYTE OBJ_C_secSig[] = { 0x2B, 0x0E, 0x03, 0x02 }; // "1.3.14.3.2"

tUnit secSig_T[] =
{
	uObjHP( "2", NULL, "md4WithRSA", pgphaMD4, pgppkaRSA ),
	uObjHP( "3", NULL, "md5WithRSA", pgphaMD5, pgppkaRSA ),
	uObjHP( "4", NULL, "md4WithRSAEncryption", pgphaMD4, pgppkaRSA ),
	uObjS( "6", NULL, "desECB", pgpsa2KeyTripleDES, pgpsamodeECB ),
	uObjS( "7", NULL, "desCBC", pgpsa2KeyTripleDES, pgpsamodeCBC ),
	uObj( "8", NULL, "desOFB" ),
	uObjS( "9", NULL, "desCFB", pgpsa2KeyTripleDES, pgpsamodeCFB ),
	uObj( "10", NULL, "desMAC" ),
	uObj( "11", NULL, "rsaSignature" ),
	uObjP( "12", NULL, "dsa", pgppkaDSA ),
	uObj( "13", NULL, "dsaWithSHA" ),
	uObj( "14", NULL, "mdc2WithRSASignature" ),
	uObj( "15", NULL, "shaWithRSASignature" ),
	uObj( "16", NULL, "dhWithCommonModulus" ),
	uObj( "17", NULL, "desEDE" ),
	uObj( "18", NULL, "sha" ),
	uObj( "19", NULL, "mdc-2" ),
	uObjP( "20", NULL, "dsaCommon", pgppkaDSA ),
	uObjHP( "21", NULL, "dsaCommonWithSHA", pgphaSHA1, pgppkaDSA ),
	uObj( "22", NULL, "rsaKeyTransport" ),
	uObj( "23", NULL, "keyed-hash-seal" ),
	uObjHP( "24", NULL, "md2WithRSASignature", pgphaMD2, pgppkaRSA ),
	uObjHP( "25", NULL, "md5WithRSASignature", pgphaMD5, pgppkaRSA ),
	uObjH( "26", "SHA1","SHA-1 hash algorithm", pgphaSHA1 ),
	uObjHP( "27", NULL, "dsa With SHA1", pgphaSHA1, pgppkaDSA ),
	uObjHP( "28", NULL, "dsa With SHA1 with Common Parameters", pgphaSHA1, pgppkaDSA ),
	uObjHP( "29", NULL, "SHA1 with RSA signature", pgphaSHA1, pgppkaRSA )
};

//
// end of secSig OID
//
///////////

//////////
//
// X.500 attribute types OID { id-at } "2.5.4"
//

tUnit collectiveT[] =
{
	uObj( "1", "(c)"	, "collective" )
};

tUnit id_at_T[] = 
{
	uObj( "0", NULL, "objectClass" ),
	uObj( "1", NULL, "aliasedEntryName" ),
	uObj( "2", NULL, "knowldgeinformation" ),
	uObj( "3", "CN", "commonName" ),
	uObj( "4", "S",	"surname" ),
	uObj( "5", "SN", "serialNumber" ),
	uObj( "6", "C", "countryName" ),
	tObj( collectiveT, "7", "L", "localityName" ),
	tObj( collectiveT, "8", "ST", "stateOrProvinceName" ),
	tObj( collectiveT, "9", "STREET", "streetAddress" ),
	tObj( collectiveT, "10", "O", "organizationName" ),
	tObj( collectiveT, "11", "OU", "organizationalUnitName" ),
	uObj( "12", "T", "title" ),
	uObj( "13", "D", "description" ),
	uObj( "14", NULL, "searchGuide" ),
	uObj( "15", NULL, "businessCategory" ),
	tObj( collectiveT, "16", "PA", "postalAddress" ),
	tObj( collectiveT, "17", NULL, "postalCode" ),
	tObj( collectiveT, "18", "POBOX", "postOfficeBox" ),
	tObj( collectiveT, "19", NULL, "physicalDeliveryOfficeName" ),
	tObj( collectiveT, "20", "TN", "telephoneNumber" ),
	tObj( collectiveT, "21", NULL, "telexNumber" ),
	tObj( collectiveT, "22", NULL, "teletexTerminalIdentifier" ),
	tObj( collectiveT, "23", "FAX", "facsimileTelephoneNumber" ),
	uObj( "24", NULL, "x121Address" ),
	tObj( collectiveT, "25", "ISDN", "internationalISDNNumber" ),
	uObj( "26", NULL, "registeredAddress" ),
	uObj( "27", NULL, "destinationIndicator" ),
	uObj( "28", NULL, "preferredDeliveryMethod" ),
	uObj( "29", NULL, "presentationAddress" ),
	uObj( "30", NULL, "supportedApplicationContext" ),
	uObj( "31", NULL, "member" ),
	uObj( "32", NULL, "owner" ),
	uObj( "33", NULL, "roleOccupant" ),
	uObj( "34", NULL, "seeAlso" ),
	uObj( "35", NULL, "userPassword" ),
	uObj( "36", NULL, "userCertificate" ),
	uObj( "37", NULL, "cACertificate" ),
	uObj( "38", NULL, "authorityRevocationList" ),
	uObj( "39", NULL, "certificateRevocationList" ),
	uObj( "40", NULL, "crossCertificatePair" ),
	uObj( "41", "N",  "name" ),
	uObj( "42", "G",  "givenName" ),
	uObj( "43", "I",  "initials" ),
	uObj( "44", "GQ", "generationQualifier" ),
	uObj( "45", "UI", "uniqueIdentifier" ),
	uObj( "46", "DQ", "dnQualifier" ),
	uObj( "47", NULL, "enhancedSearchGuide" ),
	uObj( "48", NULL, "protocolInformation" ),
	uObj( "49", NULL, "distinguishedName" ),
	uObj( "50", NULL, "uniqueMember" ),
	uObj( "51", NULL, "houseIdentifier" ),
	uObj( "52", NULL, "supportedAlgorithms" ),
	uObj( "53", NULL, "deltaRevocationList" ),
	uObj( "58", NULL, "ute Certificate (id-at-attributeCertificate)" ),
	uObj( "65", NULL, "Pseudonim" )
};

//
//	End of id-at X.500 attribute types
//
///////////


//////////
//
// IETF PKIX Attribute branch
// ietf-at         					OBJECT IDENTIFIER ::= {1 3 6 1 5 5 7 9}
// PKCS #9 v2.0
//
// { 0x2b, 0x06, 0x01, 0x05, 0x05, 0x07, 0x09 };
//
tUnit ietf_at_T[] =
{
	uObj( "1" , NULL, "dateOfBirth" ),
	uObj( "2" , NULL, "placeOfBirth" ),
	uObj( "3" , NULL, "gender" ),
	uObj( "4" , NULL, "countryOfCitizenship" ),
	uObj( "5" , NULL, "countryOfResidence" )
};

//
// end of ietf_at names OID
//
///////////

//////////
//
// IETF PKIX  Key usege purposes 
//	RFC 2459 - 4.2.1.13  Extended key usage field
// id-kp OBJECT IDENTIFIER ::= { id-pkix 3 }
// id-kp OBJECT IDENTIFIER ::= { 1 3 6 1 5 5 7 3 }
//
tUnit pkix_kp_T[] =
{
	uObj( "1", NULL, "serverAuth" ),
	//	TLS Web server authentication
	//	Key usage bits that may be consistent: digitalSignature,
	//	                        keyEncipherment or keyAgreement
	//
	uObj( "2", NULL, "clientAuth" ),
	//	TLS Web client authentication
	//	Key usage bits that may be consistent: digitalSignature and/or
	//	                           keyAgreement
	//
	uObj( "3", NULL, "codeSigning" ),
	//	Signing of downloadable executable code
	//	Key usage bits that may be consistent: digitalSignature
	//
	uObj( "4", NULL, "emailProtection" ),
	//	E-mail protection
	//	Key usage bits that may be consistent: digitalSignature,
	//	                        nonRepudiation, and/or (keyEncipherment
	//	                        or keyAgreement)
	uObj( "8", NULL, "timeStamping" )
	//	Binding the hash of an object to a time from an agreed-upon time
	//	source. Key usage bits that may be consistent: digitalSignature,
	//	                        nonRepudiation
};


//
// end of id_kp names OID
//
///////////

//////////
//
// x509 id_ce: 2.5.29 
//
// -- Object identifier assignments for ISO certificate extensions --
// -- PKIX 1 extensions
// id-ce   OBJECT IDENTIFIER       ::=     {joint-iso-ccitt(2) ds(5) 29}
//
// RFC 2459        Internet X.509 Public Key Infrastructure    January 1999
//
// p.4.2 Standard Certificate Extensions
// { 0x55, 0x1d };
// "2.5.29"
//
tUnit id_ce_T[] =
{
	uObj( "9", NULL, "subjectDirectoryAttributes" ),
	uObj( "14", NULL, "subjectKeyIdentifier" ),
	uObj( "15", NULL, "keyUsage" ),
	uObj( "16", NULL, "privateKeyUsagePeriod" ),
	uObj( "17", NULL, "subjectAltName" ),
	uObj( "18", NULL, "issuerAltName" ),
	uObj( "19", NULL, "basicConstraints" ),
	uObj( "20", NULL, "cRLNumber" ),
	uObj( "21", NULL, "reasonCode" ),
	uObj( "23", NULL, "instructionCode" ),
	uObj( "24", NULL, "invalidityDate" ),
	uObj( "27", NULL, "deltaCRLIndicator" ),
	uObj( "28", NULL, "issuingDistributionPoint" ),
	uObj( "29", NULL, "certificateIssuer" ),
	uObj( "30", NULL, "nameConstraints" ),
	uObj( "31", NULL, "cRLDistributionPoints" ),
	uObj( "32", NULL, "certificatePolicies" ),
	uObj( "33", NULL, "policyMappings" ),
	uObj( "36", NULL, "policyConstraints" ),
	uObj( "35", NULL, "authorityKeyIdentifier" ),
	uObj( "37", NULL, "extKeyUsage"),

	// From "WINCRYPT.h"

	uObj( "1", NULL, "AUTHORITY_KEY_IDENTIFIER" ),
	uObj( "2", NULL, "KEY_ATTRIBUTES" ),
	uObj( "4", NULL, "KEY_USAGE_RESTRICTION" ),
	uObj( "7", NULL, "SUBJECT_ALT_NAME" ),
	uObj( "8", NULL, "ISSUER_ALT_NAME" ),
	uObj( "10", NULL, "BASIC_CONSTRAINTS" ),

	uObj( "5", NULL, "POLICY_MAPPINGS" )
};

//
// end of id_ce names OID
//
///////////

//////////
//
// x9-57 x9algorithm
//
// "1.2.840.10040.4"
//
//	RFC 2459        Internet X.509 Public Key Infrastructure
//	7.2.2  DSA Signature AlgorithmtUnit
//
tUnit x9algorithm_T[] =
{
	uObjP( "1", "id-dsa", "DSA signature key", pgppkaDSA ),
	uObjHP( "3", "id-dsa-with-sha1", "DSA signature algorithm", pgphaSHA1, pgppkaDSA )
};

//
// end of x9-57 x9algorithm OID
//
///////////


//////////
//
//	Netscape extensions within X.509 version 3 certificates
//	OID "2.16.840.1.113730.1"
//  see http://www.netscape.com/eng/security/cert-exts.html
//
tUnit netscapeCertExt_T[] =
{
	uObj( "1", NULL, "CertType" ), 
	uObj( "2", NULL, "Base URL" ),
	uObj( "3", NULL, "Revocation URL" ),
	uObj( "4", NULL, "CA Revocation URL" ),
	uObj( "7", NULL, "Renewal URL" ),
	uObj( "8", NULL, "Netscape CA policy URL" ),
	uObj( "12", NULL, "SSL server name" ),
	uObj( "13", NULL, "Netscape certificate comment" )
};


//
// 2.16.840.1.113730 - Netscape Communications Corp
//
tUnit netscape_T[] = 
{
	uObj( "4.1", "nsSGC", "Netscape Server Gated Crypto"),
	tObj( netscapeCertExt_T, "1", "NetscapeCertExt", "Netscape extensions within X.509 version 3 certificates" )
};

//
// end of x9-57 x9algorithm OID
//
///////////

//////////
//
//	Microosft OIDs 1.3.6.1.4.1.311
//
tUnit microsoft_T[] =
{
	uObj( "10.3.3", "msSGC", "Microsoft Server Gated Crypto" ),
	uObj( "10.1", "CTL", "Microsoft PKCS #7 ContentType Object Identifiers" ),
	uObj( "2.1.14", "CERT_EXTENSIONS", NULL ),
	uObj( "10.2", "NEXT_UPDATE_LOCATION", NULL )
};

//
//	End of Microsoft OIDs
//
//////////////

//////////////
//
//	2.16.840.1.101.3.4 - nistAlgorithms

tUnit nistAlgorithms_T[] =
{
	uObjS( "1.1", NULL, "aes128-ECB", pgpsaAES_128, pgpsamodeECB ),
	uObjS( "1.2", NULL, "aes128-CBC", pgpsaAES_128, pgpsamodeCBC ),
	uObjS( "1.3", NULL, "aes128-OFB", pgpsaAES_128, pgpsamodeOFB ),
	uObjS( "1.4", NULL, "aes128-CFB", pgpsaAES_128, pgpsamodeCFB ),

	uObjS( "1.21", NULL, "aes192-ECB", pgpsaAES_192, pgpsamodeECB ),
	uObjS( "1.22", NULL, "aes192-CBC", pgpsaAES_192, pgpsamodeCBC ),
	uObjS( "1.23", NULL, "aes192-OFB", pgpsaAES_192, pgpsamodeOFB ),
	uObjS( "1.24", NULL, "aes192-CFB", pgpsaAES_192, pgpsamodeCFB ),

	uObjS( "1.41", NULL, "aes256-ECB", pgpsaAES_256, pgpsamodeECB ),
	uObjS( "1.42", NULL, "aes256-CBC", pgpsaAES_256, pgpsamodeCBC ),
	uObjS( "1.43", NULL, "aes256-OFB", pgpsaAES_256, pgpsamodeOFB ),
	uObjS( "1.44", NULL, "aes256-CFB", pgpsaAES_256, pgpsamodeCFB ),

	uObjH( "2.1", NULL, "SHA-256", pgphaSHA256 ),	// 2.16.840.1.101.3.4.2.1 - sha-256 [RFC4055]
	uObjH( "2.2", NULL, "SHA-384", pgphaSHA384 ),	// 2.16.840.1.101.3.4.2.2 - sha-384	[RFC4055]
	uObjH( "2.3", NULL, "SHA-512", pgphaSHA512 ),	// 2.16.840.1.101.3.4.2.3 - SHA-512 [RFC4055]
	uObjH( "2.4", NULL, "SHA-224", pgphaSHA224 ),	// 2.16.840.1.101.3.4.2.4 - sha-224	[RFC4055]

};

//2.16.840.1.101.3.4.1.1 - id-aes128-ECB
//2.16.840.1.101.3.4.1.2 - id-aes128-CBC
//2.16.840.1.101.3.4.1.3 - id-aes128-OFB
//2.16.840.1.101.3.4.1.4 - id-aes128-CFB
//
//2.16.840.1.101.3.4.1.21 - id-aes192-ECB
//2.16.840.1.101.3.4.1.22 - id-aes192-CBC
//2.16.840.1.101.3.4.1.23 - id-aes192-OFB
//2.16.840.1.101.3.4.1.24 - id-aes192-CFB
//
//2.16.840.1.101.3.4.1.41 - id-aes256-ECB
//2.16.840.1.101.3.4.1.42 - id-aes256-CBC
//2.16.840.1.101.3.4.1.43 - id-aes256-OFB
//2.16.840.1.101.3.4.1.44 - id-aes256-CFB
//
//////////////

//////////////
//
//	Main OID table
//
tUnit OID_T[] = 
{
   uObjP( "1.2.840.10046.2.1", "dhpublicnumber", "X9.42 Diffie-Hellman", pgppkaElgamal ),
   uObjP( "2.5.8.1.1", "id-ea-rsa", "X.509 defined RSA encryption algorithm", pgppkaRSA ),
	tObj( x9algorithm_T, "1.2.840.10040.4", "x9algorithm", "IETF PKIX X9.57 x9algorithm" ),
	tObj( pkcs_T,	"1.2.840.113549.1", "pkcs", "Public Key Cryptosystems" ),
	tObj( id_at_T,	"2.5.4",			"id-at", "X.500 attribute types" ),
	tObj( ietf_at_T,"1.3.6.1.5.5.7.9",	"ietf-at", "IETF PKIX Attribute branch" ), // PKCS #9 v2.0 
	tObj( pkix_kp_T,"1.3.6.1.5.5.7.3",	"pkix-kp", "IETF PKIX Key purpouses" ), 
	tObj( id_ce_T,	"2.5.29",			"id-ce", "Object identifier for ISO certificate extensions" ),
	tObj( secSig_T,	"1.3.14.3.2",		"secSIG", "OIW Security Special Interest Group" ),
	tObj( netscape_T, "2.16.840.1.113730", "Netscape", "Netscape Communications Corp" ),
	tObj( microsoft_T, "1.3.6.1.4.1.311", NULL, "Microsoft" ),
	tObj( nistAlgorithms_T, "2.16.840.1.101.3.4", "nistAlgorithms", "Algorithms proposed by NIST" )
};

int OID_Q = sizeof( OID_T ) / sizeof( *OID_T );

//
//	End of main OID table
//
/////////////
