#include <windows.h>
#include "objectIdentifiers.h"

//
// http://www.alvestrand.no/objectid/
//

//
// RFC 1779                   DN Representation
//
//	Key     Attribute (X.520 keys)
//	------------------------------
//	CN      CommonName
//	L       LocalityName
//	ST      StateOrProvinceName
//	O       OrganizationName
//	OU      OrganizationalUnitName
//	C       CountryName
//	STREET  StreetAddress ??
//


//
//	RFC 2459        Internet X.509 Public Key Infrastructure    January 1999
//

//////////
//
// X.500 attribute types OID { id-at } "2.5.4"
//

BYTE OBJ_C_x500_at[] = { 0x55, 0x04 };

objUnit OBJ_T_x500_at[] =
{
	{ 0x03 , "CN",			"common name" },
	{ 0x04 , "S",			"surname" },
	{ 0x05 , "SN",			"serial number" },
	{ 0x06 , "C",			"country" },
	{ 0x07 , "L",			"locality name" },
	{ 0x08 , "ST",			"state or province name" },
	{ 0x09 , "STREET",		"street address" },
	{ 0x0a , "O",			"organization" },
	{ 0x0b , "OU",			"organizational-unit" },
	{ 0x0C , "T",			"title" },
	{ 0x0D , "D",			"description" },
	{ 0x0F , "BC",			"business category" },
	{ 0x10 , "PA"			"postal address" },
	{ 0x11 , NULL,			"postal code" },
	{ 0x12 , "POBOX",		"post office box" },
	{ 0x13 , NULL,			"physical delivery office name" },
	{ 0x14 , "TN",			"telephone number"  },
	{ 0x16 , NULL,			"teletex terminal identifier"  },
	{ 0x17 , "FAX",			"fax number"  },
	{ 0x18 , NULL,			"x121Address" },
	{ 0x19 , "ISDN",		"international ISDN number" },
	{ 0x1b , "DI",			"destinationIndicator" },
	{ 0x23 , "PWD"			"userPassword" },

	{ 0x29 , "N",			"name" },
	{ 0x2a , "G",			"given name" },
	{ 0x2b , "I",			"initials" },
	{ 0x2c , "GQ",			"generation qualifier" },
	{ 0x2d , "UI",			"unique identifier" },
	{ 0x2e , "DQ",			"distinguished name qualifier" },
	{ 0x41 , "Pseudonim",	"Pseudonim" }, // { id-at 65 }

	// All from http://www.alvestrand.no/objectid/2.5.4.html

	{ 0, NULL, "objectClass" },
	{ 1, NULL, "aliasedEntryName" },
	{ 2, NULL, "knowldgeinformation" },
//	{ 3, NULL, "commonName" },
//	{ 4, NULL, "surname" },
//	{ 5, NULL, "serialNumber" },
//	{ 6, NULL, "countryName" },
//	{ 7, NULL, "localityName" },
//	{ 8, NULL, "stateOrProvinceName" },
//	{ 9, NULL, "streetAddress" },
//	{ 10, NULL, "organizationName" },
//	{ 11, NULL, "organizationalUnitName" },
//	{ 12, NULL, "title" },
//	{ 13, NULL, "description" },
	{ 14, NULL, "searchGuide" },
//	{ 15, NULL, "businessCategory" },
//	{ 16, NULL, "postalAddress" },
//	{ 17, NULL, "postalCode" },
//	{ 18, NULL, "postOfficeBox" },
//	{ 19, NULL, "physicalDeliveryOfficeName" },
//	{ 20, NULL, "telephoneNumber" },
	{ 21, NULL, "telexNumber" },
//	{ 22, NULL, "teletexTerminalIdentifier" },
//	{ 23, NULL, "facsimileTelephoneNumber" },
//	{ 24, NULL, "x121Address" },
//	{ 25, NULL, "internationalISDNNumber" },
	{ 26, NULL, "registeredAddress" },
//	{ 27, NULL, "destinationIndicator" },
	{ 28, NULL, "preferredDeliveryMethod" },
	{ 29, NULL, "presentationAddress" },
	{ 30, NULL, "supportedApplicationContext" },
	{ 31, NULL, "member" },
	{ 32, NULL, "owner" },
	{ 33, NULL, "roleOccupant" },
	{ 34, NULL, "seeAlso" },
	{ 35, NULL, "userPassword" },
	{ 36, NULL, "userCertificate" },
	{ 37, NULL, "cACertificate" },
	{ 38, NULL, "authorityRevocationList" },
	{ 39, NULL, "certificateRevocationList" },
	{ 40, NULL, "crossCertificatePair" },
//	{ 41, NULL, "name" },
//	{ 42, NULL, "givenName" },
//	{ 43, NULL, "initials" },
//	{ 44, NULL, "generationQualifier" },
//	{ 45, NULL, "uniqueIdentifier" },
//	{ 46, NULL, "dnQualifier" },
	{ 47, NULL, "enhancedSearchGuide" },
	{ 48, NULL, "protocolInformation" },
	{ 49, NULL, "distinguishedName" },
	{ 50, NULL, "uniqueMember" },
	{ 51, NULL, "houseIdentifier" },
	{ 52, NULL, "supportedAlgorithms" },
	{ 53, NULL, "deltaRevocationList" },
	{ 58, NULL, "attributeCertificate" }
};

//
// end of x509 names OID
//
///////////


//////////
//
// x509 id_ce: 2.5.29 
//
// -- Object identifier assignments for ISO certificate extensions --
// -- PKIX 1 extensions
// id-ce   OBJECT IDENTIFIER       ::=     {joint-iso-ccitt(2) ds(5) 29}
//
// RFC 2459        Internet X.509 Public Key Infrastructure    January 1999
//

BYTE OBJ_C_id_ce[] = { 0x55, 0x1d }; // "2.5.29"

objUnit OBJ_T_id_ce[] =
{
	{  9, NULL, "subjectDirectoryAttributes" },
	{ 14, NULL, "subjectKeyIdentifier" },
	{ 15, NULL, "keyUsage" },
	{ 16, NULL, "privateKeyUsagePeriod" },
	{ 17, NULL, "subjectAltName" },
	{ 18, NULL, "issuerAltName" },
	{ 19, NULL, "basicConstraints" },
	{ 20, NULL, "cRLNumber" },
	{ 21, NULL, "reasonCode" },
	{ 23, NULL, "instructionCode" },
	{ 24, NULL, "invalidityDate" },
	{ 27, NULL, "deltaCRLIndicator" },
	{ 28, NULL, "issuingDistributionPoint" },
	{ 29, NULL, "certificateIssuer" },
	{ 30, NULL, "nameConstraints" },
	{ 31, NULL, "cRLDistributionPoints" },
	{ 32, NULL, "certificatePolicies" },
	{ 33, NULL, "policyMappings" },
	{ 36, NULL, "policyConstraints" },
	{ 35, NULL, "authorityKeyIdentifier" },
	{ 37, NULL, "extKeyUsage" }
};

//
// end of id_ce names OID
//
///////////

//////////
//
// IETF PKIX Attribute branch
// ietf-at         					OBJECT IDENTIFIER ::= {1 3 6 1 5 5 7 9}
// PKCS #9 v2.0
//

BYTE OBJ_C_ietf_at[] = { 0x2b, 0x06, 0x01, 0x05, 0x05, 0x07, 0x09 };

objUnit OBJ_T_ietf_at[] =
{
	{ 0x01 , NULL, "dateOfBirth" },
	{ 0x02 , NULL, "placeOfBirth" },
	{ 0x03 , NULL, "gender" },
	{ 0x04 , NULL, "countryOfCitizenship" },
	{ 0x05 , NULL, "countryOfResidence" }
};

//
// end of ietf_at names OID
//
///////////

//////////
//
// pkcs #1 OID
//
// The DER for this in hexadecimal is:
// 06 08 
//    2A 86 48 86 F7 0D 01 01
//
// pkcs-1 OBJECT IDENTIFIER ::= {
//   iso(1) member-body(2) us(840) rsadsi(113549) pkcs(1) 1 }
//

BYTE OBJ_C_pkcs1[] = { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x01 };

objUnit OBJ_T_pkcs1[] =
{
	{ 0x01, NULL, "rsaEncryption"},
	{ 0x02, NULL, "md2WithRSAEncryption"},
	{ 0x03, NULL, "md4WithRSAEncryption"},
	{ 0x04, NULL, "md5WithRSAEncryption"},
	{ 0x05, NULL, "sha1WithRSAEncryption"},
	{ 0x06, NULL, "rsaOAEPEncryptionSET"},
	{ 0x07, NULL, "id-RSAES-OAEP"},
	{ 0x08, NULL, "id-mgf1"},
	{ 0x09, NULL, "id-pSpecified" }
};

//
// end of pkcs #1 OID
//
///////////

//////////
//
//
// OIW Security Special Interest Group OIDs. http://www.alvestrand.no/objectid/1.3.14.3.2.html
//
// id-sha1                OBJECT IDENTIFIER ::= { 
//   iso(1) identified-organization(3) oiw(14) secsig(3)
//   algorithms(2) 26 }
// { 1.3.14.3.2.26 }
//

BYTE OBJ_C_secSig[] = { 0x2B, 0x0E, 0x03, 0x02 }; // "1.3.14.3.2"

objUnit OBJ_T_secSig[] =
{
	{ 0x02, NULL, "md4WithRSA" },
	{ 0x03, NULL, "md5WithRSA" },
	{ 0x04, NULL, "md4WithRSAEncryption" },
	{ 0x06, NULL, "desECB" },
	{ 0x07, NULL, "desCBC" },
	{ 0x08, NULL, "desOFB" },
	{ 0x09, NULL, "desCFB" },
	{ 0x0A, NULL, "desMAC" },
	{ 0x0B, NULL, "rsaSignature" },
	{ 0x0C, NULL, "dsa" },
	{ 0x0D, NULL, "dsaWithSHA" },
	{ 0x0E, NULL, "mdc2WithRSASignature" },
	{ 0x0F, NULL, "shaWithRSASignature" },
	{ 0x10, NULL, "dhWithCommonModulus" },
	{ 0x11, NULL, "desEDE" },	// 017
	{ 0x12, NULL, "sha" },	// 018
	{ 0x13, NULL, "mdc-2" },	// 019
	{ 0x14, NULL, "dsaCommon" },	// 020
	{ 0x15, NULL, "dsaCommonWithSHA" },	// 021
	{ 0x16, NULL, "rsaKeyTransport" },	// 022
	{ 0x17, NULL, "keyed-hash-seal" },	// 023
	{ 0x18, NULL, "md2WithRSASignature" },	// 024
	{ 0x19, NULL, "md5WithRSASignature" },	// 025
	{ 0x1A, "SHA1","SHA-1 hash algorithm" },	// 0026
	{ 0x1B, NULL, "dsa With SHA1" },	// 027
	{ 0x1C, NULL, "dsa With SHA1 with Common Parameters" },	// 028
	{ 0x1E, NULL, "SHA1 with RSA signature" }	// 029
};

//
// end of secSig OID
//
///////////

//////////
//
// pkcs7 OID
//

BYTE OBJ_C_pkcs7[] = { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x07 };

objUnit OBJ_T_pkcs7[] =
{
	{ 0x01, NULL, "data" },
	{ 0x02, NULL, "signedData" },
	{ 0x03, NULL, "envelopedData" },
	{ 0x07, NULL, "signedAndEnvelopedData" },
	{ 0x05, NULL, "digestedData" },
	{ 0x06, NULL, "encryptedData" }
};


//
// end of pkcs7 OID
//
///////////

//////////
//
// pkcs9 OID
//

BYTE OBJ_C_pkcs9[] = { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x09 };

objUnit OBJ_T_pkcs9[] =
{
	{ 0x01, "E-mail", "email address" },
	{ 0x02, NULL, "unstructured name" },
	{ 0x03, NULL, "content type" },
	{ 0x04, NULL, "message digest" },
	{ 0x05, NULL, "signing time" },
	{ 0x06, NULL, "counter signature" },
	{ 0x07, NULL, "challenge rassword" },
	{ 0x08, NULL, "unstructured address" },
	{ 0x09, NULL, "extCertAttributes" },
//-- Obsolete (?) attribute identifiers, purportedly from "tentative PKCS #9 draft"
		{ 0x0A, NULL, "issuerAndSerialNumber" },// 10
		{ 0x0B, NULL, "passwordCheck" },		// 11
		{ 0x0C, NULL, "publicKey" },			// 12

	{ 0x0D, NULL, "signingDescription" },	// 13
	{ 0x0E, NULL, "extensionRequest" },		// 14
	{ 0x0F, NULL, "smimeCapabilities" },	// 15

//-- Unused (?)
//-- pkcs-9-at-? 	OBJECT IDENTIFIER ::= {pkcs-9 17}
//-- pkcs-9-at-? 	OBJECT IDENTIFIER ::= {pkcs-9 18}
//-- pkcs-9-at-? 	OBJECT IDENTIFIER ::= {pkcs-9 19}

	{ 0x14, NULL, "friendlyName" },			// 20
	{ 0x15, NULL, "localKeyId" }			// 21

};

//
// end of pkcs9 OID
//
///////////

//////////
//
// pkcs9 attributes branch OIDs
// pkcs-9-at { pkcs-9 25 }
//

BYTE OBJ_C_pkcs9at[] = { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x09, 0x19 };

objUnit OBJ_T_pkcs9at[] =
{
	{ 0x01, NULL, "pkcs15Token" },
	{ 0x02, NULL, "encryptedPrivateKeyInfo" },
	{ 0x03, NULL, "randomNonce" },
	{ 0x04, NULL, "sequenceNumber" },
	{ 0x05, NULL, "pkcs7PDU" }
};

//
// end of pkcs-9-at OID
//
///////////


//////////
//
// pkcs12 Password based encrypted OID
//

BYTE OBJ_C_pkcs12_PBE[] = { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x0C, 0x01 };

objUnit OBJ_T_pkcs12_PBE[] =
{
	{ 0x01, NULL, "pbeWithSHAAnd128BitRC4" },
	{ 0x02, NULL, "pbeWithSHAAnd40BitRC4" },
	{ 0x03, NULL, "pbeWithSHAAnd3-KeyTripleDES-CBC" },
	{ 0x04, NULL, "pbeWithSHAAnd2-KeyTripleDES-CBC" },
	{ 0x05, NULL, "pbeWithSHAAnd128BitRC2-CBC" },
	{ 0x06, NULL, "pbewithSHAAnd40BitRC2-CBC" }
};

//
// end of pkcs12 PBE OID
//
///////////

//////////
//
// pkcs12 bagtypes OBJECT IDENTIFIER ::= {pkcs-12 10 1}
//

BYTE OBJ_C_pkcs12_BT[] = { 0x2A,0x86,0x48,0x86,0xF7,0x0D,0x01,0x0C, 0x0A, 0x01 };

objUnit OBJ_T_pkcs12_BT[] =
{
	{ 0x01, NULL, "KeyBag" },
	{ 0x02, NULL, "PKCS8ShroudedKeyBag" },
	{ 0x03, NULL, "CertBag" },
	{ 0x04, NULL, "CRLBag" },
	{ 0x05, NULL, "SecretBag" },
	{ 0x06, NULL, "SafeContents" }
};

//
// end of pkcs12 bagtypes OID
//
///////////

////////
//
//	Common OID table
//

#define TObj( obj, name, desc ) { OBJ_C_##obj, sizeof( OBJ_C_##obj ), name, desc, OBJ_T_##obj, sizeof( OBJ_T_##obj ) / sizeof( *OBJ_T_##obj ) }

tableUnit COID_T[] = 
{
	TObj( pkcs12_BT,	"pkcs-12-bt",	"pkcs #12 BagTypes" ),
	TObj( pkcs12_PBE,	"pkcs-12-pbe",	"pkcs #12 Password Based Encryption types" ),
	TObj( pkcs9,		"pkcs-9",		"pkcs #9" ),
	TObj( pkcs7,		"pkcs-7",		"pkcs #7" ),
	TObj( pkcs1,		"pkcs-1",		"pkcs #1" ),
	TObj( secSig,		"secSIG",		"Security Special Interest Group" ),
	TObj( id_ce,		"id-ce",		"PKI Certificate Extensions" ),
	TObj( ietf_at,		"ietf-at",		"IETF PKIX Attribute branch" ),
	TObj( x500_at,		"id-at",		"X.500 attribute types" )
};
/*

tableUnit COID_T[] = 
{
	{ OBJ_pkcs12_BT_C,	sizeof( OBJ_pkcs12_BT_C ),	"pkcs-12-bt",	"pkcs #12 BagTypes",				OBJ_pkcs12_BT_T,	TSize( OBJ_pkcs12_BT_T ) },
	{ OBJ_pkcs12_PBE_C, sizeof( OBJ_pkcs12_PBE_C ), "pkcs-12-pbe",	"pkcs #12 Password Based Encryption types",OBJ_pkcs12_PBE_T,	TSize( OBJ_pkcs12_PBE_T ) },
	{ OBJ_pkcs9_C,		sizeof( OBJ_pkcs9_C ),		"pkcs-9",		"pkcs #9",							OBJ_pkcs9_T,		TSize( OBJ_pkcs9_T ) },
	{ OBJ_pkcs7_C,		sizeof( OBJ_pkcs7_C ),		"pkcs-7",		"pkcs #7",							OBJ_pkcs7_T,		TSize( OBJ_pkcs7_T ) },
	{ OBJ_pkcs1_C,		sizeof( OBJ_pkcs1_C ),		"pkcs-1",		"pkcs #1",							OBJ_pkcs1_T,		TSize( OBJ_pkcs1_T ) },
	{ OBJ_secSig_C,		sizeof( OBJ_secSig_C ),		"secSIG",		"Security Special Interest Group",		OBJ_secSig_T,		TSize( OBJ_secSig_T ) },
	{ OBJ_id_ce_C,		sizeof( OBJ_id_ce_C ),		"id-ce",		"PKI Certificate Extensions",			OBJ_id_ce_T,		TSize( OBJ_id_ce_T ) },
	{ OBJ_ietf_at_C,	sizeof( OBJ_ietf_at_C ),	"ietf-at",		"IETF PKIX Attribute branch",			OBJ_ietf_at_T,		TSize( OBJ_ietf_at_T ) },
	{ OBJ_x500_at_C,	sizeof( OBJ_x500_at_C ),	"id-at",		"X.500 attribute types",		OBJ_x500_at_T,			TSize( OBJ_x500_at_T ) }
};
*/
const int COID_Q = TSize( COID_T );

//
// End of Common OID table
//
////////



////////////////
//
//
//

#include <stdio.h>

//
//	Function: oid2string
//
//	converts a DER encoding ASN.1 object-identifier into dot-string format
// 
char *oid2string( char *buf, BYTE *oidVal, DWORD oidSize )
{
	DWORD ioff;
	DWORD ibeg;
	int a1;
	int t;

	if ( oidVal[0] < 40 )
		a1 = 0;
	else if ( oidVal[0] < 80 )
		a1 = 1;
	else
		a1 = 2;

    sprintf( buf, "%d.%d", a1, oidVal[0] - 40 * a1 );
    
    for ( ioff = 1; ioff < oidSize; ioff++ )
    {
	    ibeg = ioff;
	    while ( oidVal[ioff] & 0x80 && ioff < oidSize )
	        ioff++;
	    for ( t = 0; ibeg <= ioff; ibeg++ )
	        t += (int)((oidVal[ibeg] & 0x7f)) * ( int(1) << ( 7 * (ioff - ibeg) ));
	    sprintf( buf + strlen (buf), ".%d", t );
    }

    return buf;
}

//
//	Function: string2oid
//		Converts OID string to BYTE buffer.
//		Returns -1 if *pOidSize too small, otherwise return filled size.
//
int string2oid( BYTE *oid, int *pOidSize, const char *str )
{
    int num;
    int count = 0;
    int first = 0;
    int offset = 0;
    unsigned char buf[256];

    while (*str)
    {
		for ( num = 0; *str && *str != '.'; str++ )
		{
			num *= 10;
			num += *str - '0';
		}
		if ( *str )
			str++;
		if ( count == 0 )
			first = num;
		else
		{
			if ( count == 1 )
				num += 40 * first;
			if ( num > 0x200000 ) /* 0x80^3 */
			{
				buf[offset++] = 0x80 | (num / 0x200000);
				num -= 0x200000 * (num / 0x200000);
			}
			if ( num > 0x4000 ) /* 0x80^2 */
			{
				buf[offset++] = 0x80 | (num / 0x4000);
				num -= 0x4000 * (num / 0x4000);
			}
			if ( num > 128 )
			{
				buf[offset++] = 0x80 | (num / 0x80);
				num -= 0x80 * (num / 0x80);
			}
			buf[offset++] = num;
		}
		count++;
    }

	if ( *pOidSize < offset )
	{
		*pOidSize = offset;
		return -1;
	}

	*pOidSize = offset;
    memcpy ( oid, buf, offset );

    return offset;
}

//
//	Function: findOID
//		Tries to find an OID in Common oid table.
//		Returns -1 if specified OID was not found.
//
int findOID( int *pNum, int *pSubNum, BYTE *OID, int OIDSize )
{
	int n = COID_Q;
	int subNum = 0;
	while( n-- )
	{
		if (( OIDSize - 1 == COID_T[ n ].codeSize ) &&
			( memcmp( COID_T[ n ].code, OID, OIDSize - 1 ) == 0 )
		   )
		{
			//
			//	Find in subtable..
			//

			subNum = COID_T[ n ].Q;

			while ( subNum-- )
			{
				if ( COID_T[ n ].T[ subNum ].code == OID[ OIDSize - 1 ] )
					break;
			}

			if ( subNum != -1 )
				break;
		}
	}

	*pNum = n;
	*pSubNum = subNum;

	return n;
}

//
//	Function: getOIDName
//		Extracts the oid name and name of main table. 
//
char *getOIDName( char **pName, char **pSubName, int tabN, int subTabN )
{
	char *name, *subName;

	if ( ( tabN == -1 ) || ( subTabN == -1 ) )
		subName = name = "UDF";
	else
	{
		subName = COID_T[ tabN ].T[ subTabN ].name;
		if ( subName == NULL )
			subName = COID_T[ tabN ].T[ subTabN ].description;
		if ( subName == NULL )
			subName = "NULL";

		name = COID_T[ tabN ].name;
		if ( name == NULL )
			name = COID_T[ tabN ].description;
		if ( name == NULL )
			name = "NULL";
	}

	if ( pName )	*pName		= name;
	if ( pSubName ) *pSubName	= subName;

	return subName;
}