/////////////////////////////////////////////////////////////////
//
//	Copyright (c) 1994-2008 Jetico Inc., Finland
//	All rights reserved.
//
//	File: x509_tags.h
//	Revision: 
//	Created:
//	Description: ASN.1 tags used in certificate library
//
/////////////////////////////////////////////////////////////////

#ifndef __x509tags__
#define __x509tags__

//
//	ASN.1 codes
//
//	Built-in Types
//
//	Simple Types 
//	1	: BOOLEAN		- Model logical, two-state variable vaules 
//	2	: INTEGER		- Model integer variable values 
//	3	: BIT STRING	- Model binary data of arbitrary length 
//	4	: OCTET STRING	- Model binary data whose length is a multiple of eight 
//	5	: NULL			- Indicate effective absence of a sequence element 
//	6	: OBJECT IDENTIFIER  - Name information objects 
//	9	: REAL			- Model real variable values 
//	10	: ENUMERATED	- Model values of variables with at least three states 
//
//	CHARACTER STRING 
//	* - Models values that are strings of characters from a specified character setn (* means more than one tag)
//	Character String types
//	12	: NumericString (18)	- 0,1,2,3,4,5,6,7,8,9 and space 
//	13	: PrintableString (19)	- Upper and lower case letters, digits, space, apostrophe, left/right parenthesis, plus sign, comma, hyphen, full stop, solidus, colon, equal sign, question mark 
//	14	: TelextexString (T62String) (20) - The CCITT T61 character set 
//	15	: VideotexString (21)	- CCITT's T.100 and T.101 character sets 
//	1A	: VisibleString (ISO646String) (26) - Printing character sets fo International ASCII 
//	16	: IA5String (22)		- International Alphabet 5 (International ASCII) 
//	19	: GraphicString (25)	- All registered G sets, and space 
//	1B	: GeneralString (27)	- All registered C and G sets, space, and delete
//
//	ASN.1 Structured Types
//	30	: SEQUENCE (16) - Models an ordered collection of variables of different types 
//	30	: SEQUENCE OF (16) - Models an ordered collection of variables of the same type 
//	31	: SET (17) - Model of an unordered collection of variables of different types 
//	31	: SET OF (17) - Model of an unordered collection of variables of the same type 
//	*	: CHOICE (*) - Specify a collection of distinct types from which to choose one type 
//	*	: SELECTION (*) - Slelect a component type from a specified CHOICE type 
//	*	: ANY (*) - Enable an application to specify type
//

//
//	ASN.1 tags enum.
//	See the 'Built-in Types' description above.
//
enum
{
//	Simple Types 

	asn1tag_END_OF_CONTENTS_MARKER	= 0x00 ,
	asn1tag_BOOLEAN					= 0x01 ,
	asn1tag_INTEGER					= 0x02 ,
	asn1tag_BIT_STRING				= 0x03 ,
	asn1tag_OCTET_STRING			= 0x04 ,
	asn1tag_NULL					= 0x05 ,
	asn1tag_OBJECT_IDENTIFIER		= 0x06 ,
	asn1tag_REAL					= 0x09 ,
	asn1tag_ENUMERATED				= 0x0A ,
	asn1tag_UndefinedString		= 0x0C,	// (12)	// !! The was used at one real certificate as tag for RelativeDistinguishedNames see x509_name::readAttributes
	asn1tag_SEQUENCE				= 0x10 , // (16)
	asn1tag_SET					= 0x11 , // (17)

//	Character String types

	asn1tag_NumericString		= 0x12	, // (18)
	asn1tag_PrintableString 	= 0x13	, // (19)
	asn1tag_TelextexString		= 0x14	, // (T62String) (20)
	asn1tag_VideotexString		= 0x15	, // (21)
	asn1tag_IA5String		= 0x16	, // (22)

	asn1tag_UTCTime			= 0x17	,  // - YYMMDDHHMMSSZ
	asn1tag_GeneralizedTime		= 0x18	,  // - YYYYMMDDHHMMSSZ

	asn1tag_GraphicString		= 0x19	, // (25)
	asn1tag_VisibleString		= 0x1A	, // (ISO646String) (26)
	asn1tag_GeneralString		= 0x1B	, // (27)

	asn1tag_UnicodeString		= 0x1E	, // ?

	asn1tag_TYPE			= 0x1F,

//	Structured Types

	asn1tag_CONSTRUCTED		= 0x20	, // (32), if not -> UNIVERSAL


//	Optional field tag

	asn1tag_OPTIONAL		= 0x80	, // full value is asn1tag_OPTIONAL | asn1tag_CONSTURCTED ( 0xA0 ) + field number( 0x0X )

	asn1tag_CONSTRUCTED_SEQUENCE	= asn1tag_CONSTRUCTED | asn1tag_SEQUENCE,
	asn1tag_CONSTRUCTED_SET			= asn1tag_CONSTRUCTED | asn1tag_SET,
	asn1tag_CONSTRUCTED_OPTIONAL	= asn1tag_CONSTRUCTED | asn1tag_OPTIONAL
};

const char *getTagString( unsigned char T );

#define makeOptionalTag( num ) ( asn1tag_CONSTRUCTED_OPTIONAL | (0x0F & num ) )
#define makeChoiceTag( num ) ( asn1tag_CONSTRUCTED_OPTIONAL | (0x0F & num ) )

#endif // __x509tags__
