/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 */

#include "cipher.h"

#include <string.h>


/// Zero pool for NULL passed as pool parameter.
static const char s_zero_pool[kBCKernel_PoolSize] = {0};


BCCipher::BCCipher()
{
	memset(&m_alg, 0, sizeof(m_alg));
}


int BCCipher::init(bc_alg_id alg_id, bc_mode_id mode_id) 
{
	bc_kern_result rc;
	rc = BCKernelLookupAlgId(alg_id, &m_alg);
	if(rc != kBCKernel_Success) {
		return rc;
	}

    if(!BCKernelIsModeSupported(&m_alg, mode_id)) {
        return kBCKernel_UnsupportedMode;
    }

	m_mode = mode_id;
	return kBCKernel_Success;
}


int BCCipher::init(BCAlgInfo* alg, bc_mode_id mode_id)
{
    if(!BCKernelIsModeSupported(alg, mode_id)) {
		return kBCKernel_UnsupportedMode;
	}

	m_alg = *alg;
	m_mode = mode_id;
	return kBCKernel_Success;
}


void BCCipher::process(uint64_t iv, bc_proc_type type, bc_key_handle key_handle, const char* pool, void* data, size_t nbytes)
{
	if(pool == NULL) {
		pool = s_zero_pool;
	}
	
	if(nbytes % this->blockSize()) {
		throw BCCipherException(kBCKernel_InvalidBuffer);
	}
	
	bc_kern_result rc = BCKernelProcess(m_alg.id, m_mode, type, key_handle, iv, pool, (char*) data, nbytes);
	if(rc != kBCKernel_Success) {
		throw BCCipherException(rc);
	}
}


void BCCipher::process(uint64_t iv, bc_proc_type type, const char* key, size_t keysize, const char* pool, void* data, size_t nbytes) 
{
	if(pool == NULL) {
		pool = s_zero_pool;
	}

	if(keysize < this->keySize()) {
		throw BCCipherException(kBCKernel_InvalidKey);
	}
	
	if(nbytes % this->blockSize()) {
		throw BCCipherException(kBCKernel_InvalidBuffer);
	}

	bc_kern_result rc = BCKernelProcessWithKey(m_alg.id, m_mode, type, key, keysize, iv, pool, (char*) data, nbytes);
	if(rc != kBCKernel_Success) {
		throw BCCipherException(rc);
	}
}


BCEnumAlgorithms::BCEnumAlgorithms() {
	bc_kern_result rc = BCKernelListAlgs(&this->algs, (unsigned*)&this->nalgs);
	if(rc != kBCKernel_Success) {
		throw BCCipherException(rc);
	}
}

BCEnumAlgorithms::~BCEnumAlgorithms() {
	BCKernelFreeAlgList(this->algs);
}


BCEnumModes::BCEnumModes() {
	this->modes[0] = kBCMode_CBC;
	this->modes[1] = kBCMode_LRW;
	this->modes[2] = kBCMode_XTS;
}


