/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 * Library error codes.
 */

#ifndef BC_CONTAINER_ERROR_H
#define BC_CONTAINER_ERROR_H

/// Library return codes.
typedef enum {
    kBCContainer_Success = 0,
    kBCContainer_UnknownFormat,     // Unknown container format
    kBCContainer_InvalidArgument,   // Invalid function argument
    kBCContainer_InvalidAlgorithm,  // Invalid encryption algorithm
    kBCContainer_InvalidPassword,   // Incorrect user password
	kBCContainer_Damaged,		    // Container data is damaged
	kBCContainer_NoKeySpace,		// No space left in for new password
    kBCContainer_IOError,           // Read/write error on container file
    kBCContainer_NoMemory,          // Memory allocation error
	kBCContainer_Unsupported,		// Keygen op not supported
    kBCContainer_Cancelled,         // Operation was cancelled by the user
    kBCContainer_InternalError      // Unknown internal error
} BCContainerResult;


/// Human-readable description of the library error code
static inline const char* BCContainerErrorDecription(BCContainerResult error)
{
	switch (error) {
        case kBCContainer_UnknownFormat:    return "Unknown container format";
        case kBCContainer_InvalidArgument:  return "Invalid argument";   
        case kBCContainer_InvalidAlgorithm: return "Invalid encryption algorithm";
        case kBCContainer_InvalidPassword:  return "Invalid password";
	    case kBCContainer_Damaged:          return "Damaged container file";
	    case kBCContainer_NoKeySpace:       return "No space for new password";
        case kBCContainer_IOError:          return "Container file IO error";
        case kBCContainer_NoMemory:         return "No memory";
        case kBCContainer_Cancelled:        return "Operation cancelled by user";
	    case kBCContainer_Unsupported:      return "Operation not supported";
        case kBCContainer_InternalError:    return "Internal system failure";
		default:                            return "Unknown error"; // FIXIT!
	}
}

#endif

