#ifndef EXCEPTION_H
#define EXCEPTION_H

#include <stdio.h>
#include <exception>
#include <string>

#define BC_EXCEPTION(msg) BCException(__FILE__, __LINE__, __func__, (msg))
#define BC_NOT_IMPLEMENTED  throw BC_EXCEPTION("This function is not implemented.");

class BCException : public std::exception
{
protected:
    std::string msg;
    std::string info;

public:
    BCException() { }

    explicit BCException(std::string msg) 
    {
        this->msg.assign(msg);
    }

    explicit BCException(const char *msg)
    {
        this->msg.assign(msg);
    }

    BCException(const char *fileName, const int lineNumber, 
            const char *functionName, const char *msg) 
    {
        char buffer[256];
        this->msg.assign(msg);
        snprintf(buffer, sizeof(buffer), "%s (%s, line %i): %s", functionName, 
                fileName, lineNumber, this->msg.c_str());
        this->info.assign(buffer);

    }

    const char *GetMessage() const
    {
        return this->msg.c_str();
    }

    const char *GetExceptionInfo() const
    {
        if (this->info.length() == 0)
            return this->msg.c_str();
        return this->info.c_str();
    }

    const char *what() const throw()
    {
        return GetExceptionInfo();
    }

    ~BCException() throw() { }
};

class BCSystemException : public BCException
{
public:
    BCSystemException() :BCException() {}
    BCSystemException(std::string msg) : BCException(msg) { }
    BCSystemException(const char *msg) : BCException(msg) { }
    BCSystemException(const char *fileName, const int lineNumber, 
            const char *functionName, const char *msg) : BCException(fileName, lineNumber,
                functionName, msg) { }
};

#endif // EXCEPTION_H
