#include <stdlib.h>
#include <string.h>

#include "isaac.h"

#include "isaac64.h"

void *IsaacInit(const void* seed, unsigned seedLength)
{
    randctx64 *context = (randctx64*)malloc(sizeof(randctx64));
    const char *pseed = (const char *)seed;
    word i;

    if (!context)
        return NULL;

    // Move seed to context.
    if (pseed)
    {
        for (i = 0; i < RANDSIZ; ++i)
        {
            context->randrsl[i] = pseed[i % seedLength];
        }
    }

    randinit64(context, (seed != NULL));
    return context;
}

void IsaacRelease(void *context_)
{
    randctx64 *context = (randctx64*)context_;
    free(context);
}

void IsaacGenerate(void *context_, void *buffer_, unsigned bufferSize)
{
    randctx64 *context = (randctx64 *)context_;
    char *buffer = (char*)buffer_;
    char *rand = NULL;
    unsigned randSize = 0;
    unsigned i;
    unsigned bytesLeft = bufferSize;
    unsigned copyBytes = bufferSize;

    for (i = 0; i < bufferSize;)
    {
        isaac64(context);
        rand = (char*)context->randrsl;
        randSize = RANDSIZ * sizeof(ub8);
        copyBytes = bytesLeft > randSize? randSize : bytesLeft;
        memcpy((buffer + i), rand, copyBytes);
        bytesLeft -= copyBytes;
        i += copyBytes;
    }
}
//#define NEVER
#ifdef NEVER
int main(int argc, char **argv)
{
    unsigned seedLength = 512;
    char seed[512] = { 0 };
    unsigned bufferSize = 1024 * 1024;
    void *buffer = malloc(bufferSize);
    void *context = IsaacInit(seed, seedLength);
    int i = 0;
    if (!context)
    {
        return -1;
    }
    IsaacGenerate(context, buffer, bufferSize);

    free(buffer);
    IsaacRelease(context);
    return 0;
}
#endif
