/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 * Standalone tool to test disk event monitoring.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>

#include "disk.h"

#define DIE(message, arg...) { fprintf(stderr, message # arg); exit(EXIT_FAILURE); }


static void onDiskEvent(BCDiskRef* diskref, int event, void* arg)
{
    switch (event) {
    case kBCDiskAppeared:
        printf("Disk %s appeared, backing store %s\n", diskref->nodeName(), diskref->backingStore());
        break;
    case kBCDiskDisappeared:
        printf("Disk %s disappeared\n", diskref->nodeName());
        break;
    case kBCDiskChanged:
        printf("Disk %s changed\n", diskref->nodeName());
        break;
    default:
        printf("Unknown event for disk %s\n", diskref->nodeName());
        break;
    };
}

#ifndef TEST_CANCEL
static void sighandler(int sig)
{
    BCStopDiskEventListener();
    exit(0);
}
#endif

int main(int argc, char** argv) 
{
    BCRegisterForDiskEvents(onDiskEvent, kBCDiskAllEvents, NULL);

    if(!BCStartDiskEventListener()) {
        DIE("Could not start disk event listener\n");
    }

#ifdef TEST_CANCEL
    sleep(2);
    BCStopDiskEventListener();
#else
    // install interrupt handlers and create container
    signal(SIGINT, sighandler);
    signal(SIGQUIT, sighandler);
    signal(SIGTERM, sighandler);
    signal(SIGABRT, sighandler);

    while(1) ;
#endif

    return 0;
}

