/*
 *  Created by Evgeny Yakovlev on 7/22/11.
 *  Copyright 2011 Jetico, Inc. All rights reserved.
 *
 *  FAT12/16/32 volume interface.
 *  Currently only BestCrypt required interface is implemented.
 */

#ifndef LIBFS_FAT_H
#define LIBFS_FAT_H

#include "fat_format.h"

struct fs_volume;
struct fs_extent;


/**
 * Internal fat volume superblock structure.
 */
typedef struct fat_superblock {
	fat_bootsect_t	bs;		// bootsector copy
	uint32_t	fat_mask;	// FAT table sector number mask. Also used as a FAT type flag.
	uint32_t	fat_skip;	// Offset relative to FAT table to skip reserved entries
	uint32_t	data_offset;	// Absolute sector number where cluster region mapped in FAT tables begins.
	uint32_t	maxdataclust;	// Maximum data region cluster number. Clusters are in range 2..maxcluster
} fat_superblock_t;

extern int			fat_init_volume(struct fs_volume* vol);
extern void			fat_free_volume(struct fs_volume* vol);
extern void			fat_display_volume(struct fs_volume* vol);

extern struct fs_extent*	fat_get_free_space(struct fs_volume* vol);
extern struct fs_extent*	fat_get_file_by_name(struct fs_volume* vol, const char *fname);
extern struct fs_extent*	fat_get_file_by_id(struct fs_volume* vol, uint32_t id);

#endif
