//
//  Copyright (c) 2011 Jetico, Inc. All rights reserved.
//

#ifndef FSLIB_DISK_H
#define FSLIB_DISK_H


enum {
    kFSPartition_Empty          = 0x00,
    kFSPartition_FAT12          = 0x01,
    kFSPartition_FAT16          = 0x0E,
    kFSPartition_FAT32          = 0x0C,
    kFSPartition_FAT32_CHS      = 0x0B,
    kFSPartition_NTFS_or_EXFAT  = 0x07,
    kFSPartition_HFSP           = 0xaf,
    kFSPartition_LinuxSwap      = 0x82,
    kFSPartition_LinuxNative    = 0x83,
    kFSPartition_Extended       = 0x0F,
    kFSPartition_LinuxExtended  = 0x85,
    kFSPartition_EFI            = 0xEF,    
    kFSPartition_EFI_MBR        = 0xEE, // emulated protected MBR 
    
    kFSPartition_Unknown        = 0xFF
};


static inline const char* fslib_partition_type(int type)
{
    switch(type) {
        case kFSPartition_Empty:            return "Empty";
        case kFSPartition_FAT12:            return "FAT12";
        case kFSPartition_FAT16:            return "FAT16";
        case kFSPartition_FAT32_CHS:        /* fall thru */
        case kFSPartition_FAT32:            return "FAT32";
        case kFSPartition_NTFS_or_EXFAT:    return "NTFS or ExFAT";
        case kFSPartition_HFSP:             return "HFSP";
        case kFSPartition_LinuxSwap:        return "Linux Swap";
        case kFSPartition_LinuxNative:      return "Linux Native";
        case kFSPartition_Extended:         return "Extended";
        case kFSPartition_LinuxExtended:    return "Linux Extended";
        case kFSPartition_EFI:              return "EFI System Partition";
        case kFSPartition_EFI_MBR:          return "EFI Protective MBR";
        default:                            return "Unknown partition type";
    }
}


/**
 * Disk partition extent.
 */
typedef struct fs_partition {
    off_t               start_sector;       // first disk sector of the partition
    off_t               total_sectors;      // total sectors of the partition
    int                 fstype;             // type of the file system
} fs_partition_t;


/**
 * Whole disk info.
 */
typedef struct fs_disk {
    const char*         type;               // partition table type (MBR, GPT)
    char                signature[16];      // disk signature
    unsigned            total_partitions;   // total partitions on disk, the size of the partitions array
    fs_partition_t*     partitions;         // array of partition extents
} fs_disk_t;


#ifdef __cplusplus
extern "C" {
#endif
    
    
    /**
     * Parse disk partition table on specified device
     * @whole_dev       Name of the whole device node.
     * @out_disk        Parsed disk description on success.
     */
    int fslib_parse_disk(const char* whole_dev, fs_disk_t* out_disk);

    
    /**
     * Release disk resources.
     * @disk            Disk to release.
     */
    void fslib_put_disk(fs_disk_t* disk);
    
    
#ifdef __cplusplus
}
#endif


#endif
