//
//  Copyright (c) 2011 Jetico, Inc. All rights reserved.
//  On-disk GUID partition table
//

#ifndef FSLIB_GPT_H
#define FSLIB_GPT_H

#include <stdint.h>
#include "mbr.h"

#pragma pack(push, 1)

#define GPT_SIGNATURE "EFI PART"

/*
typedef union {
    struct {
        uint32_t    a;
        uint8_t     b;
        uint8_t     c;
        uint8_t     d;
        uint64_t    e;
    } values;
    
    char data[16];
} GUID;
*/

typedef struct {
    uint32_t    a;
    uint16_t    b;
    uint16_t    c;
    uint8_t     d[8];
} GUID;

static const GUID gGUID_Empty       = {0x00000000, 0x0000, 0x0000, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}};
static const GUID gGUID_EFI         = {0xC12A7328, 0xF81F, 0x11D2, {0xBA, 0x4B, 0x00, 0xA0, 0xC9, 0x3E, 0xC9, 0x3B}};
static const GUID gGUID_HFSP        = {0x48465300, 0x0000, 0x11AA, {0xAA, 0x11, 0x00, 0x30, 0x65, 0x43, 0xEC, 0xAC}};
static const GUID gGUID_AppleBoot   = {0x426F6F74, 0x0000, 0x11AA, {0xAA, 0x11, 0x00, 0x30, 0x65, 0x43, 0xEC, 0xAC}};
/*
 * GUID partition table header.
 * Located in LBA 1 with a backup copy near the end of the disk
 */
struct gpt_header
{
    char            signature[8];   // "EFI PART"
    uint32_t        revision;       // Revision (For GPT version 1.0 (through at least UEFI version 2.3.1), the value is 00 00 01 00)
    uint32_t        header_size;    // Header size in little endian 
    uint32_t        header_crc;     // CRC32 of header (0 to header size), with this field zeroed during calculation
    uint32_t        reserved1;      // Reserved; must be zero
    uint64_t        header_sector;  // Current LBA (location of this header copy)
    uint64_t        backup_sector;  // Backup LBA (location of the other header copy)
    uint64_t        first_sector;   // First usable LBA for partitions (primary partition table last LBA + 1) 
    uint64_t        last_sector;    // Last usable LBA (secondary partition table first LBA - 1)
    uint8_t         disk_guid[16];  // Disk GUID
    
    uint64_t        entries_sector; // Partition entries starting LBA (always 2 in primary copy)
    uint32_t        total_entries;  // Number of partition entries
    uint32_t        entry_size;     // Size of a partition entry (usually 128)
    uint32_t        entries_crc;    // CRC32 of partition array

    char            reserved2[420]; // Reserved; must be zeroes for the rest of the block
};


#define GPT_ENTRY_ATTRIBUTE_SYSTEM_BIT          0
#define GPT_ENTRY_ATTRIBUTE_LEGACY_BIT          2
#define GPT_ENTRY_ATTRIBUTE_READONLY_BIT        60
#define GPT_ENTRY_ATTRIBUTE_HIDDEN_BIT          62
#define GPT_ENTRY_ATTRIBUTE_NOAUTOMOUNT_BIT     63

#define GPT_ENTRY_ATTRIBUTE_VALUE(entry, attribute) ((entry)->attributes & (1 << (attribute)))

#define GPT_ENTRY_SYSTEM(entry)         GPT_ENTRY_ATTRIBUTE_VALUE((entry), GPT_ENTRY_ATTRIBUTE_SYSTEM_BIT)
#define GPT_ENTRY_LEGACY(entry)         GPT_ENTRY_ATTRIBUTE_VALUE((entry), GPT_ENTRY_ATTRIBUTE_LEGACY_BIT)
#define GPT_ENTRY_READONLY(entry)       GPT_ENTRY_ATTRIBUTE_VALUE((entry), GPT_ENTRY_ATTRIBUTE_READONLY_BIT)
#define GPT_ENTRY_HIDDEN(entry)         GPT_ENTRY_ATTRIBUTE_VALUE((entry), GPT_ENTRY_ATTRIBUTE_HIDDEN_BIT)
#define GPT_ENTRY_NOAUTOMOUNT(entry)    GPT_ENTRY_ATTRIBUTE_VALUE((entry), GPT_ENTRY_ATTRIBUTE_NOAUTOMOUNT_BIT)

/*
 * Partition entry
 * GPT has space reserved for up to 128 of these.
 */
struct gpt_entry
{
    GUID            type_guid;          // Partition type GUID
    uint8_t         partition_guid[16]; // Unique partition GUID
    uint64_t        start_sector;       // First LBA (little-endian)
    uint64_t        last_sector;        // Last LBA (inclusive, usually odd)
    uint64_t        attributes;         // Attribute flags (e.g. bit 60 denotes read-only)
    char            name[72];           // Partition name (36 UTF-16LE code units)
};


/*
 * GUID partition table primary data at LBA 0.
 */
struct gpt 
{
    struct mbr_sector   protective_mbr; 
    struct gpt_header   header;
    struct gpt_entry    entries[128];
};

#pragma pack(pop)

#endif
