#ifndef __HFSP_H__
#define __HFSP_H__

#include "hfs_format.h"
#include "fslib.h"

#include <iconv.h>

#define HFSP_BUFSIZE	(1024*1024)


typedef struct hfsp_info {
    HFSPlusVolumeHeader	head;		// a copy of the volume header
    int                 blksz;		// fs allocation block size in sectors;
    iconv_t             hfs2utf;
} hfsp_info_t;


typedef struct bt_context {
    fs_volume_t         *vol;		// current volume; filled by the caller
    fs_extent_t         *ext;		// btree file extent list; filled by the caller
    uint64_t            total;		// total btree size in bytes
    uint32_t            size;		// node size
    uint32_t            root;		// root node id
    uint32_t            depth;		// overall tree depth
    uint32_t            attr;		// tree attributes
    int                 keylen;		// fixed key length
} bt_context_t;

struct bt_param;

typedef int (*bt_compare_f)(bt_context_t *ctx, void *node_key, void *sample);
typedef int (*bt_leaf_f)(bt_context_t *ctx, void *data, struct bt_param *param);

typedef struct bt_param {
    bt_compare_f        compare;	// compare() function
    bt_leaf_f           leaf;		// a function to process leaf nodes
    void                *key;		// the key to compare with
    uint32_t            node;       // btree node number
    uint16_t            offset;     // offset inside btree node where found record begins
    uint16_t            length;     // length of the record inside btree node
    uint32_t            left;		// detected left border
    uint8_t             height;		// current tree level
    void                *result;    // found node data
} bt_param_t;

extern int              hfsp_init_volume(fs_volume_t *vol);
extern fs_extent_t *	hfsp_get_free_space(fs_volume_t *vol);
extern void             hfsp_display_volume(fs_volume_t* vol);
extern void             hfsp_free_volume(fs_volume_t* vol);

extern fs_extent_t *	hfsp_build_extent_list(fs_volume_t *vol, uint32_t fileID, uint8_t fork, HFSPlusForkData *p);
extern int              hfsp_iterate_btree(bt_context_t *ctx, bt_param_t *param);
extern int              hfsp_catalog_tree(fs_volume_t *vol, uint32_t fileID, fs_file_t* out_file);
extern int              hfsp_attrib_tree(fs_volume_t *vol, uint32_t fileID, fs_file_t* out_file);
extern int              hfsp_extents_tree(fs_volume_t *v, uint32_t fileID, uint8_t fork, uint32_t start, uint32_t total, fs_extent_t *ext);

extern void             hfsp_print_fork_data(HFSPlusForkData *p);
extern void             hfsp_print_extent_record(HFSPlusExtentRecord *p);
extern void             hfsp_print_volume_header(HFSPlusVolumeHeader *p);
extern void             hfsp_print_bt_node(BTNodeDescriptor *p);
extern void             hfsp_print_bt_header(BTHeaderRec *p);
extern int              hfsp_print_btree_node(void *p, uint32_t node, uint32_t nodeSize);
extern void		print_unistr(HFSUniStr255 *s);



#endif // __HFSP_H__
