#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdint.h>

#include "fs_endian.h"
#include "hfsp.h"


void hfsp_print_fork_data(HFSPlusForkData *p) 
{
    int	i;

    fprintf(stderr, "logical size: %6lld ", be64(p->logicalSize));
    //    fprintf(stderr, ("clump size  : %10d\n",   be32(p->clumpSize));
    fprintf(stderr, "total blocks: %6d\t",   be32(p->totalBlocks));

    for (i = 0; i < 8; i++) {
	if (0 == p->extents[i].blockCount) {
	    break;
	}
	fprintf(stderr, "%d:%d ", 
	       be32(p->extents[i].startBlock), 
	       be32(p->extents[i].blockCount));
    }
    fprintf(stderr, "\n");
}

void hfsp_print_extent_record(HFSPlusExtentRecord *pr) 
{
    int	i;
	
	HFSPlusExtentDescriptor* p = (HFSPlusExtentDescriptor*) pr;
    fprintf(stderr, "Extent: ");
    for (i = 0; i < 8; i++) {
	if (0 == p[i].blockCount) {
	    break;
	}
	fprintf(stderr, "%d:%d ", 
	       be32(p[i].startBlock), 
	       be32(p[i].blockCount));
    }
    fprintf(stderr, "\n");
}

fs_extent_t* hfsp_copy_fork_data(HFSPlusForkData *p)
{
    return (fs_extent_t*)0;
}

void hfsp_print_volume_header(HFSPlusVolumeHeader *p)
{
    fprintf(stderr, "volume %04hX, version: %04hX\n", be16(p->signature), be16(p->version));
    /*
    fprintf(stderr, "attributes  : %08X\n", be32(p->attributes));
    fprintf(stderr, "last mounted: %08X\n", be32(p->lastMountedVersion));
    fprintf(stderr, "journal info: %08X\n", be32(p->journalInfoBlock));

    fprintf(stderr, ("file count  : %d\n", be32(p->fileCount));
    fprintf(stderr, ("folder count: %d\n", be32(p->folderCount));
    */
    fprintf(stderr, "block size  : %d\n", be32(p->blockSize));
    fprintf(stderr, "total blocks: %d\n", be32(p->totalBlocks));
    fprintf(stderr, "free blocks : %d\n", be32(p->freeBlocks));

    fprintf(stderr, "\n");

    fprintf(stderr, "Allocation file:\n");
    hfsp_print_fork_data(&p->allocationFile);
    
    fprintf(stderr, "Extents file:\n");
    hfsp_print_fork_data(&p->extentsFile);

    fprintf(stderr, "Catalog file:\n");
    hfsp_print_fork_data(&p->catalogFile);

    fprintf(stderr, "Attributes file:\n");
    hfsp_print_fork_data(&p->attributesFile);
    
}

void hfsp_print_bt_node(BTNodeDescriptor *p) 
{
    const char *kind;

    switch(p->kind) {
    case kBTLeafNode:	kind = "Leaf   ";  break;
    case kBTIndexNode:	kind = "Index  ";  break;
    case kBTHeaderNode:	kind = "Header ";  break;
    case kBTMapNode:	kind = "Map    ";  break;
    default:		kind = "Unknown"; 
    }
    fprintf(stderr, "%s node bLink=%d fLink=%d Height=%hhd Records=%hd\n", 
	   kind, 
	   be32(p->bLink), be32(p->fLink), 
	   p->height, be16(p->numRecords));
}

void hfsp_print_bt_header(BTHeaderRec *p) 
{
    const char *type;

    switch(p->keyCompareType) {
    case kHFSCaseFolding:	type = "Case insensitive"; break;
    case kHFSBinaryCompare:	type = "Case sensitive"; break;
    default: type = "Unknown";
    }

    fprintf(stderr, "TREE HEAD: Depth=%hd Root=%d Leaves=%d:%d(%d) Size=%hd Nodes=%d(%d) Attr=%08X keylen=%d %s\n",
	   be16(p->treeDepth), be32(p->rootNode), 
	   be32(p->firstLeafNode), be32(p->lastLeafNode), be32(p->leafRecords),
	   be16(p->nodeSize), 
	   be32(p->totalNodes), be32(p->freeNodes), 
	   be32(p->attributes), 
	   be16(p->maxKeyLength),
	   type);
}

int hfsp_print_btree_node(void *p, uint32_t node, uint32_t nodeSize) 
{
    BTNodeDescriptor	*pn = (BTNodeDescriptor *)p;
    uint16_t		*po = (uint16_t *)(((char *)p) + nodeSize);
    int			i, x;

    fprintf(stderr, "Node %6d: ", node);
    hfsp_print_bt_node(pn);

    switch (pn->kind) {
    case kBTHeaderNode:
	{
	    BTHeaderRec		*ph = (BTHeaderRec *)(pn+1);

	    hfsp_print_bt_header(ph);
	    po = (uint16_t *)(((char *)p) + be16(ph->nodeSize));
	    break;
	}

    case kBTLeafNode:
	break;
    case kBTIndexNode:
	break;
    case kBTMapNode:
	break;
    default:
	fprintf(stderr, "Error: unknown node type\n");
	return -1;
    }
    return 0;

    x = be16(pn->numRecords);
    po -= x;
    for (i = 0; i < x; i++) {
	fprintf(stderr, "Offset %d: %hu\n",  i, be16(po[x-i-1]));
    }

    return 0;
}

void print_unistr(HFSUniStr255 *p)
{
    char                *iname, *oname, buf[512];
    size_t              ileft, oleft, sz;
    iconv_t             cvt;

    sz = be16(p->length);
    if (!sz) {
        fprintf(stderr, "NULL");
        return;
    }
    cvt  = iconv_open("UTF-8", "UTF-16");

    iname = (char *)&(p->unicode[0]);
    ileft = sz * 2;
    oname = buf;
    oleft = sizeof(buf);

    iconv(cvt, &iname, &ileft, &oname, &oleft);
    iconv_close(cvt);
    fprintf(stderr, "[%d] %s ", be16(p->length), buf);
}



