//
//  Copyright (c) 2011 Jetico, Inc. All rights reserved.
//  On-disk MBR partition table
//  

#ifndef FSLIB_MBR_H
#define FSLIB_MBR_H

#include <stdint.h>
#include "fat/fat_format.h"

#pragma pack(push, 1)

#define MBR_STATUS_ACTIVE 0x80
#define MBR_STATUS_INACTIVE 0x00
#define MBR_EXTENDED_PARTITION_TYPE 0x0F

#define MBR_IS_VALID_PARTITION(partition)      ( (((partition)->status == MBR_STATUS_ACTIVE) || ((partition)->status == MBR_STATUS_INACTIVE)) && ((partition)->type != 0x00) )
#define MBR_IS_EXTENDED_PARTITION(partition)   ( ((partition)->type == MBR_EXTENDED_PARTITION_TYPE) )


// Sector address in cylinder/head/sector format.
struct chs_address
{
    int head : 8;
    int sector : 6;
    int cylinder : 10;
};


// MBR partition table entry
struct mbr_partition
{
    uint8_t                 status;                 // Active (bootable) or not
    struct chs_address      first_sector;           // CHS address of the first absolute sector of the partition. Unused if LBA value is set
    uint8_t                 type;                   // Partition type: fs type or one of the extended types 
    struct chs_address      last_sector;            // CHS address of the last absolute sector of the partition.
    uint32_t                first_sector_lba;       // First partition absolute sector address. If set then CHS address is ignored
    uint32_t                total_sectors;          // Total partition sectors
};


// First MBR disk sector 
struct mbr_sector 
{
    char                    bootcode[440];          // OS boot code
    char                    disk_signature[4];      // optional disk signature
    uint16_t                unused;                 // usually zeroes
    struct mbr_partition    partition_table[4];     // four primary partitions entries
    uint16_t                signature;              // 0x55, 0xaa
};


// Extended boot record sector.
// Contains extended partition table
struct ebr_sector 
{
    char                    unused[446];
    struct mbr_partition    partition_table[2];     // 2 extended partitions
    struct mbr_partition    unused2[2];
    uint16_t                signature;              // 0x55, 0xaa
};


#pragma pack(pop)

#endif
