//
//  Copyright (c) 2011 Jetico, Inc. All rights reserved.
//
//  Tool to parse whole disk partition table
//  diskinfo device
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "fs_disk.h"


int main(int argc, char** argv)
{
    if(argc < 2) {
        fprintf(stderr, "diskinfo device\n");
        return EXIT_FAILURE;
    }
    
    fs_disk_t disk;
    int error = fslib_parse_disk(argv[1], &disk);
    if(error) {
        fprintf(stderr, "Failed parsing disk %s: %s\n", argv[1], strerror(error));
        return EXIT_FAILURE;
    }
    
    printf("Parsed disk %s (%s):\n", argv[1], disk.type);
    printf(" signature: ");
    unsigned i;
    for(i = 0; i < sizeof(disk.signature); ++i) {
        printf("%02hhX ", disk.signature[i]);
    }
    printf("\n");
    
    printf(" %u partitions:\n", disk.total_partitions);
    for(i = 0; i < disk.total_partitions; ++i) {
        printf(" > partition %d: %llu:%llu,\t type 0x%02x (%s)\n", 
               i, 
               disk.partitions[i].start_sector, 
               disk.partitions[i].total_sectors,
               disk.partitions[i].fstype, 
               fslib_partition_type(disk.partitions[i].fstype));
    }
    
    fslib_put_disk(&disk);
    return EXIT_SUCCESS;
}