#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <dirent.h>
#include <errno.h>
#include <assert.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "fslib.h"

void usage(const char *s) 
{
    printf("Usage: %s file [directory]\n", s);
}

int main(int argc, char **argv) 
{
    int             ret, frozen = 0;
    uint32_t		fileid;
    struct stat		st, sd;
    DIR             *dev;
    struct dirent	*d;
    char            devname[255];
    fs_volume_t		vol;
    fs_extent_t		*ext;
    
    memset(&vol, 0, sizeof(vol));
    memset(&st,  0, sizeof(st));
    
    
    if (2 > argc || 3 < argc) {
        usage(argv[0]);
        return -1;
    }
    
    ret = lstat(argv[1], &st);
    if (ret) {
        printf("Failed to lstat on '%s': %s\n", argv[1], strerror(errno));
        return -1;
    }
    fileid = st.st_ino;
    printf("CNID: %d, device: %08X\n", fileid, st.st_dev);
    
    dev = opendir("/dev");
    if (!dev) {
        printf("opendir(dev) failed: %s\n", strerror(errno));
        return -1;
    }
    
    while (NULL != (d = readdir(dev))) {
        snprintf(devname, sizeof(devname), "/dev/%s", d->d_name);
        ret = lstat(devname, &sd);
        if (ret) {
            continue;
        }
        if (!S_ISCHR(sd.st_mode)) {
            continue;
        }
        if (sd.st_rdev == st.st_dev) {
            break;
        }
    }
    if (!d) {
        printf("Device not found\n");
        return -1;
    }
    
    if(0 != fslib_open_volume(devname, O_RDONLY, &vol)) {
        printf("Failed to open '%s': %s\n", argv[1], strerror(errno));
        return -1;
    }
    

    
    fprintf(stderr, "Source: %s, offset: 0x%llX\n", argv[1], vol.offset);
    
    fs_file_t file;
    memset(&file, 0, sizeof(file));
    if (vol.get_file_by_id) {
        
        /*
         ret = fcntl(vol.fd, F_FREEZE_FS, NULL);
         if (ret) {
         frozen = 0;
         printf("Failed to freeze '%s': %s\n", argv[1], strerror(errno));
         } else {
         frozen = 1;
         }
         */
        
        ret = vol.get_file_by_id(&vol, fileid, &file);
        
        /*
         if (frozen) {
         ret = fcntl(vol.fd, F_THAW_FS, NULL);
         if (ret) {
         printf("Failed to thaw '%s': %s\n", argv[1], strerror(errno));
         }
         }
         */
        
        if(ret != 0) {
            fprintf(stderr, "Error: failed getting file '%s' info: 0x%x\n", argv[1], ret);
            fslib_close_volume(&vol);
            return EXIT_FAILURE;
        }
    }
    

    fs_area_t* area = file.areas;
    while(area) {
        fprintf(stdout, "Area %8s\n", area->tag);
        
        if(area->extents) {
            fs_print_extent_list(area->extents, 512);
        }
        
        area = area->next;
    }
    
    fslib_close_volume(&vol);
    
    return 0;
}

