/*
 *  Copyright 2011 Jetico, Inc. All rights reserved.
 *
 *  Utility to test free space mapping correctness.
 *  WARNING: Can damage the filesystem and/or data, please use with care.
 *  Default usecase assumes empty partition.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#define FSLIB_DEBUG
#include "fslib.h"


int main(int argc, char** argv)
{
	if(argc < 2) {
		printf("test_free_space device\n");
		return EXIT_FAILURE;
	}
    printf("Press Enter to continue...\n");
    getchar();
	
	const char* dev = argv[1];
	
	fs_volume_t vol;
	if(0 != fslib_open_volume(dev, O_RDWR, &vol)) {
		printf("Failed initializing volume\n");
		return EXIT_FAILURE;
	}
	
	fs_extent_t* free_space = vol.get_free_space(&vol);
	if(!free_space) {
		printf("Failed mapping volume free space\n");
		return EXIT_FAILURE;
	}
	
	fs_print_extent_list(free_space, 1);
	fs_extent_t* ext = free_space;
	
	char data[512] = {0};
	memset(data, 'A', sizeof(data));
	
	while(ext) {
        off_t offset = ext->start;
        size_t nbytes = ext->size;
		
        printf("Processing free space extent: %llu:%llu (0x%x:0x%x)\n", ext->start, ext->size, ext->start, ext->size);
        while(nbytes) {
			
            ssize_t nwritten = pwrite(vol.fd, data, sizeof(data), offset);
            if(nwritten != sizeof(data)) {
                printf("pwrite failed at offset %llu (written only %d bytes, %d left): %s\n", offset, nwritten, nbytes, strerror(errno));
                return EXIT_FAILURE;
            }
			
            offset += nwritten;
            nbytes -= nwritten;
        }
		
		ext = ext->next;
	}
	
	fs_put_extent_list(free_space);
    fslib_close_volume(&vol);
	
	return EXIT_SUCCESS;
}


