#!/bin/sh
# Copyright 2011 Jetico, Inc. All rights reserved.
# Script tests that volume free space extent is correct

mpoint=$1

if [ ! -d "$mpoint" ]; then
	echo "Mount point '$mpoint' does not exist" 
	exit -1;
fi

# find disk that is mounted here
disk=`diskutil info "$mpoint" | grep "Device Identifier" | awk '{printf $3 "\n"}'`
if [ -z $disk ]; then
	echo "Could not find device node mounted on '$mpoint'"
	exit -1;
fi

rdisk="/dev/r$disk"

# Calculate contents checksum
checksum=`find "$mpoint" -type f -exec md5 -q {} \; | sort | md5`
echo "$mpoint contents checksum: $checksum"

# Call util to write into free space
./test_free_space $rdisk
if [ $? != 0 ]; then
	echo "Failed filling free space on disk $rdisk"
	exit -1;
fi

# Verify checksum
checksum2=`find "$mpoint" -type f -exec md5 -q {} \; | sort | md5`
if [ $checksum != $checksum2 ]; then
	echo "Checksum mismatch: $checksum != $checksum2.\nTest failed"
	exit -1;
fi

echo "Test passed"
exit 0


