/*
 * Copyright (c) 2010 Jetico, Inc., Finland
 * All rights reserved.
 */

#ifndef BC_KERNEL_CONTROLLER_H
#define BC_KERNEL_CONTROLLER_H

#include <linux/module.h>
#include <crypto/cipher.h>


/// Opaque plugin referece.
typedef struct bc_plugin* bc_plugin_ref;


/**
 * Register new cipher definition.
 * Called by external block cipher modules typically in module init function.
 * @mod		Plugin kmod structure.
 * @cipher	Plugin cipher definition.
 * @out_ref	Returned reference to initialized plugin on success. Will be required to unregister plugin later.
 * @return 	Error code.
 */
extern int bc_register_plugin(struct module* mod, const struct bc_cipher* cipher, bc_plugin_ref* out_ref);


/**
 * Remove plugin.
 * Called by external block cipher modules typically in module exit function.
 * @ref		Plugin reference obtained when it was registered.
 */
extern void bc_unregister_plugin(bc_plugin_ref ref);


/**
 * Get block cipher plugin by its id and increase its usecount.
 * Caller should decrease the plugin's usecount with bc_put_plugin when done using the cipher.
 */
const struct bc_cipher* bc_get_plugin(int id);


/**
 * Decrease plugin usecount.
 */
void bc_put_plugin(const struct bc_cipher* cipher);


#endif

