/*
 * Copyright (c) 2010 Jetico, Inc., Finland
 * All rights reserved.
 *
 * Common file to compile build in plugin modules.
 */

#include "bc_mod.h"
#include "crypto/cipher.h"

#include "algorithm.h" 	// symlink to algorithm definitions, created by build system.

#include <linux/init.h>
#include <linux/module.h>


// Exported cipher definitions. 
// All values are taken from algorithm.h
static const struct bc_cipher g_plugin_cipher = 
{
	.id			= BC_ALGORITHM_ID,
	.name	 		= BC_ALGORITHM_NAME,
	.key_size		= BC_KEY_LENGTH,
	.block_size		= BC_BLOCK_SIZE,
	.extended_key_size	= BC_EXTENDED_KEY_SIZE,

	.extend_key		= KeyExtend,
	.encrypt_ecb 		= Encrypt_ECB,
	.decrypt_ecb		= Decrypt_ECB,
#ifdef OVERRIDE_CBC
	.encrypt_cbc		= Encrypt,
	.decrypt_cbc		= Decrypt,
#else
    .encrypt_cbc        = NULL,
    .decrypt_cbc        = NULL,
#endif
};


// Registered plugin reference.
static struct bc_plugin* g_plugin_ref;


int __init bc_plugin_init(void)
{
	int rc = bc_register_plugin(THIS_MODULE, &g_plugin_cipher, &g_plugin_ref);
	if(0 != rc) {
		printk(KERN_ERR "bc_ctl: plugin %s failed to register with 0x%x\n", g_plugin_cipher.name, rc);
		return rc;
	}

	return 0;
}


void __exit bc_plugin_exit(void)
{
	bc_unregister_plugin(g_plugin_ref);
}


module_init(bc_plugin_init);
module_exit(bc_plugin_exit);

MODULE_AUTHOR("Jetico, Inc.");
MODULE_DESCRIPTION("BestCrypt cipher plugin " BC_PLUGIN_NAME);
MODULE_LICENSE("GPL");

