#!/bin/bash

BC_CTL=bestcrypt
BC_SYSFS_DIR=/sys/class/misc/bestcrypt
BC_SYSFS_PLUGINS=$BC_SYSFS_DIR/plugins

BC_PLUGINS=( `exec ./list_plugins.sh` )
BC_TOTAL_PLUGINS=${#BC_PLUGINS[@]}

case $1 in
	start) 
		echo "Starting BestCrypt.."
		
		sudo insmod ../build/$BC_CTL.ko

		for (( i=0; i<$BC_TOTAL_PLUGINS; i++ )); do
			plugin=bc_${BC_PLUGINS[$i]}.ko
			sudo insmod ../build/$plugin
		done
		
        sudo chmod 0755 /dev/bestcrypt
		echo "started"
		;;
	stop) 
		echo "stopping BestCrypt.."
		for (( i=0; i<$BC_TOTAL_PLUGINS; i++ )); do
			plugin=bc_${BC_PLUGINS[$i]}
			sudo rmmod $plugin
		done
	
		sudo rmmod $BC_CTL

		echo "stopped"
		;;
	restart)
		./bcrypt.sh stop
		./bcrypt.sh start
		;;
	status)
		# list plugins
		if [ -d $BC_SYSFS_DIR ] ; then
			plugins=( `ls $BC_SYSFS_PLUGINS` )
			printf "BestCrypt kernel driver version %d.%d is initialized, with %d algorithm plugin(s):\n" `cat $BC_SYSFS_DIR/ver_major` `cat $BC_SYSFS_DIR/ver_minor` ${#plugins[@]}
			for dir in ${plugins[@]} ; do
				dir=$BC_SYSFS_PLUGINS/$dir;
				name=`cat $dir/name`;
				id=`cat $dir/id`;
				block_size=`cat $dir/block_size`;
				key_length=`cat $dir/key_length`;
				cbc_key_size=`cat $dir/cbc_key_size`;
				lrw_key_size=`cat $dir/lrw_key_size`;
				xts_key_size=`cat $dir/xts_key_size`;

				printf "%16s(0x%02x):\tkey length: %d\tblock size: %d\tsupported modes: " $name $id $key_length $block_size;
				[ $cbc_key_size != 0 ] && printf "cbc ";
				[ $lrw_key_size != 0 ] && printf "lrw ";
				[ $xts_key_size != 0 ] && printf "xts ";
				printf "\n";
			done
		else 
			echo "BestCrypt modules not loaded.";
			exit 0;
		fi

		# list mounted disks.
		;;
	
	*)
		echo "invalid command"
		echo "usage: bcrypt start|stop|restart|status"
		;;
esac

exit 0



