/***************************************************************************************
 * File: bc_rijn.h
 *  
 * Purpose: Declarations of the Rijndael functions (ECB, CBC and CFB mode)
 *          We use it with 16-bytes blocks and 256-bit key
 *
 *  Rijndael algorithm's authors:
 *		Joan Daemen
 *		Vincent Rijmen
 *
 * Code in aes_tab.h, aes.h, aes.c files are written by Dr B. R. Gladman <brg@gladman.uk.net>
 *
 ***************************************************************************************/

#ifndef __BC_RIJNDAEL
#define __BC_RIJNDAEL

#include <crypto/wintypes.h>

#define SERVICE_NAME	"BC_RIJN"
#define DISPLAY_NAME    "RIJNDAEL"
#define ALGORITHM_ID     0xF0

#define DRIVER_MAJOR_VERSION	0
#define DRIVER_MINOR_VERSION	1


// Key length in bits - required for external definitions of 
// BestCrypt Algorithm modules
#define KEY_LENGTH 256

// number of bits per block
#define	R_BLOCK_SIZE 128


// We have to reserve space both for encryption and decryption keys,
// The following structure is the same as defined in aes.c, 
// excluding the fields that are not used in our case, when we 
// use a definite block size (16 bytes)
#define BLOCK_LENGTH 16

typedef struct
{
    DWORD    Nkey;       // the number of 32-bit words in the key input block
    DWORD    Nrnd;       // the number of cipher rounds
    DWORD    e_key[4 * BLOCK_LENGTH];  // the encryption key schedule
    DWORD    d_key[4 * BLOCK_LENGTH];  // the decryption key schedule
    BYTE     mode;       // encrypt, decrypt or both
} BC_AES_STRUCT;

typedef BC_AES_STRUCT plugin_key_t;

// the value is in bits
#define KEY_LENGTH_EXTENDED (sizeof(BC_AES_STRUCT) * 8)

/******* Exported functions *********/

#define IN

#ifdef __cplusplus
extern "C" {
#endif

BOOL
KeyExtend( PUCHAR KeySource,
           PDWORD KeyDestination );

/* 
   Encryption and decryption in Cipher Block Chaining (CBC) mode
*/


VOID
Encrypt( DWORD *IVector, 
         DWORD *KeyAdress,
         DWORD *SrcBuffer,
         DWORD *DstBuffer,
         DWORD Length );    // in bytes

VOID
Decrypt( DWORD *IVector, 
         DWORD *KeyAdress,
         DWORD *SrcBuffer,
         DWORD *DstBuffer,
         DWORD Length );    // in bytes

/*
   Encryption and decryption in 32-bit Cipher-Feedback (CFB) mode
*/
VOID
encrypt_CFB( DWORD *IVector, 
             DWORD *KeyAddress,
             DWORD *SrcBuffer,
             DWORD *DstBuffer,
             DWORD Length );    /* in bytes */

VOID
decrypt_CFB( DWORD *IVector, 
             DWORD *KeyAddress,
             DWORD *SrcBuffer,
             DWORD *DstBuffer,
             DWORD Length );    /* in bytes */


/*
	Encrypt\decrypt in electronic code book mode
*/

VOID
Encrypt_ECB( 	BYTE* Input,
		BYTE* Output,
		BYTE* ExtendedKey);


VOID
Decrypt_ECB( 	BYTE* Input,
		BYTE* Output,
		BYTE* ExtendedKey);

#ifdef __cplusplus
}
#endif


/*
 * Plugin build defs.
 */
#define BC_PLUGIN_NAME		SERVICE_NAME
#define BC_ALGORITHM_NAME	DISPLAY_NAME
#define BC_ALGORITHM_ID		ALGORITHM_ID
#define BC_KEY_LENGTH		KEY_LENGTH
#define BC_BLOCK_SIZE		BLOCK_LENGTH
#define BC_EXTENDED_KEY_SIZE	sizeof(BC_AES_STRUCT)

#endif // __BC_RIJNDAEL
