/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#ifndef BC_CRYPTO_CIPHER_H
#define BC_CRYPTO_CIPHER_H

#include "intsize.h"
#include "wintypes.h"


/// Compatibility with existing windows alg drivers code. We use macros below.
typedef BOOL(*bc_extkey_proc)(PUCHAR keysrc, PDWORD keydst);
typedef void(*bc_ecb_proc)(BYTE* src, BYTE* dst, BYTE* key);
typedef void(*bc_cbc_proc)(DWORD* iv, DWORD* key, DWORD* src, DWORD* dst, DWORD nbytes);

		   
/**
 * Block cipher definitions.
 * Supplied by plugin, used by modes.
 * Historically, we require a block cipher to implement ECB and CBC modes.
 * For the CBC mode in library we use a modified version that calls cipher CBC mode.
 */
typedef struct bc_cipher
{
	int		id;			// unique id.
	const char*	name;			// display name.
	size_t		key_size;		// raw key size in bits.
	size_t		block_size;		// single atomic block size.
	size_t		extended_key_size;	// size of the extended key buffer in bytes.
	
	bc_extkey_proc	extend_key;
	bc_ecb_proc	encrypt_ecb;
	bc_ecb_proc	decrypt_ecb;
	bc_cbc_proc	encrypt_cbc;
	bc_cbc_proc	decrypt_cbc;
} bc_cipher;


static inline void bc_cipher_extend_key(const bc_cipher* cipher, const unsigned char* keysrc, void* keydst) {
	(void) cipher->extend_key((PUCHAR) keysrc, (PDWORD) keydst);
}

static inline void bc_cipher_encrypt_cbc(const bc_cipher* cipher, uint64_t iv, void* key, unsigned char* data, size_t nbytes) {
	cipher->encrypt_cbc((DWORD*) &iv, (DWORD*) key, (DWORD*) data, (DWORD*) data, nbytes);
}

static inline void bc_cipher_decrypt_cbc(const bc_cipher* cipher, uint64_t iv, void* key, unsigned char* data, size_t nbytes) {
	cipher->decrypt_cbc((DWORD*) &iv, (DWORD*) key, (DWORD*) data, (DWORD*) data, nbytes);
}

static inline void bc_cipher_encrypt_ecb(const bc_cipher* cipher, void* key, unsigned char* data) {
	cipher->encrypt_ecb((BYTE*) data, (BYTE*) data, (BYTE*) key);
}

static inline void bc_cipher_decrypt_ecb(const bc_cipher* cipher, void* key, unsigned char* data) {
	cipher->decrypt_ecb((BYTE*) data, (BYTE*) data, (BYTE*) key);
}

#endif
