/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#ifndef BC_CRYPTO_MODE_H
#define BC_CRYPTO_MODE_H

#include "intsize.h"
#include "cipher.h"

//struct bc_cipher;


/// Internal mode tags.
enum
{
	kCBC = 0,
	kLRW,
	kXTS,
	
	kBCMode_Total
};

enum {
	kNOOP = 0xff,
};


/// Encryption mode definition.
typedef struct bc_mode
{
	int		id;
	const char*	name;
	
	// Does this mode can be used with this cipher?
	int(*is_supported)(const bc_cipher* cipher);
	
	// Tell the packed raw key size required to use the cipher under this mode.
	size_t (*key_size) (const bc_cipher* cipher);
	
	// Tell the full extended context size for this mode under this cipher.
	size_t (*context_size) (const bc_cipher* cipher);
	
	// Init key context from raw key data into preallocated buffer.
	void (*init_context) (void* context, const bc_cipher* cipher, const unsigned char* key, size_t keysize);
	
	// Encrypt data.
	void (*encrypt) (void* context, const bc_cipher* cipher, uint64_t iv, unsigned char* pool, unsigned char* data, size_t nbytes);
	
	// Decrypt data.
	void (*decrypt) (void* context, const bc_cipher* cipher, uint64_t iv, unsigned char* pool, unsigned char* data, size_t nbytes);

} bc_mode;


#ifdef __cplusplus 
extern "C" {
#endif

/// Lookup mode definition by tag.
extern const bc_mode* bc_lookup_mode(unsigned tag);

#ifdef __cplusplus
}
#endif

	
#endif
