/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#include "mode.h"
#include "utils.h"

static int noop_is_supported(const bc_cipher* cipher) {
	return 1;
}

static size_t noop_key_size(const bc_cipher* cipher) {
	return bc_bits_to_bytes(cipher->key_size);
}

static size_t noop_context_size(const bc_cipher* cipher) {
	return cipher->extended_key_size;
}

static void noop_init_context(void* context, const bc_cipher* cipher, const unsigned char* key, size_t keysize) {
	bc_cipher_extend_key(cipher, key, context);
}

static void noop_encrypt(void* key, const bc_cipher* cipher, uint64_t iv, unsigned char* pool, unsigned char* data, size_t nbytes) {
	bc_cipher_encrypt_ecb(cipher, key, data);
}

static void noop_decrypt(void* key, const bc_cipher* cipher, uint64_t iv, unsigned char* pool, unsigned char* data, size_t nbytes) {
	bc_cipher_decrypt_ecb(cipher, key, data);
}

// exported mode def.
bc_mode noop_exports = 
{
	kNOOP, 
	"NOOP",
	noop_is_supported,
	noop_key_size,
	noop_context_size,
	noop_init_context,
	noop_encrypt,
	noop_decrypt
};
