/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 * Implementation uses a thin wrapper over certificate lib hash algorithms.
 */

#include "hash.h"

#include <CertificateLib/hashAlgs/cHashAlg.h>
#include <CertificateLib/hashAlgs/hashAlgs.h>
#include <CertificateLib/hashAlgs/SHA1/Sha1.h>
#include <CertificateLib/hashAlgs/SHA256/sha256.h>
#include <CertificateLib/x509pkts/pkcs12_funcs.h>

#include <assert.h>
#include <time.h>
#include "container/isaac.h"
#include "container/user.h"


extern const char *PROG_NAME;

static cHashAlg* get_hash(int id)
{
	cHashAlg* palg;
	if(hae_invalidAlgorithmID == ha_createAlgorithm(&palg, id)) {
		assert(0);
	}
	
	return palg;
}


/*
size_t KGSHA_DigestSize(int hash_id)
{
	cHashAlg* alg = get_hash(hash_id);
	if(!alg) {
		assert(0);
	}
	
	return alg->getDigestSize();
}
*/


namespace kgghost {
	
	void calculate_digest(int hash_id, const void* message, size_t nbytes, char* out_digest)
	{
		cHashAlg* alg = get_hash(hash_id);
		if(!alg) {
			fprintf(stderr, "%s: Warning: Unknown hash_ID %d\n", PROG_NAME, hash_id);
			snprintf(out_digest, nbytes, "Unknown hash");
			return;
		}

		alg->makeDigest((BYTE*)out_digest, (BYTE*)message, nbytes);
        ha_free(alg);
	}
	
	
	void shred(void* data, size_t nbytes)
	{
		if ((!data) || (nbytes <= 0)) return;
        unsigned t = time(NULL);
        const void *seed = &t;
        unsigned seedLength = sizeof(unsigned);

        //srand( (unsigned int)time( NULL ) );
        void *isaacContext = IsaacInit(seed, seedLength);
        IsaacGenerate(isaacContext, data, nbytes);
        IsaacRelease(isaacContext);
		SHA256_RandomGenerate((BYTE*)data, nbytes, (BYTE*)data, nbytes);
	}

	
	void generate_key_from_password(int hash_id, const char* pwdbuf, size_t pwdlen, const char* salt, size_t saltsize, unsigned num_iter, char* out_key, size_t keysize)
	{
		int res = P12_producePseudoRandomBytes((BYTE*)out_key, keysize, (BYTE*)salt, saltsize, num_iter, 
						       (BYTE*)pwdbuf, pwdlen,
						       P12_prgm4key, (BYTE)hash_id);
		assert(res == x509e_ok);
	}
	
	
	void generate_key(int hash_id, const char* seed, size_t seedlen, char* out_key, size_t keylen)
	{
		int res = P12_producePseudoRandomBytes((BYTE*)out_key, keylen, NULL, 0, 1, 
						       (BYTE*)seed, seedlen, 
						       P12_prgm4key, (BYTE)hash_id);
		assert(res == x509e_ok);
	}

	
	BCEnumHashes::BCEnumHashes() {
		hashes[0] = haID_MD5;
		hashes[1] = haID_SHA1;
		hashes[2] = haID_Ripemd;
		hashes[3] = haID_SHA256;
	}
}
