/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 *  Small procedural hashing interface.
 */

#ifndef KGGHOST_HASH_H
#define KGGHOST_HASH_H

#include <stdlib.h>
#include <assert.h>
#include <CertificateLib/hashAlgs/hashAlgs.h>



// begin of copy-paste    ---------

/* This copy-paste this enum from Certification library
   fixes compilation of project.

  Since svn rev. 204 it was commented out and instead of it
  non-equivalent constant was used:

   Old:              New:  (svn -r 204)
     haID_MD5 = 1,      pgphaMD5 = 1
     haID_SHA1 = 2,     pgphaSHA1 = 2
!!!  haID_SHA256 = 128, pgphaSHA256 = 8   // !!! This constant does not match!
     haID_Ripemd = 3,   pgphaRIPEMD160 = 3
     haID_MD2 = 5       pgphaMD2 = 5

  I have no time to check if haID_SHA256 = 128 is used some where
  as a part of file format, so I just pasted it here.

  TODO: check that there is no 128 byte read from file for identify hash algorithm
        and replace all usages to pgpha* constants.
*/

enum enum_haID {
    haID_MD5 = 1,
    haID_SHA1 = 2,
    haID_SHA256 = 128,
    haID_Ripemd = 3,
    haID_MD2 = 5
};

// ---------  end of copy-paste





namespace kgghost {
	
	
	/**
	 * Calculate message digest.
	 * @hash_id	Hash algorithm id to use.
	 * @message	Message data.
	 * @nbytes	Number of bytes in message to calculate digest for.
	 * @out_digest	Returned calculated digest.
	 */
	void calculate_digest(int hash_id, const void* message, size_t nbytes, char* out_digest);
	
	
	/**
	 * Overwrite data with random garbage.
	 * Here because implementation will use SHA256 to generate random garbage.
	 * @data	Data to overwrite.
	 * @nbytes	Nuber of bytes of data.
	 */
	void shred(void* data, size_t nbytes);
	
	
	/**
	 * Generate encryption key from password and seed.
	 * @hash_id	Hash algorithm to use for key generation.
	 * @password	NULL-terminated password string.
	 * @salt	Password salt buffer.
	 * @saltsize	Total salt bytes.
	 * @num_iter	Number of hash iteration to perform.
	 * @out_key	Returned key buffer.
	 * @keysize	Size of the key in bytes.
	 */
	void generate_key_from_password(int hash_id, const char* pwdbuf, size_t pwdlen, const char* salt, size_t saltsize, unsigned num_iter, char* out_key, size_t keysize);
	
	
	/**
	 * Generate container encryption key data.
	 * @hash_id	Hash algorithm to use for key generation.
	 * @seed	Random seed buffer.
	 * @seedlen	Number of seed bytes.
	 * @out_key	Initialized key buffer.
	 * @keylen	Number of bytes of key to generate.
	 */
	void generate_key(int hash_id, const char* seed, size_t seedlen, char* out_key, size_t keylen);
	
	
	/**
	 * Enumeration of all supported hash algorithms.
	 */
	struct BCEnumHashes {
		enum {
			kTotalHashes = 4,
		};
		
		int hashes[kTotalHashes];
		
		BCEnumHashes();
		
		unsigned total() const {
			return kTotalHashes;
		}
		
		int get(unsigned idx) {
			assert(idx < kTotalHashes);
			return hashes[idx];
		}
	};
}


#endif
