/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 *
 * SHA1/SHA256 interface.
 * We have an internal id for both supported hash algorithms.
 * Currently the implementation just piggybacks on CertificateLib and we basically map our interface to it.
 */

#ifndef KGSHA_SHA_H
#define KGSHA_SHA_H

#include "defs.h"
#include <stdlib.h>




// begin of copy-paste    ---------

/* This copy-paste this enum from Certification library
   fixes compilation of project.

  Since svn rev. 204 it was commented out and instead of it
  non-equivalent constant was used:

   Old:              New:  (svn -r 204)
     haID_MD5 = 1,      pgphaMD5 = 1
     haID_SHA1 = 2,     pgphaSHA1 = 2
!!!  haID_SHA256 = 128, pgphaSHA256 = 8   // !!! This constant does not match!
     haID_Ripemd = 3,   pgphaRIPEMD160 = 3
     haID_MD2 = 5       pgphaMD2 = 5

  I have no time to check if haID_SHA256 = 128 is used some where
  as a part of file format, so I just pasted it here.

  TODO: check that there is no 128 byte read from file for identify hash algorithm
        and replace all usages to pgpha* constants.
*/

enum enum_haID {
    haID_MD5 = 1,
    haID_SHA1 = 2,
    haID_SHA256 = 128,
    haID_Ripemd = 3,
    haID_MD2 = 5
};

// ---------  end of copy-paste



// Hash algorithm internal id.
// This is what we get from container header and map to CertificateLib.
enum {
	kSHA_ID			= 0x53484131,
	kSHA256_ID		= 0x53323536,
};


/// Get digest size for sha version.
size_t KGSHA_DigestSize(int hash_id);


/// Calculate message digest.
void KGSHA_CalculateDigest(int hash_id, const char *data, size_t length, char *out_digest);


/// Generate random data with hash algorithm.
void KGSHA_RandomGenerate(int hash_id, const char* seed, size_t seed_size, char* out_buf, size_t nbytes);


/// Overwrite a memory area with sha-generated garbage.
void KGSHA_ShredData(char *data, size_t nbytes);


/// Generate random encryption key data using sha hash.
void KGSHA_RandomGenerateKey(int hash_id, const char* seed, size_t seed_length, char *out_key, size_t out_key_size);


#endif
