/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 *
 * hidden_sector.h
 * V7 container header format.
 */

#ifndef BC_HIDDEN_SECTOR_H
#define BC_HIDDEN_SECTOR_H

#include "container/intsize.h"

#pragma pack(push, 1)

#define BCV7_REENCRYPTED_LABEL	"RE_KeyGenID"
#define BCV7_VOLUME_LABEL       "CryptedDisk"
#define BCV7_OEMID		        "LOCOS94 "
#define BCV7_VERSION_TAG	    3

/**
 * Boot record structure.
 * Part of the container header, this structure was added for old BestCrypt Windows version to make containers look more like boot drives.
 * On unixes and modern windows this is mostly unused except for a few fields marked explicitely by comments.
 */
struct BootRecord 
{
	bc_u8	jmpCode[3];		// 0xEB if container is locked.
	char	OEMid[8];		// LOCOSXX signature.
	bc_u16	sectSize;
	bc_u8	sectPerCluster;
	bc_u16	reservedSectors;
	bc_u8	NumberOfFat;
	bc_u16	maxRootDirEntry;
	bc_u16	totalSectors;
	bc_u8	mediaDesc;
	bc_u16	sectorsPerFat;
	bc_u16	sectorsPerTrack;
	bc_u16	numberOfHeads;
	bc_u32	hiddenSectors;
	bc_u32	totalSectorsLong;	// container data size in sectors.
	bc_u8	driveNo;
	bc_u8	reserved;
	bc_u8	extBootSign;
	bc_u32	serialNumber;
	char	volumeLabel[11];
	char	FatType[8];
};


enum {
	kBCV7_DescriptionSize = 66,
	kBCV7_SignatureSize = 8,
	kBCV7_KeyDataSize = 4,
	kBCV7_HiddenSectorSize = 512,
};


/**
 * Container hidden sector.
 * This is the first 512 bytes of the container header.
 * Describes most container non-secret info.
 */
struct HiddenSector
{
	struct BootRecord	br;
	char			description[kBCV7_DescriptionSize];
	bc_u16			flags;
	bc_u16			version;
	bc_u8			keyData[kBCV7_KeyDataSize];	

    // push the rest of the sector to the end
	bc_u8 reserved[
		       kBCV7_HiddenSectorSize -
		       sizeof(struct BootRecord) -
		       kBCV7_DescriptionSize - 
		       sizeof(bc_u16) - 
		       sizeof(bc_u16) -
		       kBCV7_KeyDataSize - 
		       sizeof(bc_u32) -			
		       sizeof(bc_u32) -			
		       sizeof(bc_u32) -			
		       sizeof(bc_u32) -			
		       sizeof(bc_u32) -			
		       kBCV7_SignatureSize	
		       ];
	
	bc_u32			dwKeySize;			// size of the key block data region that follows the header.
	bc_u32			dwDataOffset;		// offset of the encrypted data in container file aligned to sector boundary.
	bc_u32			fsId;				// file system id. used only on unixes.
	bc_u32			algId;				// encryption algorithm id.
	bc_u32			keygenId;			// key generation module id.
	bc_u8			signature[kBCV7_SignatureSize];	// container unique signature.
};


#pragma pack(pop)

#endif
