/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 *
 * KGSHA keygen library interface.
 */

#ifndef KGSHA_KGSHA_H
#define KGSHA_KGSHA_H

#include "container/keygen.h"
#include "hidden_sector.h"
#include "datablock.h"

/**
 * kgsha keygen interface implementation.
 */
class KGSHA_Header : public IBCHeader {
public:

    KGSHA_Header(HiddenSector* hs, DATA_BLOCK* db);

    virtual bool isEncrypted() {
        return false;
    }

    virtual BCContainerResult encrypt(const char* password) {
        return kBCContainer_Unsupported;
    }

    virtual BCContainerResult decrypt(const char* password) {
        return kBCContainer_Unsupported;
    }

    virtual void mainVolumeInfo(struct BCVolumeInfo* info);
    virtual BCContainerResult init(int algid, int modeid, bc_u64 size, const char* description,
                                   const char* password, BCProgressCallback progress,
                                   void *progressArg);
    virtual BCContainerResult createHiddenKey(SpaceMap *map,
                            const BCPassword* mainPassword, const BCPassword* hiddenPassword,
                            BCProgressCallback progress, void *progressArg);
    virtual BCContainerResult verifyPassword(const char* password, struct BCVolumeInfo* out_info);
    virtual BCContainerResult decodeKey(const char* password, struct BCVolumeInfo* out_info,
                                int* key_handle, char* out_pool);
    virtual BCContainerResult addPassword(const char* password, const char* newPassword);
    virtual BCContainerResult changePassword(const char* password, const char* newPassword);
    virtual BCContainerResult removePassword(const char* password);
    virtual BCContainerResult store(const char* file);
    virtual void setDescription(const char* description);
    virtual void markForReencryption(bool mark);
    virtual bool isMarkedForReencryption();

    virtual ~KGSHA_Header();

protected:

    BCContainerResult passwd(const char* password, const char* newPassword, BCKeyOp opflag);
    BCContainerResult volumeInfoForKey(const kgsha_key* key, struct BCVolumeInfo* out_info);

private:
        
    HiddenSector    m_hs;
    DATA_BLOCK      m_db;
};


#endif

