/*
 *	Copyright Jetico, Oy, 2011.
 *	Common utilities.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>


int bc_get_dev_major(void) 
{
    FILE* proc = fopen ( "/sys/class/misc/bestcrypt/block_major", "r" );
	if ( !proc ) {
		return 0;
	}

	int major = 0;
    if (EOF == fscanf(proc, "%d", &major))
        major = 0;

	fclose ( proc );
	return major;
}


int bc_get_root(void)
{
	return setreuid(-1, 0);
}

int bc_drop_root(void)
{
	return setreuid(-1, getuid());
}

int bc_spawn(const char* name, ...)
{
	char* pargs[1024] = {NULL};
	va_list vargs;
	va_start(vargs, name);

	unsigned i = 0;
	for(;;) {
		if( i >= (sizeof(pargs) / sizeof(*pargs)) ) {
			fprintf(stderr, "Error: too many arguments\n");
			return EINVAL;
		}

		if( NULL == (pargs[i++] = va_arg(vargs, char*)) ) {
			break;
		}
	}

	va_end(vargs);

	
	pid_t pid = fork();
	if(pid < 0) {
		fprintf(stderr, "Error: fork failed\n");
		return EXIT_FAILURE;
	}

	if( 0 == pid ) 
	{
		const char* env[] = {"PATH=/sbin:/usr/sbin:/bin:/usr/bin", NULL};
		execve( name, (char* const*) pargs, (char* const*) env );

		// exec failed if we're here
		return EXIT_FAILURE;

	}

	// parent
	int status;
	waitpid(pid, &status, 0);
	return WEXITSTATUS(status);
}

int is_bc_device(dev_t dev)
{
	return (bc_get_dev_major() == major(dev));
}


int is_owner(const char* path) 
{
	struct stat statbuf;
	if(0 != stat(path, &statbuf)) {
		return -1;
	}

    return (!getuid()/* root owns everything */) || (statbuf.st_uid == getuid());
}
	
