/*
 *	Copyright Jetico, Oy, 2011.
 *	
 *  Suid helper tool to send a BLKRRPART ioctl to our device.
 *  bcrrpart_helper device
 */

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <linux/fs.h>

#include "common.h"

int main(int argc, char** argv)
{
    if (argc != 2) {
        fprintf(stderr, "bcrrpart_helper device\n");
        return EXIT_FAILURE;
    }

    const char* dev = argv[1];

	// verify that this is our block device.
	struct stat statbuf;
	if (0 != stat(dev, &statbuf)) {
		fprintf(stderr, "Error: stat failed\n");
		return EXIT_FAILURE;
	}

	if (!S_ISBLK(statbuf.st_mode)) {
		fprintf(stderr, "Error: %s is not a block device\n", dev);
		return EXIT_FAILURE;
	}

	if (bc_get_dev_major() != major(statbuf.st_rdev)) {
		fprintf(stderr, "Error: %s not a bestcrypt block device\n", dev);
        return EXIT_FAILURE;
	}

    int fd = open(dev, O_RDONLY);
    if (fd < 0) {
		fprintf(stderr, "Error: failed opening %s: %s\n", dev, strerror(errno));
        return EXIT_FAILURE;
    }
        
    if (0 != ioctl(fd, BLKRRPART, 0)) {
        close(fd);
		fprintf(stderr, "Error: ioctl failed: %s\n", strerror(errno));
        return EXIT_FAILURE;
    }

    close(fd);

    return EXIT_SUCCESS;
}


