/*
 * Copyright (c) 2012 Jetico, Inc., Finland
 * All rights reserved.
 */

#ifndef EXTENT_H
#define EXTENT_H

#include "defs.h"

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(push)
#pragma pack(1)

typedef struct _Extent
{
    bc_u64 start;
    bc_u64 length;
} Extent;

#pragma pack(pop)


static inline bc_u64 ExtentStart(const Extent *e)
{
    return e->start;
}

static inline bc_u64 ExtentEnd(const Extent *e)
{
    if (e->start != 0 || e->length != 0)
        return e->start + e->length - 1; // Enumerating from zero.
    return 0;
}

static inline bool ExtentContainsPoint(const Extent *who, bc_u64 point)
{
    if (who->length > 0)
        return (ExtentStart(who) <= point) && (ExtentEnd(who) >= point);
    return false;
}

static inline bool ExtentContains(const Extent *who, const Extent *what)
{
    return ExtentContainsPoint(who, ExtentStart(what)) && ExtentContainsPoint(who, ExtentEnd(what));
}

static inline bool ExtentAreIntersecting(const Extent *e1, const Extent *e2)
{
    return ExtentContainsPoint(e1, ExtentStart(e2)) || ExtentContainsPoint(e1, ExtentEnd(e2))
            || ExtentContainsPoint(e2, ExtentStart(e1)) || ExtentContainsPoint(e2, ExtentEnd(e1));
}

static inline bool ExtentIsEndBefore(const Extent *before, const Extent *after)
{
    return ExtentEnd(before) < ExtentStart(after);
}

static inline bool ExtentAreEqual(const Extent *e1, const Extent *e2)
{
    return (ExtentStart(e1) == ExtentStart(e2))
            && (ExtentEnd(e1) == ExtentEnd(e2));
}

bool ExtentUnion(Extent *result, const Extent *e1, const Extent *e2);

#ifdef __cplusplus
}
#endif

#endif // EXTENT_H
