#include "bctool.h"

bool BCTool::Benchmark(const char *algName, const char *modeName, std::vector<bc_bench_info> *out_result)
{
    // Does benchmark for selected alg with specified mode.
    // Supposed all alg(mode), if no alg(mode) is selected.

    if (!out_result)
    {
        this->ShowError("Result parameter cannot be null here.\n");
        return false;
    }

    BCAlgInfo algInfo;
    bc_kern_result result;
    std::vector<BCAlgInfo> algsToTest;
    std::vector<bc_mode_id> modesToTest;
    unsigned i = 0;

    if (algName)
    {
        // If alg name is set, will test only this alg...
        result = BCKernelLookupAlgName(algName, &algInfo);
        if (kBCKernel_Success != result)
        {
            this->ShowError(BCKernelGetErrorString(result));
            return false;
        }
        algsToTest.push_back(algInfo);
    }
    else
    {
        // otherwise test all available algs.
        BCAlgInfo *algs = NULL;
        unsigned algCount = 0;
        result = BCKernelListAlgs(&algs, &algCount);
        if (kBCKernel_Success != result)
        {
            this->ShowError(BCKernelGetErrorString(result));
            return false;
        }

        for (i = 0; i < algCount; i++)
        {
            algsToTest.push_back(algs[i]);
        }

        BCKernelFreeAlgList(algs);
    }

    // Similar to alg.
    if (modeName)
    {
        bc_mode_id modeId = BCKernelModeId(modeName);
        modesToTest.push_back(modeId);
    }
    else
    {
        modesToTest.push_back(kBCMode_CBC);
        modesToTest.push_back(kBCMode_LRW);
        modesToTest.push_back(kBCMode_XTS);
    }

    std::vector<BCAlgInfo>::iterator algI = algsToTest.begin();
    std::vector<bc_mode_id>::iterator modeI;

    for (; algI != algsToTest.end(); algI++)
    {
        BCAlgInfo *alg = &(*algI);

        modeI = modesToTest.begin();
        for (; modeI != modesToTest.end(); modeI++)
        {
            bc_mode_id modeId = *modeI;
            if (BCKernelIsModeSupported(alg, modeId))
            {
                unsigned long benchResult;
                result = BCKernelBenchmark(alg->id, modeId, &benchResult);
                if (kBCKernel_Success != result)
                {
                    this->ShowError(BCKernelGetErrorString(result));
                    return false;
                }

                bc_bench_info bench;
                memset(&bench, 0, sizeof(bench));

                strncpy(bench.algName, alg->name, BC_ALG_NAME_SIZE - 1);
                strncpy(bench.modeName, BCKernelModeName(modeId), BC_ALG_MODE_NAME_SIZE - 1);
                bench.averageTime = benchResult;
                out_result->push_back(bench);
            }
        }
    }

    return true;
}
