/*
 * Copyright 2012 Jetico, Inc. All rights reserved.
 * BestCrypt container reencryption interface.
 */

#include "bctool.h"

#include <signal.h>

bool BCTool::Reencrypt(const char *file, const char *algName, const char *modeName)
{
    if (!BCTool::WantChange(file))
        return false;

    if (!algName) {
        this->ShowError("Please specify algorithm name.\n");
        return false;
    }

    if (!modeName) {
        this->ShowError("Please specify encryption mode name.\n");
        return false;
    }

    if (this->CheckContainerInUse(file)) {
        return false;
    }

    // get specified algorithm and mode
    BCAlgInfo alg;
    if (kBCKernel_Success != BCKernelLookupAlgName(algName, &alg)) {
        this->ShowError("Unknown algorithm: %s.\n", algName);
        return false;
    }

    bc_mode_id modeid = BCKernelModeId(modeName);

    if (!BCKernelIsModeSupported(&alg, modeid)) {
        this->ShowError("Algorithm %s does not support mode %s\n", algName, modeName);
        return false;
    }

    if (!this->AskUser("Warning: you are about to reencrypt your container.\n"
                "Please note that any additional passwords and hidden parts (if any) "
                       "will be lost. Are you sure you want to continue? (y/[n])", false))
    {
        this->ShowInfo("Reencryption cancelled.\n");
        return false;
    }

    // install interrupt handlers to terminate process correctly
    signal(SIGINT,  this->HandleSignal);
    signal(SIGQUIT, this->HandleSignal);
    signal(SIGTERM, this->HandleSignal);
    signal(SIGABRT, this->HandleSignal);

    int error = BCReencrypt(file, alg.id, modeid, this->ReportProgress, NULL);
    if(kBCContainer_Cancelled == error) {
        this->ShowInfo("Reencryption cancelled.\n");
    } else if (kBCContainer_Success != error) {
        this->ShowError("Failed to reencrypt container.\n");
        return false;
    } /*else {
        this->ShowMessage("Container successfully reencrypted.\n");
    }*/

    return true;
}
