#include "additionalactionswidget.h"
#include "ui_additionalactionswidget.h"

#include <QAction>
#include <QToolButton>
#include <QMenu>

#include "entity/containerdescriptor.h"

AdditionalActionsWidget::AdditionalActionsWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AdditionalActionsWidget)
{
    ui->setupUi(this);
    FillMenus();
}

void AdditionalActionsWidget::FillMenus()
{
    QList<QAction *> list;

// "Backup" actions
    list.append(this->_backupBackupHeaderAction = new QAction(tr("Backup header"), this->ui->backupToolButton));
    list.append(this->_backupRestoreHeaderAction = new QAction(tr("Restore header"), this->ui->backupToolButton));

    connect(this->_backupBackupHeaderAction, SIGNAL(triggered()), SIGNAL(OnBackupHeader()));
    connect(this->_backupRestoreHeaderAction, SIGNAL(triggered()), SIGNAL(OnRestoreHeader()));

    this->ui->backupToolButton->addActions(list);
    list.clear();

// "Passwords" actions
    list.append(this->_passwordsAddAction = new QAction(tr("Add password"), this->ui->passwordsToolButton));
    list.append(this->_passwordsRemoveAction = new QAction(tr("Remove password"), this->ui->passwordsToolButton));
    list.append(this->_passwordsChangeAction = new QAction(tr("Change password"), this->ui->passwordsToolButton));

    connect(this->_passwordsAddAction, SIGNAL(triggered()), SIGNAL(OnPasswordAdd()));
    connect(this->_passwordsRemoveAction, SIGNAL(triggered()), SIGNAL(OnPasswordRemove()));
    connect(this->_passwordsChangeAction, SIGNAL(triggered()), SIGNAL(OnPasswordChange()));

    this->ui->passwordsToolButton->addActions(list);
    list.clear();

// "Advanced" actions
    list.append(this->_advancedReencryptAction = new QAction(tr("Reencrypt container"), this->ui->advancedToolButton));
    list.append(this->_advancedEncryptHeaderAction = new QAction(tr("Encrypt header"), this->ui->advancedToolButton));
    list.append(this->_advancedDecryptHeaderAction = new QAction(tr("Decrypt header"), this->ui->advancedToolButton));
    list.append(this->_advancedCreateHiddenAction = new QAction(tr("Create hidden part"), this->ui->advancedToolButton));
    list.append(this->_advancedFormatAction = new QAction(tr("Format container"), this->ui->advancedToolButton));
    list.append(this->_advancedFsckAction = new QAction(tr("Check filesystem"), this->ui->advancedToolButton));
    list.append(this->_advancedLinkAction = new QAction(tr("Link container"), this->ui->advancedToolButton));
    list.append(this->_advancedShowLogAction = new QAction(tr("Show log"), this->ui->advancedToolButton));

    connect(this->_advancedReencryptAction, SIGNAL(triggered()), SIGNAL(OnReencrypt()));
    connect(this->_advancedEncryptHeaderAction, SIGNAL(triggered()), SIGNAL(OnEncryptHeader()));
    connect(this->_advancedDecryptHeaderAction, SIGNAL(triggered()), SIGNAL(OnDecryptHeader()));
    connect(this->_advancedCreateHiddenAction, SIGNAL(triggered()), SIGNAL(OnCreateHiddenPart()));
    connect(this->_advancedFormatAction, SIGNAL(triggered()), SIGNAL(OnFormat()));
    connect(this->_advancedFsckAction, SIGNAL(triggered()), SIGNAL(OnFsck()));
    connect(this->_advancedShowLogAction, SIGNAL(triggered()), SIGNAL(OnShowLog()));
    connect(this->_advancedLinkAction, SIGNAL(triggered()), SIGNAL(OnLink()));

    this->ui->advancedToolButton->addActions(list);
    list.clear();

    this->_advancedCreateHiddenAction->setStatusTip(tr("Create hidden part inside selected container"));
    this->_advancedDecryptHeaderAction->setStatusTip(tr("Decrypt header of selected container"));
    this->_advancedEncryptHeaderAction->setStatusTip(tr("Encrypt header of selected container"));
    this->_advancedFormatAction->setStatusTip(tr("Format selected container"));
    this->_advancedFsckAction->setStatusTip(tr("Check file system on selected container"));
    this->_advancedLinkAction->setStatusTip(tr("Create block device with selected container without mounting"));
    this->_advancedShowLogAction->setStatusTip(tr("Show log"));

    this->_passwordsAddAction->setStatusTip(tr("Add new password for main part of selected container"));
    this->_passwordsChangeAction->setStatusTip(tr("Change one of the container passwords"));
    this->_advancedReencryptAction->setStatusTip(tr("Change container algorithm, mode and/or encryption key"));

    this->_backupBackupHeaderAction->setStatusTip(tr("Backup header of selected container"));
    this->_backupRestoreHeaderAction->setStatusTip(tr("Restore header of selected container"));
}

AdditionalActionsWidget::~AdditionalActionsWidget()
{
    delete ui;
    delete this->_advancedReencryptAction;
    delete this->_advancedCreateHiddenAction;
    delete this->_advancedDecryptHeaderAction;
    delete this->_advancedEncryptHeaderAction;
    delete this->_advancedFormatAction;
    delete this->_advancedFsckAction;
    delete this->_advancedLinkAction;
    delete this->_advancedShowLogAction;
    delete this->_passwordsAddAction;
    delete this->_passwordsChangeAction;
    delete this->_backupBackupHeaderAction;
    delete this->_backupRestoreHeaderAction;
}

void AdditionalActionsWidget::SetDescriptor(ContainerDescriptor *descriptor)
{
    this->ui->backupToolButton->setEnabled(!!descriptor);
    this->ui->passwordsToolButton->setEnabled(!!descriptor);
    this->_advancedLinkAction->setEnabled(!!descriptor);

    if (descriptor)
    {
        this->_advancedDecryptHeaderAction->setEnabled(!descriptor->IsParsed());
        this->_advancedEncryptHeaderAction->setEnabled(descriptor->IsParsed());
        this->_advancedLinkAction->setEnabled(!descriptor->IsLinked());
    }
}
