#include "bcapplication.h"

#include "model/bcoperation.h"
#include "entity/containerdescriptor.h"
#include "src/version.h"

#include <QFile>
#include <QMessageBox>
#include <QProcess>
#include <QStringList>

#include <semaphore.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

BCApplication::BCApplication(int &argc, char *argv[])
    : QApplication(argc, argv)
{
    //this->semaphore_ = 0;
}

BCApplication::~BCApplication()
{
    //this->UnlockSemaphore();
}

void BCApplication::CheckOneInstance()
{
    QString thisProgramName = "bestcrypt-panel";
    QString psCommand = QString("ps -C %0").arg(thisProgramName);
    QProcess ps;
    ps.setReadChannel(QProcess::StandardOutput);
    ps.setReadChannelMode(QProcess::MergedChannels);
    ps.start(psCommand);
    ps.waitForStarted(-1);
    ps.waitForFinished(-1);
    QByteArray output = ps.readAll();
    QString outputString(output);
    QStringList list = outputString.split("\n");

    bool singleInstance = list.count() == 3; // Only 'grep bctool-gui' itself is launched.

    if (!singleInstance)
    {
        QMessageBox msgBox;
        msgBox.setText("You have already launched another instance of BestCrypt. You should quit it before launching one more.");
        msgBox.setIcon(QMessageBox::Critical);
        msgBox.setWindowTitle("BestCrypt Control Panel");
        msgBox.exec();
        ::exit(EXIT_FAILURE);
    }
}

bool BCApplication::NeedProcessArgs()
{
    return this->arguments().count() > 1;
}

int BCApplication::ProcessArgs()
{
    QStringList args = this->arguments();
    bool result = true;

    for (int i = 1; i < args.count(); i++)
    {
        QString containerFile = args.at(i);
        ContainerDescriptor descriptor;
        descriptor.SetPath(containerFile);
        BCOperationUpdateDescriptor(&descriptor);

        switch(descriptor.GetState())
        {
        case ContainerDescriptor::kBCContainerState_Linked:
            result &= BCOperationUnlinkContainer(&descriptor, false);
            break;

        case ContainerDescriptor::kBCContainerState_Mounted:
            result &= BCOperationUmountContainter(&descriptor, false);
            break;

        case ContainerDescriptor::kBCContainerState_Unlinked:
            result &= BCOperationMountContainer(&descriptor);
            break;

        default:
            qDebug("Unimplemented case in BCApplication::ProcessArgs");
            break;
        }
    }

    if (result)
        return EXIT_SUCCESS;

    return EXIT_FAILURE;
}
