#include "containerview.h"
#include "ui_containerview.h"

#include "entity/containerdescriptor.h"
#include "container/intsize.h"
#include "model/bcoperation.h"
#include "model/helper.h"

#include <QMouseEvent>
#include <QPainter>

ContainerView::ContainerView(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ContainerView)
{
    ui->setupUi(this);
    connect(this->ui->ejectToolButton, SIGNAL(clicked()), this, SLOT(EjectClickSlot()));
}

ContainerView::~ContainerView()
{
    delete ui;
}

void ContainerView::SetDescriptor(ContainerDescriptor *d)
{
    this->descriptor_ = d;
    this->UpdateView();
}

void ContainerView::Select()
{
    if (!this->IsSelected())
    {
        this->setFocus();
        QPalette palette(this->ui->containerNameLabel->palette());
        QPalette defaultPalette(QApplication::palette());
        palette.setColor(QPalette::Foreground, defaultPalette.highlightedText().color());
        this->ui->containerNameLabel->setPalette(palette);
        this->ui->containerInfoLabel->setPalette(palette);
        this->isSelected_ = true;
        this->repaint();
    }

}

void ContainerView::Deselect()
{
    if (this->IsSelected())
    {
        QPalette defaultPalette(QApplication::palette());
        this->ui->containerNameLabel->setPalette(defaultPalette);
        this->ui->containerInfoLabel->setPalette(defaultPalette);

        this->isSelected_ = false;
        this->repaint();
    }
}

void ContainerView::UpdateView()
{
    this->ui->containerNameLabel->setText(this->descriptor_->GetFileName());
    if (this->descriptor_->IsExists() && this->descriptor_->IsParsed())
    {
        this->ui->containerInfoLabel->setText(
                    QString(tr("%0")).arg(BCHelperGetSizeString(this->descriptor_->size)));
    }
    else
    {
        if (!this->descriptor_->IsExists())
            this->ui->containerInfoLabel->setText(tr("File not found."));
        else
            this->ui->containerInfoLabel->setText("Encrypted header or corrupted.");
    }

    if (this->descriptor_->IsLinked())
    {
        this->ui->ejectToolButton->setVisible(true);
    }
    else
    {
        this->ui->ejectToolButton->setVisible(false);
    }
}

void ContainerView::paintEvent(QPaintEvent *e)
{
    if (!this->IsSelected())
    {
        return this->QWidget::paintEvent(e);
    }

    QPainter p(this);
    QPalette defaultPalette(QApplication::palette());
    p.fillRect(this->rect(), defaultPalette.highlight().color());
}

void ContainerView::mouseDoubleClickEvent(QMouseEvent *)
{
    emit this->OnSelected(this);
    emit this->OnMount(this);
}

void ContainerView::keyPressEvent(QKeyEvent *event)
{
    switch(event->key())
    {
        case Qt::Key_Return:
        case Qt::Key_Enter:
            if (this->descriptor_->IsExists())
            {
                event->accept();
                if (this->descriptor_->IsLinked())
                    emit this->OnEject(this);
                else
                    emit this->OnMount(this);
            }
            break;

        case Qt::Key_Up:
            event->accept();
            emit this->OnSelectPrevious(this);
            break;

        case Qt::Key_Down:
            event->accept();
            emit this->OnSelectNext(this);
            break;

        default:
            event->ignore();
            break;
    }
}

void ContainerView::mousePressEvent(QMouseEvent */*event*/)
{
    emit this->OnSelected(this);
}

void ContainerView::EjectClickSlot()
{
    emit this->OnEject(this);
}
