#include "createdialog.h"
#include "ui_createdialog.h"

#include "container/keygen.h"
#include "model/bcoperation.h"
#include "model/helper.h"
#include "entity/containerdescriptor.h"

#include <QList>
#include <QFile>
#include <QDir>
#include <QFileInfo>
#include <QDialog>

CreateDialog::CreateDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CreateDialog)
{
    ui->setupUi(this);

    //QObject::connect(this->ui->browseButton, SIGNAL(clicked()), this, SLOT(OnBrowseButtonSlot()));
    connect(this, SIGNAL(rejected()), this, SLOT(OnCancelSlot()));
    connect(this->ui->okButton, SIGNAL(clicked()), this, SLOT(OnOkSlot()));
    connect(this->ui->cancelButton, SIGNAL(clicked()), this, SLOT(OnCancelSlot()));
    connect(this->ui->advancedCheckBox, SIGNAL(toggled(bool)), this, SLOT(OnAdvancedCheckBoxCheckedSlot(bool)));

    this->ui->advancedCheckBox->setChecked(false);

    // CheckBox.setChecked() should generate toggled(bool) signal, but at now it doesn't, so call method by hand.
    this->OnAdvancedCheckBoxCheckedSlot(false);

    this->ui->okButton->setDefault(true);
    this->FillControls();
    this->ResetResult();
}

CreateDialog::~CreateDialog()
{
    delete ui;
}

QDialog::DialogCode CreateDialog::GetResult()
{
    return this->dialogResult_;
}

ContainerDescriptor *CreateDialog::GetData()
{
    QString algName = this->ui->selectCipherWidget->GetAlgorythmName();
    bc_alg_id algId = this->ui->selectCipherWidget->GetAlgorythmId();
    QString modeName = this->ui->selectCipherWidget->GetModeName();
    bc_mode_id modeId = BCOperationGetModeId(modeName);
    QString filePath = ""; //this->ui->locationText->text() + "/" + this->ui->nameText->text();
    QString fileSystem = this->ui->fsWidget->GetFileSystem();

    // If user selected 'None', clear FS-type string.
//    if (!fileSystem.compare(NO_FILESYSTEM_STRING, Qt::CaseInsensitive))
//        fileSystem.clear();

    QString description = this->ui->descriptionText->text();

    // Get container size.
    QString sizeStr;
    QString sizeUnit = this->ui->sizeUnitCombo->currentText().at(0); // Get first letter (K/M/G).
    sizeStr = sizeStr.append("%0%1").arg(this->ui->sizeSpinBox->value()).arg(sizeUnit);
    bc_u64 size = BCHelperGetSize(sizeStr);

    return ContainerDescriptor::CreateForNewContainer(filePath, size, algId, algName,
                                                      modeId, modeName, fileSystem, description);
}

void CreateDialog::FillControls()
{
    // Fill size combo.
    this->ui->sizeUnitCombo->addItem("KB");
    this->ui->sizeUnitCombo->addItem("MB");
    this->ui->sizeUnitCombo->addItem("GB");

    this->ui->sizeUnitCombo->setCurrentIndex(2);
}

void CreateDialog::OnOkSlot()
{
    if (!this->ui->sizeSpinBox->value())
    {
        BCOperationShowErrorString(tr("Container size cannot be zero."));
        return;
    }
    this->dialogResult_ = QDialog::Accepted;
    this->setResult(QDialog::Accepted);
    this->haveResult_ = true;
}

void CreateDialog::OnCancelSlot()
{
    this->setResult(QDialog::Rejected);
    this->dialogResult_ = QDialog::Rejected;
    this->haveResult_ = true;
}

void CreateDialog::OnAdvancedCheckBoxCheckedSlot(bool checked)
{
    this->ui->advancedWidget->setVisible(checked);
    BCHelperFixSize(this);
}
