#ifndef BC_SETTINGS_H
#define BC_SETTINGS_H

#include "entity/hotkey.h"

#include <QList>
#include <QSize>
#include <QPoint>
#include <QVariant>
#include <QSettings>
#include <QKeySequence>

class ContainerDescriptor;

/**
  @brief Used for accessing application settings. All property changes
        are reflecting in configuration file.
*/
class BCSettings : QObject
{
    Q_OBJECT

public:
    BCSettings();


    // General Properties.


    /**
      @brief Gets value indicating whether settings are stored in config.
    */
    bool IsConfigUsed();

    /**
      @brief Sets value indicating whether settings should be saved in config.
            If @param store is set to false, config file will be cleared and
            only this setting will be saved.
    */
    void SetConfigUsed(bool used);

    /**
      @brief Clears all saved settings.
    */
    void Clear();

    /**
      @brief Returns list of container pathes loaded from user config file.
      @warning Caller should free function result by delete operator.
    */
    QList<ContainerDescriptor *> *LoadContainerList();

    /**
      @brief Stores container path list in user config file.

      @param pathList QList<QString> object containing pathes of container
                      files to store in user config.
      @warning Caller should free parameter's memory.
    */
    void SaveContainerList(QList<ContainerDescriptor*> *descriptorList);

    bool IsEjectAllHotKeyEnabled();
    void SetEjectAllHotKeyEnabled(bool enabled);

    BCHotkey EjectAllHotKey(const BCHotkey& defaultHotkey = BCHotkey());
    void SetEjectAllHotKey(const BCHotkey& hotkey);

    void SetEjectAllForced(bool forced);
    bool IsEjectAllForced();

    void SetContainerListSaving(bool save);
    bool IsContainerListSaving();

    void SetMinimizeToTray(bool use);
    bool IsMinimizingToTray();


    // Main Form Properties.


    bool MainFormIsMaximized();
    void SetMainFormMaximized(bool maximized);

    QSize MainFormSize(const QSize& defaultSize);
    void SetMainFormSize(const QSize& size);

    QPoint MainFormLocation(const QPoint& defaultLocation);
    void SetMainFormLocation(const QPoint& location);

private:
    QSettings       userSettings_;

    void SaveVariant(const QString& elementName, const QVariant& val, const QString& group = QString());
};
#endif // BC_SETTINGS_H
