#include "filesystemwidget.h"
#include "ui_filesystemwidget.h"

#include "model/helper.h"

#define NO_FILESYSTEM_STRING (QObject::tr("<No, I will format it later>"))

FileSystemWidget::FileSystemWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::FileSystemWidget)
{
    ui->setupUi(this);

    this->fileSystems = new QMap<QString, QString>();

    this->PrepareFileSystems();
    this->ReloadFilesystems(true);
}

FileSystemWidget::~FileSystemWidget()
{
    delete ui;
}

QString FileSystemWidget::GetFileSystem()
{
    if (!this->ui->fsCombo->currentText().compare(NO_FILESYSTEM_STRING, Qt::CaseInsensitive))
        return QString();

    return this->fileSystems->value(this->ui->fsCombo->currentText());
}

void FileSystemWidget::ReloadFilesystems(bool isNoFSVisible)
{
    // Loading list of filesystems.
    QStringList filesystems;

    QList<QString> fsNames = this->fileSystems->keys();
    for (int i = 0; i < fsNames.count(); i++)
    {
        filesystems.append(fsNames.at(i));
    }

    if (isNoFSVisible)
    {
        filesystems.insert(0, NO_FILESYSTEM_STRING);
    }

    this->ui->fsCombo->clear();
    this->ui->fsCombo->addItems(filesystems);

    // Select default FS. Select the most popular first.
    int defaultFs = 0;

    if (-1 == (defaultFs = BCHelperGetIndexInList(filesystems, "FAT")))
    {
        if (-1 == (defaultFs = BCHelperGetIndexInList(filesystems, "ext3")))
        {
            if (this->ui->fsCombo->count() > 1)
                defaultFs = 1;
        }
    }

    // Select default fs in combo.
    this->ui->fsCombo->setCurrentIndex(defaultFs);
}

static void AddFileSystemToMap(QMap<QString, QString> *map, const QString& fs)
{
    if (((fs == "msdos") || (fs == "vfat") || fs == "fat"))
    {
        if (!map->contains("FAT"))
        {
            map->insert("FAT", fs);
        }
    }
    else
    {
        map->insert(fs, fs);
    }
}

void FileSystemWidget::PrepareFileSystems()
{
    // Get all filesystems, that are supported on the system.
    QStringList filesystems;
    bool succeeded = BCHelperGetSupportedFS(&filesystems);
    if (!succeeded)
        return;

    // Prepare list of known unsupported file systems.
    QList<QString> unsupportedFs;
    unsupportedFs.append("cramfs");
    unsupportedFs.append("fuseblk");

    this->fileSystems->clear();

    // Create [FS User Name] => [FS Sys Name] map.
    for (int i = 0; i < filesystems.count(); i++)
    {
        QString fs = filesystems.at(i);
        if (!unsupportedFs.contains(fs))
        {
            AddFileSystemToMap(this->fileSystems, fs);
        }
    }
}
