#include "hiddenpartdialog.h"
#include "ui_hiddenpartdialog.h"

#include "spacemap/spacemap.h"

HiddenPartDialog::HiddenPartDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::HiddenPartDialog)
{
    ui->setupUi(this);

    this->ui->widget->ReloadFilesystems(false);

    connect(this->ui->finishButton, SIGNAL(clicked()), this, SLOT(FinishSlot()));
    connect(this->ui->protectButton, SIGNAL(clicked()), this, SLOT(ProtectSlot()));
    connect(this->ui->cancelButton, SIGNAL(clicked()), this, SLOT(CancelSlot()));
    connect(this, SIGNAL(rejected()), this, SLOT(CancelSlot()));
    this->ResetResult();
}

HiddenPartDialog::~HiddenPartDialog()
{
    delete ui;
}

void HiddenPartDialog::SetParams(HiddenPartDialogParameters *parameters)
{
    this->parameters_ = parameters;
    this->UpdateControls();
}

void HiddenPartDialog::UpdateControls()
{
    if (this->parameters_)
    {
        bc_u64 length;
        sm_result result = SpaceMapGetLength(this->parameters_->freeSpaceMap, &length);

        if (kSpaceMap_Success != result)
            return;

        // Set maximum size in megabytes.
        this->ui->sizeSpinBox->setMaximum(length / (1024L * 1024L));
    }
}

void HiddenPartDialog::FinishSlot()
{
    QString fsType = this->ui->widget->GetFileSystem();
    if (fsType.length() && this->parameters_->fsType)
    {
        std::string fsStr = fsType.toStdString();
        strncpy(this->parameters_->fsType, fsStr.c_str(), BC_MAX_PATH);
    }
    else
    {
        qDebug("Error getting fstype inside hidden part dialog: empty string returned.\n");
    }

    bc_u64 size = this->ui->sizeSpinBox->value() * 1024L * 1024L;
    if (size)
    {
        // Truncate map to the specified size.
        sm_result smres = SpaceMapGetMapOfSize(this->parameters_->freeSpaceMap, size,
                                               &this->parameters_->resizedMap);
        if (kSpaceMap_Success != smres)
        {
            qDebug("Error while resizing space map in hidden part dialog.\n");
        }
    }

    this->result_ = QDialog::Accepted;
    this->haveResult_ = true;
    this->accept();
    this->close();
}

void HiddenPartDialog::ProtectSlot()
{
    if (this->parameters_ && this->parameters_->protectCallback)
    {
        this->parameters_->protectCallback(this->parameters_->header,
                   this->parameters_->containerFile, this->parameters_->freeSpaceMap);
        this->UpdateControls();
    }
}

void HiddenPartDialog::CancelSlot()
{
    this->result_ = QDialog::Rejected;
    this->haveResult_ = true;
    this->close();
}
