#ifndef HIDDENPARTDIALOG_H
#define HIDDENPARTDIALOG_H

#include <QDialog>

#include "container/user.h"

namespace Ui {
class HiddenPartDialog;
}

typedef struct _HiddenPartDialogParameters
{
    void                                *header;        // keygen header with operations.
    const char                          *containerFile; // full path to the container file.
    SpaceMap                            *freeSpaceMap;  // space map containing a free space of the container.
    SpaceMap                            *resizedMap;    // resized space map.
    char                                *fsType;        // buffer, will contain a file system.
    BCProtectExistingHiddenPartCallback protectCallback;
}
HiddenPartDialogParameters;

class HiddenPartDialog : public QDialog
{
    Q_OBJECT
    
public:
    explicit HiddenPartDialog(QWidget *parent = 0);
    ~HiddenPartDialog();

    bool HaveResult()
    {
        return this->haveResult_;
    }

    void SetParams(HiddenPartDialogParameters *parameters);

    int GetHiddenPartSize();

    DialogCode GetResult()
    {
        return this->result_;
    }

    void ResetResult()
    {
        this->haveResult_ = false;
    }
    
private:
    Ui::HiddenPartDialog        *ui;

    HiddenPartDialogParameters  *parameters_;
    bool                        haveResult_;
    DialogCode                  result_;
    long                        maxSize_;

    void UpdateControls();

private slots:
    void FinishSlot();
    void ProtectSlot();
    void CancelSlot();
};

#endif // HIDDENPARTDIALOG_H
