#include "hotkeydialog.h"
#include "ui_hotkeydialog.h"

#include <QKeyEvent>
#include <QLabel>
#include <QDebug>

HotKeyDialog::HotKeyDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::HotKeyDialog)
{
    ui->setupUi(this);
    this->keyLabel_ = this->ui->hotkeyLabel;
    this->hotkey_= new BCHotkey();
}

HotKeyDialog::~HotKeyDialog()
{
    delete ui;
    if (this->hotkey_)
        delete this->hotkey_;
}

void HotKeyDialog::SetHotkey(const BCHotkey& hotkey)
{
    this->hotkey_->mainKey = hotkey.mainKey;
    this->hotkey_->modifiers = hotkey.modifiers;
    this->keyLabel_->setText(this->hotkey_->ToString());
}

void HotKeyDialog::keyPressEvent(QKeyEvent *e)
{
    long int keys = e->key();

    if ((e->key() == Qt::Key_Enter) || (e->key() == Qt::Key_Return))
        this->accept();

    if (e->key() == Qt::Key_Escape)
        this->reject();

    if (keys == Qt::Key_Control || keys == Qt::Key_Shift || keys == Qt::Key_Alt || keys == Qt::Key_Meta)
    {
        // Ignore single special key press.
        return;
    }

    Qt::KeyboardModifiers modifiers = e->modifiers();
    BCHotkey hotkey((Qt::Key)keys, modifiers);
    QString modifiersString;

    if (modifiers & Qt::ShiftModifier)
    {
        modifiersString += "Shift";
        keys += Qt::SHIFT;
    }

    if (modifiers & Qt::ControlModifier)
    {
        modifiersString += (modifiersString.length()? QString("+") : QString()) + QString("Ctrl");
        keys += Qt::CTRL;
    }

    if (modifiers & Qt::AltModifier)
    {
        modifiersString += (modifiersString.length()? QString("+") : QString()) + QString("Alt");
        keys += Qt::ALT;
    }

    if (modifiers & Qt::MetaModifier)
    {
        modifiersString += (modifiersString.length()? QString("+") : QString()) + QString("Meta");
        keys += Qt::META;
    }

    QKeySequence keySequence(keys);
    QKeySequence modifiersSequence(modifiersString);

    if ((modifiersSequence.matches(keySequence) != QKeySequence::ExactMatch) // Ignore strings with modifiers only.
            && modifiersString.contains("+"))                               // Ignore string with less than two modifiers.
    {
        this->SetHotkey(hotkey);
    }
}

BCHotkey HotKeyDialog::GetHotkey()
{
    return BCHotkey(*(this->hotkey_));
}

