#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>


#include "entity/settings.h"

class QSystemSemaphore;
class ContainerModel;
class QItemSelection;
class ContainerInfoWidget;
class ContainerDescriptor;
class ContainerTreeWidget;
class MainToolbarWidget;
class BCToolUI;
class HotkeyListener;
class TrayIconManager;

namespace Ui {
class MainWindow;

}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    void closeEvent(QCloseEvent * event);

private:
    Ui::MainWindow      *ui;
    ContainerModel      *containerModel_;
    ContainerInfoWidget *containerInfo_;
    ContainerTreeWidget *containerTree_;
    MainToolbarWidget   *mainToolbar_;
    HotkeyListener      *hotkeyListener_;
    TrayIconManager     *trayIcon_;
    QSystemSemaphore    *singleInstSemaphore_;
    bool                isRestoring_;
    //bool                isExiting_; ///< When set to false, 'close' events cause form hiding.

    BCSettings          settings_;

    void changeEvent(QEvent *e);

    /// @brief Updates enability of tool buttons depending of currently selected descriptor.
    void UpdateToolButtonState();

    /// @brief Updates all necessary controls after some changes have occured in selected descriptor.
    void ProcessDescriptorChanges();

    /// @brief Loads stored coords and sizes (if any) and sets them to widgets.
    void LoadPreviousViewSettings();

    /// @brief Saves necessary coords and sizes.
    void SaveViewSettings();

    /// @brief Updates the hot-key listener state from the application config.
    void UpdateHotkeyListener();

private slots:
    void SelectedDescriptorChangedSlot(ContainerDescriptor *);
    void TrayIconDescriptorSelectedSlot(ContainerDescriptor * descriptor);
    void TrayIconShowControlPanelSlot();
    void BrowseSlot();
    void MountSlot();
    void LinkSlot();
    void RawLinkSlot();
    void CreateSlot();
    void ShowSettingsSlot();
    void EjectForceSlot();
    void EjectSlot();
    void EjectAllSlot();
    void ExitSlot();
    void DeleteSlot();
    void EncryptHeaderSlot();
    void DecryptHeaderSlot();
    void BackupHeaderSlot();
    void RestoreHeaderSlot();
    void ReencryptSlot();
    void CreateHiddenSlot();
    void AddPasswordSlot();
    void RemovePasswordSlot();
    void ChangePasswordSlot();
    void FormatSlot();
    void FsckSlot();
    void ShowLogSlot();
    void AboutSlot();
    void AboutQtSlot();
};

#endif // MAINWINDOW_H
