#ifndef CONTAINERLISTMODEL_H
#define CONTAINERLISTMODEL_H

#include <QObject>

class ContainerDescriptor;

/**
    @brief Model for containers' list.
*/
class ContainerModel :public QObject
{
    Q_OBJECT

public:
    bool ignoreEvents;

    ContainerModel(QObject *parent = 0);
    ~ContainerModel();

    QList<ContainerDescriptor*> *GetDescriptors()
    {
        return this->descriptors_;
    }

    /// @brief Adds specified container descriptor to model.
    void Add(ContainerDescriptor *descriptor);

    /// @brief Removes specified container descriptor from model.
    void Remove(ContainerDescriptor *descriptor);

    /// @brief Checks whether specified container is already present in the model.
    int IndexOfContainer(const QString &fullPath);

    /// @brief Updates all information about each container in the model.
    void Update();

    /// @brief Removes all containers from the model.
    void Clear();

    /// @brief Loads list of existing container files from config file.
    void LoadFromConfig();

    /// @brief Loads list of currently mounted and existing containers and adds them to model.
    void LoadFromDevices();

    /// @brief Loads descriptors for all existing containers in list.
    void LoadFromList(QList<ContainerDescriptor *>* descriptors);

    /// @brief Saves list of containers stored in the model, to config file.
    void SaveToConfig();

signals:
    void OnDataChanged();

private slots:
    void DiskEventSlot();

private:
    QList<ContainerDescriptor *> *descriptors_;

    void InternalAdd(ContainerDescriptor *descriptor);
    void EmitDataChanged();
};

#endif // CONTAINERLISTMODEL_H
