#include "devicemonitor.h"

#include "disk/disk.h"
#include "model/bcoperation.h"
#include "model/helper.h"

#include <stdio.h>

extern void DiskEventCallback(BCDiskRef *, int /*event*/, void *arg)
{
    DeviceMonitor *instance = static_cast<DeviceMonitor*>(arg);
    // NOTE: When mounting some file, this callback is first time executing when BC device is appearing,
    // i.e. kBCDiskAppeared event is coming. If this callback complete execution after
    // kBCDiskChanged came, and kBCDiskChanged came after ContainerDescriptor is created, we will
    // think descriptor is only linked, but it will be mounted. So now simply sleep before sending signal,
    // in order to container mount is completed.
    BCHelperSleep(2000);
    instance->RaiseDiskEvent();
}

// static
DeviceMonitor *DeviceMonitor::GetInstance()
{
    static DeviceMonitor *instance = 0;

    if (!instance)
    {
        instance = new DeviceMonitor();
    }

    return instance;
}

DeviceMonitor::DeviceMonitor()
    : QObject(0)
{
    this->isListening = false;
}

void DeviceMonitor::RaiseDiskEvent()
{
    emit this->OnDiskEvent();
}

void DeviceMonitor::StartListening()
{
    if (!this->isListening)
    {
        this->isListening = true;
        BCStartDiskEventListener();
        BCRegisterForDiskEvents(DiskEventCallback, kBCDiskAllEvents, this);
    }
    else
    {
        fprintf(stderr, "StartListening: listening is already started.\n");
    }
}

void DeviceMonitor::StopListening()
{
    if (this->isListening)
    {
        this->isListening = false;
        BCUnregisterForDiskEvents(DiskEventCallback);
        BCStopDiskEventListener();
    }
    else
    {
        fprintf(stderr, "StopListening: listening is already stopped.\n");
    }
}
