/**
  This class contains functions that are useful for some part of the program,
  but should not be put into separate entity because of their small amount.
*/

#ifndef HELPER_H
#define HELPER_H

#include "container/intsize.h"

#include <QString>
#include <QIcon>

void BCHelperPrintStackTrace(const QString &msg);

/**
  @brief Returns string with container size formatted with appropriate size units.
*/
QString BCHelperGetSizeString(bc_u64 size);

/**
  @brief Converts string of format ex. "10M" to size in bytes.
*/
bc_u64 BCHelperGetSize(QString sizeString);

/**
  @brief Gets a location from a user by using modal dialog.
  If user cancels dialog, an empty string will be returned.
*/
QString BCHelperGetLocation(const QString& dialogCaption, const QString &basePath);

/**
  @brief Fills specified string list with list of filesystems supported by system.
*/
bool BCHelperGetSupportedFS(QStringList *out_supportedFS);

QStringList BCHelperLoadFiles(const QString& dialogCaption,
                              const QString& extensions, bool allowMultiple);

/**
  @brief Gets a full path to save file got from a user.
  If a user cancels the dialog, an empty string will be returned.
*/
QString BCHelperSaveFile(const QString& defaultName, const QString &dialogCaption,
                         const QString &extensions);

void BCHelperDeleteFileIfExists(const QString &fileName);

/**
  @brief Suspends current thread for msec milliseconds.
*/
void BCHelperSleep(unsigned long msec);

/**
  @brief Loads icon with specified name from application resources.
*/
bool BCHelperLoadIcon(const QString& iconName, QIcon *out_icon);

/**
  @brief Gets index of specified word in string list.
*/
int BCHelperGetIndexInList(QStringList list, QString word);

/**
  @brief Updates size of the widget specified to a size hint value and sets it fixed.
*/
void BCHelperFixSize(QWidget *widget);

#endif // HELPER_H
