#include "logging.h"

#include <errno.h>
#include <unistd.h>
#include <fcntl.h>

static QString g_logText = QString();

static int g_outReadDescriptor = -1;
static int g_errReadDescriptor = -1;

void BCLogStart()
{
    // Prevent second launch.
    if (g_outReadDescriptor != -1)
        return;

    int outDescriptors[2]; // stdout pipe, 0 - read, 1 - write
    int errDescriptors[2]; // stderr pipe

    if (pipe2(outDescriptors, O_NONBLOCK) || pipe2(errDescriptors, O_NONBLOCK))
    {
        int error = errno;
        BCLogAdd(QString("Cannot enable logging. Error opening pipe: %0").arg(strerror(error)));
        return;
    }

    // Replace standard output and error output descriptors by pipe descriptor.
    dup2(outDescriptors[1], 1);// stdout
    dup2(errDescriptors[1], 2);// stderr

    // Save read descriptor.
    g_outReadDescriptor = outDescriptors[0];
    g_errReadDescriptor = errDescriptors[0];

    // Close write descriptors, as we duplicate them.
    close(outDescriptors[1]);
    close(errDescriptors[1]);

    BCLogAdd("Logging started.");
}

static void ReadLog(int fd)
{
    char b;
    QString str;
    while (read(fd, &b, 1) > 0)
    {
        if ((b != '\0') && (b != '\n'))
            str.append(b);
        else
        {
            BCLogAdd(str);
            str.clear();
        }
    }
}

void BCLogUpdate()
{
    ReadLog(g_outReadDescriptor);
    ReadLog(g_errReadDescriptor);
}

void BCLogClear()
{
    g_logText.clear();
}

QString BCLogGetText()
{
    return g_logText;
}

void BCLogAdd(const QString& text)
{
    g_logText.append("\n");
    g_logText.append(text);

    // Truncate log.
    if (g_logText.length() > 16000)
        g_logText = g_logText.mid(16000);
}
