#!/bin/bash

set -e
GUI_BINARY=bestcrypt-panel
GUI_DESKTOP_FILE=jetico-bestcrypt-panel.desktop
GUI_ICON_NAME=application-x-bestcrypt
GUI_MIME_NAME=application-x-bestcrypt-container
GUI_MIME_FILE=jbc-mime.xml

if [[ "$1" = "-u" ]]
then
    #
    # Uninstalling desktop information.
    #
	echo "Uninstalling desktop information..."
    
    # Unregister application from X menu.
    rm -f /usr/share/applications/$GUI_DESKTOP_FILE

    # Removing executable.
    rm -f /usr/bin/$GUI_BINARY

    # Uninstall MIME type information.
    xdg-mime uninstall ./$GUI_MIME_FILE > /dev/null

    # Uninstall MIME icon.
    xdg-icon-resource uninstall --context mimetypes --size 64 $GUI_MIME_NAME > /dev/null
    
    # Uninstall application icons.
    xdg-icon-resource uninstall --context apps --size 48 $GUI_ICON_NAME > /dev/null
    xdg-icon-resource uninstall --context apps --size 64 $GUI_ICON_NAME > /dev/null

    echo "Done."
    exit
fi

if [[ "$1" = "-i" ]]
then
    #
    # Installing desktop information.
    #

    # Install application executable.
    install -o root -m 0755 ../$GUI_BINARY /usr/bin/

    cp ./$GUI_DESKTOP_FILE /usr/share/applications/
    # Register icon for mime type.
    xdg-icon-resource install --noupdate --context mimetypes --size 64 ../res/icon_crypt_64x64.png $GUI_MIME_NAME

    # Register icon for program.
    xdg-icon-resource install --noupdate --context apps --size 64 ../res/bestcrypt-64.png $GUI_ICON_NAME
    xdg-icon-resource install --noupdate --context apps --size 48 ../res/bestcrypt-48.png $GUI_ICON_NAME

    # Register mime type.
    xdg-mime install $GUI_MIME_FILE

    # Install application to X menu.
    xdg-desktop-icon install $GUI_DESKTOP_FILE

    # Associate bctool-gui with mimetype created.
    mkdir -p ~/.local/share/applications/
    echo >> ~/.local/share/applications/mimeapps.list # Create a file if it isn't exists.
    xdg-mime default $GUI_DESKTOP_FILE application/x-bestcrypt-container 

    # Force desktop icon system update.
    xdg-icon-resource forceupdate

    echo "Desktop information installed."
    exit
fi

echo "Script is a part of BestCrypt for Linux installation, you shouldn't use it directly. Used for managing GUI desktop information. Use -i for install or -u for uninstall."
