#include "selectcipherwidget.h"
#include "ui_selectcipherwidget.h"

#include "model/helper.h"
#include "model/bcoperation.h"

SelectCipherWidget::SelectCipherWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::SelectCipherWidget)
{
    ui->setupUi(this);

    connect(this->ui->algCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(OnSelectedAlgChangedSlot(int)));
    this->FillAlgList();
}

void SelectCipherWidget::FillAlgList()
{
    // Loading list of ciphers.
    this->algList_ = new QList<BCAlgInfo>();

    if (!BCOperationGetAlgList(this->algList_))
        return;

    QStringList algNameList;
    for (int i=0; i<this->algList_->count(); i++)
    {
        BCAlgInfo algInfo = this->algList_->at(i);
        algNameList.insert(i, QString(algInfo.name)); // Keep indexes while inserting items.
    }

    if (algNameList.count())
    {
        this->ui->algCombo->addItems(algNameList);

        // Select default cipher to 'rijndael'.
        int defaultAlg;
        if (-1 == (defaultAlg = BCHelperGetIndexInList(algNameList, "rijndael")))
        {
            defaultAlg = 0;
        }

        this->ui->algCombo->setCurrentIndex(defaultAlg);

    }
}

QString SelectCipherWidget::GetModeName()
{
    return this->ui->modeCombo->currentText();
}

QString SelectCipherWidget::GetAlgorythmName()
{
    int algIndex = this->ui->algCombo->currentIndex();
    if (algIndex == -1)
        return QString();
    return QString(this->algList_->at(algIndex).name);
}

bc_alg_id SelectCipherWidget::GetAlgorythmId()
{
    int algIndex = this->ui->algCombo->currentIndex();
    if (algIndex == -1)
        return BC_INVALID_ALG;
    bc_alg_id algId = this->algList_->at(algIndex).id;
    return algId;
}

void SelectCipherWidget::AddModeIfSupported(BCAlgInfo &alg, bc_mode_id mode)
{
    if (BCOperationIsModeSupported(alg, mode))
        this->ui->modeCombo->addItem(BCOperationGetModeName(mode));
}

void SelectCipherWidget::OnSelectedAlgChangedSlot(int index)
{
    BCAlgInfo selectedAlg = this->algList_->at(index);
    this->ui->modeCombo->clear();

    this->AddModeIfSupported(selectedAlg, kBCMode_CBC);
    this->AddModeIfSupported(selectedAlg, kBCMode_LRW);
    this->AddModeIfSupported(selectedAlg, kBCMode_XTS);
    this->ui->modeCombo->setEnabled(true);

    int modeCount = this->ui->modeCombo->count();
    if (modeCount)
        this->ui->modeCombo->setCurrentIndex(modeCount - 1);
}

SelectCipherWidget::~SelectCipherWidget()
{
    delete this->algList_;
    delete ui;
}
