#include "settingsdialog.h"
#include "ui_settingsdialog.h"

#include "hotkeydialog.h"

SettingsDialog::SettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SettingsDialog),
    settings()
{
    ui->setupUi(this);

    connect(this->ui->changeHotKeyButton, SIGNAL(clicked()), this, SLOT(OnHotKeyChangeSlot()));
    connect(this->ui->useConfigCheckBox, SIGNAL(toggled(bool)), this, SLOT(OnConfigEnabledCheckedSlot(bool)));
    connect(this->ui->okButton, SIGNAL(clicked()), this, SLOT(OnAcceptSlot()));
    connect(this->ui->cancelButton, SIGNAL(clicked()), this, SLOT(OnCancelSlot()));

    LoadSettings();
}

SettingsDialog::~SettingsDialog()
{
    delete ui;
}

void SettingsDialog::OnHotKeyChangeSlot()
{
    HotKeyDialog hotkeyDlg;
    BCHotkey currentHotKey;

    currentHotKey = this->settings.EjectAllHotKey();
    hotkeyDlg.SetHotkey(currentHotKey);
    QDialog::DialogCode result = (QDialog::DialogCode)hotkeyDlg.exec();

    if (QDialog::Accepted == result)
    {
        BCHotkey newHotkey = hotkeyDlg.GetHotkey();
        this->settings.SetEjectAllHotKey(newHotkey);
        this->DisplayHotkeyString(newHotkey);
    }
    else
    {
        this->DisplayHotkeyString(currentHotKey);
    }
}

void SettingsDialog::OnAcceptSlot()
{
    SaveSettings();
    this->close();
}

void SettingsDialog::OnCancelSlot()
{
    this->close();
}

void SettingsDialog::LoadSettings()
{
    BCHotkey hotkey;

    this->ui->useConfigCheckBox->setChecked(this->settings.IsConfigUsed());
    this->ui->keepContainerListCheckBox->setChecked(this->settings.IsContainerListSaving());
    this->ui->forceEjectAllCheckBox->setChecked(this->settings.IsEjectAllForced());
    this->ui->useHotKeyCheckBox->setChecked(this->settings.IsEjectAllHotKeyEnabled());
    this->ui->minimizeToTrayCheckBox->setChecked(this->settings.IsMinimizingToTray());

    hotkey = this->settings.EjectAllHotKey();

    this->DisplayHotkeyString(hotkey);
    this->OnConfigEnabledCheckedSlot(this->settings.IsConfigUsed());
}

void SettingsDialog::SaveSettings()
{
    bool useConfig = this->ui->useConfigCheckBox->isChecked(),
            saveContainerList = this->ui->keepContainerListCheckBox->isChecked(),
            forceEjectAll = this->ui->forceEjectAllCheckBox->isChecked(),
            useEjectAllHotkey = this->ui->useHotKeyCheckBox->isChecked(),
            minimizeToTray = this->ui->minimizeToTrayCheckBox->isChecked();

    this->settings.SetConfigUsed(useConfig);
    this->settings.SetContainerListSaving(saveContainerList);
    this->settings.SetEjectAllForced(forceEjectAll);
    this->settings.SetEjectAllHotKeyEnabled(useEjectAllHotkey);
    this->settings.SetMinimizeToTray(minimizeToTray);
}

void SettingsDialog::DisplayHotkeyString(const BCHotkey& hotkey)
{
    this->ui->hotKeyLabel->setText(hotkey.ToString());
}

void SettingsDialog::OnConfigEnabledCheckedSlot(bool set)
{
    this->ui->keepContainerListCheckBox->setEnabled(set);
    this->ui->ejectAllTab->setEnabled(set);
}
