#ifndef TRAYICONMANAGER_H
#define TRAYICONMANAGER_H

#include <QObject>
#include <QIcon>
#include <QSystemTrayIcon>

class ContainerDescriptor;
class ContainerModel;
class QAction;
class QMenu;

class TrayIconManager : public QObject
{
    Q_OBJECT
public:
    /**
      @brief Initializes new instance of TrayIconManager.
      @param model      Pointer to the container model.
      @param icon       Tray icon.
    */
    TrayIconManager(ContainerModel *model);
    ~TrayIconManager();

    void ShowIcon();
    void HideIcon();
    void UpdateMenu();

signals:
    void OnShowControlPanel();
    void OnQuit();
    void OnDescriptorSelected(ContainerDescriptor *descriptor);

private:
    QSystemTrayIcon     *trayIcon_;
    ContainerModel      *containerModel_;
    QAction             *showControlPanelAction_;
    QAction             *exitAction_;
    QMenu               *menu;

private slots:
    void ContainerModelChangedSlot();
    void MenuActionTriggeredSlot(QAction *action);
    void TrayIconActivatedSlot(QSystemTrayIcon::ActivationReason reason);
};

#endif // TRAYICONMANAGER_H
