#include "../../container/file.h"
#include "../../container/intsize.h"
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

void TestContainerFile()
{
    bc_file_handle handle;
    const int bufferSize = 1024 * 1024;
    char buffer[bufferSize] = {0};
    char fileName[] = "test-file.txt";
    bc_u64 bytesRead;
    // Create file for testing.
    int fd = creat(fileName, S_IRWXU);
    bc_file_result openResult;

    if (-1 == fd)
        CU_FAIL("Can't create test file!");

    if (close(fd))
        CU_FAIL("Can't close test file after creating.");

    openResult = BCFileOpen(fileName, kBCFileMode_ReadWrite, &handle);
    CU_ASSERT(kBCFileResult_Success == openResult);

    // Trying to read from empty file.
    bytesRead = BCFileRead(&handle, buffer, 256);
    CU_ASSERT(0 == bytesRead);

    bytesRead = BCFileReadWithOffset(&handle, buffer, 256, 0);
    CU_ASSERT(0 == bytesRead);

    CU_ASSERT(0 == BCFileSeek(&handle, 0, kBCFileSeek_Begin));

    // Add empty bytes.
    CU_ASSERT(256 == BCFileWrite(&handle, buffer, 256));
    CU_ASSERT(256 == BCFileGetPosition(&handle));

    // Now must read all 256 bytes.
    CU_ASSERT(0 == BCFileSeek(&handle, 0, kBCFileSeek_Begin));

    bytesRead = BCFileRead(&handle, buffer, 256);
    CU_ASSERT(256 == bytesRead);
    CU_ASSERT(256 == BCFileGetPosition(&handle));

    // And can't read more.
    bytesRead = BCFileRead(&handle, buffer, 256);
    CU_ASSERT(0 == bytesRead);
    
    // But can read with offset.
    bytesRead = BCFileReadWithOffset(&handle, buffer, 256, 0);
    CU_ASSERT(256 == bytesRead);

    // Check that reading with offset is not changing file pointer.
    CU_ASSERT(0 == BCFileSeek(&handle, 0, kBCFileSeek_Begin));
    CU_ASSERT(0 == BCFileGetPosition(&handle));

    CU_ASSERT(256 == BCFileReadWithOffset(&handle, buffer, 256, 0));
    CU_ASSERT(0 == BCFileGetPosition(&handle));

    // Try to write a big amount of data.
    for (int i = 0; i < 100; i++)
    {
        CU_ASSERT(bufferSize == BCFileWrite(&handle, buffer, bufferSize));
    }

    BCFileClose(&handle);

    CU_ASSERT(0 == remove(fileName));
}
