#ifndef TESTS_ALG_LOOKUP
#define TESTS_ALG_LOOKUP

#include "../tests-common.c"

#include <string.h>
#include <stdio.h>
#include <unistd.h>

void TestBCKernelListAlgs(void)
{
    BCAlgInfo *list = NULL;
    unsigned int list_count = -1;
    bc_kern_result result = BCKernelListAlgs(&list, &list_count);
    CU_ASSERT(kBCKernel_NotInitialized == result);
    CU_ASSERT(NULL == list);
    
    InstallBCKernel();
    
    result = BCKernelListAlgs(&list, &list_count);
    CU_ASSERT(kBCKernel_Success == result);
    CU_ASSERT(0 == list_count);
    CU_ASSERT(NULL == list);
    
    if (Install3DESModule())
    {
        UninstallBCKernel();
        CU_FAIL("Cannot install 3DES module.");
        return;
    }
    
    result = BCKernelListAlgs(NULL, &list_count);
    CU_ASSERT(kBCKernel_InvalidBuffer == result);
    
    result = BCKernelListAlgs(&list, NULL);
    CU_ASSERT(kBCKernel_InvalidBuffer == result);
    
    result = BCKernelListAlgs(NULL, NULL);
    CU_ASSERT(kBCKernel_InvalidBuffer == result);
    
    result = BCKernelListAlgs(&list, &list_count);
    CU_ASSERT(kBCKernel_Success == result);

    if (NULL == list || 1 != list_count)
    {
        Uninstall3DESModule();
        UninstallBCKernel();
        CU_FAIL("Unexpected list came.\n");
        return;
    }
    
    BCKernelFreeAlgList(list);
    list_count = 0;
    
    Uninstall3DESModule();
    result = BCKernelListAlgs(&list, &list_count);
    CU_ASSERT(NULL == list && 0 == list_count);
    UninstallBCKernel();
}

static int AlgsEqual(BCAlgInfo *first, BCAlgInfo *second) // Expect pointers to not copy structures.
{
    CU_ASSERT(first != NULL);
    CU_ASSERT(second != NULL);
    return first->id == second->id
        && !strncmp(first->name, second->name, kBCKernel_AlgNameSize)
        && first->key_length == second->key_length
        && first->block_size == second->block_size
        && first->cbc_key_size == second->cbc_key_size
        && first->lrw_key_size == second->lrw_key_size
        && first->xts_key_size == second->xts_key_size;
}

void TestBCKernelLookupAlgIdName()
{
    // Assume here that list_algs already tested and working.
    BCAlgInfo *list = NULL;
    BCAlgInfo *alg_wrong_ptr = NULL;
    BCAlgInfo alg;
    unsigned int list_count = 0;
    bc_kern_result result;
    
    alg.id = 0;
    strncpy(alg.name, "Noname", kBCKernel_AlgNameSize);
    
    result = BCKernelLookupAlgId(123, &alg);
    CU_ASSERT(kBCKernel_NotInitialized == result);
    
    result = BCKernelLookupAlgName("Test alg", &alg);
    CU_ASSERT(kBCKernel_NotInitialized == result);
    
    InstallBCKernel();
    
    result = BCKernelLookupAlgId(123, &alg);
    CU_ASSERT(kBCKernel_UnknownAlg == result);
    
    result = BCKernelLookupAlgName("Test name", &alg);
    CU_ASSERT(kBCKernel_UnknownAlg == result);
    
    result = BCKernelLookupAlgId(123, alg_wrong_ptr);
    CU_ASSERT(kBCKernel_InvalidBuffer == result);
    
    result = BCKernelLookupAlgName("Test name", alg_wrong_ptr);
    CU_ASSERT(kBCKernel_InvalidBuffer == result);
    
    if (Install3DESModule())
    {
        UninstallBCKernel();
        CU_FAIL_FATAL("Cannot install 3DES module.");
    }
    
    BCKernelListAlgs(&list, &list_count);
    if (NULL != list && 1 == list_count)
    {
        BCAlgInfo des = list[0];
        const char *name = des.name;
        const int id = des.id;
        DEBUG_MSG("\nWill look for id = %i and name = %s\n", id, name);
        result = BCKernelLookupAlgId(id, &alg);
        CU_ASSERT(kBCKernel_Success == result);
        DEBUG_MSG("\nlookup_alg_id result: %s\n", get_result_string(result));
        CU_ASSERT(AlgsEqual(&alg, &des));
        
        result = BCKernelLookupAlgName(name, &alg);
        CU_ASSERT(kBCKernel_Success == result);
        DEBUG_MSG("\nlookup_alg_name result: %s\n", get_result_string(result));
        CU_ASSERT(AlgsEqual(&alg, &des));
    }
    else
    {
        CU_FAIL("Unexpected list came.\n");
    }
    Uninstall3DESModule();
    UninstallBCKernel();
}

#endif //TESTS_ALG_LOOKUP
