#ifndef TESTS_ENCRYPTION_ALG_BASE
#define TESTS_ENCRYPTION_ALG_BASE
#include "../tests-common.c"

void TestAlgBase(const char *alg_name, bc_mode_id mode, const char *ko_name, const char *plaintext, 
    const char *ciphertext, size_t text_size, const char *key, size_t key_size, uint64_t iv, const char *pool)
{
    bc_kern_result result;
    char *buf = NULL;
    BCAlgInfo alg;
    
    InstallBCKernel();
    if (InstallModule(ko_name))
    {
        UninstallBCKernel();
        CU_FAIL_FATAL("Cannot install cipher module.");
    }

    result = BCKernelIsInitialized();
    ASSERT_RESULT(kBCKernel_Success, result);
    
    result = BCKernelLookupAlgName(alg_name, &alg);
    ASSERT_RESULT(kBCKernel_Success, result);

    buf = (char*) malloc(text_size);
    memcpy(buf, plaintext, text_size);
    result = BCKernelProcessWithKey(alg.id, mode, kBCProcess_Encrypt, 
                                   key, key_size, iv, pool, 
                                   buf, text_size);
                                   
    ASSERT_RESULT(kBCKernel_Success, result);
    
    CU_ASSERT(0 == memcmp(buf, ciphertext, text_size));
    result = BCKernelProcessWithKey(alg.id, mode, kBCProcess_Decrypt, 
                                   key, key_size, iv, pool, 
                                   buf, text_size);
    ASSERT_RESULT(kBCKernel_Success, result);
    CU_ASSERT(0 == memcmp(buf, plaintext, text_size));
    free(buf);
    
    if (UninstallModule(ko_name))
    {
        CU_FAIL_FATAL("Cannot uninstall kernel module.");
    }
    
    UninstallBCKernel();
}

#endif
