#include "keygens/kgsha/hash.h"
#include "keygens/kgsha/datablock.h"
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

static void WriteToFile(const char *buffer, uint64_t bufferSize, const char *fileName)
{
    FILE *f;
    uint64_t i;

    printf("Writing result of %lu bytes to '%s'...\n", bufferSize, fileName);
    f = fopen(fileName, "w");
    for (i = 0; i < bufferSize; i++)
    {
        if (i % 16 == 0)
            fprintf(f, "\n");
        fprintf(f, "0x%02hhx, ", buffer[i]);
    }
    fclose(f);
}

static void GetRandomString(char *buffer, uint64_t size)
{
    int fd = open("/dev/urandom", O_RDONLY);
    read(fd, buffer, size);
    close(fd);
}

int main()
{
    const int PasswordLength = 128;
    DATA_BLOCK dataBlock;
    int algId               = 208;
    int hashId              = kSHA256_ID;
    int i                   = 0;
    bool hidden             = false;
    unsigned int dataOffset = 0;
    kgsha_key key;
    char passwordBuffer[PasswordLength];
    kgsha_password password;

    GetRandomString(passwordBuffer, PasswordLength);

    WriteToFile(passwordBuffer, PasswordLength, "password.txt");

    char seed[kKGSHA_SeedSize];

    GetRandomString(seed, kKGSHA_SeedSize);

    WriteToFile(seed, kKGSHA_SeedSize, "seed.txt");
    
    DataBlock_Init(&dataBlock, algId, hashId);

    DataBlock_GenerateKey(&dataBlock, seed, hidden, dataOffset, &key);

    WriteToFile(key.data, kKGSHA_KeySize, "key.txt");

    DataBlock_GenerateStorageKey(&dataBlock, passwordBuffer, PasswordLength, &password);

    WriteToFile(password.storage_key, kKGSHA_KeySize, "storage-key.txt");

    return 0;
}
