#include "../tests-common.c"

#include "keygens/kgsha/hash.h"
#include "keygens/kgsha/datablock.h"
#include "container/cipher.h"
#include "kernel/BCKernel.h"

void TestDataBlockFunctions()
{
    try
    {
        DATA_BLOCK dataBlock;
        int algId = 208; // TripleDES
        int hashId = kSHA256_ID;
        int i = 0;
        bc_mode_id mode = kBCMode_CBC;
        unsigned emptyBlockCount = 0;
        const char passwordData[] = 
        {
            0xde, 0xfb, 0xea, 0xd4, 0x39, 0xa4, 0x3f, 0xfb, 0x73, 0xfc, 0x70, 0x7c, 0x81, 0x8c, 0x69, 0x0a, 
            0x5b, 0xd0, 0xc6, 0x58, 0xd7, 0x2d, 0xae, 0xdf, 0xb5, 0x9f, 0x47, 0x9c, 0x0b, 0x0e, 0x47, 0x20, 
            0x7f, 0xcc, 0x05, 0x01, 0x09, 0xb1, 0x6a, 0x57, 0xe1, 0xec, 0x2e, 0x78, 0x96, 0xd9, 0xf3, 0xfc, 
            0x53, 0x3f, 0x75, 0x02, 0x45, 0xaf, 0xb5, 0xa2, 0x23, 0x76, 0x0c, 0xef, 0xd3, 0xd5, 0x1a, 0x1b, 
            0x22, 0x5a, 0xf8, 0x53, 0xe0, 0x1e, 0xf3, 0xe7, 0xdf, 0x56, 0x19, 0x48, 0x5a, 0xa7, 0x0d, 0x10, 
            0x04, 0xe2, 0xf8, 0x3f, 0xf7, 0x12, 0xed, 0x68, 0x8a, 0x9b, 0x6f, 0xe4, 0xe8, 0x02, 0x48, 0xeb, 
            0x38, 0x9c, 0x10, 0xbc, 0xa3, 0x86, 0x19, 0x72, 0x1c, 0x5d, 0x88, 0x6b, 0x3f, 0xb4, 0xeb, 0x01, 
            0x35, 0x01, 0xa8, 0x13, 0x65, 0x8a, 0x9c, 0xe8, 0x9a, 0xf3, 0x59, 0x26, 0xae, 0x4e, 0x0b, 0x99   
        };
        const unsigned passwordLength = 128;
        const char seedData[] = 
        {
            0xfa, 0xec, 0x7e, 0x1f, 0x6e, 0x09, 0xca, 0x80, 0x5e, 0x9f, 0x46, 0xa9, 0xf5, 0x40, 0xb4, 0xb2, 
            0xd2, 0xc6, 0x83, 0x93, 0x05, 0x89, 0x23, 0x78, 0xe3, 0x58, 0x10, 0x6c, 0xaf, 0x60, 0x1f, 0x0b, 
            0x8b, 0x2c, 0x81, 0x58, 0x12, 0x32, 0x59, 0x1c, 0x31, 0x93, 0x42, 0x5e, 0xc6, 0x83, 0x4e, 0xaa, 
            0x45, 0x3c, 0x03, 0xee, 0x1c, 0x05, 0x73, 0xc1, 0x3f, 0xd4, 0xe8, 0x63, 0x76, 0x1e, 0xe0, 0xd9, 
            0xe9, 0xb7, 0xc3, 0x23, 0x17, 0x1e, 0x83, 0x8e, 0x91, 0x11, 0xd6, 0xeb, 0x14, 0x4d, 0x49, 0x8b, 
            0xf0, 0xb1, 0x2d, 0x46, 0x27, 0x44, 0x2c, 0x91, 0xb9, 0x9f, 0x86, 0x0b, 0xed, 0x43, 0xa9, 0x01, 
            0x78, 0x97, 0x94, 0x43, 0x40, 0xfa, 0x98, 0x31, 0x3a, 0x64, 0x4c, 0xc1, 0xad, 0x9f, 0x7c, 0x13, 
            0xf2, 0xd3, 0x6a, 0x20, 0xd0, 0x81, 0xa8, 0x7a, 0x5f, 0xdd, 0x37, 0x1b, 0x58, 0x23, 0x56, 0x63
        };
        const unsigned seedSize = 128;
        const char keyData[] = 
        {
            0x87, 0xa9, 0xbe, 0x37, 0x22, 0x04, 0xe5, 0x0e, 0xfd, 0x42, 0x13, 0x79, 0xd7, 0xae, 0xef, 0x55, 
            0x09, 0x43, 0xf8, 0x44, 0x52, 0x40, 0xcf, 0x0d, 0xe8, 0xbd, 0xda, 0xaa, 0xd8, 0x38, 0xb6, 0xa6, 
            0xab, 0x1a, 0x14, 0xdf, 0x86, 0x41, 0x1f, 0x9d, 0x44, 0xa3, 0x03, 0xa0, 0xfb, 0x7a, 0x80, 0x1c, 
            0x2b, 0x68, 0xdd, 0xb1, 0xea, 0x7b, 0xbf, 0x2c, 0x9b, 0xdc, 0x18, 0x31, 0x98, 0xb0, 0x3e, 0x85   
        };
        const unsigned keyLength = 64;
        const char storageKeyData[] = 
        {
            0xdf, 0x79, 0x6d, 0x40, 0xa2, 0xca, 0xd4, 0x72, 0xb3, 0x89, 0x49, 0x0d, 0x3b, 0xc4, 0xa1, 0xca, 
            0xed, 0x2b, 0x4f, 0x47, 0x3e, 0xb5, 0xe3, 0x40, 0x5e, 0x3c, 0xb0, 0x13, 0xa7, 0x95, 0xf7, 0xfb, 
            0x53, 0xb6, 0x37, 0x2d, 0x60, 0x59, 0x1d, 0x1c, 0xc1, 0xe9, 0xf3, 0xbb, 0x49, 0xbb, 0x80, 0xad, 
            0x32, 0x47, 0x4f, 0x19, 0x3c, 0x69, 0xf5, 0xcc, 0x33, 0xbc, 0x8f, 0xde, 0x6a, 0xa5, 0xc9, 0xfe
        };
        const unsigned storageKeyLength = 64;
        char pool[kKGSHA_PoolSize];
        
        kgsha_key key;
        kgsha_password password;
        KEY_BLOCK *keyBlock;
        BCCipher cipher;

        InstallAllModules();

        CU_ASSERT(!cipher.init(algId, mode));
        DataBlock_Init(&dataBlock, algId, hashId);

        // Check correct parameter initialization.
        CU_ASSERT(dataBlock.header.hashAlgorithmId == hashId);
        CU_ASSERT(dataBlock.header.encrAlgorithmId == algId);
        
        // Check digest verification working.
        CU_ASSERT(DataBlock_Verify(&dataBlock));
        
        // Check empty block count.
        emptyBlockCount = DataBlock_CountFreeKeyBlocks(&dataBlock);
        ASSERT_INT(kKGSHA_TotalKeys, emptyBlockCount);

        // Check key generation.
        DataBlock_GenerateKey(&dataBlock, seedData, false, 0, &key);
        CU_ASSERT(0 == memcmp(keyData, key.data, kKGSHA_KeySize));

        // Check correct storage key generation.
        DataBlock_GenerateStorageKey(&dataBlock, passwordData, passwordLength, &password);
        CU_ASSERT(0 == memcmp(storageKeyData, password.storage_key, kKGSHA_KeySize));

        // Check that storing data block is correct and verification block is added.
        keyBlock = DataBlock_StoreKeyWithPassword(&dataBlock, &key, &password, &cipher);
        CU_ASSERT(keyBlock == &(dataBlock.keys[0]));
        CU_ASSERT(!KeyBlock_IsEmpty(keyBlock));
        CU_ASSERT(KeyBlock_IsVerificationBlock(&(dataBlock.keys[1])));

        emptyBlockCount = DataBlock_CountFreeKeyBlocks(&dataBlock);
        CU_ASSERT((kKGSHA_TotalKeys - 2) == emptyBlockCount);
        CU_ASSERT(DataBlock_VerifyPassword(&dataBlock, &password, &cipher) == keyBlock);

        // Check that can load stored block correctly.
        keyBlock = DataBlock_LoadKeyWithPassword(&dataBlock, &password, &cipher, &key, pool);
        CU_ASSERT(keyBlock == &(dataBlock.keys[0]));
        CU_ASSERT(0 == memcmp(keyData, key.data, kKGSHA_KeySize));
        // Pool is creating inside DataBlock_GenerateKey and is not accessible 
        // until key block is being loaded.

        // Erasing test.
        KeyBlock_Erase(keyBlock);
        CU_ASSERT(KeyBlock_IsEmpty(keyBlock));
        keyBlock = DataBlock_LoadKeyWithPassword(&dataBlock, &password, &cipher, &key, pool);
        CU_ASSERT(NULL == keyBlock);

        // Key buffer overflow handling.
        emptyBlockCount = DataBlock_CountFreeKeyBlocks(&dataBlock);
        CU_ASSERT((kKGSHA_TotalKeys - 1) == emptyBlockCount);
        for (i = 0; i < emptyBlockCount; i++)
        {
            // In level of DataBlock_StoreKeyWithPassword there is no check that password is unique,
            // so simply insert same key multiple times.
            keyBlock = DataBlock_StoreKeyWithPassword(&dataBlock, &key, &password, &cipher);
            CU_ASSERT(NULL != keyBlock);
        }
        keyBlock = DataBlock_StoreKeyWithPassword(&dataBlock, &key, &password, &cipher);
        CU_ASSERT(NULL == keyBlock);

        // Check that after all data block is correct.
        CU_ASSERT(DataBlock_Verify(&dataBlock));

        // Remove all content from data block.
        DataBlock_Init(&dataBlock, algId, hashId);

        emptyBlockCount = DataBlock_CountFreeKeyBlocks(&dataBlock);
        CU_ASSERT(kKGSHA_TotalKeys == emptyBlockCount);

        key.is_hidden = false;

        // Store dummy key in data block. Use key data to change password.
        memcpy(password.storage_key, keyData, (keyLength > storageKeyLength)? storageKeyLength : keyLength);
        keyBlock = DataBlock_StoreKeyWithPassword(&dataBlock, &key, &password, &cipher);
        emptyBlockCount = DataBlock_CountFreeKeyBlocks(&dataBlock);
        CU_ASSERT((kKGSHA_TotalKeys - 2) == emptyBlockCount);

        key.is_hidden = true;
        memcpy(password.storage_key, storageKeyData, storageKeyLength);
        keyBlock = DataBlock_StoreKeyWithPassword(&dataBlock, &key, &password, &cipher);
        // Hidden key must be now in the last key block...
        CU_ASSERT(keyBlock == &(dataBlock.keys[kKGSHA_TotalKeys - 1]));
        // ...and it must be marked as empty...
        CU_ASSERT(KeyBlock_IsEmpty(keyBlock));
        // ...and empty block count must not change...
        emptyBlockCount = DataBlock_CountFreeKeyBlocks(&dataBlock);
        CU_ASSERT(kKGSHA_TotalKeys - 2 == emptyBlockCount);
        // ...and it must be able to load key...
        keyBlock = DataBlock_LoadKeyWithPassword(&dataBlock, &password, &cipher, &key, pool);
        CU_ASSERT(NULL != keyBlock);
        CU_ASSERT(keyBlock == &(dataBlock.keys[kKGSHA_TotalKeys - 1]));
    }
    catch(...)
    {
        CU_FAIL_FATAL("Unknown exception while testing DataBlock functions.");
    }
    UninstallAllModules();
    
}
