#ifndef TESTS_COMMON_C
#define TESTS_COMMON_C

#include <CUnit/CUnit.h>
#include <stdlib.h>
#include <string.h>
#include "../kernel/BCKernel.h"

#define ASSERT_RESULT(expected, result) if (result != expected) { printf("\n[FAILED, line %i] Actual result '%s', expected '%s'.\n", __LINE__, BCKernelGetErrorString(result), BCKernelGetErrorString(expected)); CU_FAIL(result); }

#define ASSERT_INT(expected, actual) if (actual != expected) { printf("\n[FAILED, line %i] Actual result %i, expected %i.\n", __LINE__, actual, expected); CU_FAIL(""); }

#define ADD_TEST(suite, test_name, test_func) \
    if (NULL == CU_add_test(suite, test_name, test_func)) \
    { \
        return ProcessError(); \
    }

void InstallAllModules(void)
{
    system("sudo sh insert-modules.sh");
}


void UninstallAllModules(void)
{
    system("sudo sh remove-modules.sh");
}


int IsKernelLoaded(void)
{
    return system("bash bcmod-loaded.sh");
}


void InstallBCKernel(void)
{
    if (IsKernelLoaded())
    {
        UninstallAllModules();
    }
    if (system("sudo insmod ../kernel/bestcrypt.ko") || system("sudo chmod +r /dev/bestcrypt"))
    {
        CU_FAIL_FATAL("Cannot install BestCrypt kernel module.\n");
    }
}


void UninstallBCKernel(void)
{
    if (system("sudo rmmod bestcrypt"))
    {
        CU_FAIL_FATAL("Cannot remove BestCrypt kernel module.\n");
    }
}


int Install3DESModule(void)
{
    return system("sudo insmod ../kernel/bc_3des.ko");
}


int Uninstall3DESModule(void)
{
    return system("sudo rmmod bc_3des");
}


int InstallAESModule(void)
{
    return system("sudo insmod ../kernel/bc_rijn.ko");
}


int UninstallAESModule(void)
{
    return system("sudo rmmod bc_rijn");
}


int InstallModule(const char *ko_name)
{
    CU_ASSERT_FATAL(NULL != ko_name);
    const int buf_size = 50;
    char cmd_buffer[buf_size];
    snprintf(cmd_buffer, buf_size - 1, "sudo insmod ../kernel/%s", ko_name);
    cmd_buffer[buf_size - 1] = '\0';
    return system(cmd_buffer);
}


int UninstallModule(const char *ko_name)
{
    const int buf_size = 50;
    char cmd_buffer[buf_size];
    char name_buf[buf_size];
    const char *point_ptr = strchr(ko_name, '.');
    CU_ASSERT_FATAL(NULL != point_ptr);
    size_t point_position = point_ptr - ko_name;
    CU_ASSERT_FATAL(buf_size > point_position + 1);
    strncpy(name_buf, ko_name, point_position);
    name_buf[point_position] = '\0';
    
    snprintf(cmd_buffer, buf_size - 1, "sudo rmmod %s", name_buf);
    return system(cmd_buffer);
}

#endif

