#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <CUnit/Basic.h>

#include "../kernel/BCKernel.h"
#include "kernel/tests-kernel.c"
#include "kgsha/tests-kgsha.c"
#include "container/tests-container.cpp"
#include "spacemap/tests-spacemap.c"

#define ADD_SUITE(AddSuiteFunc) if (AddSuiteFunc) return ProcessError();

int Dummy0(void)
{
    return 0;
}

int ProcessError()
{
    CU_cleanup_registry();
    printf("CUnit Error: %s\n", CU_get_error_msg());
    return CU_get_error();
}

void test_failed(void)
{
    CU_ASSERT(0);
}

/*******************************************************************************************************************************
********************************************************************************************************************************
*******************************************************************************************************************************/

int AddSuiteForBCKernel()
{
    CU_pSuite suite = CU_add_suite("BCKernel Test Suite", Dummy0, Dummy0);
    
    if (NULL == suite)
    {
        return -1;
    }
    
    ADD_TEST(suite, "BCKernel Init Test", TestBCKernelIsInitialized);
    
    ADD_TEST(suite, "BCKernel Struct Sizes Test", TestStructSize);

    ADD_TEST(suite, "BCKernel Alg List Test", TestBCKernelListAlgs);
    
    ADD_TEST(suite, "BCKernel Alg Id & Name Lookup Test", TestBCKernelLookupAlgIdName);
    
    ADD_TEST(suite, "BCKernel Store & Free Key Test", TestBCKernelStoreFreeKey);

    ADD_TEST(suite, "BCKernel Encryption Common Test", TestBCKernelEncryptionCommon);
    
    ADD_TEST(suite, "BCKernel Encryption Test for RIJNDAEL", TestEncryptionRijn);
    
    ADD_TEST(suite, "BCKernel Encryption Test for 3DES", TestEncryption3DES);
    
    ADD_TEST(suite, "BCKernel Encryption Test for BlowFish", TestEncryptionBlowFish);
    
    ADD_TEST(suite, "BCKernel Encryption Test for BlowFish-128", TestEncryptionBlowFish128);
    
    ADD_TEST(suite, "BCKernel Encryption Test for BlowFish-448", TestEncryptionBlowFish448);
    
    ADD_TEST(suite, "BCKernel Encryption Test for CAST", TestEncryptionCAST);
    
    ADD_TEST(suite, "BCKernel Encryption Test for DES", TestEncryptionDES);
    
    ADD_TEST(suite, "BCKernel Encryption Test for GOST", TestEncryptionGOST);
    
    ADD_TEST(suite, "BCKernel Encryption Test for Idea", TestEncryptionIDEA);
    
    return 0;
}

int AddSuiteForKgsha()
{
    CU_pSuite suite = CU_add_suite("KeyGenerator kgSHA Test Suite", Dummy0, Dummy0);

    if (NULL == suite)
    {
        return -1;
    }

    ADD_TEST(suite, "KGSHA DataBlock Test", TestDataBlockFunctions);

    return 0;
}

int AddSuiteForContainer()
{
    CU_pSuite suite = CU_add_suite("Container Functions Test Suite", Dummy0, Dummy0);
    if (NULL == suite)
    {
        return -1;
    }

    ADD_TEST(suite, "Container File Test", TestContainerFile);

    return 0;
}

int AddSuiteForSpaceMap()
{
    CU_pSuite suite = CU_add_suite("Space Map Test Suite", Dummy0, Dummy0);
    if (NULL == suite)
        return -1;

    ADD_TEST(suite, "Extents Test", TestExtentFunctions);

    ADD_TEST(suite, "Space Map", TestSpaceMap);

    return 0;
}

int main()
{
    CU_ErrorCode error = CU_initialize_registry();
    if (CUE_SUCCESS != error)
        return ProcessError();

    //ADD_SUITE(AddSuiteForBCKernel());
    //ADD_SUITE(AddSuiteForKgsha());
    ADD_SUITE(AddSuiteForContainer());
    ADD_SUITE(AddSuiteForSpaceMap());

    CU_basic_set_mode(CU_BRM_VERBOSE);
    error = CU_basic_run_tests();
    CU_basic_show_failures(CU_get_failure_list());
    
    printf("\n");
    
    int number_failed = CU_get_number_of_tests_failed();
    CU_cleanup_registry();
    return -number_failed;
}
